/**
 * Created by janis dähne.
 */

import * as genericLayer from './genericLayer'
import {
  CustomProjectDescriptionFullBase, CustomProjectFullBase, CustomProjectPropertiesFromFrontend,
  CustomProjectSolutionFullBase, CustomProjectTestFullBase
} from "../types/customProjects";
import {StoredTestResultFromBackend} from "../types/testResults";


const callingName = 'editCustomProjectLayer'
const controllerPrefix = 'custom/projects/edit'


export async function getCustomProject(customProjectId: number, shouldLoadTestAssetContents: boolean): Promise<CustomProjectFullBase> {
  return genericLayer.get<CustomProjectFullBase>(callingName,
    `${controllerPrefix}/${customProjectId}/${shouldLoadTestAssetContents}`)
}



export async function createCustomProject(customProject: CustomProjectFullBase): Promise<number> {
  return genericLayer.update<CustomProjectFullBase, number>(callingName,
    `${controllerPrefix}/create`, customProject)
}

export async function getOwnCustomProjectTestResults(customProjectId: number, pLangId: number): Promise<ReadonlyArray<StoredTestResultFromBackend>> {
  return genericLayer.get<ReadonlyArray<StoredTestResultFromBackend>>(callingName,
    `${controllerPrefix}/${customProjectId}/testresults/${pLangId}`)
}


export async function updateCustomProjectProperties(customProjectId: number, customProjectProperties : CustomProjectPropertiesFromFrontend): Promise<void> {
  return genericLayer.update<CustomProjectPropertiesFromFrontend, void>(callingName,
    `${controllerPrefix}/${customProjectId}/properties`, customProjectProperties)
}

export async function updateCustomProjectSolution(customProjectId: number, solution: CustomProjectSolutionFullBase): Promise<CustomProjectSolutionFullBase> {
  return genericLayer.update<CustomProjectSolutionFullBase, CustomProjectSolutionFullBase>(callingName,
    `${controllerPrefix}/update/${customProjectId}/solution`, solution)
}

export async function updateCustomProjectDescription(customProjectId: number, description: CustomProjectDescriptionFullBase): Promise<void> {
  return genericLayer.update<CustomProjectDescriptionFullBase, void>(callingName,
    `${controllerPrefix}/update/${customProjectId}/description`, description)
}


export async function createCustomProjectTest(customProjectId: number, test: CustomProjectTestFullBase): Promise<CustomProjectTestFullBase> {
  return genericLayer.update<CustomProjectTestFullBase, CustomProjectTestFullBase>(callingName,
    `${controllerPrefix}/update/${customProjectId}/tests/create`, test)
}


export async function updateCustomProjectTest(customProjectId: number, test: CustomProjectTestFullBase): Promise<CustomProjectTestFullBase> {
  return genericLayer.update<CustomProjectTestFullBase, CustomProjectTestFullBase>(callingName,
    `${controllerPrefix}/update/${customProjectId}/tests/update`, test)
}


export async function deleteCustomProjectTest(customProjectId: number, testId: number): Promise<void> {
  return genericLayer.del<void>(callingName,
    `${controllerPrefix}/update/${customProjectId}/tests/delete/${testId}`)
}