/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../state/reducers/index";

//const css = require('./styles.styl');

export interface MyProps {

  /**
   * the left area
   */
  readonly leftArea?: JSX.Element
  /**
   * the middle area
   */
  readonly middleArea?: JSX.Element
  /**
   * the right area
   */
  readonly rightArea?: JSX.Element

  /**
   * true: disable interaction,
   * false: not
   */
  readonly disabled?: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

/**
 * a common header bar with 3 areas (left, middle, right) for a site (a bar under the main app header bar)
 * you might want to use SimpleVDivider inside the areas
 *
 * this is normally used inside the header bars from the sites
 */
class CommonHeaderBarContentWrapper extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className={'flexed fh full-w' + (this.props.disabled ? ' div-disabled' : '')}>
        <div className="flexed fh full-w">
          <div className="site-content-header--left">
            {this.props.leftArea}
          </div>


          <div className="site-content-header--middle">
            {this.props.middleArea}
          </div>

          <div className="site-content-header--right">
            {this.props.rightArea}
          </div>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(CommonHeaderBarContentWrapper)