/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";
import {Form, Checkbox, Radio} from "semantic-ui-react";
import {
  CheckboxOnChangeEventData, InputOnChangeEvent, RadioOnChangeEventData,
  SelectOnChangeEventData
} from "../../types/reactEvents";
import MaterialInput from '../material/materialInput'
import MaterialSelectWrapper from '../material/materialSelectWrapper'
import {validationMessageKeys, validationRules} from "../../state/reducers/editorSettingsDialog/editorSettingsReducer";
import {
  setFontSize, setHighlightCurrentLine,
  setShowInvisibles, setShowLineIndentions, setShowLineNumbers, setTabSize, setTheme, setUseWrapping
} from "../../state/actions/editorSettingsDialog/editorSettingsActions";
import {parseNotNegativeOrNullNumber} from "../../helpers/convertersAndTransformers";
import {availableEditorThemeNames, InternalCodeEditorThemes} from "../../helpers/themes";
import {translate} from "../../../i18n/translationHelper";
import {TranslateIds} from "../../../i18n/ids";
import {errorDialog} from "../../helpers/dialogHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    editorSettings: rootState.editorSettingsDialogState.editorSettings
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setFontSize,
  setHighlightCurrentLine,
  setShowInvisibles,
  setShowLineNumbers,
  setTabSize,
  setUseWrapping,
  setShowLineIndentions,
  setTheme
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorSettingsView extends React.Component<Props, any> {
  render(): JSX.Element {

    const isThemeValid = validationRules.theme.every(validator => validator(this.props.editorSettings.theme))

    return (
      <div>

        <div className="ui form divided-checkboxes">
          <Form.Group widths='equal'>

            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.theme)
                }
              </label>

              <MaterialSelectWrapper
                value={this.props.editorSettings.theme}
                validations={validationRules.theme}
                validationMessageKey={validationMessageKeys.theme}
              >
                <select
                  value={this.props.editorSettings.theme}
                  onChange={(e: SelectOnChangeEventData) => {
                    this.props.setTheme(e.currentTarget.value)
                  }}
                >
                  <optgroup label={translate(TranslateIds.light)}>

                    <option key={InternalCodeEditorThemes.light} value={InternalCodeEditorThemes.light}>
                      {
                        translate(TranslateIds.defaultLightTheme)
                      }
                    </option>

                  </optgroup>

                  <optgroup label={translate(TranslateIds.dark)}>

                    <option key={InternalCodeEditorThemes.dark} value={InternalCodeEditorThemes.dark}>
                      {
                        translate(TranslateIds.defaultDarkTheme)
                      }
                    </option>

                  </optgroup>

                </select>
              </MaterialSelectWrapper>
            </Form.Field>


            <Form.Field required>
              <label>{translate(TranslateIds.fontSize)}</label>
              <MaterialInput type="number"
                             value={this.props.editorSettings.fontSize.toString()}
                             onChange={(e: InputOnChangeEvent) => {

                               const fontSize = parseNotNegativeOrNullNumber(e.currentTarget.value)

                               if (fontSize === null) {

                                 errorDialog(translate(TranslateIds.error),
                                   translate(TranslateIds.couldNotConvertStringToInt)
                                 )

                               } else {
                                 this.props.setFontSize(fontSize)
                               }
                             }}
                             validations={validationRules.fontSize}
                             validationMessageKey={validationMessageKeys.fontSize}
              />
            </Form.Field>

            <Form.Field required>
              <label>{translate(TranslateIds.tabSize)}</label>
              <div className="material-padding">

                <Radio className="mar-right"
                       checked={this.props.editorSettings.tabSize === 2}
                       label="2"
                       value={2}
                       onChange={(e, data: RadioOnChangeEventData) => {
                         this.props.setTabSize(2)
                       }}
                />

                <Radio
                  checked={this.props.editorSettings.tabSize === 4}
                  label="4"
                  value={4}
                  onChange={(e, data: RadioOnChangeEventData) => {
                    this.props.setTabSize(4)
                  }}
                />
              </div>

            </Form.Field>

          </Form.Group>

          {
            /*
           cannot get this to work with ace editor props (method call could work...)... (it should work somehow though...)
            <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.showLineNumbers}
                      label={translate(TranslateIds.showLineNumbers)}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setShowLineNumbers(data.checked)
                      }}
            />
          </Form.Field>
           */
          }


          {
            /* no editor support
            <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.highlightCurrentLine}
                      label="Aktuelle Zeile hervorheben"
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setHighlightCurrentLine(data.checked)
                      }}
            />
          </Form.Field>
             */
          }

          <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.useWrapping}
                      label={`${translate(TranslateIds.useWrapping)} (${translate(TranslateIds.useWrapping_desc)})`}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setUseWrapping(data.checked)
                      }}
            />
          </Form.Field>

          {
            /*
            cannot get this to work with ace editor props (method call could work...)... (it should work somehow though...)
            <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.showLineIndentions}
                      label={translate(TranslateIds.showLineIndentations)}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setShowLineIndentions(data.checked)
                      }}
            />
          </Form.Field>
             */
          }


          {
            /* no editor support
            <Form.Field>
            <Checkbox toggle
                      checked={this.props.editorSettings.showInvisibles}
                      label="Nicht druckbare Zeichen anzeigen"
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setShowInvisibles(data.checked)
                      }}
            />
          </Form.Field>
             */
          }


        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorSettingsView)