/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";
import {SiteViewState} from "../../types/siteView";
import {Icon} from "semantic-ui-react";

//const css = require('./styles.styl');

export interface MyProps {
  /**
   * the view state from the parent to bind to
   */
  readonly viewState: SiteViewState
  /**
   * the change view state func from the parent
   * because every parent comp has a different set view state func
   * usage in the parent: (viewState) => setViewState(viewState)
   *
   * the parent must import the right setViewState on it's own (for the right store)
   */
  readonly setViewStateFunc: (viewSate: SiteViewState) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

/**
 * this component renders all possible view states as icons
 * lets the user change the view state | calls the this.props.setViewStateFunc
 * displays the currently selected view state | used this.props.viewState
 *
 * usage:
 *  <ViewStateChanger viewState={this.props.viewState} setViewStateFunc={(viewState) => this.props.setViewState(viewState)} />
 *
 *  where
 *    viewState comes from a redux store
 *    setViewState is a redux action
 *
 */
class ViewStateChanger extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed"
      >

        <div
          className={'clickable mar-right-half' +
          (this.props.viewState === SiteViewState.Cards ? ' toggled-active--color' : '')}
          onClick={() => {
            this.props.setViewStateFunc(SiteViewState.Cards)
          }}>
          <Icon name="grid layout"/>
        </div>


        <div
          className={'clickable' +
          (this.props.viewState === SiteViewState.List ? ' toggled-active--color' : '')}
          onClick={() => this.props.setViewStateFunc(SiteViewState.List)}>
          <Icon name="align justify"/>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ViewStateChanger)