/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {FormattedMessage} from 'react-intl';
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";
import {Icon, Popup} from "semantic-ui-react";
import SiteHeaderUserArea from './siteHeaderUserArea'
import SiteMenu from './siteMenu'
import {setIsSiteMenuOpen, setIsSiteHeaderBarCollapsed} from "../../state/actions/siteHeaderBarActions";
import RelatedLinksView from './relatedLinksView'
import {withRouter} from "react-router";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    siteMenuIsOpen: rootState.siteHeaderBarState.isSiteMenuOpen,
    isDisplayedAndEnabled: rootState.siteHeaderBarState.isSiteHeaderBarDisplayedAndEnabled,
    isSiteHeaderBarCollapsed: rootState.siteHeaderBarState.isSiteHeaderBarCollapsed
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsSiteMenuOpen,
  setIsSiteHeaderBarCollapsed
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class SiteHeaderBar extends React.Component<Props, any> {

  siteMenuOpened(): void {
    this.props.setIsSiteMenuOpen(true)
  }

  siteMenuClosed(): void {
    this.props.setIsSiteMenuOpen(false)
  }


  render(): JSX.Element {
    return (
      <div>
        {
          this.props.isDisplayedAndEnabled &&
          !this.props.isSiteHeaderBarCollapsed &&
          <div className="site-header-bar flexed card-shadow">
            <div className="site-header-bar--location flexed">
              <div className="mar-right clickable" onClick={() => {
                this.props.setIsSiteMenuOpen(!this.props.siteMenuIsOpen)
              }}>


                <Icon size="big" name="sidebar"></Icon>
                {
                  /*
                TODO ask if we should try the new menu

                */
                }

                {
                  /*
                  TODO ask if we should try the new menu

                <Popup
                  trigger={<Icon size="big" name="sidebar"></Icon>}
                  position='bottom left'
                  on='click'
                  open={this.props.siteMenuIsOpen}
                  onOpen={() => this.siteMenuOpened()}
                  onClose={() => this.siteMenuClosed()}
                >
                  <SiteMenu></SiteMenu>
                </Popup>
 */
                }
              </div>

              <RelatedLinksView/>
            </div>

            <div className="site-header-bar--status">
              {
                //currently not used
              }
            </div>

            <div className="site-header-bar--user-area">
              <SiteHeaderUserArea></SiteHeaderUserArea>
            </div>

            <div className="site-menu-collapse-arrow arrow-up" onClick={() => {
              this.props.setIsSiteHeaderBarCollapsed(true)
            }}>
            </div>

          </div>
        }

        {
          this.props.isDisplayedAndEnabled &&
          this.props.isSiteHeaderBarCollapsed &&
          <div className="site-menu-expand-arrow arrow-down" onClick={() => {
            this.props.setIsSiteHeaderBarCollapsed(false)
          }}>
          </div>
        }
      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(SiteHeaderBar))