/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../state/reducers/index";
import {Icon, Popup, List, Divider} from "semantic-ui-react";
import {setIsLoggerPanelOpen} from "../../state/actions/loggerPanelActions";
import {leftIconPopupOffset} from "../../constants";
import * as loginLayer from '../../communicationLayer/loginLayer'
import history from "../../routerHistory";
import * as constants from '../../constants'
import {Link} from "react-router-dom";
import {notifyError} from "../../helpers/notificationHelper";
import {translate} from "../../../i18n/translationHelper";
import {TranslateIds} from "../../../i18n/ids";
import {FrontendSettingsManager} from "../../helpers/frontendSettingsManager";
import {setIsUserMenuOpen} from "../../state/actions/siteHeaderBarActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    id: rootState.userDataSettingsSate.userData.id,
    firstName: rootState.userDataSettingsSate.userData.firstName,
    lastName: rootState.userDataSettingsSate.userData.lastName,
    systemRoleDisplayName: rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole.displayName
      : translate(TranslateIds.noSystemRole),
    systemRoleId: rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole.id
      : -1,
    isLoggerPanelOpen: rootState.loggerPanelState.isOpen,
    isUserMenuOpen: rootState.siteHeaderBarState.isUserMenuOpen
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsLoggerPanelOpen,

  setIsUserMenuOpen

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

const userMenuOffset = 5

class SiteHeaderUserArea extends React.Component<Props, any> {

  userMenuEntryClicked(): void {
    this.props.setIsUserMenuOpen(false)
  }

  render(): JSX.Element {
    return (
      <div className="flexed">
        <div className="mar-right">
          <span className="mar-right-half">
            {this.props.firstName}
          </span>
          <span>
            {this.props.lastName}
          </span>
        </div>

        <div className="clickable">
          <Popup
            className="site-user-menu"
            trigger={<Icon size="large" name="user circle"></Icon>}
            position="bottom right"
            on="click"
            offset={leftIconPopupOffset - userMenuOffset}
            open={this.props.isUserMenuOpen}
            onOpen={() => this.props.setIsUserMenuOpen(true)}
            onClose={() => this.props.setIsUserMenuOpen(false)}
          >
            <Popup.Content>
              <List>
                <List.Item icon="user circle" content={this.props.firstName + ' ' + this.props.lastName}/>

                <List.Item>
                  <List.Icon name="law"/>
                  <List.Content>
                    <span className="mar-right-half">
                      {
                        `${translate(TranslateIds.systemRole)}:`
                      }
                    </span>
                    <span>{this.props.systemRoleDisplayName}</span>
                  </List.Content>
                </List.Item>

                <List.Item>
                  <Link to={constants.assessmentStatisticsSiteToLinkPath + "/"}
                        onClick={() => this.userMenuEntryClicked()}
                  >
                    <Icon name='area chart' className="mar-right-half"/>
                    <span className="mar-left-half">
                        {
                          translate(TranslateIds.myAssessmentStatisticsForUser)
                        }
                        </span>
                  </Link>
                </List.Item>
                <Divider/>

                <List.Item className="clickable">
                  <Link to={constants.ownSettingsLinkPath} onClick={() => this.userMenuEntryClicked()}>
                    <Icon name="settings"/>
                    <span className="mar-left-half">
                      {
                        translate(TranslateIds.ownSettingsSite)
                      }
                    </span>
                  </Link>
                </List.Item>

                <List.Item className={'clickable' + (this.props.isLoggerPanelOpen ? ' accent-colored' : '')}
                           onClick={() => {
                             this.props.setIsLoggerPanelOpen(!this.props.isLoggerPanelOpen)
                             this.userMenuEntryClicked()
                           }}
                >
                  {
                    //we need the forced padding because .item > .list but the list class is only for the icon
                  }
                  <List.Icon name="unordered list" style={{paddingLeft: 0}} />
                  <List.Content>
                    <span>
                      {
                        translate(TranslateIds.log)
                      }
                    </span>
                  </List.Content>
                </List.Item>

                <Divider/>

                <List.Item className="clickable"
                           onClick={async () => {

                             try {
                               await loginLayer.logout()

                               FrontendSettingsManager.clear()

                             } catch (err) {
                               notifyError(translate(TranslateIds.logoutError))
                               return
                             }

                             this.userMenuEntryClicked()

                             history.replace(constants.homeLinkPath)
                             //window.location.reload(true) //forces to ignore the cache...
                             window.location.reload()

                           }}
                >
                  <List.Icon name="sign out"/>
                  <List.Content>
                    <span>
                      {
                        translate(TranslateIds.logout)
                      }
                    </span>
                  </List.Content>
                </List.Item>

              </List>
            </Popup.Content>
          </Popup>
        </div>
        {
          /*
          not yet used
          <div className="mar-right clickable">
          <Icon size="large" name="alarm"></Icon>
        </div>
           */
        }


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(SiteHeaderUserArea)