/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Divider} from "semantic-ui-react";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


enum LibraryCategory {
  /**
   * important/big ones
   */
  important = 0,
  /**
   * normal ones (side functionality)
   * @type {number}
   */
  normal = 1
}

interface LibraryInfo {
  readonly title: string
  readonly officialWebsite: string
  readonly sourceWebsite: string
  readonly license: string
  readonly licenseLink: string
  //readonly category: LibraryCategory //they are all important!
}

interface DesignInfo {
  readonly title: string
  readonly website: string
  /**
   * description what was used
   */
  readonly what?: string
}

interface PersonInfo {

  readonly name: string
  /**
   * what the person have done for the project
   */
  readonly what: string

}


const aboutInfo = {
  title: translate(TranslateIds.projectTitle),
  titleLong: 'Yet Another Practical EXercise Platform', //Yet Another Practical EXercise Platform
  description: translate(TranslateIds.projectDescription),
  createdBy: ['Janis Dähne'],
  cc: '2016-2018'
}

const projectPeopleInfo: ReadonlyArray<PersonInfo> = [
  {
    name: 'Janis Dähne',
    what: 'initial idea, initiator, programming'
  },
  {
    name: 'Steffen Schiele',
    what: 'initiator, testing, feature ideas'
  },
  {
    name: 'Annett Thüring',
    what: 'initiator, testing, feature ideas'
  }
]

const designInspirations: ReadonlyArray<DesignInfo> = [
  {
    title: 'Semantic UI',
    website: 'https://semantic-ui.com/'
  },
  {
    title: 'Material design',
    website: 'https://material.io/guidelines/',
  },
  {
    title: 'MacOS',
    website: 'https://www.apple.com/de/macos/sierra/',
    what: 'Layout'
  },
  {
    title: 'YoutTube',
    website: 'https://www.youtube.com/'
  },
  {
    title: 'CodeinGame',
    website: 'https://www.codingame.com/'
  },
  {
    title: 'GitLab',
    website: 'https://about.gitlab.com',
  },
  {
    title: 'Visual Studio Code',
    website: 'https://code.visualstudio.com',
  },
  {
    title: 'AppStore (install operation animation)',
    website: 'https://www.apple.com/de/ios/app-store/',
  },
  {
    title: 'Gradients 1',
    website: 'https://webgradients.com/'
  },
  {
    title: 'Gradients 2',
    website: 'https://www.gradient-animator.com/'
  }
]

const usedLibraries: ReadonlyArray<LibraryInfo> = [
  {
    title: 'React',
    officialWebsite: 'https://facebook.github.io/react/',
    sourceWebsite: 'https://github.com/facebook/react',
    license: 'BSD 3-clause',
    licenseLink: 'https://github.com/facebook/react/blob/master/LICENSE'
  },
  {
    title: 'React Router',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/ReactTraining/react-router',
    license: 'MIT',
    licenseLink: 'https://github.com/ReactTraining/react-router/blob/master/LICENSE'
  },
  {
    title: 'Webpack',
    officialWebsite: 'https://webpack.github.io/',
    sourceWebsite: 'https://github.com/webpack/webpack',
    license: 'MIT',
    licenseLink: 'https://github.com/webpack/webpack/blob/master/LICENSE',
  },
  {
    title: 'Stylus',
    officialWebsite: 'http://stylus-lang.com',
    sourceWebsite: 'https://github.com/stylus/stylus/',
    license: 'MIT',
    licenseLink: 'https://github.com/stylus/stylus/blob/dev/LICENSE',
  },
  {
    title: 'TypeScript',
    officialWebsite: 'https://www.typescriptlang.org',
    sourceWebsite: 'https://github.com/Microsoft/TypeScript',
    license: 'Apache-2.0',
    licenseLink: 'https://github.com/Microsoft/TypeScript/blob/master/LICENSE.txt',
  },
  {
    title: 'Redux',
    officialWebsite: 'https://redux.js.org',
    sourceWebsite: 'https://github.com/reactjs/redux',
    license: 'MIT',
    licenseLink: 'https://github.com/reactjs/redux/blob/master/LICENSE.md',
  },
  {
    title: 'React-Redux',
    officialWebsite: 'https://redux.js.org/docs/basics/UsageWithReact.html',
    sourceWebsite: 'https://github.com/reactjs/react-redux',
    license: 'MIT',
    licenseLink: 'https://github.com/reactjs/react-redux/blob/master/LICENSE.md',
  },
  {
    title: 'Awesome-Typescript-Loader',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/s-panferov/awesome-typescript-loader',
    license: 'MIT',
    licenseLink: 'https://github.com/s-panferov/awesome-typescript-loader/blob/master/LICENSE.md',
  },
  {
    title: 'Semantic UI React',
    officialWebsite: 'https://react.semantic-ui.com/introduction',
    sourceWebsite: 'https://github.com/Semantic-Org/Semantic-UI-React',
    license: 'MIT',
    licenseLink: 'https://github.com/Semantic-Org/Semantic-UI-React/blob/master/LICENSE.md',
  },
  {
    title: 'CryptoJS',
    officialWebsite: 'https://code.google.com/archive/p/crypto-js/',
    sourceWebsite: 'https://github.com/sytelus/CryptoJS',
    license: 'MIT',
    licenseLink: 'https://github.com/sytelus/CryptoJS/blob/master/copyright.txt',
  },
  {
    title: 'TsLint',
    officialWebsite: 'https://palantir.github.io/tslint/',
    sourceWebsite: 'https://github.com/palantir/tslint',
    license: 'Apache-2.0',
    licenseLink: 'https://github.com/palantir/tslint/blob/master/LICENSE',
  },
  {
    title: 'Axios',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/axios/axios',
    license: 'MIT',
    licenseLink: 'https://github.com/axios/axios/blob/master/LICENSE',
  },
  {
    title: 'Sweet Alert 2',
    officialWebsite: 'https://limonte.github.io/sweetalert2/',
    sourceWebsite: 'https://github.com/limonte/sweetalert2',
    license: 'MIT',
    licenseLink: 'https://github.com/limonte/sweetalert2/blob/master/LICENSE',
  },
  {
    title: 'Moment.js',
    officialWebsite: 'https://momentjs.com',
    sourceWebsite: 'https://github.com/moment/moment/',
    license: 'MIT',
    licenseLink: 'https://github.com/moment/moment/blob/develop/LICENSE',
  },
  {
    title: 'Mousetrap',
    officialWebsite: 'https://craig.is/killing/mice',
    sourceWebsite: 'https://github.com/ccampbell/mousetrap',
    license: 'Apache-2.0',
    licenseLink: 'https://github.com/ccampbell/mousetrap/blob/master/LICENSE',
  },
  {
    title: 'FileSaver.js',
    officialWebsite: 'https://eligrey.com/blog/saving-generated-files-on-the-client-side/',
    sourceWebsite: 'https://github.com/eligrey/FileSaver.js/',
    license: 'MIT',
    licenseLink: 'https://github.com/eligrey/FileSaver.js/blob/master/LICENSE.md',
  },
  {
    title: 'jsdiff',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/kpdecker/jsdiff',
    license: 'BSD-3-clause',
    licenseLink: 'https://github.com/kpdecker/jsdiff/blob/master/LICENSE',
  },
  {
    title: 'React resize detector',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/maslianok/react-resize-detector',
    license: 'MIT',
    licenseLink: 'https://github.com/maslianok/react-resize-detector/blob/master/README.md',
  },
  {
    title: 'React Notifications',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/minhtranite/react-notifications',
    license: 'MIT',
    licenseLink: 'https://github.com/minhtranite/react-notifications/blob/master/package.json',
  },
  {
    title: 'React Color',
    officialWebsite: 'https://casesandberg.github.io/react-color/',
    sourceWebsite: 'https://github.com/casesandberg/react-color/',
    license: 'MIT',
    licenseLink: 'https://github.com/casesandberg/react-color/blob/master/LICENSE',
  },
  {
    title: 'Recharts',
    officialWebsite: 'http://recharts.org/#/en-US/',
    sourceWebsite: 'https://github.com/recharts/recharts',
    license: 'MIT',
    licenseLink: 'https://github.com/recharts/recharts/blob/master/LICENSE',
  },
  {
    title: 'Select/Choose icon of FontAwesome as png',
    officialWebsite: 'http://fa2png.io/r/font-awesome/sort/',
    sourceWebsite: '-',
    license: '-',
    licenseLink: '-',
  },
  {
    title: 'Ace Editor',
    officialWebsite: 'https://ace.c9.io',
    sourceWebsite: 'https://github.com/ajaxorg/ace',
    license: 'BSD',
    licenseLink: 'https://github.com/ajaxorg/ace/blob/master/LICENSE',
  },
  {
    title: 'React-Ace',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/securingsincity/react-ace',
    license: 'MIT',
    licenseLink: 'https://github.com/securingsincity/react-ace/blob/master/LICENSE',
  },
  {
    title: 'Webpack Bundle Size Analyzer',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/robertknight/webpack-bundle-size-analyzer',
    license: 'ISC',
    licenseLink: 'https://github.com/robertknight/webpack-bundle-size-analyzer/blob/master/package.json',
  },
  {
    title: 'UglifyJs',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/mishoo/UglifyJS',
    license: 'BSD',
    licenseLink: 'https://github.com/mishoo/UglifyJS/blob/master/README.org',
  },
  {
    title: 'Lodash',
    officialWebsite: 'https://lodash.com',
    sourceWebsite: 'https://github.com/lodash/lodash',
    license: 'MIT',
    licenseLink: 'https://github.com/lodash/lodash/blob/master/LICENSE',
  },
  {
    title: 'Highlight.js',
    officialWebsite: 'https://highlightjs.org',
    sourceWebsite: 'https://github.com/isagalaev/highlight.js',
    license: 'BSD-3-clause',
    licenseLink: 'https://github.com/isagalaev/highlight.js/blob/master/LICENSE',
  },
  {
    title: 'marked',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/chjj/marked',
    license: 'MIT',
    licenseLink: 'https://github.com/chjj/marked/blob/master/LICENSE',
  },
  {
    title: 'Webstorm',
    officialWebsite: 'https://www.jetbrains.com/webstorm/',
    sourceWebsite: '-',
    license: '-',
    licenseLink: '-',
  },
  {
    title: 'PostStylus',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/seaneking/poststylus',
    license: 'MIT',
    licenseLink: 'https://github.com/seaneking/poststylus/blob/master/README.md',
  },
  {
    title: 'Autoprefixer',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/postcss/autoprefixer',
    license: 'MIT',
    licenseLink: 'https://github.com/postcss/autoprefixer/blob/master/LICENSE',
  },
  {
    title: 'Postcss',
    officialWebsite: 'http://postcss.org',
    sourceWebsite: 'https://github.com/postcss/postcss',
    license: 'MIT',
    licenseLink: 'https://github.com/postcss/postcss/blob/master/LICENSE',
  },

  {
    title: 'react-datetime',
    officialWebsite: '-',
    sourceWebsite: 'https://github.com/YouCanBookMe/react-datetime',
    license: 'MIT',
    licenseLink: 'https://github.com/YouCanBookMe/react-datetime/blob/master/LICENSE.md',
  },

  {
    title: 'See also package.json!',
    officialWebsite: 'https://gitlab.informatik.uni-halle.de/Syndrom/frontend-react/blob/master/package.json',
    sourceWebsite: '-',
    license: '-',
    licenseLink: '-',
  },

  //TODO
]


class AboutSite extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="about-site">

        <div className="about-logo">
          <img src="about_icon.png" alt="logo"/>
        </div>

        <div className="about-title">
          <span>{aboutInfo.title}</span>
        </div>

        <div className="about-title-long">
          <span className="about-big-letter">Y</span>
          <span className="mar-right-half about-not-important-colored">et</span>

          <span className="about-big-letter">A</span>
          <span className="mar-right-half about-not-important-colored">nother</span>

          <span className="about-big-letter">P</span>
          <span className="mar-right-half about-not-important-colored">ractical</span>

          <span className="about-big-letter">EX</span>
          <span className="mar-right-half about-not-important-colored">ercise</span>

          <span className="about-not-important-colored">Platform</span>
        </div>

        <div className="about-description">
          <span>{aboutInfo.description}</span>
        </div>

        <Divider className="about-divider"/>

        <h1>
          {
            translate(TranslateIds.design)
          }
        </h1>

        <div className="about-card-wrapper">
          {
            designInspirations.map((value, index) => {
              return (
                <div className="about-card card-shadow">
                  <h1>
                    {value.title}
                  </h1>

                  <Divider/>

                  <div className="about-card-details">
                    <div>
                      <span>Website</span>
                      {
                        value.website.length > 3 &&
                        <a href={value.website} target="_blank">
                          {value.website}
                        </a>
                      }
                      {
                        value.website.length <= 3 &&
                          <span>{value.website}</span>
                      }

                    </div>
                    {
                      value.what &&
                      <div>
                        <span>What</span>
                        <span>
                        {value.what}
                      </span>
                      </div>
                    }
                  </div>

                </div>
              )
            })
          }

        </div>

        <Divider className="about-divider"/>

        <h1>
          {
            translate(TranslateIds.frameworks)
          }
        </h1>

        <div className="about-card-wrapper">
          {
            usedLibraries.map((value, index) => {
              return (
                <div className="about-card card-shadow">
                  <h1>
                    {value.title}
                  </h1>

                  <Divider/>

                  <div className="about-card-details">
                    <div>
                      <span>Website</span>
                      {
                        value.officialWebsite.length > 3 &&
                        <a href={value.officialWebsite} target="_blank">
                          {value.officialWebsite}
                        </a>
                      }
                      {
                        value.officialWebsite.length <= 3 &&
                        <span>{value.officialWebsite}</span>
                      }

                    </div>
                    <div>
                      <span>Source</span>
                      {
                        value.sourceWebsite.length > 3 &&
                        <a href={value.sourceWebsite} target="_blank">
                          {value.sourceWebsite}
                        </a>
                      }
                      {
                        value.sourceWebsite.length <= 3 &&
                          <span>{value.sourceWebsite}</span>
                      }
                    </div>
                    <div>
                      <span>License</span>
                      <a href={value.licenseLink} target="_blank">
                        {value.license}
                      </a>
                    </div>
                  </div>
                </div>
              )
            })
          }

        </div>


        <Divider className="about-divider"/>

        <h1>
          {
            translate(TranslateIds.projectPeople)
          }
        </h1>

        <div className="about-card-wrapper">
          {
            projectPeopleInfo.map((value, index) => {
              return (
                <div className="about-card card-shadow">
                  <h1>
                    {value.name}
                  </h1>

                  <Divider/>

                  <div className="about-card-details">
                    <div>
                      <span>What</span>
                      <span>
                        {value.what}
                      </span>
                    </div>
                  </div>
                </div>
              )
            })
          }

        </div>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AboutSite)