/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Card, Icon, Divider, Image, Label} from "semantic-ui-react";
import {ReleaseDurationType} from "../../../../types/ReleaseDurationType";
import {ReleaseStartType} from "../../../../types/ReleaseStartType";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {copyToClipboard} from "../../../../helpers/clipboard";
import {getDoExerciseSiteToLink} from "../../../../constants";
import {Link} from "react-router-dom";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import Tag from '../../../tagComponents/tag'
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly exercisePreviews: ReadonlyArray<ExercisePreviewFromBackend>

  /**
   * true: display the releases as they were accessed via code,
   * false: display the releases as they are always accessible/accessed (release is visible to all setting)
   */
  readonly displayAsViaCode: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    exercisePreviews: props.exercisePreviews,
    displayAsViaCode: props.displayAsViaCode,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class AllAccessibleExercisesCards extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="material-table--caption-line-padding">

        <Card.Group>
          {
            this.props.exercisePreviews.map((exercise, index) => {
              //we can't use the id for the key because one exercise can have multiple releases...
              return (
                <Card key={exercise.generatedCode}>
                  <Card.Content>
                    {
                      /*
                      display icon of p lang
                      <Image floated='right' size='mini'
                           src='https://react.semantic-ui.com/assets/images/avatar/large/steve.jpg'/>
                       */
                    }

                    {
                      //language icon or span...
                      //                    <Image floated='right' size='mini'
                      //src='https://react.semantic-ui.com/assets/images/avatar/large/steve.jpg'/>
                      //<span style={{float: 'right'}}>Java</span>
                    }

                    <Card.Header>
                      <span>{exercise.displayName}</span>
                    </Card.Header>

                    {
                      this.props.displayAsViaCode &&
                      <Card.Meta>
                        <Icon name="clone" className="clickable mar-right-half"
                              onClick={() => {
                                copyToClipboard(exercise.generatedCode)
                              }}
                        />
                        <span>{exercise.generatedCode}</span>
                      </Card.Meta>
                    }

                    {
                      exercise.shortDescription &&
                      <Card.Description>
                        <Icon name="help circle" className="mar-right-half"></Icon>
                        <span>{exercise.shortDescription}</span>
                      </Card.Description>
                    }

                    <Divider/>

                    <Card.Description>
                      <span className="mar-right-half">{translate(TranslateIds.availableWorkingTimeInMin)}:</span>
                      {
                        (exercise.hasLimitedWorkingTime ?
                            <span>{exercise.availableWorkingTimeInMinutes}</span> :
                            <span>&infin;</span>
                        )
                      }
                    </Card.Description>
                    <Card.Description>
                      <span className="mar-right-half">{translate(TranslateIds.leftWorkingTime)}:</span>
                      {
                        //todo check time is up instead of true, calc left time in preview proxy
                        (exercise.hasLimitedWorkingTime && true ?
                            <span>test</span> :
                            <span>&infin;</span>
                        )
                      }
                    </Card.Description>
                    <Card.Description>
                      <span className="mar-right-half">{translate(TranslateIds.releaseEndAt_label)}:</span>
                      {
                        exercise.releaseStartType === ReleaseStartType.Manual ? <span>{translate(TranslateIds.manual)}</span> : null
                      }
                      {
                        exercise.releaseStartType === ReleaseStartType.Automatic
                        && exercise.releaseDurationType === ReleaseDurationType.Limited ?
                          <span>end at {displayDateAndTime(exercise.automaticEndAt, -1)}</span> : null
                      }
                    </Card.Description>


                    {
                      (exercise.tags && exercise.tags.length > 0) ?
                        <div>
                          <Divider/>
                          <Card.Description className="exercise-tag-wrapper">
                            {
                              exercise.tags.map((tag, index2) => {
                                return (
                                  <Tag key={tag.id} tag={tag}/>
                                )
                              })
                            }
                          </Card.Description>
                        </div>
                        : null
                    }
                  </Card.Content>

                  <Card.Content extra>
                    <div>
                      <Link to={getDoExerciseSiteToLink(exercise.generatedCode)} className="clickable">
                        <Icon name="share"/>
                      </Link>
                    </div>
                  </Card.Content>

                </Card>
              )
            })
          }
        </Card.Group>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AllAccessibleExercisesCards)