/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";

import ListView from './listView'
import CardsView from './cardsView'
import AllAccessibleExercisesCards from './cardsView'
import SinglePanelSiteWrapper from '../../../singlePanelSiteWrapper'
import {SiteViewState} from "../../../../types/siteView";
import {ExerciseState} from "../../../../types/exercisePreview";
import MaterialSearchInput from '../../../material/materialSearchInput'

import EnterReleaseCodeView from '../dialogs/enterReleaseCodeView'
import {
  getClosedExercisePreviewsAsync, loadClosedExercisePreviewsAsync,
  setSortByKey
} from "../../../../state/actions/allAccessibleExercises/closedExercisesSite/closedExercisesSiteActions";
import Spinner from "../../../helpers/Spinner";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {sortAndFilterExercisePreviews} from "../../../../helpers/tablesHelper";
import TagFilterPanel from '../../../tagComponents/tagFilterPanel'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  /**
   * true: load data,
   * false: not
   * this is ued in the all closed exercises site because not both (via code, via visibility) should load data
   * and they use the same state...
   */
  readonly shouldLoadData: boolean

  /**
   * true: can display, false not (probably because parent does)
   */
  readonly canDisplayTagFilterPanel: boolean

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    viewState: rootState.closedExercisesState.viewState,
    closedExercisePreviews: rootState.closedExercisesState.closedExercisePreviewsViaCode,
    isLoading: rootState.closedExercisesState.isLoading
    || rootState.manageTagsState.isLoading,
    shouldLoadData: props.shouldLoadData,
    sortByKey: rootState.closedExercisesState.sortByKey,
    sortDirection: rootState.closedExercisesState.sortDirection,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    canDisplayTagFilterPanel: props.canDisplayTagFilterPanel,
    isTagsFilterDisplayed: rootState.closedExercisesState.isTagsFilterDisplayed,
    tags: rootState.manageTagsState.tags,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadClosedExercisePreviewsAsync,
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ClosedExercisesSite extends React.Component<Props, any> {


  componentWillMount(): void {
    if (this.props.shouldLoadData)
      this.props.loadClosedExercisePreviewsAsync()
  }

  render(): JSX.Element {

    const previewsList = sortAndFilterExercisePreviews(this.props.closedExercisePreviews, this.props.sortDirection,
      this.props.sortByKey, this.props.selectedFilterTagsIds, this.props.selectedNegativeFilterTagsIds)

    return (

      <div className="multiple-single-panels">

        {
          this.props.isTagsFilterDisplayed &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
          />
        }

        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>{translate(TranslateIds.closedExercisesViaCode)}</h1>
              </div>
              <div className="view-options">
                <MaterialSearchInput/>
              </div>
            </div>

            <div className="view-content">
              {
                this.props.viewState === SiteViewState.List &&
                <ListView exercisePreviews={previewsList} displayAsViaCode={true}
                          sortByKey={this.props.sortByKey}
                          sortDirection={this.props.sortDirection}
                          setSortByKey={this.props.setSortByKey}
                />
              }
              {
                this.props.viewState === SiteViewState.Cards &&
                <CardsView exercisePreviews={previewsList} displayAsViaCode={true}/>
              }
            </div>
          </div>

        </SinglePanelSiteWrapper>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ClosedExercisesSite)