/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import OpenExercisesViaCodeSite from './openExercisesViaCodeSite'
import OpenExercisesViaVisibilitySite from './openExercisesViaVisibilitySite'
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import TagFilterPanel from '../../../tagComponents/tagFilterPanel'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

export interface ExercisePreviewsFrontendExclusive extends ExercisePreviewFromBackend {

  /**
   * the left working time as string
   */
  readonly leftTimeString: string

  /**
   * the left time is seconds
   * e.g. if we want to warn to user if he/she has only xxx seconds left
   *
   * or null if not set
   */
  readonly leftTimeInS: number | null

  /**
   * true: user can edit his/her solution,
   * false: not (e.g. working time is up or the release ended...)
   *
   * we set it to FALSE BUT if the release is open & has no limited working time (and no automatic end)
   * then the user can still do the exercise...
   *
   * we can't use isReleased because this is not updated in the frontend only on get from server
   */
  readonly isStillOpen: boolean

  readonly isDisplayed: boolean


}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.openExercisesState.isLoading
    || rootState.manageTagsState.isLoading,


    isEnterReleaseCodeDialogDisplayed: rootState.openExercisesState.isEnterReleaseCodeDialogDisplayed,

    isTagsFilterDisplayed: rootState.openExercisesState.isTagsFilterDisplayed,
    tags: rootState.manageTagsState.tags,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class allOpenExercisesSite extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="multiple-single-panels">

        {
          (!this.props.isEnterReleaseCodeDialogDisplayed && this.props.isTagsFilterDisplayed) &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
          />
        }

        <OpenExercisesViaCodeSite shouldLoadData={true} canDisplayEnterReleaseCodeDialog={true} canDisplayTagFilterPanel={false}/>

        <OpenExercisesViaVisibilitySite shouldLoadData={false} canDisplayEnterReleaseCodeDialog={false} canDisplayTagFilterPanel={false}/>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(allOpenExercisesSite)