/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {Icon, Input} from "semantic-ui-react";
import CommonHeaderBarContentWrapper from '../../../commonHeaderBarContentWrapper'
import {
  setIsEnterReleaseCodeDialogDisplayed, setIsTagsFilterDisplayed,
  setOpenExercisesSiteViewState
} from "../../../../state/actions/allAccessibleExercises/openExercisesSite/openExercisesSiteActions";
import ViewStateChanger from '../../../helpers/viewStateChanger'
import {Link} from "react-router-dom";
import * as constants from '../../../../constants'
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {RootState} from "../../../../state/reducers";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    viewState: rootState.openExercisesState.viewState,
    isEnterReleaseCodeDialogDisplayed: rootState.openExercisesState.isEnterReleaseCodeDialogDisplayed,
    isLoading: rootState.openExercisesState.isLoading,

    isTagsFilterDisplayed: rootState.openExercisesState.isTagsFilterDisplayed,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setViewState: setOpenExercisesSiteViewState,
  setIsEnterReleaseCodeDialogDisplayed,
  setIsTagsFilterDisplayed
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class HeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {

    const leftArea = (
      <div>

        <div className="clickable"
             onClick={() => this.props.setIsEnterReleaseCodeDialogDisplayed(true)}
        >
          <Icon name="privacy"/>
          <span>
            {
              translate(TranslateIds.enterReleaseCode)
            }
          </span>
        </div>

      </div>
    )

    const rightArea = (
      <div className="flexed">

        <div className={`clickable mar-right-half ${this.props.isTagsFilterDisplayed ? 'toggled-colored' : ''}`}
             onClick={() => {
               this.props.setIsTagsFilterDisplayed(!this.props.isTagsFilterDisplayed)
             }}
        >
          <Icon name="filter" />
        </div>

        <ViewStateChanger viewState={this.props.viewState}
                          setViewStateFunc={(viewState) => this.props.setViewState(viewState)}/>
      </div>
    )

    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea} rightArea={rightArea} disabled={this.props.isLoading}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)