/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";

import ListView from './listView'
import CardsView from './cardsView'

import SinglePanelSiteWrapper from '../../../singlePanelSiteWrapper'
import {SiteViewState} from "../../../../types/siteView";
import {ExerciseState} from "../../../../types/exercisePreview";
import MaterialSearchInput from '../../../material/materialSearchInput'

import EnterReleaseCodeDialog from '../dialogs/enterReleaseCodeDialog'

import {
  getOpenExercisePreviewsAsync, loadOpenExercisePreviewsAsync, setSortByKey
} from "../../../../state/actions/allAccessibleExercises/openExercisesSite/openExercisesSiteActions";
import Spinner from "../../../helpers/Spinner";
import {TranslateIds} from "../../../../../i18n/ids";
import {translate} from "../../../../../i18n/translationHelper";
import {sortAndFilterExercisePreviews} from "../../../../helpers/tablesHelper";
import TagFilterPanel from '../../../tagComponents/tagFilterPanel'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  /**
   * true: load data,
   * false: not
   * this is ued in the all closed exercises site because not both (via code, via visibility) should load data
   * and they use the same state...
   */
  readonly shouldLoadData: boolean

  /**
   * if we display both via code & via visibility we would get 2 dialogs...
   */
  readonly canDisplayEnterReleaseCodeDialog: boolean

  /**
   * true: can display, false not (probably because parent does)
   */
  readonly canDisplayTagFilterPanel: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    viewState: rootState.openExercisesState.viewState,
    openExercisePreviews: rootState.openExercisesState.openExercisePreviewsViaVisibility,
    isLoading: rootState.openExercisesState.isLoading
    || rootState.manageTagsState.isLoading,
    isEnterReleaseCodeDialogDisplayed: rootState.openExercisesState.isEnterReleaseCodeDialogDisplayed,
    shouldLoadData: props.shouldLoadData,
    sortByKey: rootState.openExercisesState.sortByKey,
    sortDirection: rootState.openExercisesState.sortDirection,
    canDisplayEnterReleaseCodeDialog: props.canDisplayEnterReleaseCodeDialog,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    canDisplayTagFilterPanel: props.canDisplayTagFilterPanel,
    isTagsFilterDisplayed: rootState.openExercisesState.isTagsFilterDisplayed,
    tags: rootState.manageTagsState.tags,

  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadOpenExercisePreviewsAsync,
  getOpenExercisePreviewsAsync,
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps

class OpenExercisesViaVisibilitySite extends React.Component<Props, any> {

  componentWillMount(): void {
    if (this.props.shouldLoadData)
      this.props.loadOpenExercisePreviewsAsync()
  }

  render(): JSX.Element {

    if (this.props.isEnterReleaseCodeDialogDisplayed && !this.props.canDisplayEnterReleaseCodeDialog) {
      //the other site (via code) displays the dialog already
      return null
    }

    const previewsList = sortAndFilterExercisePreviews(this.props.openExercisePreviews, this.props.sortDirection,
      this.props.sortByKey, this.props.selectedFilterTagsIds, this.props.selectedNegativeFilterTagsIds)

    return (

      <div className="multiple-single-panels">

        {
          (this.props.canDisplayTagFilterPanel && !this.props.isEnterReleaseCodeDialogDisplayed && this.props.isTagsFilterDisplayed) &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
          />
        }

        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          {
            !this.props.isEnterReleaseCodeDialogDisplayed &&
            <div className="view-padding">
              <div className="view-head-line">
                <div className="view-caption">
                  <h1>
                    {
                      translate(TranslateIds.openExercisesViaVisibility)
                    }
                  </h1>
                </div>
                <div className="view-options">
                  <MaterialSearchInput/>
                </div>
              </div>

              <div className="view-content">
                {
                  this.props.viewState === SiteViewState.List &&
                  <ListView exercisePreviews={previewsList} displayAsViaCode={false}
                            sortByKey={this.props.sortByKey}
                            sortDirection={this.props.sortDirection}
                            setSortByKey={this.props.setSortByKey}
                  />
                }
                {
                  this.props.viewState === SiteViewState.Cards &&
                  <CardsView exercisePreviews={previewsList} displayAsViaCode={false}/>
                }
              </div>
            </div>
          }

          {
            (!this.props.isLoading && this.props.isEnterReleaseCodeDialogDisplayed && this.props.canDisplayEnterReleaseCodeDialog) &&
            <EnterReleaseCodeDialog/>
          }


        </SinglePanelSiteWrapper>

      </div>

    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(OpenExercisesViaVisibilitySite)