/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import Spinner from "../../helpers/Spinner";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {
  CartesianGrid, Area, AreaChart, ResponsiveContainer, XAxis,
  YAxis
} from "recharts";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {convertSizeToHumanReadableSize} from "../../../helpers/convertersAndTransformers";
import {HelpPopup} from "../../helpers/helpPopup";
import {Popup} from "semantic-ui-react";
import {leftIconPopupOffset} from "../../../constants";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    hardwareUsage: rootState.dashboardSiteState.hardwareUsageStatistic,
    isLoading: rootState.dashboardSiteState.isLoading
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

interface UsageDataPoint {
  readonly time: string
  readonly value: number
}

class hardwareUsageView extends React.Component<Props, any> {

  render(): JSX.Element {


    const cpuUsageData: ReadonlyArray<UsageDataPoint> = this.props.hardwareUsage.map(p => {
      const temp: UsageDataPoint = {
        value: p.cpuUsageInPercentage,
        time: p.time
      }
      return temp
    })


    const ramUsageData: ReadonlyArray<UsageDataPoint> = this.props.hardwareUsage.map(p => {
      // noinspection TsLint
      const temp: UsageDataPoint = {
        value: (p.totalRamInKb - p.freeRamInKb) / 1024, //mb
        time: p.time
      }
      return temp
    })

    console.log(this.props.hardwareUsage)

    // noinspection TsLint
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }


        <div className="pad-right">
          <div className="view-head-line">
            <div className="view-caption">
              <h1>
                {
                  translate(TranslateIds.hardwareCharts)
                }
              </h1>

            </div>
            <div className="view-options">

              <Popup
                offset={leftIconPopupOffset}
                trigger={
                  <div className="live-sonar">
                    <div>
                  <span>
                    <span />
                  </span>
                    </div>
                  </div>
                }
                content={
                  translate(TranslateIds.liveRefresh_helpText)
                }
              ></Popup>

            </div>
          </div>

          <div className="view-content">


            <div className="flexed">

              {
                //cpu
              }
              <div style={{width: '45%'}}>


                <div className="flexed-h">
                  <h2>
                    {
                      translate(TranslateIds.cpuUsageChartCaption)
                    }
                  </h2>

                  <ResponsiveContainer
                    width={'100%'}
                    height={200}
                    debounce={1000}
                  >

                    <AreaChart width={600} height={400} data={cpuUsageData as any[]}
                               margin={{top: 10, right: 30, left: 0, bottom: 0}}>

                      <XAxis dataKey="time"/>
                      <YAxis domain={[0, 100]}/>
                      <Area type='monotone' dataKey='value' stroke='#8884d8' fill='#8884d8'/>

                      <CartesianGrid stroke="#eee" strokeDasharray="5 5"/>
                    </AreaChart>

                  </ResponsiveContainer>


                </div>

              </div>

              {
                //ram
                ramUsageData.length > 0 &&
                <div style={{width: '45%'}}>

                  <div className="flexed-h">

                    <h2>
                      {
                        translate(TranslateIds.ramUsageChartCaption) + ' '
                      }
                      (
                      {
                        `${convertSizeToHumanReadableSize(this.props.hardwareUsage[this.props.hardwareUsage.length - 1].freeRamInKb * 1024,
                          2)} ${translate(TranslateIds.free)}/${convertSizeToHumanReadableSize(this.props.hardwareUsage[this.props.hardwareUsage.length - 1].totalRamInKb * 1024,
                          2)}`
                      }
                      )
                    </h2>

                    <ResponsiveContainer
                      width={'100%'}
                      height={200}
                      debounce={1000}
                    >

                      <AreaChart width={600} height={400} data={ramUsageData as any[]}
                                 margin={{top: 10, right: 30, left: 0, bottom: 0}}>

                        <XAxis dataKey="time"/>
                        <YAxis domain={[0, this.props.hardwareUsage[0].totalRamInKb / 1024]}/>
                        <Area type='monotone' dataKey='value' stroke='#8884d8' fill='#8884d8'/>

                        <CartesianGrid stroke="#eee" strokeDasharray="5 5"/>
                      </AreaChart>

                    </ResponsiveContainer>

                  </div>

                </div>
              }


            </div>


          </div>
        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(hardwareUsageView)