/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {List} from "semantic-ui-react";
import {AssessedSubmitTestFromBackend, TestProtocolItem, TestProtocolType} from "../../../../types/testResults";
import ConsoleItem from './consoleItem'
import {convertTestResultToProtocolItems} from "../../../../helpers/convertersAndTransformers";
import {RunnableExerciseTestFrontendExclusive} from "../../../../types/exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {KnownInternalTestTypes, TestTypeFullBase} from "../../../../types/testTypes";
import {Icon, Popup} from "semantic-ui-react";
import Logger from "../../../../helpers/logger";
import {leftIconPopupOffset} from "../../../../constants";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {HelpPopup} from "../../../helpers/helpPopup";

//const css = require('./styles.styl');

//only test OR protocolItems is valid
export interface MyProps {
  //readonly test: string
  readonly test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFromBackend | null
  readonly testTypes: ReadonlyArray<TestTypeFullBase>

  readonly protocolItems: ReadonlyArray<TestProtocolItem> | null

  readonly useDiffIfOutputMismatched: boolean

  readonly useReplaceWhiteSpacesWithBullets: boolean
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    test: props.test,
    testTypes: props.testTypes.concat(
      [{id: -1, displayName: 'Just Run Program', internalName: KnownInternalTestTypes.justRunTest}]),
    protocolItems: props.protocolItems,
    useDiffIfOutputMismatched: props.useDiffIfOutputMismatched,
    useReplaceWhiteSpacesWithBullets: props.useReplaceWhiteSpacesWithBullets,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps)
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ConsolePanelView extends React.Component<Props, any> {
  render(): JSX.Element {

    if (!this.props.test && !this.props.protocolItems) {
      //if we clear the console view we set both to null...
      //Logger.error('console panel view can only use test or protocolItems property')
      return null
    }

    // noinspection TsLint
    let protocolItems: ReadonlyArray<TestProtocolItem> = []

    if (this.props.test) {
      protocolItems = convertTestResultToProtocolItems(this.props.test, this.props.testTypes)
      Logger.debug('consolePanelView', 'using test as content')
    }

    if (this.props.protocolItems) {
      protocolItems = this.props.protocolItems
      Logger.debug('consolePanelView', 'using protocol items as content')
    }

    return (
      <div className="fh">

        <List divided relaxed>
          {
            //only display header if we have some normal output... (or just don't display headers if all items are errors)
            (protocolItems && protocolItems.some(p => p.type !== TestProtocolType.isError)) &&
            <div className="console-input-output-line-header">
              <div className="input-part">
                <div className="flexed-well-spaced">

                  <div className="flexed">
                    <div className="mar-right-half">
                      {
                        translate(TranslateIds.input)
                      }
                    </div>
                    {
                      this.props.useReplaceWhiteSpacesWithBullets &&
                      <HelpPopup defaultText={translate(TranslateIds.allWhitespacesOfTheInputAreReplaced)}/>
                    }

                  </div>

                  <Icon className="not-important-colored" name="chevron right"/>
                </div>
              </div>
              <div className="program-part-invisible">

                <div className="absolute-centered">
                  <div>
                    {
                      translate(TranslateIds.program)
                    }
                  </div>
                </div>

              </div>
              <div className="output-part">
                <div className="flexed-well-spaced">
                  <Icon className="not-important-colored" name="chevron right"/>

                  <div className="flexed">
                    <div className="mar-right-half">
                      {
                        translate(TranslateIds.output)
                      }
                    </div>
                    {
                      this.props.useReplaceWhiteSpacesWithBullets &&
                      <HelpPopup defaultText={translate(TranslateIds.allWhitespacesOfTheOutputAreReplaced)}/>
                    }

                  </div>
                </div>
              </div>
            </div>
          }
          {
            (this.props.test
              && this.props.testTypes.find(
                p => this.props.test.testTypeId === p.id).internalName === KnownInternalTestTypes.compileTest) &&
            <div>
              {
                protocolItems.length === 0 &&
                <div className="ok-colored">
                  {
                    translate(TranslateIds.compileSingleFileSuccessful)
                  }
                </div>
              }
            </div>
          }
          {
            protocolItems &&
            protocolItems.map((item: TestProtocolItem, index) => {

                const nextItem = index + 1 >= protocolItems.length ? null : protocolItems[index + 1]
                return (
                  <ConsoleItem key={index} item={item}
                               nextItem={nextItem}
                               useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                               useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                  />
                )
              }
            )
          }
        </List>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ConsolePanelView)