/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon} from "semantic-ui-react";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {openCreateCustomTestDialog} from "../../../../state/actions/doExerciseSite/dialogs/changeCustomTestDialog/changeCustomTestActions";
import {RunTestCommandForBackend} from "../../../../types/testsAndTestProtocol";
import {
  runCustomDoExerciseTest,
  saveDoExerciseSolution
} from "../../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {HelpPopup} from "../../../helpers/helpPopup";
import Logger from "../../../../helpers/logger";
import {isTestResultValid} from "../../../../helpers/testHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {resetCustomTestResults} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {MultiActions} from "../../../../state/actions/types";
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {RunnableCustomProjectTestFrontendExclusive} from "../../../../types/customProjects";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly customTests: ReadonlyArray<RunnableCustomTestFrontendExclusive | RunnableCustomProjectTestFrontendExclusive>
  readonly canUserCreateCustomTests: boolean

  readonly openCreateCustomTestDialog: () => void
  readonly runCustomTests: (tests: ReadonlyArray<RunnableCustomTestFrontendExclusive | RunnableCustomProjectTestFrontendExclusive>) => Promise<boolean>
  readonly resetCustomTestResults: () => void

  /**
   * e.g. used to save changes in the editor
   */
  readonly beforeRunningTests: () => Promise<void>
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    customTests: props.customTests,

    canUserCreateCustomTests: props.canUserCreateCustomTests,

    openCreateCustomTestDialog: props.openCreateCustomTestDialog,
    beforeRunningTests: props.beforeRunningTests,
    runCustomTests: props.runCustomTests,
    resetCustomTestResults: props.resetCustomTestResults,

    systemSettings: rootState.systemSettingsState.systemSettings,
    maxNumberOfTestsWithOneRequest: rootState.systemSettingsState.systemSettings
    && rootState.systemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest >= 0
      ? rootState.systemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest : 1,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class testsPanelHeader extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className={`${this.props.canUserCreateCustomTests ? 'clickable' : 'div-disabled' }`}
             onClick={() => {
               this.props.openCreateCustomTestDialog()
             }}
        >
          <Icon name="add"/>
          <span>
            {
              translate(TranslateIds.add)
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div className={`mar-right-half ${(this.props.canUserCreateCustomTests  && this.props.customTests.length > 0) ? 'clickable' : 'div-disabled' }`}
             onClick={async () => {

               //save changes if code changed
               await this.props.beforeRunningTests()

               const maxNumberOfTestsWithOneRequest = this.props.maxNumberOfTestsWithOneRequest

               Logger.debug("test panel header bar",
                 `running all test with ${maxNumberOfTestsWithOneRequest} test in parallel`)


               const validTests = this.props.customTests.filter(p => isTestResultValid(p))


               // noinspection TsLint
               let ranTests: RunnableCustomTestFrontendExclusive[] = []

               // noinspection TsLint
               for (let i = 0; i < this.props.customTests.length; i++) {
                 const test = this.props.customTests[i]

                 if (isTestResultValid(test) === false || validTests.length === this.props.customTests.length) {
                   ranTests.push(test)
                 } else {
                   //test was already valid
                 }


                 //we reached the limit
                 if (ranTests.length === maxNumberOfTestsWithOneRequest) {

                   const hasAnyTestFailed = await this.props.runCustomTests(ranTests)

                   //clear ran test
                   ranTests = []

                   //stop if at least 1 test failed
                   if (hasAnyTestFailed) {
                     //return instead of break else we would run the last check below which would be always true
                     return
                   }
                 }
               }

               //we might have some last tests but not reached the max num of tests with one request...
               if (ranTests.length > 0) {
                 //we missed the last < maxNumberOfTestsWithOneRequest tests
                 //these are still in the last tempCommand
                 await this.props.runCustomTests(ranTests)
               }


             }}
        >
          <Icon name="forward"/>
          <span>
            {
              translate(TranslateIds.runAllTests)
            }
          </span>
        </div>

        <HelpPopup
          defaultText={
            `${translate(TranslateIds.runAllTests_desc)} ${translate(TranslateIds.maxParallelTests)}: ${this.props.maxNumberOfTestsWithOneRequest}`}
          wide
        />

        <SimpleVDivider/>

        <div className={this.props.canUserCreateCustomTests ? 'clickable' : 'div-disabled'}
             onClick={() => {
               this.props.resetCustomTestResults()
             }}
        >
          <Icon name="remove"/>
          <span>
          {
            translate(TranslateIds.clearCustomTestResults)
          }
          </span>
        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(testsPanelHeader)