/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {List, Icon} from "semantic-ui-react";
import TestNumber from '../testsPanel/testNumber'
import {RunnableCustomTestFrontendExclusive} from "../../../../types/customTests";
import {setSelectedConsoleViewTest} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {RunTestCommandForBackend} from "../../../../types/testsAndTestProtocol";
import {
  runCustomDoExerciseTest,
  saveDoExerciseSolution
} from "../../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {openUpdateCustomTestDialog} from "../../../../state/actions/doExerciseSite/dialogs/changeCustomTestDialog/changeCustomTestActions";
import {deleteCustomTestDoEditor} from "../../../../state/actions/doExerciseSite/sub/customTestActions";
import {ReleaseDurationType} from "../../../../types/ReleaseDurationType";
import {askDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import orderBy from 'lodash-es/orderBy'
import {AssessedSubmitTestFrontendExclusive} from "../../../../types/testResults";
import {RunnableExerciseTestFrontendExclusive} from "../../../../types/exerciseEditor";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly customTests: ReadonlyArray<RunnableCustomTestFrontendExclusive>

  readonly canUserEditExercise: boolean

  readonly isSomeTestRunning: boolean

  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null


  readonly setSelectedConsoleViewTest: (test: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => void

  readonly openUpdateCustomTestDialog: (test: RunnableCustomTestFrontendExclusive) => void

  readonly runCustomTest: (test: RunnableCustomTestFrontendExclusive) => Promise<boolean>

  readonly deleteCustomTestDoEditor: (test: RunnableCustomTestFrontendExclusive) => void

  /**
   * e.g. used to save changes in the editor
   */
  readonly beforeRunningTests: () => Promise<void>

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    customTests: props.customTests,
    selectedConsoleViewTest: props.selectedConsoleViewTest,

    isSomeTestRunning: props.isSomeTestRunning,

    runCustomTest: props.runCustomTest,
    setSelectedConsoleViewTest: props.setSelectedConsoleViewTest,
    canUserEditExercise: props.canUserEditExercise,
    beforeRunningTests: props.beforeRunningTests,
    openUpdateCustomTestDialog: props.openUpdateCustomTestDialog,
    deleteCustomTestDoEditor: props.deleteCustomTestDoEditor,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class TestsPanel extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="fh">

        <List selection divided relaxed>
          {
            orderBy<RunnableCustomTestFrontendExclusive>(this.props.customTests,
              (test) => test.displayIndex)
              .map((customTest, index) => {
                return (
                  <List.Item key={customTest.id}
                             className={`test-line pad-left-right default-cursor fg-colored ` +
                             (this.props.selectedConsoleViewTest
                               && this.props.selectedConsoleViewTest.id === customTest.id
                                 ? 'line-active' : ''
                             )}

                  >
                    <List.Content className="v-centered-i" floated="right">

                    </List.Content>
                    <List.Content>
                      <div className="flexed">
                        <TestNumber number={(index + 1)} isValid={customTest.result ? customTest.result.passed : null}
                                    className="mar-right"
                                    isTestRunning={customTest.uiIsRunning}
                                    isEnabled={!this.props.isSomeTestRunning && this.props.canUserEditExercise}
                                    onClick={async () => {

                                      //save changes if code changed
                                      await this.props.beforeRunningTests()

                                      this.props.runCustomTest(customTest)

                                    }}
                        />

                        <div className="flexed v-centered flex-stretch">
                          <div>
                            {
                              customTest.displayName
                            }
                          </div>

                          {
                            customTest.result &&
                            <div className="clickable mar-left v-centered"
                                 onClick={() => {
                                   this.props.setSelectedConsoleViewTest(customTest)
                                 }}
                            >
                              <Icon name="info circle"/>
                              <span>
                                {
                                  translate(TranslateIds.result)
                                }
                              </span>
                            </div>
                          }
                        </div>

                        <div className="clickable v-centered mar-right"
                             onClick={() => {
                               this.props.openUpdateCustomTestDialog(customTest)
                             }}
                        >
                          <Icon name="write"/>
                          <span>
                            {
                              translate(TranslateIds.change)
                            }
                          </span>
                        </div>

                        <div className="clickable v-centered"
                             onClick={async () => {

                               const shouldRemove = await askDialog(translate(TranslateIds.deleteCustomTest_title),
                                 `${translate(TranslateIds.deleteCustomTest_content)} (${customTest.displayName})`
                               )

                               if (shouldRemove) {
                                 // this.props.deleteCustomTestDoEditor(this.props.releaseCode, customTest,
                                 //   this.props.customTests)
                                 this.props.deleteCustomTestDoEditor(customTest)
                               }
                             }}
                        >
                          <Icon name="remove"/>
                          <span>
                            {
                              translate(TranslateIds.delete_)
                            }
                          </span>
                        </div>

                      </div>
                    </List.Content>
                  </List.Item>
                )
              })
          }

        </List>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TestsPanel)