/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {RouteComponentProps, withRouter} from "react-router";
import {isNotEmptyOrSpaces, isNotNullOrUndefined} from "../../../validation/validations";
import Spinner from "../../helpers/Spinner";
import InterceptionOverlay from '../../helpers/InterceptOverlay'
import PanelGrid from "../../panelSystem/panelGrid/panelGrid";
import Panel from "../../panelSystem/panel/panel";
import TabContentHost from "../../panelSystem/panel/tabContentHost";
import PreviewPanelView from '../exerciseEditorSite/previewPanel/previewPanelView'
import {
  loadDoExerciseSite, resetCustomTestResults, setBotLeftPanelViewSize, setBotRightPanelViewSize,
  setBottomLeftSelectedTabIndex,
  setBottomRightSelectedTabIndex, setDoExerciseDummy,
  setDoExerciseHDividerPosOffset,
  setDoExerciseVDividerPosOffset, setExercisePreviewFrontendDoExercise, setIsLeftTimeStringDisplayed,
  setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite,
  setSelectedConsoleViewTest, setSolutionVersion,
  setTopLeftPanelViewSize,
  setTopLeftSelectedTabIndex, setTopRightPanelViewSize,
  setTopRightSelectedTabIndex, setUseDiffIfOutputMismatched, setUseReplaceWhiteSpacesWithBullets
} from "../../../state/actions/doExerciseSite/doExerciseSiteActions";

import TestsPanel from './testsPanel/testsPanelView'
import TestPanelHeaderBar from './testsPanel/testsPanelHeaderBar'

import CustomTestsPanel from './customTestsPanel/customTestsPanelView'
import CustomTestPanelHeaderBar from './customTestsPanel/customTestsPanelHeaderBar'

import ConsolePanelView from './consolePanel/consolePanelView'
import ConsolePanelHeaderBar from './consolePanel/consolePanelHeaderBar'

import EditorPanelView from './editorPanel/editorPanelView'
import EditorPanelHeaderBar from './editorPanel/editorPanelHeaderBar'

import ChangeCustomTestDialog from './customTestsPanel/dialog/customTestsDialog/changeCustomTestDialog'
import ViewTestDialog from './testsPanel/viewTestDialog/viewTestDialog'

import EditorSettingsDialog from '../../editorSettingsDialog/editorSettingsDialog'
import {getServerTime} from "../../../communicationLayer/timeLayer";
import moment = require("moment");
import {updateExercisePreviews} from "../../../helpers/editorHelper";
import {Moment} from "moment";
import {ExercisePreviewsFrontendExclusive} from "../allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {ReleaseStartType} from "../../../types/ReleaseStartType";
import {createAndRefreshAssetUrlsAndObjects, freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";


import SubmitTestsPanelHeaderBar from './submitTestsPanel/submitTestsPanelHeaderBar'

import {
  runAfterNormalTest, runAfterSubmitTest, runCustomDoExerciseTest, runDoExerciseTest,
  saveDoExerciseSolution
} from "../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {AssessedSubmitTestFromBackend, AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {
  cancelViewTestDialog,
  openViewTestDialog
} from "../../../state/actions/doExerciseSite/dialogs/viewTestDialog/viewTestDialogActions";
import {RunAfterTestCommandForBackend, RunTestCommandForBackend} from "../../../types/testsAndTestProtocol";
import {debugOriginUrl} from "../../../../debug";
import {ReleaseDurationType} from "../../../types/ReleaseDurationType";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import history from "../../../routerHistory";
import * as constants from '../../../constants'
import AssessmentResultPanelView from './assessmentResultPanel/assessmentResultPanelView'
import AssessmentResultPanelHeaderBar from './assessmentResultPanel/assessmentResultPanelHeaderBar'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";
import {askDialog, errorDialog} from "../../../helpers/dialogHelper";

//editor for solving the exercise after the release finished or the time is up?? or just release finished?
import AfterEditorPanelView from './afterEditorPanel/afterEditorPanelView'
import AfterNormalTestsPanelHeaderBar from './afterNormalTestsPanel/afterNormalTestsPanelHeaderBar'
import AfterSubmitTestsPanelHeaderBar from './afterSubmitTestsPanel/afterSubmitTestsPanelHeaderBar'
import AfterEditorPanelHeaderBar from './afterEditorPanel/afterEditorPanelHeaderBar'
import {SolutionFileDoExerciseFullBase, SolutionVersionDoExerciseFullBase} from "../../../types/exerciseSolution";
import Logger from "../../../helpers/logger";
import {
  setHasUnsavedChanges,
  setSelectedTabIdDoEditor
} from "../../../state/actions/doExerciseSite/editorActions/doEditorActions";
import {LoginTimeoutHelper} from "../../../helpers/loginTimeoutHelper";
import orderBy from 'lodash-es/orderBy'
import isEqual from 'lodash-es/isEqual'
import {
  openCreateCustomTestDialog,
  openUpdateCustomTestDialog
} from "../../../state/actions/doExerciseSite/dialogs/changeCustomTestDialog/changeCustomTestActions";
import {deleteCustomTestDoEditor} from "../../../state/actions/doExerciseSite/sub/customTestActions";
import {accessExerciseWithReleaseCode} from "../../../communicationLayer/exerciseReleasesLayer";
import {
  setIsSiteHeaderBarDisplayedAndEnabledAction,
  setIsSubHeaderBarCollapsed
} from "../../../state/actions/siteHeaderBarActions";
import {criticalLeftTimeInS} from "./headerBarContent";
import {Button, Input, Label} from "semantic-ui-react";
import {copyToClipboard} from "../../../helpers/clipboard";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    doExercise: rootState.doExerciseSiteState.exercise,
    exerciseDescription: rootState.doExerciseSiteState.exercise.exerciseDescription,
    solution: rootState.doExerciseSiteState.solution,
    solutionPLangId: rootState.doExerciseSiteState.solution.pLangId,

    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    //not needed here (inside editor panel) but keep it to know that we need to wait for loading
    //pLangs: rootState.pLangsState.pLangs,

    submitTestResults: rootState.doExerciseSiteState.submitTestResults,


    selectedConsoleViewTest: rootState.doExerciseSiteState.selectedConsoleViewTest,
    testTypes: rootState.testTypesState.testTypes,

    exercisePreview: rootState.doExerciseSiteState.exercisePreviewFrontendTemp,


    afterSolution: rootState.doExerciseSiteState.afterSolution,
    afterNormalTests: rootState.doExerciseSiteState.afterNormalTests,
    afterSubmitTests: rootState.doExerciseSiteState.afterSubmitTests,

    useDiffIfOutputMismatched: rootState.doExerciseSiteState.useDiffIfOutputMismatched,
    useReplaceWhiteSpacesWithBullets: rootState.doExerciseSiteState.useReplaceWhiteSpacesWithBullets,

    selectedPLangId: rootState.doExerciseSiteState.selectedPLangId,
    isCustomTestDialogDisplayed: rootState.doExerciseSiteState.isCustomTestDialogDisplayed,
    isViewTestDialogDisplayed: rootState.doExerciseSiteState.isViewTestDialogDisplayed,
    isCodeEditorSettingsDialogDisplayed: rootState.doExerciseSiteState.isCodeEditorSettingsDialogDisplayed,


    canUserCreateCustomTests: (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen)
    || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
      && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
      && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely),

    canUserEditExercise: (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen)
    || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
      && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
      && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely),

    // isSomeTestRunning: rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
    // || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning),

    /**
     * true: display the submit test results (not longer released and assessment finished)
     * false: not
     */
    displaySubmitTestResults: !rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
    && rootState.doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,

    displayAssessmentResult: !rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
    && rootState.doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished &&
    rootState.doExerciseSiteState.assessmentResult
    && (isNotEmptyOrSpaces(
      rootState.doExerciseSiteState.assessmentResult.feedbackForStudent) || rootState.doExerciseSiteState.assessmentResult.manualPoints),

    topRightSelectedTabIndex: rootState.doExerciseSiteState.topRightSelectedTabIndex,
    topLeftSelectedTabIndex: rootState.doExerciseSiteState.topLeftSelectedTabIndex,
    bottomRightSelectedTabIndex: rootState.doExerciseSiteState.bottomRightSelectedTabIndex,
    bottomLeftSelectedTabIndex: rootState.doExerciseSiteState.bottomLeftSelectedTabIndex,

    topRightPanelViewSize: rootState.doExerciseSiteState.topRightPanelViewSize,
    topLeftPanelViewSize: rootState.doExerciseSiteState.topLeftPanelViewSize,
    botRightPanelViewSize: rootState.doExerciseSiteState.botRightPanelViewSize,
    botLeftPanelViewSize: rootState.doExerciseSiteState.botLeftPanelViewSize,

    hDividerPosOffset: rootState.doExerciseSiteState.hDividerPosOffset,
    vDividerPosOffset: rootState.doExerciseSiteState.vDividerPosOffset,

    isSomeTestRunning: rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
    || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning)
    || rootState.doExerciseSiteState.afterNormalTests.some(p => p.uiIsRunning)
    || rootState.doExerciseSiteState.afterSubmitTests.some(p => p.uiIsRunning)
    ,

    isNoDialogDisplayed: !rootState.doExerciseSiteState.isCustomTestDialogDisplayed
    && !rootState.doExerciseSiteState.isViewTestDialogDisplayed
    && !rootState.doExerciseSiteState.isCodeEditorSettingsDialogDisplayed,

    canUserSolveExercise: (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen
      && rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased)
    || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
      && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
      && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely),

    editorHasUnsavedChanges: rootState.doExerciseSiteState.doEditorState.hasUnsavedChanges,

    //release should be not active and assessment finished then the after tests can be run
    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased &&
    rootState.doExerciseSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,

    dummy: rootState.doExerciseSiteState.dummy,

    //see loadDoExerciseSite used data
    isLoading: rootState.doExerciseSiteState.isLoading
    || rootState.testTypesState.isLoading
    || rootState.pLangsState.isLoading
    || rootState.systemSettingsState.isLoading,

    isOnGotFocusInterceptionOverlayDisplayed: rootState.doExerciseSiteState.isOnGotFocusInterceptionOverlayDisplayed,

    isSiteHeaderBarDisplayed: rootState.siteHeaderBarState.isSiteHeaderBarDisplayedAndEnabled,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setTopRightSelectedTabIndex,
  setBottomRightSelectedTabIndex,
  setTopLeftSelectedTabIndex,
  setBottomLeftSelectedTabIndex,
  loadDoExerciseSite,
  setDoExerciseHDividerPosOffset,
  setDoExerciseVDividerPosOffset,

  setExercisePreviewFrontendDoExercise,

  setTopRightPanelViewSize,
  setTopLeftPanelViewSize,
  setBotRightPanelViewSize,
  setBotLeftPanelViewSize,

  saveDoExerciseSolution,
  setSelectedConsoleViewTest,

  runDoExerciseTest,
  openViewTestDialog,
  cancelViewTestDialog,
  setUseDiffIfOutputMismatched,
  setUseReplaceWhiteSpacesWithBullets,
  setSolutionVersion,

  runAfterNormalTest,
  runAfterSubmitTest,
  setHasUnsavedChanges,
  setSelectedTabIdDoEditor,

  openCreateCustomTestDialog,
  runCustomDoExerciseTest,
  resetCustomTestResults,

  openUpdateCustomTestDialog,
  deleteCustomTestDoEditor,
  setIsSiteHeaderBarDisplayedAndEnabledAction,
  setIsSubHeaderBarCollapsed,
  setIsLeftTimeStringDisplayed,

  setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite
}, dispatch)


enum DoExerciseMode {
  /**
   * hide the menu bar so that the user
   * cannot navigate (some other program should hide the url e.g. safe exam browser)
   *
   * should not count is also not displayed (because it's an exam ;))
   */
  exam = 'exam'
}

interface MatchedProps {
  readonly releaseCode: string
  readonly mode: DoExerciseMode | null
}


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


// noinspection TsLint
let updateLeftTimeIntervals = -1
const onGotFocusBackInputId = 'ongotfocushelper'

class doExerciseSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  lastPageVisibilityWasHidden: boolean | null = null
  hasFocus = false
  firstOnFocus = true


  async componentWillMount(): Promise<void> {

    const releaseCode = this.props.match.params.releaseCode
    const mode = this.props.match.params.mode

    // noinspection TsLint
    let inExamMode = false

    //it's safer to set to disabled and when we know that we should enable it
    //then set it back to enabled
    this.props.setIsSiteHeaderBarDisplayedAndEnabledAction(false)

    if ((mode && mode === DoExerciseMode.exam)) {
      Logger.log('setting exam mode for do exercise site (url)')
      inExamMode = true
    }


    if (isNotNullOrUndefined(releaseCode) === false) {
      errorDialog(translate(TranslateIds.error), translate(TranslateIds.someUrlParametersNotFound))
      return
    }

    if (LoginTimeoutHelper.isReLogin && releaseCode === LoginTimeoutHelper.lastKnownDoEditorReleaseCode) {
      //do not reload or reset site state...
      //user re logged in he/she want's to continue
      //if we would reload then the code would be gone (even though the frontend settings would store a copy... but that's bad style)


      //but we need to re-create the assets because componentWillUnmount cleared everything

      createAndRefreshAssetUrlsAndObjects(this.props.exerciseDescription.assets)
      setDoExerciseDummy(false) //re render because we now have assets --> only right 1 render

      //we also need to renew the timer
      Logger.debug('doExerciseSite', 'setting up timers (refresh)')
      await this.setupTimeUsedUpIntervals()

      return
    } else {


      //if the user (or via a link) the release code is entered before it was entered in the
      //enter release code dialog to it here...

      try {
        const test = await accessExerciseWithReleaseCode(releaseCode, true)
      } catch (err) {
        //already handled by setup.ts
        return
      }

      LoginTimeoutHelper.lastKnownDoEditorReleaseCode = releaseCode
    }


    //we need to freeze the manager because the multi tab editor would overwrite the solution when the
    //sessions were created

    FrontendSettingsManager.setIsFrozen(true)

    try {
      await this.props.loadDoExerciseSite(releaseCode)
    } catch (err) {
      //TODO better error

      FrontendSettingsManager.setIsFrozen(false)
      return
    }


    if (inExamMode || this.props.doExercise.exerciseRelease.hideSiteHeaderBar) {
      Logger.log('setting exam mode for do exercise site (release param)')
    } else {
      this.props.setIsSiteHeaderBarDisplayedAndEnabledAction(true)
    }


    if (this.props.doExercise.exerciseRelease.shouldClearClipboard) {

      //we can't move this to the header because we need to await the exercise.release to be loaded

      //we cannot clear the clipboard in js...
      //copyToClipboard is not working even with settimeout
      this.lastPageVisibilityWasHidden = document.hidden

      document.addEventListener('visibilitychange', this.onPageVisibilityChanged.bind(this))
      // document.addEventListener('focus', this.onWindowGotFocus.bind(this), true)
      // document.addEventListener('blur', this.onWindowLostFocus.bind(this), true)
      window.onfocus = this.onWindowGotFocus.bind(this)
      window.onblur = this.onWindowLostFocus.bind(this)
    }


    Logger.debug('doExerciseSite', 'setting up timers')
    await this.setupTimeUsedUpIntervals()

    FrontendSettingsManager.setIsFrozen(false)

    Logger.debug('doExerciseSite', 'checking for newer frontend solution version')
    await this.checkForNewerFrontendSolutionVersion()

  }

  /**
   * called when the tab changed (or when minimized)
   * but NOT when the user changes windows focus
   */
  onPageVisibilityChanged(): void {

    if (this.lastPageVisibilityWasHidden && document.hidden === false) {
      //window got focus back
      this.props.setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(true)
    }
    //
    console.log("new state hidden: " + document.hidden)
    this.lastPageVisibilityWasHidden = document.hidden
  }

  onWindowLostFocus(e: FocusEvent): void {

    const el = e.target as HTMLElement

    if (el && el.id === onGotFocusBackInputId) {
      //ignore
      return
    }

    if (this.hasFocus === true) {
      this.hasFocus = false;
      this.props.setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(true)
    }

    console.log("lost focus")
    this.hasFocus = false
  }

  onWindowGotFocus(e: FocusEvent): void {


    if (this.firstOnFocus) {
      this.props.setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(true)
      this.firstOnFocus = false
      return
    }

    const el = e.target as HTMLElement

    if (el && el.id === onGotFocusBackInputId) {
      //ignore
      return
    }
    this.hasFocus = true
    console.log("got focus")
    console.log(this.hasFocus)
  }

  async checkForNewerFrontendSolutionVersion(): Promise<void> {

    //check if we have a newer frontend only version but only if the use can edit...

    if (!this.props.canUserSolveExercise) {
      Logger.debug('doExerciseSite',
        ' cancel checking for newer frontend solution version because canUserSolveExercise is false/not set')
      return
    }

    //only if there is an old saved version
    const frontendSettings = FrontendSettingsManager.getFrontendSettings()
    if (frontendSettings.doEditorSolution) {

      //this implies  FrontendSettings.lastKnownDoEditorReleaseId === frontendSettings.doEditorReleaseId
      //because ids are unique, sequential
      if (frontendSettings.doEditorSolution.id === this.props.solution.id) {
        //frontend version could be newer duo to browser crashes

        //ignore file visibility
        const frontendVersion = JSON.stringify(frontendSettings.doEditorSolution,
          (key: string, value: any) => {
            if (key === 'createdAt') {
              return ''//value.toString() //ignore because we only wont to compare ids & contents...
            }
            if (key === 'isDisplayed') {
              return 'true'
            }
            if (key === 'displayIndex') {
              return '0'
            }
            if (key === 'uiEditSession') {
              return undefined
            }
            if (key === 'shouldNotCount') {
              return ''
            }
            return value
          })

        //ignore file visibility & case date time moment object to string
        const backendVersion = JSON.stringify(this.props.solution,
          (key: string, value: any) => {
            if (key === 'createdAt') {
              return ''//value.toString()  //ignore because we only wont to compare ids & contents...
            }
            if (key === 'isDisplayed') {
              return 'true'
            }
            if (key === 'displayIndex') {
              return '0'
            }
            if (key === 'uiEditSession') {
              return undefined
            }
            if (key === 'shouldNotCount') {
              return ''
            }
            return value
          })


        //both have som props unset that don't matter
        // noinspection TsLint
        let cleanedFrontendVersion: SolutionVersionDoExerciseFullBase = JSON.parse(frontendVersion)
        // noinspection TsLint
        let cleanedBackendVersion: SolutionVersionDoExerciseFullBase = JSON.parse(backendVersion)


        cleanedFrontendVersion = {
          ...cleanedFrontendVersion,
          solutionFiles: orderBy(cleanedFrontendVersion.solutionFiles, (p: SolutionFileDoExerciseFullBase) => p.id)
        }

        cleanedBackendVersion = {
          ...cleanedBackendVersion,
          solutionFiles: orderBy(cleanedBackendVersion.solutionFiles, (p: SolutionFileDoExerciseFullBase) => p.id)
        }


        //if (frontendVersion === backendVersion) {
        const areEqual = isEqual(cleanedFrontendVersion, cleanedBackendVersion)

        if (areEqual) {
          //all ok same
          Logger.debug("doExerciseSite", "compared frontend code version and backend ... EQUAL!");

        } else {
          //take the frontend version

          //TODO remove but there is still a bug where this occurs even if the code is equal...
          Logger.debug("doExerciseSite", "not equal: cleanedFrontendVersion", cleanedFrontendVersion)
          Logger.debug("doExerciseSite", "not equal: cleanedBackendVersion", cleanedBackendVersion)

          const shouldOverwrite = await askDialog(translate(TranslateIds.doEditorRecovery_title),
            translate(TranslateIds.doEditorRecovery_content))

          if (shouldOverwrite) {

            Logger.debug("doExerciseSite",
              "setting frontend solution version as the solution version (because not equal)");

            this.props.setSolutionVersion({
              ...frontendSettings.doEditorSolution,
              createdAt: frontendSettings.doEditorSolution.createdAt.clone(),
              solutionFiles: frontendSettings.doEditorSolution.solutionFiles.map(p => {
                return {
                  ...p,
                  uiEditSession: null
                }
              })
            })

            //we need to pre select the first file...
            //see multiFileAceEditor > componentWillUpdate for explanation
            if (frontendSettings.doEditorSolution.solutionFiles.length > 0) {

              const visibleFiles = frontendSettings.doEditorSolution.solutionFiles.filter(p => p.displayIndex >= 0)
              if (visibleFiles.length === 0) {
                //else we would still have the old file selected that we just overwritten with frontend files
                this.props.setSelectedTabIdDoEditor(-1)
              }
            }


            //also reset the selected tab index

            //mark changes
            this.props.setHasUnsavedChanges(true)
          }

        }
      } else {

        //this can happen if
        //the user solves the solution on another pc then changes back --> take greater id (because unique, sequential)
        //the user starts solving another exercise (or just release)

        if (frontendSettings.lastKnownDoEditorReleaseId !== frontendSettings.doEditorReleaseId
          || frontendSettings.lastKnownDoEditorReleaseId === -1 && frontendSettings.doEditorReleaseId === -1) {
          //user started to solve another exercise (release)... clear old solution
          //overwrite the old with the new one...
          FrontendSettingsManager.setDoEditorSolution({...this.props.solution})

        } else {
          //user starts solving the same release but the frontend version id is higher --> take it
          if (frontendSettings.doEditorSolution.id > this.props.solution.id) {

            const shouldOverwrite = await askDialog(translate(TranslateIds.doEditorRecovery_title),
              translate(TranslateIds.doEditorRecovery_content))

            if (shouldOverwrite) {

              Logger.debug("doExerciseSite",
                "setting frontend solution version as the solution version (because newer id)");
              this.props.setSolutionVersion({...frontendSettings.doEditorSolution})

              //we need to pre select the first file...
              //see multiFileAceEditor > componentWillUpdate for explaination
              if (frontendSettings.doEditorSolution.solutionFiles.length > 0) {

                const visibleFiles = frontendSettings.doEditorSolution.solutionFiles.filter(p => p.displayIndex >= 0)
                if (visibleFiles.length === 0) {
                  //else we would still have the old file selected that we just overwritten with frontend files
                  this.props.setSelectedTabIdDoEditor(-1)
                }
              }

              //mark changes
              this.props.setHasUnsavedChanges(true)
            }

          }
        }
      }

      Logger.debug("doExerciseSite", "end checking frontend & backend version!");

    }
  }

  componentWillUnmount(): void {

    clearInterval(updateLeftTimeIntervals)
    freeAssetUrlsAndObjects()
    setDoExerciseDummy(true) //enable re rendering

    if (this.props.doExercise.exerciseRelease.shouldClearClipboard) {
      document.removeEventListener('visibilitychange', this.onPageVisibilityChanged)
      // document.removeEventListener('focus', this.onWindowGotFocus)
      // document.removeEventListener('blur', this.onWindowLostFocus)
      window.onfocus = null
      window.onblur = null

    }
  }

  async setupTimeUsedUpIntervals(): Promise<void> {


    // noinspection TsLint
    let serverTime: Moment = moment()
    try {
      serverTime = await getServerTime();
    } catch (err) {
      //TODO better error
      Logger.error('doExerciseSite',
        `cannot get time from server, thus cannot calculate left time ... redirect to ${constants.homeLinkPath}`)
      history.push(constants.homeLinkPath)
      return
    }

    const clientServerTimeDiff = moment().diff(serverTime, 'seconds')

    updateLeftTimeIntervals = setInterval(() => {

      const releaseCopy: ExercisePreviewsFrontendExclusive = {
        id: this.props.doExercise.exerciseRelease.id,
        isDisplayed: true,
        isReleased: this.props.doExercise.exerciseRelease.isReleased,
        displayName: '',
        automaticEndAt: this.props.doExercise.exerciseRelease.automaticEndAt,
        hasLimitedWorkingTime: this.props.doExercise.exerciseRelease.hasLimitedWorkingTime,
        availableWorkingTimeInMinutes: this.props.doExercise.exerciseRelease.availableWorkingTimeInMinutes,
        firstAccessedAt: this.props.doExercise.firstAccessedAt,
        shortDescription: '',
        shouldNotCount: false,
        releaseDurationType: this.props.doExercise.exerciseRelease.releaseDurationType,
        tags: [],
        generatedCode: this.props.doExercise.exerciseRelease.releaseCode,
        isStillOpen: false,
        leftTimeString: '',
        releaseStartType: this.props.doExercise.exerciseRelease.releaseStartType,
        isVisibleToAllAfterRelease: false,
        releasedForPLangId: null,
        leftTimeInS: null
      }

      const rest = updateExercisePreviews(
        [this.props.exercisePreview ? this.props.exercisePreview : releaseCopy], clientServerTimeDiff)

      const copy = rest[0]

      //Logger.debug('doExerciseSite', `setExercisePreviewFrontendDoExercise with ${JSON.stringify(copy)}`)

      this.props.setExercisePreviewFrontendDoExercise(copy)

      if (!copy.isStillOpen) {
        clearInterval(updateLeftTimeIntervals)
      }

      //is released and no limited working time --> always allowed to solve
      if (copy.isStillOpen && copy.isReleased && !copy.hasLimitedWorkingTime
        && (copy.releaseStartType === ReleaseStartType.Manual || copy.releaseDurationType === ReleaseDurationType.Infinitely)) {
        clearInterval(updateLeftTimeIntervals)
      }

      if (copy.leftTimeInS !== null && copy.leftTimeInS <= criticalLeftTimeInS) {
        this.props.setIsSubHeaderBarCollapsed(false)
        this.props.setIsLeftTimeStringDisplayed(true)
      }


    }, 1000)

  }

  render(): JSX.Element {


    return (
      <div className="fh">

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          this.props.isOnGotFocusInterceptionOverlayDisplayed &&
          <InterceptionOverlay>

            {
              //copy only work if the input is visible, enabled...
              //because of react the user cannot change the value of the input...
              //so this should be ok (even though it's ugly)
            }


            <p className='fg-colored'>
              {
                translate(TranslateIds.onFocusInterceptionLayerLabel)
              }
            </p>

            <Input id={onGotFocusBackInputId} value=' '/>
            <Button onClick={() => {
              const input = document.getElementById(onGotFocusBackInputId) as HTMLInputElement
              input.select()
              document.execCommand("Copy");

              this.props.setIsOnGotFocusInterceptionOverlayDisplayedDoExerciseSite(false)
            }}>
              {
                translate(TranslateIds.resume)
              }
            </Button>

          </InterceptionOverlay>
        }
        {
          this.props.isCustomTestDialogDisplayed &&
          <ChangeCustomTestDialog/>
        }
        {
          this.props.isViewTestDialogDisplayed &&
          <ViewTestDialog
            cancelViewTestDialog={() => {
              this.props.cancelViewTestDialog()
            }}
            getDownloadTestAssetLink={(testId: number, fileId: number, isSubmitTest: boolean) => {
              return `${debugOriginUrl}/api/assets/${this.props.releaseCode}/${testId}/${fileId}/${isSubmitTest}/false`
            }}
          />
        }
        {
          this.props.isCodeEditorSettingsDialogDisplayed &&
          <EditorSettingsDialog/>
        }

        {
          this.props.isNoDialogDisplayed &&
          <PanelGrid
            width="100%"
            height="100%"

            topRightPanelViewSize={this.props.topRightPanelViewSize}
            topLeftPanelViewSize={this.props.topLeftPanelViewSize}
            botRightPanelViewSize={this.props.botRightPanelViewSize}
            botLeftPanelViewSize={this.props.botLeftPanelViewSize}

            hDividerPosOffset={this.props.hDividerPosOffset}
            vDividerPosOffset={this.props.vDividerPosOffset}
            onHDividerPosOffsetChanged={(Offset) => {
              this.props.setDoExerciseHDividerPosOffset(Offset)
            }}
            onVDividerPosOffsetChanged={(Offset) => {
              this.props.setDoExerciseVDividerPosOffset(Offset)
            }}

            topLeftPanel={
              <Panel
                applyDefaultPadding={false}
                selectedTabIndex={this.props.topLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopLeftSelectedTabIndex(index)
                }}

                onSizeChangedClicked={this.props.setTopLeftPanelViewSize}
                panelViewSize={this.props.topLeftPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.editor)}
                  icon={'write'}
                  applyDefaultPadding={false}
                  tabHeaderBar={
                    <EditorPanelHeaderBar/>
                  }
                >
                  <EditorPanelView/>
                </TabContentHost>

                {
                  this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                  <TabContentHost
                    title={translate(TranslateIds.subsequentEditor)}
                    icon={'life ring'}
                    applyDefaultPadding={false}
                    defaultHelpText={translate(TranslateIds.subsequentEditor_helpText)}
                    tabHeaderBar={
                      <AfterEditorPanelHeaderBar/>
                    }
                  >
                    <AfterEditorPanelView/>
                  </TabContentHost>
                }

              </Panel>
            }

            topRightPanel={
              <Panel
                selectedTabIndex={this.props.topRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopRightPanelViewSize}
                panelViewSize={this.props.topRightPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.exerciseTaskDescription)}
                  icon={'align left'}
                >
                  <PreviewPanelView descriptionContent={this.props.doExercise.exerciseDescription.content}
                                    assets={this.props.doExercise.exerciseDescription.assets}
                                    dummy={this.props.dummy}
                  />
                </TabContentHost>

              </Panel>
            }

            bottomLeftPanel={
              <Panel
                selectedTabIndex={this.props.bottomLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotLeftPanelViewSize}
                panelViewSize={this.props.botLeftPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.normalTests)}
                  icon={'bug'}
                  tabHeaderBar={
                    <TestPanelHeaderBar/>
                  }
                  defaultHelpText={translate(TranslateIds.autoSaveFilesOnTestRunIfNecessary)}
                >
                  <TestsPanel
                    displayWeightIndicatorsForResults={false}
                    tests={this.props.doExercise.tests}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    isUserAllowedToRunTests={this.props.canUserSolveExercise}
                    editorHasUnsavedChanges={this.props.editorHasUnsavedChanges}
                    isSomeTestRunning={this.props.isSomeTestRunning}
                    onSaveSolution={async () => {
                      await this.props.saveDoExerciseSolution(this.props.releaseCode, this.props.solution)
                    }}
                    setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive
                      | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.setSelectedConsoleViewTest(test)
                    }}
                    onRunTest={(test: RunnableExerciseTestFrontendExclusive
                      | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                      const command: RunTestCommandForBackend = {
                        generatedCode: this.props.doExercise.exerciseRelease.releaseCode,
                        plangId: this.props.solutionPLangId,
                        testIds: [test.id],
                        isNormalTest: true,
                        isCustomTest: false,
                        isSubmitTest: false,
                        solutionVersionId: null
                      }

                      this.props.runDoExerciseTest(command)
                    }}
                    openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                      this.props.openViewTestDialog(runnableTest)
                    }}
                  />

                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.customTests)}
                  icon={'shield'}
                  tabHeaderBar={
                    <CustomTestPanelHeaderBar

                      customTests={this.props.doExercise.customTests}
                      canUserCreateCustomTests={this.props.canUserCreateCustomTests}

                      openCreateCustomTestDialog={this.props.openCreateCustomTestDialog}

                      runCustomTests={async (tests: ReadonlyArray<RunnableCustomTestFrontendExclusive>) => {

                        //a template
                        // noinspection TsLint
                        let tempCommand: RunTestCommandForBackend = {
                          generatedCode: this.props.releaseCode,
                          plangId: this.props.solutionPLangId,
                          testIds: tests.map(p => p.id),
                          isNormalTest: false,
                          isCustomTest: true,
                          isSubmitTest: false,
                          solutionVersionId: null
                        }

                        const hasAnyTestFailed = await this.props.runCustomDoExerciseTest(tempCommand)

                        return hasAnyTestFailed as any //wrong infered because magic redux binding
                      }}

                      resetCustomTestResults={this.props.resetCustomTestResults}

                      beforeRunningTests={async () => {
                        if (this.props.editorHasUnsavedChanges) {
                          await this.props.saveDoExerciseSolution(this.props.releaseCode, this.props.solution)
                        }
                      }}

                    />
                  }
                  defaultHelpText={translate(TranslateIds.autoSaveFilesOnTestRunIfNecessary)}
                >
                  <CustomTestsPanel
                    customTests={this.props.doExercise.customTests}
                    canUserEditExercise={this.props.canUserEditExercise}
                    isSomeTestRunning={this.props.isSomeTestRunning}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    setSelectedConsoleViewTest={this.props.setSelectedConsoleViewTest}
                    openUpdateCustomTestDialog={this.props.openUpdateCustomTestDialog}

                    runCustomTest={async (test: RunnableCustomTestFrontendExclusive) => {

                      //a template
                      // noinspection TsLint
                      let tempCommand: RunTestCommandForBackend = {
                        generatedCode: this.props.releaseCode,
                        plangId: this.props.solutionPLangId,
                        testIds: [test.id],
                        isNormalTest: false,
                        isCustomTest: true,
                        isSubmitTest: false,
                        solutionVersionId: null
                      }

                      const hasAnyTestFailed = await this.props.runCustomDoExerciseTest(tempCommand)

                      return hasAnyTestFailed as any //wrong infered because magic redux binding
                    }}

                    beforeRunningTests={async () => {
                      if (this.props.editorHasUnsavedChanges) {
                        await this.props.saveDoExerciseSolution(this.props.releaseCode, this.props.solution)
                      }
                    }}
                    deleteCustomTestDoEditor={(test: RunnableCustomTestFrontendExclusive) => {
                      this.props.deleteCustomTestDoEditor(this.props.releaseCode, test,
                        this.props.doExercise.customTests)
                    }}

                  />
                </TabContentHost>

                {
                  this.props.displaySubmitTestResults &&
                  <TabContentHost
                    title={translate(TranslateIds.submitTests)}
                    icon={'flag checkered'}
                    tabHeaderBar={
                      <SubmitTestsPanelHeaderBar submitTestResults={this.props.submitTestResults}/>

                    }
                    defaultHelpText={translate(TranslateIds.submitTestResultStudentHelp)}
                  >

                    {
                      //submit test panel for submit tests ... don't allow the user to run tests
                    }
                    <TestsPanel
                      displayWeightIndicatorsForResults={true}
                      tests={this.props.submitTestResults}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={false}
                      editorHasUnsavedChanges={null}
                      isSomeTestRunning={false}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}
                      onRunTest={null}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />

                  </TabContentHost>
                }

                {
                  this.props.displayAssessmentResult &&
                  <TabContentHost
                    title={translate(TranslateIds.manualAssessment)}
                    icon={'talk'}
                    tabHeaderBar={
                      <AssessmentResultPanelHeaderBar/>
                    }
                    defaultHelpText={translate(TranslateIds.feedbackFromTutorStudentHelp)}
                  >
                    {
                      //assessment result panel
                    }
                    <AssessmentResultPanelView/>

                  </TabContentHost>
                }

                {
                  //--- after finish normal tests, submit tests
                }
                {
                  this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                  <TabContentHost
                    title={translate(TranslateIds.normalTests)}
                    icon={'life ring'}
                    tabHeaderBar={
                      <AfterNormalTestsPanelHeaderBar/>
                    }
                    defaultHelpText={translate(TranslateIds.noAutomaticTestResultReset)}
                  >
                    <TestsPanel
                      displayWeightIndicatorsForResults={false}
                      tests={this.props.afterNormalTests}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={this.props.releaseFinishedAndAutomaticAssessmentHasFinished}
                      editorHasUnsavedChanges={false}
                      isSomeTestRunning={this.props.isSomeTestRunning}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}
                      onRunTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                        const command: RunAfterTestCommandForBackend = {
                          releaseCode: this.props.doExercise.exerciseRelease.releaseCode,
                          isSubmitTest: false,
                          testIds: [test.id],
                          isNormalTest: true,
                          plangId: this.props.afterSolution.pLangId,
                          mainFileId: this.props.afterSolution.mainFileId,
                          solutionFiles: this.props.afterSolution.solutionFiles
                        }

                        this.props.runAfterNormalTest(command)

                      }}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />

                  </TabContentHost>
                }

                {
                  this.props.releaseFinishedAndAutomaticAssessmentHasFinished &&
                  <TabContentHost
                    title={translate(TranslateIds.submitTests)}
                    icon={'life ring'}
                    tabHeaderBar={
                      <AfterSubmitTestsPanelHeaderBar submitTestResults={this.props.afterSubmitTests}/>
                    }
                    defaultHelpText={translate(TranslateIds.noAutomaticTestResultReset)}
                  >
                    <TestsPanel
                      displayWeightIndicatorsForResults={true}
                      tests={this.props.afterSubmitTests}
                      selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                      isUserAllowedToRunTests={this.props.releaseFinishedAndAutomaticAssessmentHasFinished}
                      editorHasUnsavedChanges={false}
                      isSomeTestRunning={this.props.isSomeTestRunning}
                      onSaveSolution={null}
                      setSelectedConsoleViewTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.setSelectedConsoleViewTest(test)
                      }}
                      onRunTest={(test: RunnableExerciseTestFrontendExclusive
                        | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive) => {

                        const command: RunAfterTestCommandForBackend = {
                          releaseCode: this.props.doExercise.exerciseRelease.releaseCode,
                          isSubmitTest: true,
                          testIds: [test.id],
                          isNormalTest: false,
                          plangId: this.props.afterSolution.pLangId,
                          mainFileId: this.props.afterSolution.mainFileId,
                          solutionFiles: this.props.afterSolution.solutionFiles
                        }

                        this.props.runAfterSubmitTest(command)

                      }}
                      openViewTestDialog={(runnableTest: RunnableExerciseTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null) => {
                        this.props.openViewTestDialog(runnableTest)
                      }}
                    />

                  </TabContentHost>
                }


              </Panel>
            }

            bottomRightPanel={
              <Panel
                selectedTabIndex={this.props.bottomRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotRightPanelViewSize}
                panelViewSize={this.props.botRightPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.consoleView)}
                  icon={'terminal'}
                  tabHeaderBar={
                    <ConsolePanelHeaderBar test={this.props.selectedConsoleViewTest} testTypes={this.props.testTypes}
                                           useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                                           setUseDiffIfOutputMismatched={this.props.setUseDiffIfOutputMismatched}
                                           useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                                           setUseReplaceWhiteSpacesWithBullets={this.props.setUseReplaceWhiteSpacesWithBullets}
                    />
                  }
                >
                  <ConsolePanelView test={this.props.selectedConsoleViewTest} protocolItems={null}
                                    testTypes={this.props.testTypes}
                                    useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                                    useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                  />
                </TabContentHost>
              </Panel>
            }


          >
          </PanelGrid>
        }


      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(doExerciseSite))