/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon, Popup} from "semantic-ui-react";
import {HelpPopup} from "../../../helpers/helpPopup";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {
  resetDoExerciseSolutionToTemplate,
  setIsDoCodeEditorSettingsDialogDisplayed
} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {openEditEditorSettingsDialog} from "../../../../state/actions/editorSettingsDialog/editorSettingsDialogActions";
import {askDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import {leftIconPopupOffset} from "../../../../constants";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    codeEditorSettings: rootState.userDataSettingsSate.userData.settings ?
      rootState.userDataSettingsSate.userData.settings.codeEditorSettings
      : null,

    pLang: rootState.doExerciseSiteState.solution.pLangId,
    codeTemplates: rootState.doExerciseSiteState.exercise.codeTemplates,
    solutionCreatedAt: rootState.doExerciseSiteState.solution.createdAt,

    canUserEditExercise: rootState.doExerciseSiteState.exercisePreviewFrontendTemp
    && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsDoCodeEditorSettingsDialogDisplayed,
  openEditEditorSettingsDialog,
  resetDoExerciseSolutionToTemplate,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelHeaderBar extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {
               this.props.openEditEditorSettingsDialog(this.props.codeEditorSettings,
                 this.props.setIsDoCodeEditorSettingsDialogDisplayed)
             }}
        >
          <Icon name="settings"/>
          <span>
            {
              translate(TranslateIds.editorSettings)
            }
          </span>
        </div>

        <SimpleVDivider/>

        <div className={`mar-right-half ${this.props.canUserEditExercise ? 'clickable' : 'div-disabled' }`}
             onClick={async () => {


               const shouldReset = await askDialog(translate(TranslateIds.resetCode_title),
                 translate(TranslateIds.resetCode_content))

               if (shouldReset) {
                 this.props.resetDoExerciseSolutionToTemplate(this.props.pLang, this.props.codeTemplates)

               }

             }}
        >
          <Icon name="undo"/>
          <span>
            {
              translate(TranslateIds.resetCode)
            }
          </span>
        </div>
        <HelpPopup
          defaultText={translate(TranslateIds.resetCode_helpText)}
          wide
        />

        <SimpleVDivider/>

        <Popup
          trigger={
            <div className="hoverable" style={{cursor: 'default'}}>

              <Icon className="mar-right-half" name="send"/>
              <span className="mar-right-half">
              {
                translate(TranslateIds.solutionServerVersionCreatedAt)
              }
              </span>
              <span>
              {
                displayDateAndTime(this.props.solutionCreatedAt, -1)
              }
              </span>
            </div>
          }
          content={translate(TranslateIds.solutionServerVersionCreatedAt_helpText)}
          offset={leftIconPopupOffset}
        />


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelHeaderBar)