/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import MultiTabCodeEditorWrapper from '../../../codeEditors/multiTabCodeEditorWrapper'
import {
  closeTabChangeDoEditor,
  openTabChangeDoEditor, setHasFileChangedMapDoEditor, setHasUnsavedChanges,
  setIsFileExplorerDisplayedDoEditor, setIsRenamingFileDoEditor, setNewFileNameDoEditor,
  setRenamingFileIdDoEditor,
  setRenamingFileNameDoEditor,
  setSelectedTabIdDoEditor
} from "../../../../state/actions/doExerciseSite/editorActions/doEditorActions";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import {
  setSolutionFiles,
} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";
import Logger from "../../../../helpers/logger";
import {
  setSolutionFiles_AndRefreshEditor,
} from "../../../../helpers/editorHelper";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {EditorTab, MultiTabEditorState} from "../../../codeEditors/multiTabCodeEditorProps";
import {setForceEditorUpdateSessionDummyDoEditor} from "../../../../state/actions/editorHelperState/editorHelperStateActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    doExercise: rootState.doExerciseSiteState.exercise,
    solution: rootState.doExerciseSiteState.solution,

    pLangs: rootState.pLangsState.pLangs,

    selectedTabIdDoEditor: rootState.doExerciseSiteState.doEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.doExerciseSiteState.doEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.doExerciseSiteState.doEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.doExerciseSiteState.doEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.doExerciseSiteState.doEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.doExerciseSiteState.doEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.doExerciseSiteState.doEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.doExerciseSiteState.doEditorState.hasFileChangedMap,

    forceEditorUpdateSessionDummy: rootState.editorHelperState.forceEditorUpdateSessionDummyDoEditor,

    canUserEditExercise: rootState.doExerciseSiteState.exercisePreviewFrontendTemp
    && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen
    && rootState.doExerciseSiteState.isOnGotFocusInterceptionOverlayDisplayed === false,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setSelectedTabIdDoEditor,
  setIsFileExplorerDisplayedDoEditor,
  setNewFileNameDoEditor,
  setRenamingFileIdDoEditor,
  setIsRenamingFileDoEditor,
  setRenamingFileNameDoEditor,

  openTabChangeDoEditor,
  closeTabChangeDoEditor,
  setHasUnsavedChanges,

  setSolutionFiles,
  setHasFileChangedMapDoEditor,

  setSolutionFiles_AndRefreshEditor,
  setForceEditorUpdateSessionDummyDoEditor,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelView extends React.Component<Props, any> {

  render(): JSX.Element {

    const pLang = this.props.pLangs.find(p => p.id === this.props.solution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!
      return null
    }

    const editorTabs = this.props.solution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.solution.mainFileId,
        isReadonly: !this.props.canUserEditExercise || !p.isEditableByUser,
        displayIndex: p.displayIndex,
        language: pLang.internalName,
        editSession: p.uiEditSession
      }
    })

    return (
      <div className="fh hidden-overflow-i">
        <MultiTabCodeEditorWrapper
          id="editorPanelView"
          forceEditorUpdateSessionDummy={this.props.forceEditorUpdateSessionDummy}
          height="100%"
          selectedTabId={this.props.selectedTabIdDoEditor}
          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setSelectedTabIdDoEditor(tabId)
          }}

          isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
          onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
            this.props.setIsFileExplorerDisplayedDoEditor(isDisplayed)
          }}

          newFileName={this.props.newFileNameDoEditor}
          onNewFileNameChanged={(name: string) => {
            this.props.setNewFileNameDoEditor(name)
          }}

          renamingFileId={this.props.renamingFileIdDoEditor}
          onRenamingFileIdChanged={(fileId: number) => {
            this.props.setRenamingFileIdDoEditor(fileId)
          }}

          isRenamingFile={this.props.isRenamingFileDoEditor}
          onIsRenamingFileChanged={(isRenamingFileDoEditor: boolean) => {
            this.props.setIsRenamingFileDoEditor(isRenamingFileDoEditor)
          }}

          renamingFileName={this.props.renamingFileNameDoEditor}
          onRenamingFileNameChanged={(newName: string) => {
            this.props.setRenamingFileNameDoEditor(newName)
          }}

          hasUnsavedChanges={this.props.hasUnsavedChanges}
          onHasUnsavedChangesChanged={(hasChanged: boolean) => {
            if (hasChanged === this.props.hasUnsavedChanges) return
            this.props.setHasUnsavedChanges(true)
          }}

          userCanCreateNewFiles={this.props.doExercise.canUserCreateFiles}

          language={pLang.internalName}

          onCreateFileClicked={(editorTab: EditorTab) => {

            const newFile: SolutionFileDoExerciseFullBase = {
              id: editorTab.id,
              content: editorTab.content,
              fileNameWithExtension: editorTab.fileNameWithExtensions,
              uiIsDisplayed: editorTab.displayIndex >= 0,
              displayIndex: editorTab.displayIndex,
              isEditableByUser: true,
              templateFileId: null,
              uiEditSession: editorTab.editSession
            }

            this.props.setSolutionFiles_AndRefreshEditor(
              [...this.props.solution.solutionFiles, newFile],
              this.props.setSolutionFiles,
              this.props.setForceEditorUpdateSessionDummyDoEditor
            )

            //this.props.setSolutionFiles_CreateFile([...this.props.solution.solutionFiles, newFile])

          }}

          onOpenTabClicked={(editorTab: EditorTab) => {
            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (open tab clicked)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            //do we still need this???
            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: true,
                }
                : p
            })
            this.props.openTabChangeDoEditor(file, mappedFiles, this.props.selectedTabIdDoEditor)

            //this.props.openTabChangeDoEditor(file, this.props.solution.solutionFiles, this.props.selectedTabIdDoEditor)
          }}

          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (closed tab clicked)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            //do we still need this???
            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: false,
                }
                : p
            })

            this.props.closeTabChangeDoEditor(file, mappedFiles,
              this.props.selectedTabIdDoEditor)

            // this.props.closeTabChangeDoEditor(file, this.props.solution.solutionFiles,
            //   this.props.selectedTabIdDoEditor)
          }}

          onFileRename={(editorTab: EditorTab, newName: string) => {

            const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {
              Logger.error("file not found (onFileRename)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            this.props.setSolutionFiles(
              this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                  ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: newName,
                    isEditableByUser: file.isEditableByUser,
                    content: file.content,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    uiEditSession: editorTab.editSession
                  }
                  : file
              })
            )
          }}

          onRemoveFile={(editorTab: EditorTab) => {

            this.props.setSolutionFiles_AndRefreshEditor(
              this.props.solution.solutionFiles.filter(p => p.id !== editorTab.id),
              this.props.setSolutionFiles,
              this.props.setForceEditorUpdateSessionDummyDoEditor
            )

            // this.props.setSolutionFiles(
            //   this.props.solution.solutionFiles.filter(p => p.id !== editorTab.id)
            // )

          }}

          editorTabs={editorTabs}
          onChange={(editorTab: EditorTab, newValue: string) => {

            const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {
              Logger.error("file not found (onChange)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            this.props.setSolutionFiles(
              this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                  ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: file.fileNameWithExtension,
                    isEditableByUser: file.isEditableByUser,
                    content: newValue,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    uiEditSession: editorTab.editSession
                  }
                  : file
              })
            )
          }}
          onSomeEditorSessionChanged={(editorTabs: ReadonlyArray<EditorTab>) => {
            this.props.setSolutionFiles(
              this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                const changedTab = editorTabs.find(p => p.id === file.id)
                return changedTab
                  ? {
                    ...file,
                    uiEditSession: changedTab.editSession
                  }
                  : file
              })
            )
          }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapDoEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}

        />
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelView)