/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon} from "semantic-ui-react";
import {
  runDoExerciseTest,
  saveDoExerciseSolution
} from "../../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {RunTestCommandForBackend} from "../../../../types/testsAndTestProtocol";
import {HelpPopup} from "../../../helpers/helpPopup";
import Logger from "../../../../helpers/logger";
import {isTestResultValid} from "../../../../helpers/testHelper";
import {ReleaseDurationType} from "../../../../types/ReleaseDurationType";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {resetNormalTestResults} from "../../../../state/actions/doExerciseSite/doExerciseSiteActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tests: rootState.doExerciseSiteState.exercise.tests,
    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    solutionPLangId: rootState.doExerciseSiteState.solution.pLangId,
    systemSettings: rootState.systemSettingsState.systemSettings,
    editorHasUnsavedChanges: rootState.doExerciseSiteState.doEditorState.hasUnsavedChanges,
    solution: rootState.doExerciseSiteState.solution,

    isSomeTestRunning: rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
    || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning),

    canUserEditExercise: (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen)
    || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
      && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
      && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely),

    maxNumberOfTestsWithOneRequest: rootState.systemSettingsState.systemSettings
    && rootState.systemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest >= 0
      ? rootState.systemSettingsState.systemSettings.maxNumberOfTestsWithOneRequest : 1,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  runDoExerciseTest,
  saveDoExerciseSolution,
  resetNormalTestResults,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class testsPanelHeader extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div className="flexed">

        <div
          className={`mar-right-half ${(this.props.canUserEditExercise && !this.props.isSomeTestRunning  && this.props.tests.length > 0) ? 'clickable' : 'div-disabled' }`}
          onClick={async () => {
            //save changes if code changed
            //this also ensures that the old/now invalid after saving test results are cleared
            if (this.props.editorHasUnsavedChanges) {
              await this.props.saveDoExerciseSolution(this.props.releaseCode, this.props.solution)
            }

            const maxNumberOfTestsWithOneRequest = this.props.maxNumberOfTestsWithOneRequest

            Logger.debug("test panel header bar",
              `running all test with ${maxNumberOfTestsWithOneRequest} test in parallel`)

            const validTests = this.props.tests.filter(p => isTestResultValid(p))

            //a template
            // noinspection TsLint
            let tempCommand: RunTestCommandForBackend = {
              generatedCode: this.props.releaseCode,
              plangId: this.props.solutionPLangId,
              testIds: [],
              isNormalTest: true,
              isCustomTest: false,
              isSubmitTest: false,
              solutionVersionId: null
            }

            // noinspection TsLint
            for (let i = 0; i < this.props.tests.length; i++) {
              const test = this.props.tests[i]

              if (isTestResultValid(test) === false || validTests.length === this.props.tests.length) {
                //add
                tempCommand = {
                  ...tempCommand,
                  testIds: [...tempCommand.testIds, test.id]
                }
              } else {
                //test was already valid
              }

              //we reached the limit
              if (tempCommand.testIds.length === maxNumberOfTestsWithOneRequest) {
                const hasAnyTestFailed = await this.props.runDoExerciseTest(tempCommand)

                //clear test ids
                tempCommand = {
                  ...tempCommand,
                  testIds: []
                }

                //stop if at least 1 test failed
                if (hasAnyTestFailed) {
                  //return instead of break else we would run the last check below which would be always true
                  return
                }
              }
            }

            //we might have some last tests but not reached the max num of tests with one request...
            if (tempCommand.testIds.length > 0) {
              //we missed the last < maxNumberOfTestsWithOneRequest tests
              //these are still in the last tempCommand
              await this.props.runDoExerciseTest(tempCommand)
            }

          }}
        >
          <Icon name="forward"/>
          <span>
            {
              translate(TranslateIds.runAllTests)
            }
          </span>
        </div>

        <HelpPopup
          defaultText={
            `${translate(TranslateIds.runAllTests_desc)} ${translate(
              TranslateIds.maxParallelTests)}: ${this.props.maxNumberOfTestsWithOneRequest}`}
          wide
        />

        <SimpleVDivider/>

        <div className={this.props.canUserEditExercise ? 'clickable' : 'div-disabled'}
             onClick={() => {
               this.props.resetNormalTestResults()
             }}
        >
          <Icon name="remove"/>
          <span>
          {
            translate(TranslateIds.clearNormalTestResults)
          }
          </span>
        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(testsPanelHeader)