/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import ChangeTestView from './changeTestView'
import FullSiteDialog from '../../../fullSiteDialog'
import {Link} from "react-router-dom";
import * as constants from '../../../../constants'
import {Icon} from "semantic-ui-react";
import {RootState} from "../../../../state/reducers/index";
import {TranslateIds} from "../../../../../i18n/ids";
import {translate} from "../../../../../i18n/translationHelper";
import {
  saveCustomProjectTestDialog,
  cancelCustomProjectTestDialog
} from "../../../../state/actions/editCustomProjectSite/dialogs/changeCustomProjectTestDialog/changeCustomProjectTestDialogActions";
import {errorDialog} from "../../../../helpers/dialogHelper";
import Logger from "../../../../helpers/logger";
import {notifySuccess} from "../../../../helpers/notificationHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isBusy: rootState.changeCustomProjectTestDialogState.isBusy || rootState.testTypesState.isLoading,
    isCreating: rootState.changeCustomProjectTestDialogState.isCreating,
    isValid: rootState.changeCustomProjectTestDialogState.isValid,
    test: rootState.changeCustomProjectTestDialogState.test,
    allTests: rootState.editCustomProjectSiteState.customProject.tests
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  saveCustomProjectTestDialog,
  cancelCustomProjectTestDialog
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class changeTestDialog extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <FullSiteDialog
        isBusy={this.props.isBusy}
        isValid={this.props.isValid}
        caption={
          <div className="flexed">
            <h2 className="mar-right">
              {
                this.props.isCreating &&
                translate(TranslateIds.createNormalTest)
              }
              {
                !this.props.isCreating &&
                translate(TranslateIds.changeNormalTest)
              }
            </h2>

            <Link target="_blank" to={constants.markdownSiteTestSyntaxGuide}
                  style={{marginTop: '0.5em'}}
            >
              <Icon className="mar-right-half" name="external share"/>
              <span>
                  {
                    translate(TranslateIds.openHelpInNewTab)
                  }
                </span>
            </Link>

          </div>
        }
        onSave={async () => {

          try {
            await this.props.saveCustomProjectTestDialog(this.props.test, this.props.isCreating, this.props.allTests)
          } catch (err) {

            if (this.props.isCreating) {
              errorDialog(translate(TranslateIds.error), translate(TranslateIds.couldNotCreateCustomTest))
            } else {
              errorDialog(translate(TranslateIds.error), translate(TranslateIds.couldNotUpdateCustomTest))
            }
            Logger.error(`could not ${this.props.isCreating ? 'create' : 'update'} custom test, err: ${err.message}`)
            return
          }

          //notify because the user needs to know that he/she not need to save something e.g. solution or exercise
          //this was already done by the save button click
          if (this.props.isCreating) {
            notifySuccess(translate(TranslateIds.createdCustomTest))
          } else {
            notifySuccess(translate(TranslateIds.updatedCustomTest))
          }


        }}
        onCancel={() => {
          this.props.cancelCustomProjectTestDialog()
        }}
      >
        <ChangeTestView/>
      </FullSiteDialog>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(changeTestDialog)