/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {errorDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {createAndRefreshAssetUrlsAndObjects, freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import Spinner from "../../helpers/Spinner";
import PanelGrid from "../../panelSystem/panelGrid/panelGrid";
import Panel from "../../panelSystem/panel/panel";
import TabContentHost from "../../panelSystem/panel/tabContentHost";
import EditorSettingsDialog from '../../editorSettingsDialog/editorSettingsDialog'
import {
  loadEditCustomProjectSite, resetCustomProjectTestResults,
  setBotLeftPanelViewSize, setBotRightPanelViewSize,
  setBottomLeftSelectedTabIndex, setBottomRightSelectedTabIndex, setCustomProjectCodeEditorSettingsDialogDisplayed,
  setCustomProjectHDividerPosOffset,
  setCustomProjectVDividerPosOffset, setInitialPLangId, setSelectedConsoleViewTest,
  setTopLeftPanelViewSize,
  setTopLeftSelectedTabIndex, setTopRightPanelViewSize, setTopRightSelectedTabIndex, setUseDiffIfOutputMismatched,
  setUseReplaceWhiteSpacesWithBullets
} from "../../../state/actions/editCustomProjectSite/editCustomProjectActions";

import {
  setCustomProjectDescriptionAssets,
  setDescriptionContent
} from "../../../state/actions/editCustomProjectSite/subSet/customProjectDescriptionActions";
import {parseId} from "../../../helpers/convertersAndTransformers";
import {RouteComponentProps, withRouter} from "react-router";

//---top right panel
import PreviewPanelView from '../exerciseEditorSite/previewPanel/previewPanelView'
import ExerciseSyntaxHelpPanelView from '../exerciseEditorSite/exerciseSyntaxHelpPanel/exerciseSyntaxHelpPanelView'
import ExerciseSyntaxHelpPanelHeaderBar from '../exerciseEditorSite/exerciseSyntaxHelpPanel/exerciseSyntaxHelpPanelHeaderBar'

import PropertiesPanelView from './propertiesPanel/propertiesPanelView'


//--- top left panel
import EditorPanelView from './editorPanel/editorPanelView'
import EditorPanelHeaderBar from './editorPanel/editorPanelHeaderBar'

import DescriptionEditorPanelView from '../exerciseEditorSite/descriptionEditorPanel/descriptionEditorPanelView'
import DescriptionEditorHeaderBar from '../exerciseEditorSite/descriptionEditorPanel/descriptionEditorHeaderBar'

//--- bottom left panel
import CustomTestsPanel from '../doExerciseSite/customTestsPanel/customTestsPanelView'
import CustomTestPanelHeaderBar from '../doExerciseSite/customTestsPanel/customTestsPanelHeaderBar'

import AssetsPanelView from '../exerciseEditorSite/assetsPanel/assetsPanelView'
import AssetsPanelHeaderBar from '../exerciseEditorSite/assetsPanel/assetsPanelHeaderBar'

//--- bottom right panel
import ConsolePanelView from '../doExerciseSite/consolePanel/consolePanelView'
import ConsolePanelHeaderBar from '../doExerciseSite/consolePanel/consolePanelHeaderBar'
import {RunnableCustomProjectTestFrontendExclusive} from "../../../types/customProjects";


import ChangeTestDialog from './dialogs/changeTestDialog'
import {
  openCreateCustomProjectTestDialog, openEditCustomProjectTestDialog,
  resetChangeCustomProjectTestDialog
} from "../../../state/actions/editCustomProjectSite/dialogs/changeCustomProjectTestDialog/changeCustomProjectTestDialogActions";
import {LoginTimeoutHelper} from "../../../helpers/loginTimeoutHelper";
import Logger from "../../../helpers/logger";
import {deleteCustomTestCustomProjectAsync} from "../../../state/actions/editCustomProjectSite/subSet/customProjectTestActions";
import {
  runCustomProjectTestAsync,
  saveCustomProjectSolution
} from "../../../state/actions/editCustomProjectSite/editCustomProjectCrudActions";
import {RunTestCommandCustomProjectForBackend} from "../../../types/testsAndTestProtocol";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemSettings: rootState.systemSettingsState.systemSettings,
    customProjectPreviews: rootState.manageCustomProjectsSiteState.customProjectPreviews,

    customProjectDescription: rootState.editCustomProjectSiteState.customProject.description,
    customProject: rootState.editCustomProjectSiteState.customProject,

    testTypes: rootState.testTypesState.testTypes,
    pLangs: rootState.pLangsState.pLangs,


    editorHasUnsavedChanges: rootState.editCustomProjectSiteState.solutionEditorState.hasUnsavedChanges,
    canCreateCustomProject: rootState.manageCustomProjectsSiteState.customProjectPreviews.length < rootState.systemSettingsState.systemSettings.maxCustomProjectsPerUser,

    selectedConsoleViewTest: rootState.editCustomProjectSiteState.selectedConsoleViewTest,
    useDiffIfOutputMismatched: rootState.editCustomProjectSiteState.useDiffIfOutputMismatched,
    useReplaceWhiteSpacesWithBullets: rootState.editCustomProjectSiteState.useReplaceWhiteSpacesWithBullets,

    topRightSelectedTabIndex: rootState.editCustomProjectSiteState.topRightSelectedTabIndex,
    topLeftSelectedTabIndex: rootState.editCustomProjectSiteState.topLeftSelectedTabIndex,
    bottomRightSelectedTabIndex: rootState.editCustomProjectSiteState.bottomRightSelectedTabIndex,
    bottomLeftSelectedTabIndex: rootState.editCustomProjectSiteState.bottomLeftSelectedTabIndex,

    topRightPanelViewSize: rootState.editCustomProjectSiteState.topRightPanelViewSize,
    topLeftPanelViewSize: rootState.editCustomProjectSiteState.topLeftPanelViewSize,
    botRightPanelViewSize: rootState.editCustomProjectSiteState.botRightPanelViewSize,
    botLeftPanelViewSize: rootState.editCustomProjectSiteState.botLeftPanelViewSize,

    vDividerPosOffset: rootState.editCustomProjectSiteState.vDividerPosOffset,
    hDividerPosOffset: rootState.editCustomProjectSiteState.hDividerPosOffset,

    isChangeTestDialogDisplayed: rootState.editCustomProjectSiteState.isChangeTestDialogDisplayed,
    isEditorSettingsDialogDisplayed: rootState.editCustomProjectSiteState.isCustomProjectCodeEditorSettingsDialogDisplayed,

    isSomeTestRunning: rootState.editCustomProjectSiteState.customProject.tests.some(p => p.uiIsRunning),

    isLoading: rootState.editCustomProjectSiteState.isLoading
    || rootState.pLangsState.isLoading
    || rootState.testTypesState.isLoading,

    isNoDialogDisplayed: !rootState.editCustomProjectSiteState.isCustomProjectCodeEditorSettingsDialogDisplayed
    && !rootState.editCustomProjectSiteState.isChangeTestDialogDisplayed
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setCustomProjectVDividerPosOffset,
  setCustomProjectHDividerPosOffset,
  setTopLeftSelectedTabIndex,
  setTopLeftPanelViewSize,
  setTopRightSelectedTabIndex,
  setTopRightPanelViewSize,
  setBottomLeftSelectedTabIndex,
  setBotLeftPanelViewSize,
  setBottomRightSelectedTabIndex,
  setBotRightPanelViewSize,

  setUseDiffIfOutputMismatched,
  setUseReplaceWhiteSpacesWithBullets,

  setCustomProjectCodeEditorSettingsDialogDisplayed,
  setDescriptionContent,
  setCustomProjectDescriptionAssets,
  loadEditCustomProjectSite,

  resetChangeCustomProjectTestDialog,
  openCreateCustomProjectTestDialog,
  openEditCustomProjectTestDialog,
  setSelectedConsoleViewTest,

  setInitialPLangId,
  deleteCustomTestCustomProjectAsync,
  runCustomProjectTestAsync,
  saveCustomProjectSolution,
  resetCustomProjectTestResults,
}, dispatch)


interface MatchedProps {
  /**
   * the id to load or undefined to create a new one
   */
  readonly customProjectId: string | undefined
}

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editCustomProjectSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  async componentWillMount(): Promise<void> {

    // noinspection TsLint
    const customProjectId = parseId(this.props.match.params.customProjectId)

    if (customProjectId !== null) {
      //load

      if (LoginTimeoutHelper.isReLogin && customProjectId === LoginTimeoutHelper.lastKnownCustomProjectId) {
        //do not reload or reset site state...
        //user re logged in he/she want's to continue

        createAndRefreshAssetUrlsAndObjects(this.props.customProject.description.assets)

        return
      } else {
        LoginTimeoutHelper.lastKnownCustomProjectId = customProjectId
      }

      try {
        await this.props.loadEditCustomProjectSite(customProjectId, false, false)
      } catch (err) {

        errorDialog(translate(TranslateIds.serverError), translate(TranslateIds.couldNotLoadCustomProject))
        Logger.error(`could not load custom project with id: ${customProjectId}`)
        return
      }

    } else {
      //create

      if (this.props.customProject.id <= 0) {
        if (LoginTimeoutHelper.isReLogin && this.props.customProject
          && this.props.customProject.id === LoginTimeoutHelper.lastKnownCustomProjectId
        ) {
          //do not reload or reset site state...
          //user re logged in he/she want's to continue
          return
        } else {
          //this would give us an old id because loadEditCustomProjectSite will create a new one...
          //LoginTimeoutHelper.lastKnownExerciseEditorId = this.props.editorExercise.id
        }
      }


      await this.props.loadEditCustomProjectSite(null, true, false)

      //ensure the user can create a new project
      if (this.props.canCreateCustomProject === false) {
        errorDialog(translate(TranslateIds.error),
          translate(TranslateIds.couldNotCreateCustomProjectBecauseExceededMaxAmount))
        return
      }

      //set initial plang
      if (this.props.pLangs.length > 0) {
        this.props.setInitialPLangId(this.props.pLangs[0].id)
      }


      //grab new id
      LoginTimeoutHelper.lastKnownCustomProjectId = customProjectId
    }
  }

  componentWillUnmount(): void {
    freeAssetUrlsAndObjects()
  }

  render(): JSX.Element {
    return (
      <div className="fh">

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          this.props.isChangeTestDialogDisplayed &&
          <ChangeTestDialog/>
        }

        {
          this.props.isEditorSettingsDialogDisplayed &&
          <EditorSettingsDialog/>
        }

        {
          this.props.isNoDialogDisplayed &&
          <PanelGrid
            width="100%"
            height="100%"

            topRightPanelViewSize={this.props.topRightPanelViewSize}
            topLeftPanelViewSize={this.props.topLeftPanelViewSize}
            botRightPanelViewSize={this.props.botRightPanelViewSize}
            botLeftPanelViewSize={this.props.botLeftPanelViewSize}

            hDividerPosOffset={this.props.hDividerPosOffset}
            vDividerPosOffset={this.props.vDividerPosOffset}
            onHDividerPosOffsetChanged={(Offset: number) => {
              this.props.setCustomProjectHDividerPosOffset(Offset)
            }}
            onVDividerPosOffsetChanged={(Offset: number) => {
              this.props.setCustomProjectVDividerPosOffset(Offset)
            }}

            topLeftPanel={
              <Panel
                applyDefaultPadding={false}
                selectedTabIndex={this.props.topLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopLeftPanelViewSize}
                panelViewSize={this.props.topLeftPanelViewSize}
              >

                <TabContentHost
                  title={translate(TranslateIds.solutionEditor)}
                  icon={'write'}
                  applyDefaultPadding={false}
                  tabHeaderBar={
                    <EditorPanelHeaderBar/>
                  }
                >
                  <EditorPanelView/>
                </TabContentHost>

                <TabContentHost
                  icon={'font'}
                  title={translate(TranslateIds.descriptionEditor)}
                  applyDefaultPadding={false}
                  tabHeaderBar={
                    <DescriptionEditorHeaderBar
                      setIsEditorSettingsDialogDisplayed={this.props.setCustomProjectCodeEditorSettingsDialogDisplayed}
                    />
                  }
                >
                  <DescriptionEditorPanelView
                    descriptionContent={this.props.customProjectDescription.content}
                    setContent={(content: string) => {
                      this.props.setDescriptionContent(content)
                    }}
                  />
                </TabContentHost>

              </Panel>
            }

            topRightPanel={
              <Panel
                selectedTabIndex={this.props.topRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setTopRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setTopRightPanelViewSize}
                panelViewSize={this.props.topRightPanelViewSize}
              >

                <TabContentHost
                  title={translate(TranslateIds.properties)}
                  icon={'options'}
                >
                  <PropertiesPanelView/>
                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.preview)}
                  icon={'unhide'}
                >
                  <PreviewPanelView descriptionContent={this.props.customProjectDescription.content}
                                    assets={this.props.customProjectDescription.assets}
                                    dummy={false /*not needed because we have tabs that will re render if tab index changed*/}
                  />
                </TabContentHost>

                <TabContentHost
                  title={translate(TranslateIds.guide)}
                  icon={'book'}
                  tabHeaderBar={
                    <ExerciseSyntaxHelpPanelHeaderBar/>
                  }
                >
                  <ExerciseSyntaxHelpPanelView/>
                </TabContentHost>

              </Panel>
            }

            bottomLeftPanel={
              <Panel
                selectedTabIndex={this.props.bottomLeftSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomLeftSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotLeftPanelViewSize}
                panelViewSize={this.props.botLeftPanelViewSize}
              >

                <TabContentHost
                  title={translate(TranslateIds.customTests)}
                  icon={'shield'}
                  disabled={this.props.customProject.id <= 0}
                  tabHeaderBar={
                    <CustomTestPanelHeaderBar

                      customTests={this.props.customProject.tests}
                      canUserCreateCustomTests={true}

                      openCreateCustomTestDialog={this.props.openCreateCustomProjectTestDialog}

                      runCustomTests={async (tests: ReadonlyArray<RunnableCustomProjectTestFrontendExclusive>) => {

                        const command: RunTestCommandCustomProjectForBackend = {
                          customProjectId: this.props.customProject.id,
                          plangId: this.props.customProject.solution.pLangId,
                          testIds: tests.map(p => p.id),
                        }

                        const hasAnyTestFailed = await this.props.runCustomProjectTestAsync(command)

                        return hasAnyTestFailed as any //wrong infered because magic redux binding
                      }}

                      resetCustomTestResults={this.props.resetCustomProjectTestResults}

                      beforeRunningTests={async () => {
                        if (this.props.editorHasUnsavedChanges) {
                          await this.props.saveCustomProjectSolution(this.props.customProject)
                        }
                      }}

                    />
                  }
                  defaultHelpText={translate(TranslateIds.autoSaveFilesOnTestRunIfNecessary)}
                >
                  <CustomTestsPanel
                    customTests={this.props.customProject.tests}
                    canUserEditExercise={true}
                    isSomeTestRunning={this.props.isSomeTestRunning}
                    selectedConsoleViewTest={this.props.selectedConsoleViewTest}
                    setSelectedConsoleViewTest={this.props.setSelectedConsoleViewTest}
                    openUpdateCustomTestDialog={this.props.openEditCustomProjectTestDialog}

                    runCustomTest={async (test: RunnableCustomProjectTestFrontendExclusive) => {

                      const command: RunTestCommandCustomProjectForBackend = {
                        customProjectId: this.props.customProject.id,
                        plangId: this.props.customProject.solution.pLangId,
                        testIds: [test.id],
                      }

                      const hasAnyTestFailed = await this.props.runCustomProjectTestAsync(command)

                      return hasAnyTestFailed as any //wrong infered because magic redux binding
                    }}

                    beforeRunningTests={async () => {
                      if (this.props.editorHasUnsavedChanges) {
                        await this.props.saveCustomProjectSolution(this.props.customProject)
                      }
                    }}
                    deleteCustomTestDoEditor={(test: RunnableCustomProjectTestFrontendExclusive) => {
                      this.props.deleteCustomTestCustomProjectAsync(this.props.customProject.id, test,
                        this.props.customProject.tests)
                    }}

                  />
                </TabContentHost>


                <TabContentHost
                  title={translate(TranslateIds.assets)}
                  icon={'file'}
                  disabled={true /*DISABLED until we find a way to limit upload size (at backend)*/}
                  tabHeaderBar={
                    <AssetsPanelHeaderBar assets={this.props.customProjectDescription.assets}
                                          setAssets={(assets) => {
                                            this.props.setCustomProjectDescriptionAssets(assets)
                                          }}
                    />
                  }
                >
                  <AssetsPanelView assets={this.props.customProjectDescription.assets} setAssets={(assets) => {
                    this.props.setCustomProjectDescriptionAssets(assets)
                  }}/>
                </TabContentHost>


              </Panel>
            }

            bottomRightPanel={
              <Panel
                selectedTabIndex={this.props.bottomRightSelectedTabIndex}
                onSelectedTabChanged={(index) => {
                  this.props.setBottomRightSelectedTabIndex(index)
                }}
                onSizeChangedClicked={this.props.setBotRightPanelViewSize}
                panelViewSize={this.props.botRightPanelViewSize}
              >
                <TabContentHost
                  title={translate(TranslateIds.consoleView)}
                  icon={'terminal'}
                  tabHeaderBar={
                    <ConsolePanelHeaderBar test={this.props.selectedConsoleViewTest} testTypes={this.props.testTypes}
                                           useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                                           setUseDiffIfOutputMismatched={this.props.setUseDiffIfOutputMismatched}
                                           useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                                           setUseReplaceWhiteSpacesWithBullets={this.props.setUseReplaceWhiteSpacesWithBullets}
                    />
                  }
                >
                  <ConsolePanelView test={this.props.selectedConsoleViewTest} protocolItems={null}
                                    testTypes={this.props.testTypes}
                                    useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                                    useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                  />
                </TabContentHost>

              </Panel>
            }


          />
        }

      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(editCustomProjectSite))