/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon} from "semantic-ui-react";
import {InputOnChangeEvent} from "../../../../types/reactEvents";
import {ExerciseAssetFullBase} from "../../../../types/exerciseEditor";
import {setExerciseEditorAssets} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseDescriptionActions";
import {getNextFrontendId} from "../../../../helpers/frontendIdHelper";
import {assetPrefix} from "../../../../constants";
import {readFileAsAsset, readFileAsText} from "../../../../helpers/fileReader";
import Logger from "../../../../helpers/logger";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {errorDialog} from "../../../../helpers/dialogHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly assets: ReadonlyArray<ExerciseAssetFullBase>

  readonly setAssets: (assets: ReadonlyArray<ExerciseAssetFullBase>) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    assets: props.assets,
    setAssets: props.setAssets
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

const importAssetFileExerciseId = 'exerciseEditor-asset-file-import'

class AssetsPanelHeaderBar extends React.Component<Props, any> {


  async addAsset(e: InputOnChangeEvent): Promise<void> {
    const file: File = e.currentTarget.files[0]

    // noinspection TsLint
    let asset: ExerciseAssetFullBase = {
      id: getNextFrontendId(),
      mimeType: file.type,
      displayName: file.name,
      assetUrl: assetPrefix + encodeURI(file.name),
      content: []
    }

    try {
      asset = {
        ...asset,
        content: await readFileAsAsset(file)
      }
    } catch (err) {
      Logger.error(`exercise editor assets panel, cannot read: ${err.message}`)
      return
    }

    this.props.setAssets([...this.props.assets, asset])
  }

  render(): JSX.Element {
    return (
      <div className="flexed">

        <div className="clickable"
             onClick={() => {

               const input = document.getElementById(importAssetFileExerciseId)
               if (!input) {
                 errorDialog(translate(TranslateIds.error),
                   translate(TranslateIds.couldNotGetHtmlElementWithSpecificId)
                 )
                 return
               }

               input.click()

             }}
        >
          <Icon name="add"/>
          <span>
            {
              translate(TranslateIds.add)
            }
          </span>
        </div>

        <input id={importAssetFileExerciseId} type="file" className="collapsed" accept="*/*"
               onChange={(e: InputOnChangeEvent) => this.addAsset(e)}
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AssetsPanelHeaderBar)