/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {ExerciseAssetFullBase} from "../../../../types/exerciseEditor";
import {List, Icon} from "semantic-ui-react";
import {copyToClipboard} from "../../../../helpers/clipboard";
import {convertSizeToHumanReadableSize} from "../../../../helpers/convertersAndTransformers";
import {setExerciseEditorAssets} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseDescriptionActions";
import {getCopyAssetUrlText} from "../../../../helpers/assetHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly assets: ReadonlyArray<ExerciseAssetFullBase>

  readonly setAssets: (assets: ReadonlyArray<ExerciseAssetFullBase>) => void
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    assets: props.assets,
    setAssets: props.setAssets,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class AssetsPanelView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>

        <List>
          {
            this.props.assets.map((asset: ExerciseAssetFullBase) => {
              return (
                <List.Item key={asset.id}>
                  <List.Icon className="clickable" name='remove' size='large' verticalAlign='middle'
                             onClick={() => {

                               this.props.setAssets(this.props.assets.filter(p => p.id !== asset.id))
                             }}
                  />
                  <List.Content>
                    <List.Header>
                      <div className="flexed">
                        <div className="mar-right-half">
                          {
                            asset.displayName
                          }
                        </div>

                        <div className="not-important-colored">
                          (
                          {convertSizeToHumanReadableSize(asset.content.length)}
                          )
                        </div>
                      </div>
                    </List.Header>
                    <List.Description>
                      <div className="flexed">
                        <div className="mar-right-half">
                          {
                            asset.assetUrl
                          }
                        </div>
                        <div className="clickable"
                             onClick={() => {
                               copyToClipboard(getCopyAssetUrlText(asset))
                             }}
                        >
                          <Icon name="copy"/>
                        </div>
                      </div>
                    </List.Description>
                  </List.Content>
                </List.Item>
              )
            })
          }
        </List>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AssetsPanelView)