/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers/index";
import {Form, Popup, Icon} from "semantic-ui-react";
import {SelectOnChangeEventData} from "../../../../../types/reactEvents";
import {
  onCreateTemplateFile,
  setChangCodeTemplate,
  setMainFileId,
  setPLangId, setSelectedEditorTabIdChangeCodeTemplate, setTemplateFiles
} from "../../../../../state/actions/exerciseEditorSite/dialogs/changeCodeTemplateDialog/changeCodeTemplateActions";
import {parseAnyId, parseFrontendId, parseId} from "../../../../../helpers/convertersAndTransformers";
import MaterialInput from '../../../../material/materialInput'
import MaterialSelectWrapper from '../../../../material/materialSelectWrapper'
import MultiTabCodeEditorWrapper from '../../../../codeEditors/multiTabCodeEditorWrapper'
import {leftIconPopupOffset} from "../../../../../constants";
import {
  closeTabChangeCodeTemplate,
  openTabChangeCodeTemplate, setChangeCodeTemplateNewFileName, setHasFileChangedMapExerciseEditor, setHasUnsavedChanges,
  setIsMonacoFileExplorerDisplayed, setIsRenamingCodeTemplateFile, setRenamingCodeTemplateFileName,
  setRenamingFileIdCodeTemplate
} from "../../../../../state/actions/exerciseEditorSite/dialogs/changeCodeTemplateDialog/changeCodeTemplateDialogActions";
import {ExerciseCodeTemplateFileFullBase} from "../../../../../types/exerciseEditor";
import {setSolutionFiles_AndRefreshEditor, updateHasFileChangedMap} from "../../../../../helpers/editorHelper";
import {translate} from "../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../i18n/ids";
import Logger from "../../../../../helpers/logger";
import {errorDialog} from "../../../../../helpers/dialogHelper";
import {EditorTab} from "../../../../codeEditors/multiTabCodeEditorProps";
import {setForceEditorUpdateSessionDummyCreateExerciseTemplateEditor} from "../../../../../state/actions/editorHelperState/editorHelperStateActions";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    allPLangs: rootState.pLangsState.pLangs,
    allCodeTemplates: rootState.exerciseEditorState.editorExercise.codeTemplates,
    codeTemplate: rootState.changeCodeTemplateDialogState.codeTemplate,
    selectedEditorTabId: rootState.changeCodeTemplateDialogState.selectedEditorTabId,
    isMonacoFileExplorerDisplayed: rootState.changeCodeTemplateDialogState.isMonacoFileExplorerDisplayed,
    changeCodeTemplateNewFileName: rootState.changeCodeTemplateDialogState.changeCodeTemplateNewFileName,
    isCreating: rootState.changeCodeTemplateDialogState.isCreating,
    isRenamingCodeTemplateFile: rootState.changeCodeTemplateDialogState.isRenamingCodeTemplateFile,
    renamingCodeTemplateFileName: rootState.changeCodeTemplateDialogState.renamingCodeTemplateFileName,
    renamingFileIdCodeTemplate: rootState.changeCodeTemplateDialogState.renamingFileIdCodeTemplate,
    hasUnsavedChanges: rootState.changeCodeTemplateDialogState.hasUnsavedChanges,
    hasFileChangedMap: rootState.changeCodeTemplateDialogState.hasFileChangedMap,

    forceEditorUpdateSessionDummy: rootState.editorHelperState.forceEditorUpdateSessionDummyCreateExerciseTemplateEditor,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setPLangId,
  setMainFileId,
  setSelectedEditorTabIdChangeCodeTemplate,
  setTemplateFiles,
  setIsMonacoFileExplorerDisplayed,
  openTabChangeCodeTemplate,
  closeTabChangeCodeTemplate,
  setChangeCodeTemplateNewFileName,
  setIsRenamingCodeTemplateFile,
  setRenamingCodeTemplateFileName,
  setRenamingFileIdCodeTemplate,

  setHasUnsavedChanges,
  setHasFileChangedMapExerciseEditor,

  setSolutionFiles_AndRefreshEditor,
  setForceEditorUpdateSessionDummyCreateExerciseTemplateEditor,
  onCreateTemplateFile,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class changeCodeTemplateView extends React.Component<Props, any> {

  componentWillMount(): void {

    const possiblePLangs = this.props.allPLangs.filter(pLang => {
      const codeTemplatePLang = this.props.allCodeTemplates.find(codeTemplate => codeTemplate.pLangId === pLang.id)
      return !codeTemplatePLang
    })
    const pLang = this.props.allPLangs.find(p => p.id === this.props.codeTemplate.pLangId)

    if (!pLang && this.props.isCreating) {
      this.props.setPLangId(possiblePLangs[0].id)
    }

  }

  render(): JSX.Element {

    //TODO only show plangs where we not already have a code template...

    const possiblePLangs = this.props.allPLangs.filter(pLang => {
      const codeTemplatePLang = this.props.allCodeTemplates.find(codeTemplate => codeTemplate.pLangId === pLang.id)
      return !codeTemplatePLang
    })

    // noinspection TsLint
    let pLang = this.props.allPLangs.find(p => p.id === this.props.codeTemplate.pLangId)

    if (!pLang && !this.props.isCreating) {
      //this also happens if we not selected a plang yet (creating code template)
    } else if (possiblePLangs.length === 0) {
      //should not happen
    }

    if (!pLang && this.props.isCreating) {
      pLang = possiblePLangs[0]
      //react warning ... render should have no side effects
      //this.props.setPLangId(possiblePLangs[0].id)
    }

    const editorTabs: ReadonlyArray<EditorTab> = this.props.codeTemplate.templateFiles.map(p => {
      const editorTab: EditorTab = {
        id: p.id,
        content: p.content,
        displayIndex: p.displayIndex,
        fileNameWithExtensions: p.fileNameWithExtension,
        isMainFile: p.id === this.props.codeTemplate.mainFileId,
        isReadonly: false,
        language: pLang.internalName,
        editSession: p.uiEditSession
      }
      return editorTab
    })

    const activeEditorTab: EditorTab = editorTabs.find(p => p.id === this.props.selectedEditorTabId)

    return (

      <div className="fh">

        <div className="ui form">

          <Form.Group widths='equal'>
            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.pLang)
                }
              </label>

              {
                //when creating and changing plang the editor changes to the selected plang...
                //during save/create we take the selected plang so everything should be ok
                this.props.isCreating &&
                <MaterialSelectWrapper
                  value={this.props.codeTemplate.pLangId}
                >
                  <select
                    value={this.props.codeTemplate.pLangId}
                    onChange={(e: SelectOnChangeEventData) => {
                      const id = parseId(e.currentTarget.value)

                      if (id === null) {

                        Logger.error(`plang with id ${id} not found`)
                        errorDialog(translate(TranslateIds.error),
                          translate(TranslateIds.plangIdNotFound)
                        )

                      } else {
                        this.props.setPLangId(id)
                      }
                    }}
                  >
                    {
                      possiblePLangs.map(pLang => {
                        return (
                          <option key={pLang.id} value={pLang.id}>
                            {
                              pLang.displayName
                            }
                          </option>
                        )
                      })
                    }
                  </select>
                </MaterialSelectWrapper>
              }
              {
                //after creating we don't allow the user to change the plang ... he/she can create a new template
                //this way he/she cannot select the plang twice or do something other bad (hopefully)
                !this.props.isCreating &&
                <MaterialInput value={pLang ? pLang.displayName : 'error'} disabled
                />
              }


            </Form.Field>

            <Form.Field required>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {
                    translate(TranslateIds.mainFile)
                  }
                </label>
                <Popup
                  inverted
                  trigger={
                    <Icon name="question circle" className="hoverable"/>
                  }
                  content={translate(TranslateIds.mainFile_desc)}
                  offset={leftIconPopupOffset}
                />
              </div>

              {
                this.props.codeTemplate.templateFiles.length === 0 &&
                <MaterialInput
                  value={translate(TranslateIds.atLeastOneFileIsNeeded)}
                  disabled
                />
              }
              {
                this.props.codeTemplate.templateFiles.length !== 0 &&
                <MaterialSelectWrapper
                  value={this.props.codeTemplate.mainFileId}
                >
                  <select
                    value={this.props.codeTemplate.mainFileId}
                    onChange={(e: SelectOnChangeEventData) => {
                      console.log(e.currentTarget.value)
                      //this can be an old or new file
                      const id = parseAnyId(e.currentTarget.value)

                      if (id === null) {
                        errorDialog(translate(TranslateIds.error),
                          translate(TranslateIds.couldNotConvertStringToInt)
                        )
                      } else {
                        this.props.setMainFileId(id)
                      }
                    }}
                  >
                    {
                      this.props.codeTemplate.templateFiles.map(file => {
                        return (
                          <option key={file.id} value={file.id}>
                            {
                              file.fileNameWithExtension
                            }
                          </option>
                        )
                      })
                    }
                  </select>
                </MaterialSelectWrapper>
              }
            </Form.Field>


          </Form.Group>

        </div>


        {
          //TODO editor sessions don't work 100%
          //when we open and close (and save) the template multiple times and pressing ctrl+z (undo)...
          //but it's ok for now
          //TODO maybe somehow reset the sessions ?? we will lose undo but thats ok here (i think)
        }
        <MultiTabCodeEditorWrapper
          id="changeCodeTemplateView"
          height="400px"
          forceEditorUpdateSessionDummy={this.props.forceEditorUpdateSessionDummy}
          isRenamingFile={this.props.isRenamingCodeTemplateFile}
          renamingFileId={this.props.renamingFileIdCodeTemplate}
          renamingFileName={this.props.renamingCodeTemplateFileName}
          onFileRename={(editorTab: EditorTab, newName: string) => {
            //replace the file name
            this.props.setTemplateFiles(
              this.props.codeTemplate.templateFiles.map(file => {
                return file.id === editorTab.id
                  ? {
                    ...file,
                    fileNameWithExtension: newName
                  }
                  : file
              })
            )
            //then reset
            this.props.setIsRenamingCodeTemplateFile(false)
            this.props.setRenamingCodeTemplateFileName('')

          }}

          hasUnsavedChanges={this.props.hasUnsavedChanges}
          onHasUnsavedChangesChanged={(hasChanged: boolean) => {
            if (hasChanged === this.props.hasUnsavedChanges) return
            this.props.setHasUnsavedChanges(true)
          }}

          onRenamingFileIdChanged={(fileId: number) => {
            this.props.setRenamingFileIdCodeTemplate(fileId)
          }}

          onIsRenamingFileChanged={(isRenaming: boolean) => {
            this.props.setIsRenamingCodeTemplateFile(isRenaming)
          }}

          onRenamingFileNameChanged={(newName: string) => {
            this.props.setRenamingCodeTemplateFileName(newName)
          }}

          onRemoveFile={(editorTab: EditorTab) => {

            // this.props.setTemplateFiles(
            //   this.props.codeTemplate.templateFiles.filter(p => p.id !== editorTab.id)
            // )

            this.props.setSolutionFiles_AndRefreshEditor(
              this.props.codeTemplate.templateFiles.filter(p => p.id !== editorTab.id),
              this.props.setTemplateFiles,
              this.props.setForceEditorUpdateSessionDummyCreateExerciseTemplateEditor
            )

          }}

          language={pLang.internalName}
          userCanCreateNewFiles={true}
          selectedTabId={this.props.selectedEditorTabId}
          editorTabs={editorTabs}
          newFileName={this.props.changeCodeTemplateNewFileName}

          onCreateFileClicked={(editorTab: EditorTab) => {

            const newFile: ExerciseCodeTemplateFileFullBase = {
              id: editorTab.id,
              displayIndex: editorTab.displayIndex,
              content: editorTab.content,
              isEditableByUser: !editorTab.isReadonly,
              fileNameWithExtension: editorTab.fileNameWithExtensions,
              uiEditSession: editorTab.editSession
            }

            this.props.onCreateTemplateFile(newFile,
              this.props.codeTemplate.templateFiles,
              this.props.selectedEditorTabId
            )

            //this don't work because we cannot trigger a property within a sync method because the editor wouldn't
            //recognize this on componentWillUpdate ...

            //const newTemplateFiles: ReadonlyArray<ExerciseCodeTemplateFileFullBase> = [...this.props.codeTemplate.templateFiles, newFile]

            // this.props.setSolutionFiles_AndRefreshEditor(
            //   [...this.props.codeTemplate.templateFiles, newFile],
            //   this.props.setTemplateFiles,
            //   this.props.setForceEditorUpdateSessionDummyCreateExerciseTemplateEditor
            // )
            //
            // if (this.props.codeTemplate.templateFiles.length === 0) {
            //   //this is the first file set it as the main file
            //   this.props.setMainFileId(newFile.id)
            // }

            // this.props.setTemplateFiles(
            //   newTemplateFiles
            // )

            //open the new file
            // this.props.openTabChangeCodeTemplate(newFile, newTemplateFiles,
            //   this.props.selectedEditorTabId)

          }}

          onNewFileNameChanged={(name: string) => {
            this.props.setChangeCodeTemplateNewFileName(name)
          }}

          isFileExplorerDisplayed={this.props.isMonacoFileExplorerDisplayed}
          onChangeIsFileExplorerDisplayed={(isDisplayed) => {
            this.props.setIsMonacoFileExplorerDisplayed(isDisplayed)
          }}
          onOpenTabClicked={(editorTab: EditorTab) => {

            const file = this.props.codeTemplate.templateFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (onOpenTabClicked)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            this.props.openTabChangeCodeTemplate(file, this.props.codeTemplate.templateFiles,
              this.props.selectedEditorTabId)
          }}
          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.codeTemplate.templateFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (onCloseTabClicked)")
              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )
              return
            }

            this.props.closeTabChangeCodeTemplate(file, this.props.codeTemplate.templateFiles,
              this.props.selectedEditorTabId)
          }}

          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setSelectedEditorTabIdChangeCodeTemplate(tabId)
          }}

          onChange={(editorTab: EditorTab) => {
            this.props.setTemplateFiles(this.props.codeTemplate.templateFiles.map(file => {
              return file.id === activeEditorTab.id ? {
                  ...file,
                  content: editorTab.content,
                  fileNameWithExtension: editorTab.fileNameWithExtensions,
                  displayIndex: editorTab.displayIndex,
                  uiEditSession: editorTab.editSession
                }
                : file
            }))
          }}
          onSomeEditorSessionChanged={(editorTabs: ReadonlyArray<EditorTab>) => {
            this.props.setTemplateFiles(
              this.props.codeTemplate.templateFiles.map<ExerciseCodeTemplateFileFullBase>(file => {
                const changedTab = editorTabs.find(p => p.id === file.id)
                return changedTab
                  ? {
                    ...file,
                    uiEditSession: changedTab.editSession
                  }
                  : file
              })
            )
          }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapExerciseEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}
        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(changeCodeTemplateView)