/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Form, Checkbox, Radio} from "semantic-ui-react";
import MaterialInput from '../../../material/materialInput'
import {
  setCanUserCreateCustomTests, setCanUserCreateFiles,
  setDisplayName, setIsOnlyVisibleToMe,
  setShortDescription, setShowSubmitResults, setUserGroupId
} from "../../../../state/actions/exerciseEditorSite/subSets/editorExercisePropertiesAction";
import {CheckboxOnChangeEventData} from "../../../../types/reactEvents";
import {
  setMaxDiskSpaceInKb,
  setMemoryLimitInKb,
  setTimeoutInMs
} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseDefaultTestSettingsActions";
import {parseId} from "../../../../helpers/convertersAndTransformers";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {errorDialog} from "../../../../helpers/dialogHelper";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    exerciseProperties: rootState.exerciseEditorState.editorExercise.exerciseProperties,
    defaultTestSettings: rootState.exerciseEditorState.editorExercise.defaultTestSettings,
    groups: rootState.exerciseEditorState.groups
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setDisplayName,
  setShortDescription,
  setIsOnlyVisibleToMe,
  setCanUserCreateCustomTests,
  setCanUserCreateFiles,
  setShowSubmitResults,
  setUserGroupId,

  setTimeoutInMs,
  setMemoryLimitInKb,
  setMaxDiskSpaceInKb,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class exerciseSettingsPanelView extends React.Component<Props, any> {

  componentWillMount(): void {

    //pre select managing group
    //done in reducer because the groups might not be loaded here
  }



  render(): JSX.Element {
    return (
      <div className="mar-right-half">

        <div className="ui form">

          <Form.Field required>
            <label>
              {
                translate(TranslateIds.displayName)
              }
            </label>
            <MaterialInput
              value={this.props.exerciseProperties.displayName}
              onChange={(e) => this.props.setDisplayName(e.currentTarget.value)}
            />
          </Form.Field>

          <Form.Field>
            <label>
              {
                translate(TranslateIds.description)
              }
            </label>
            <MaterialInput
              value={this.props.exerciseProperties.shortDescription}
              onChange={(e) => this.props.setShortDescription(e.currentTarget.value)}
            />
          </Form.Field>


          <Form.Field>
            <Form.Field>
              <Checkbox label={translate(TranslateIds.usersAreAllowedToCreateCustomTests)}
                        checked={this.props.exerciseProperties.canUserCreateCustomTests}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setCanUserCreateCustomTests(data.checked)
                        }}
              />
            </Form.Field>

            <Form.Field>
              <Checkbox label={translate(TranslateIds.usersAreAllowedToCreateCustomFiles)}
                        checked={this.props.exerciseProperties.canUserCreateFiles}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setCanUserCreateFiles(data.checked)
                        }}
              />
            </Form.Field>

            <Form.Field>
              <Checkbox label={translate(TranslateIds.exerciseIsOnlyVisibleToOwner)}
                        checked={this.props.exerciseProperties.isOnlyVisibleToMe}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setIsOnlyVisibleToMe(data.checked)
                        }}
              />
            </Form.Field>

            <Form.Field>
              <Checkbox label={translate(TranslateIds.showSubmitTestResults)}
                        checked={this.props.exerciseProperties.showSubmitResults}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setShowSubmitResults(data.checked)
                        }}
                        disabled
              />
            </Form.Field>

          </Form.Field>

          <h4 className="ui dividing header">
            {
              translate(TranslateIds.defaultTestSettingsForCustomTests)
            }
          </h4>

          <Form.Group widths='equal'>
            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.timeoutInMs)
                }
              </label>
              <MaterialInput
                type="number"
                value={this.props.defaultTestSettings.timeoutInMs.toString(10)}
                onChange={(e) => {
                  //checks for > 0
                  const limit = parseId(e.currentTarget.value)

                  if (limit === null) {
                    errorDialog(translate(TranslateIds.error),
                      translate(TranslateIds.couldNotConvertStringToInt)
                    )
                    return
                  }

                  this.props.setTimeoutInMs(limit)
                }}
              />
            </Form.Field>

            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.maxRamInKb)
                }
              </label>
              <MaterialInput
                type="number"
                value={this.props.defaultTestSettings.memoryLimitInKb.toString(10)}
                onChange={(e) => {
                  //checks for > 0
                  const limit = parseId(e.currentTarget.value)

                  if (limit === null) {
                    errorDialog(translate(TranslateIds.error),
                      translate(TranslateIds.couldNotConvertStringToInt)
                    )
                    return
                  }

                  this.props.setMemoryLimitInKb(limit)
                }}
              />
            </Form.Field>

            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.maxDiskSpaceInKb)
                }
              </label>
              <MaterialInput
                type="number"
                value={this.props.defaultTestSettings.maxDiskSpaceInKb.toString(10)}
                onChange={(e) => {
                  //checks for > 0
                  const limit = parseId(e.currentTarget.value)

                  if (limit === null) {
                    errorDialog(translate(TranslateIds.error),
                      translate(TranslateIds.couldNotConvertStringToInt)
                    )
                    return
                  }

                  this.props.setMaxDiskSpaceInKb(limit)
                }}
              />
            </Form.Field>
          </Form.Group>

          <h4 className="ui dividing header">
            {
              translate(TranslateIds.managedByGroup)
            }
          </h4>


          <Form.Field>
            {
              this.props.groups.map(group => {
                return (
                  <Form.Field key={group.id}>
                    <Radio label={group.displayName}
                           name="groupRadio"
                           checked={group.id === this.props.exerciseProperties.userGroupId}
                           value={group.id}
                           onChange={(e, data: CheckboxOnChangeEventData) => {
                             if (data.checked) {
                               this.props.setUserGroupId(group.id)
                             }
                           }}
                    />
                  </Form.Field>
                )
              })
            }
          </Form.Field>

        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(exerciseSettingsPanelView)