/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../state/reducers/index";
import {Form, Icon, Popup} from "semantic-ui-react";
import MaterialInput from '../../../../material/materialInput'
import MaterialSelectWrapper from '../../../../material/materialSelectWrapper'
import {
  validationMessageKeys,
  validationRules
} from "../../../../../state/reducers/exerciseEditorSite/dialogs/changeTestDialog/editorTestReducer";
import {
  setEditorTestContent,
  setDisplayNameAction, setTestTypeId,
  setWeight, setEditorTest_memoryLimitInKb, setEditorTest_timeoutInMs, setEditorTest_maxDiskSpaceInKb,
  setEditorTest_files
} from "../../../../../state/actions/exerciseEditorSite/dialogs/changeTestDialog/changeTestActions";
import {InputOnChangeEvent, SelectOnChangeEventData} from "../../../../../types/reactEvents";
import {
  convertTestProtocolFromString,
  parseId
} from "../../../../../helpers/convertersAndTransformers";
import SimpleCodeEditorWrapper from '../../../../codeEditors/simpleCodeEditorWrapper'
import {setSelectedTabIndex} from "../../../../../state/actions/exerciseEditorSite/dialogs/changeTestDialog/changeTestDialogActions";
import {Tab} from 'semantic-ui-react'

import ConsolePanelView from '../../../doExerciseSite/consolePanel/consolePanelView'
import {TestProtocolItem} from "../../../../../types/testResults";
import TestFilesListView from './testFilesListView'
import {debugOriginUrl} from "../../../../../../debug";
import {TestAssetForBackend} from "../../../../../types/exerciseEditor";
import {getNextFrontendId} from "../../../../../helpers/frontendIdHelper";
import Logger from "../../../../../helpers/logger";
import {readFileAsAsset} from "../../../../../helpers/fileReader";
import {translate} from "../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../i18n/ids";
import {errorDialog, warningDialog} from "../../../../../helpers/dialogHelper";
import {EditorTab} from "../../../../codeEditors/multiTabCodeEditorProps";
import {aceTextLanguage} from "../../../../../helpers/editorHelper";
import {KnownInternalTestTypes} from "../../../../../types/testTypes";
import {isNotEmptyOrSpaces} from "../../../../../validation/validations";
import {
  checkRegexs,
  convertRegexTestObject, parseRegexTestContent, RegexTestObject,
  RegexTestObjectConverted
} from "../../../../../types/testsAndTestProtocol";
import {escapeJSON} from "../../../../../helpers/jsonTransformer";

import JsonEscapeEditorView from '../../../../jsonEscapeEditorView/jsonEscapeEditorView'
import {SimpleVDivider} from "../../../../helpers/simpleVDivider";
import {leftIconPopupOffset} from "../../../../../constants";

//import jsonlint from 'jsonlint'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly className?: string
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
    test: rootState.changeTestDialogState.test,
    testTypes: rootState.testTypesState.testTypes,
    selectedTabIndex: rootState.changeTestDialogState.selectedTabIndex,

    exerciseId: rootState.exerciseEditorState.editorExercise.id,
    useDiffIfOutputMismatched: false, //not important when viewing, creating a test
    useReplaceWhiteSpacesWithBullets: true
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setDisplayNameAction,
  setWeight,
  setTestTypeId,
  setEditorTestContent,

  setEditorTest_memoryLimitInKb,
  setEditorTest_timeoutInMs,
  setEditorTest_maxDiskSpaceInKb,

  setSelectedTabIndex,
  setEditorTest_files,

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const testAssetInputId = 'testAssetInputId'

class ChangeTestView extends React.Component<Props, any> {

  componentWillMount(): void {

    if (this.props.test.testTypeId <= 0 && this.props.testTypes.length > 0) {
      //not selected ... select the first one
      this.props.setTestTypeId(this.props.testTypes[0].id)
    }
  }


  async addTestAsset(e: InputOnChangeEvent): Promise<void> {

    const file: File = e.currentTarget.files[0]

    // noinspection TsLint
    let testAssetFile: TestAssetForBackend = {
      id: getNextFrontendId(),
      mimeType: file.type,
      displayName: file.name,
      size: 0,
      content: []
    }

    try {
      const content = await readFileAsAsset(file)
      testAssetFile = {
        ...testAssetFile,
        content: content,
        size: content.length
      }
    } catch (err) {
      Logger.error(`exercise editor change test view, cannot read file: ${err.message}`)
      return
    }

    this.props.setEditorTest_files([...this.props.test.files, testAssetFile])
  }

  render(): JSX.Element {

    const testType = this.props.testTypes.find(p => p.id === this.props.test.testTypeId)

    if (this.props.test.testTypeId <= 0 || !testType) {
      //this happens when we render and we display the initial value (test where testTypeId is negative)
      //in componentWillMount we set the test type id and we should be finde on the second render
      return null
    }

    const isRegexTest = testType.internalName === KnownInternalTestTypes.regexTest

    // noinspection TsLint
    let testParseError = '' //error when parsing the json object
    // noinspection TsLint
    let regexError = '' //error when a regex was invalid

    if (isRegexTest) {

      // noinspection TsLint
      let result: RegexTestObject | string = null
      // noinspection TsLint
      let convertedResult: RegexTestObjectConverted = null

      try {
        result = parseRegexTestContent(this.props.test.content)

        if (typeof result === 'string') {
          throw new Error(result)
        }

        convertedResult = convertRegexTestObject(result)
      } catch (err) {
        testParseError = err.message
      }

      if (convertedResult === null) {

        if (isNotEmptyOrSpaces(testParseError) === false) { //if testParseError is empty
          regexError = 'some regex are not properly separated, check that every regex is enclosed in /.../\n' +
            `also check that you don't use && or || inside a regex`
        }

      } else {

        const errors = checkRegexs(convertedResult)

        if (errors && errors.length > 0) {
          regexError = errors.join('\n')
        } else {


        }
      }
    }

    //we fake a test result to reuse the console view
    const visualTest: ReadonlyArray<TestProtocolItem> | null = convertTestProtocolFromString(this.props.test.content,
      testType)


    const tempEditorTab: EditorTab = {
      id: this.props.test.id,
      content: this.props.test.content,
      language: aceTextLanguage,
      editSession: null,
      isReadonly: false,
      fileNameWithExtensions: this.props.test.displayName,
      displayIndex: this.props.test.displayIndex,
      isMainFile: true,
    }

    return (
      <div className="fh">
        <div className="ui form divided-checkboxes fh" style={{height: 'auto' /*firefox fix*/}}>

          <Form.Group widths='equal'>
            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.testType)
                }
              </label>
              <MaterialSelectWrapper
                value={this.props.test.testTypeId}
                validations={validationRules.testTypeId}
                validationMessageKey={validationMessageKeys.testTypeId}
              >
                <select
                  value={this.props.test.testTypeId}
                  onChange={(e: SelectOnChangeEventData) => {
                    const id = parseId(e.currentTarget.value)
                    if (id === null) {
                      errorDialog(translate(TranslateIds.error),
                        translate(TranslateIds.couldNotConvertStringToInt)
                      )
                    } else {
                      this.props.setTestTypeId(id)
                    }

                  }}
                >
                  {
                    this.props.testTypes.map(testType => {
                      return (
                        <option key={testType.id} value={testType.id}>
                          {
                            testType.displayName
                          }
                        </option>
                      )
                    })
                  }
                </select>
              </MaterialSelectWrapper>

            </Form.Field>

            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.points)
                }
              </label>
              <MaterialInput type="number"
                             validations={validationRules.weight}
                             validationMessageKey={validationMessageKeys.weight}
                             value={this.props.test.weight.toString(10)}
                             onChange={(e) => {

                               const weight = parseInt(e.currentTarget.value, 10)

                               if (isNaN(weight) || weight < 0) {

                                 errorDialog(translate(TranslateIds.error),
                                   translate(TranslateIds.couldNotConvertStringToInt)
                                 )

                                 return
                               }

                               this.props.setWeight(weight)
                             }}
              />
            </Form.Field>

            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.displayName)
                }
              </label>
              <MaterialInput
                validations={validationRules.displayName}
                validationMessageKey={validationMessageKeys.displayName}
                value={this.props.test.displayName}
                onChange={(e) => {
                  this.props.setDisplayNameAction(e.currentTarget.value)
                }}
              />
            </Form.Field>
          </Form.Group>

        </div>

        <Tab menu={{secondary: true, pointing: true}} panes={
          [
            {
              menuItem: `${translate(TranslateIds.testContent)}*`,
              render: () => {
                return (
                  <div id="ChangeTestView"
                       style={{height: '350px'}} className="mar-bot">
                    {
                      //TODO somehow make this div span the full dialog height...
                    }
                    {
                      //if we got a parse error
                      (isRegexTest && isNotEmptyOrSpaces(
                        testParseError)) &&
                      <div>
                        {
                          translate(TranslateIds.parseError) + ':'
                        }
                        <pre className="error-colored">
                          {
                            testParseError
                          }
                        </pre>
                      </div>
                    }

                    {
                      //if we got a regex error
                      (isRegexTest && isNotEmptyOrSpaces(
                        regexError)) &&
                      <div>
                        {
                          translate(TranslateIds.regexError) + ':'
                        }
                        <pre className="error-colored">
                          {
                            regexError
                          }
                        </pre>
                      </div>
                    }

                    {
                      //calc with because SimpleVDivider has 1em margin
                    }
                    <div className="fh flexed">
                      <div className="fh" style={isRegexTest ? {width: 'calc(50% - 1em)'} : {width: '100%'}}>
                        <SimpleCodeEditorWrapper
                          id="changeTestView"
                          resizeAnchorId="ChangeTestView"
                          editorTab={tempEditorTab}
                          onChange={(editorTab: EditorTab) => {
                            this.props.setEditorTestContent(editorTab.content)
                          }}
                          onEditorSessionChanged={null}
                        />
                      </div>
                      {
                        isRegexTest &&
                        <SimpleVDivider/>
                      }
                      {
                        isRegexTest &&
                        <div className="fh" style={{width: 'calc(50% - 1em - 1px)'}}>
                          <JsonEscapeEditorView editorResizeAnchorId="ChangeTestView"/>
                        </div>
                      }
                    </div>

                  </div>
                )
              }
            },
            {
              menuItem: translate(TranslateIds.testContentVisually),
              render: () => {
                return (
                  <div className="mar-bot">
                    <ConsolePanelView test={null} protocolItems={visualTest} testTypes={this.props.testTypes}
                                      useDiffIfOutputMismatched={this.props.useDiffIfOutputMismatched}
                                      useReplaceWhiteSpacesWithBullets={this.props.useReplaceWhiteSpacesWithBullets}
                    />
                  </div>
                )
              }
            },
            {
              menuItem: translate(TranslateIds.testSettingsAndFiles),
              render: () => {
                return (
                  <div className="ui form divided-checkboxes fh">
                    <Form.Group widths='equal'>
                      <Form.Field required>
                        <label>
                          {
                            translate(TranslateIds.timeoutInMs)
                          }
                        </label>
                        <MaterialInput
                          type="number"
                          value={this.props.test.testSettings.timeoutInMs.toString(10)}
                          onChange={(e) => {
                            //checks for > 0
                            const limit = parseId(e.currentTarget.value)

                            if (limit === null) {
                              errorDialog(translate(TranslateIds.error),
                                translate(TranslateIds.couldNotConvertStringToInt)
                              )
                              return
                            }

                            this.props.setEditorTest_timeoutInMs(limit)
                          }}
                        />
                      </Form.Field>

                      <Form.Field required>
                        <label>
                          {
                            translate(TranslateIds.maxRamInKb)
                          }
                        </label>
                        <MaterialInput
                          type="number"
                          value={this.props.test.testSettings.memoryLimitInKb.toString(10)}
                          onChange={(e) => {
                            //checks for > 0
                            const limit = parseId(e.currentTarget.value)

                            if (limit === null) {
                              errorDialog(translate(TranslateIds.error),
                                translate(TranslateIds.couldNotConvertStringToInt)
                              )
                              return
                            }

                            this.props.setEditorTest_memoryLimitInKb(limit)
                          }}
                        />
                      </Form.Field>

                      <Form.Field required>
                        <label>
                          {
                            translate(TranslateIds.maxDiskSpaceInKb)
                          }
                        </label>
                        <MaterialInput
                          type="number"
                          value={this.props.test.testSettings.maxDiskSpaceInKb.toString(10)}
                          onChange={(e) => {
                            //checks for > 0
                            const limit = parseId(e.currentTarget.value)

                            if (limit === null) {
                              errorDialog(translate(TranslateIds.error),
                                translate(TranslateIds.couldNotConvertStringToInt)
                              )
                              return
                            }

                            this.props.setEditorTest_maxDiskSpaceInKb(limit)
                          }}
                        />
                      </Form.Field>
                    </Form.Group>

                    <h4 className="ui dividing header">
                      {
                        translate(TranslateIds.files)
                      }
                      <Popup
                        trigger={
                          <Icon name="help circle" className="hoverable mar-left-half"/>
                        }
                        content={
                          <span>
                      {
                        translate(TranslateIds.downloadEditExerciseAssets_helpText)
                      }
                  </span>
                        }
                        wide
                        offset={leftIconPopupOffset}
                      />
                    </h4>

                    <div>
                      <div className="clickable"
                           onClick={() => {
                             const input = document.getElementById(testAssetInputId)

                             if (!input) {
                               errorDialog(translate(TranslateIds.error),
                                 translate(TranslateIds.couldNotGetHtmlElementWithSpecificId)
                               )
                               return
                             }

                             input.click()
                           }}
                      >
                        <Icon name="add"/>
                        <span>
                          {
                            translate(TranslateIds.add)
                          }
                        </span>
                      </div>

                      <input id={testAssetInputId} type="file" className="collapsed" accept="*/*"
                             onChange={(e: InputOnChangeEvent) => this.addTestAsset(e)}
                      />

                    </div>

                    <TestFilesListView files={this.props.test.files}
                                       canDownloadFiles={this.props.test.id > 0}
                                       getDownloadFileHRef={(file) => {
                                         return `${debugOriginUrl}/api/assets/editor/${this.props.exerciseId}/${this.props.test.id}/${file.id}`
                                       }}
                                       canRemoveFile={true}
                                       onRemoveFile={(file) => {
                                         this.props.setEditorTest_files(
                                           this.props.test.files.filter(p => p.id !== file.id))
                                       }}
                    />
                  </div>
                )
              }
            },
          ]

        }/>


      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeTestView)