/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch, Unsubscribe} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Icon, Popup} from "semantic-ui-react";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'
import {toJSONAndReplaceIds} from "../../../helpers/jsonTransformer";
import {InputOnChangeEvent} from "../../../types/reactEvents";
import {readFileAsText} from "../../../helpers/fileReader";
import Logger from "../../../helpers/logger";
import {EditorExerciseExerciseForBackend} from "../../../types/exerciseEditor";
import {setEditorExercise} from "../../../state/actions/exerciseEditorSite/exerciseEditorSiteActions";
import {
  createEditorExerciseAsync,
  updateEditorExerciseAsync
} from "../../../state/actions/exerciseEditorSite/exerciseEditorCrudActions";
import state from '../../../state/state'
import history from '../../../routerHistory'
import {getExerciseEditorEditToLink, leftIconPopupOffset} from "../../../constants";
import {addCloseWindowEvent, clearCloseWindowEvent} from "../../../helpers/eventsHelper";
import {Prompt} from "react-router";
import {freeAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {getNextFrontendId} from "../../../helpers/frontendIdHelper";
import {notifyError, notifyInfo, notifySuccess} from "../../../helpers/notificationHelper";
import * as mousetrap from "mousetrap";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {getEditorExercise} from "../../../communicationLayer/exerciseEditorLayer";
import {errorDialog} from "../../../helpers/dialogHelper";
import {mergeEditorExerciseTestAssetContents} from "../../../helpers/convertersAndTransformers";

const fileSaver = require('file-saver');

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    editorExercise: rootState.exerciseEditorState.editorExercise,
    savedNewExercise: rootState.exerciseEditorState.savedNewExercise,

    isLoading: rootState.exerciseEditorState.isLoading
    || rootState.testTypesState.isLoading
    || rootState.pLangsState.isLoading,

    isHeaderBarDisabled: rootState.exerciseEditorState.isLoading
    || rootState.exerciseEditorState.isChangeEditorSettingsDialogDisplayed
    || rootState.exerciseEditorState.isChangeCodeTemplateDialogDisplayed
    || rootState.exerciseEditorState.isChangeSelectedTagsDialogDisplayed
    || rootState.exerciseEditorState.isChangeTestDialogDisplayed,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setEditorExercise,
  createEditorExerciseAsync,
  updateEditorExerciseAsync,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const importJsonFileExerciseId = 'exerciseEditor-file-import'

class HeaderBar extends React.Component<Props, any> {

  unboundHandlers = false

  componentDidMount(): void {
    addCloseWindowEvent()
    mousetrap.bindGlobal("ctrl+s", this.onSaveKeyPress)
  }

  componentWillUnmount(): void {
    this.clearHandlers()
  }

  clearHandlers(): void {
    clearCloseWindowEvent()
    freeAssetUrlsAndObjects()
    mousetrap.unbind("ctrl+s")
  }


  onSaveKeyPress = async (e: ExtendedKeyboardEvent | null) => {

    if (e) {
      e.preventDefault()
    }

    if (this.props.editorExercise.id <= 0) {
      //creating (backend uses ids > 0

      // noinspection TsLint
      let created = false

      //this catches and returns always true/false
      created = (await this.props.createEditorExerciseAsync(this.props.editorExercise)) as any as boolean

      if (!created) {
        notifyError(translate(TranslateIds.couldNotSaveExercise))
        return
      }

      //because we only get the new id back (intentionally)
      //navigate to the load exercise editor site

      //actually we don't need to get the state because we await in the method which re populates the props
      //but to be sure
      if (this.props.savedNewExercise) { //state.getState().exerciseEditorState.savedNewExercise //works but wrong typing
        const newExerciseId = this.props.editorExercise.id //state.getState().exerciseEditorState.editorExercise.id
        this.clearHandlers()

        history.push(getExerciseEditorEditToLink(newExerciseId))
      }


    } else {
      //updating

      const success = await this.props.updateEditorExerciseAsync(this.props.editorExercise)

      if (!success) {
        notifyError(translate(TranslateIds.couldNotUpdateExercise))
        return
      }

      notifySuccess(translate(TranslateIds.updatedExercise))

    }
  }


  render(): JSX.Element {

    const leftArea = (
      <div>

        <Prompt
          when={this.props.savedNewExercise === false}
          message={translate(TranslateIds.exitExerciseEditorSiteMessage)}/>

        <div className={`clickable ${this.props.editorExercise.id <= 0 ? 'ok-colored' : 'orange-colored'}`}
             onClick={async () => {
               await this.onSaveKeyPress(null)
             }}
        >
          <Icon name={this.props.editorExercise.id <= 0 ? 'asterisk' : 'copy'}/>
          <span>
            {
              this.props.editorExercise.id <= 0 &&
              translate(TranslateIds.createExercise)
            }
            {
              this.props.editorExercise.id > 0 &&
              translate(TranslateIds.changeExercise)
            }
          </span>
        </div>

      </div>
    )

    const rightArea = (
      <div>
        <div className="clickable"
             onClick={async () => {


               //when we create a new exercise (not copying) then the test asset contents are not loaded...
               // noinspection TsLint
               let exerciseToExport = this.props.editorExercise

               if (this.props.editorExercise.id > 0) {

                 notifyInfo(translate(TranslateIds.editorExercise_loadingTestAssetContents))

                 try {
                   //load the full exercise but only take the test asset contents
                   const exerciseWithTestContents = await getEditorExercise(this.props.editorExercise.id, true)

                   exerciseToExport = mergeEditorExerciseTestAssetContents(this.props.editorExercise,
                     exerciseWithTestContents.tests)

                 } catch (err) {
                   errorDialog(translate(TranslateIds.serverError),
                     translate(TranslateIds.editorExercise_couldNotLoadTestAssetContents))

                   return
                 }
               } else {
                 //when we create a new exercise the test assets are still stored at frontend (in memory)
               }

               const value = toJSONAndReplaceIds(exerciseToExport)

               // noinspection TsLint
               let fileName = this.props.editorExercise.exerciseProperties.displayName.trim()

               if (!fileName) {
                 fileName = "exercise.json"
               }

               const blob = new Blob([value], {type: "application/json;charset=utf-8"});
               fileSaver.saveAs(blob, fileName)
             }}
        >
          <Icon name="upload"/>
          <span>
                {
                  translate(TranslateIds.exportExercise)
                }
          </span>
        </div>

        {
          this.props.editorExercise.id <= 0 &&
          <SimpleVDivider/>
        }

        {
          //only allow import for new exercises (for old ones there is the copy func)
          this.props.editorExercise.id <= 0 &&
          <div className="clickable"
               onClick={() => {

                 const inputEl = document.getElementById(importJsonFileExerciseId)
                 inputEl.click()
               }}
          >
            <Icon name="download"/>
            <span>
            {
              translate(TranslateIds.importExercise)
            }
          </span>
            <input id={importJsonFileExerciseId} type="file" className="collapsed" accept="application/json"
                   onChange={async (e: InputOnChangeEvent) => {
                     const files: FileList = e.currentTarget.files

                     // noinspection TsLint
                     let text: string = null
                     try {
                       text = await readFileAsText(files[0])
                     } catch (err) {
                       Logger.error(`exercise editor: ${err.message}`)
                     }

                     if (!text) {
                       Logger.warn(`exercise editor: something went wrong reading the file`)
                       return
                     }

                     //TODO must this is somehow insecure....
                     //check if the parsed has every needed props
                     // noinspection TsLint
                     let parsed = JSON.parse(text) as EditorExerciseExerciseForBackend

                     parsed = {
                       ...parsed,
                       id: getNextFrontendId()
                     }

                     this.props.setEditorExercise(parsed)
                   }}
            />
          </div>
        }


      </div>
    )


    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea} rightArea={rightArea}
                                     disabled={this.props.isLoading || this.props.isHeaderBarDisabled}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)