/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Card, Icon, Checkbox, Divider} from "semantic-ui-react";
import {openEditGroupRoleDialog} from "../../../state/actions/groupRolesSite/groupRolesSiteActions";
import {deleteGroupRoleAsync} from "../../../state/actions/groupRolesSite/groupRolesCrudActions";
import {askDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    groupRoles: rootState.groupRolesState.groupRoles,
    userSystemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  editGroupRole: openEditGroupRoleDialog,
  deleteGroupRoleAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class GroupRolesCardsView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <div className="material-table--caption-line-padding">
          <Card.Group>
            {
              this.props.groupRoles.map((groupRole, index) => {
                return (<Card key={groupRole.id}>
                    <Card.Content>
                      <Card.Header>
                        <span>{groupRole.displayName}</span>
                      </Card.Header>

                      <Card.Description>
                        <Checkbox label={translate(TranslateIds.canCreateExercises)}
                                  disabled
                                  checked={groupRole.canCreateExercises}/>
                        <Checkbox label={translate(TranslateIds.canChangeExercises)}
                                  disabled
                                  checked={groupRole.canChangeExercises}/>
                        <Checkbox label={translate(TranslateIds.canDeleteExercises)}
                                  disabled
                                  checked={groupRole.canDeleteExercises}/>

                        <Divider/>

                        <Checkbox label={translate(TranslateIds.canReleaseExercises)}
                                  disabled checked={groupRole.canReleaseExercises}/>
                        <Checkbox label={translate(TranslateIds.canCopyExercises)}
                                  disabled checked={groupRole.canCopyExercises}/>
                        <Checkbox label={translate(TranslateIds.canSeeExercisesFromOthersInGroup)} disabled
                                  checked={groupRole.canSeeExercisesFromOthersInGroup}/>

                        <Checkbox label={translate(TranslateIds.canAssessExercises)}
                                  disabled
                                  checked={groupRole.canAssessExercises}/>


                        <Divider/>

                        <Checkbox label={translate(TranslateIds.canAddUserToGroup)}
                                  disabled
                                  checked={groupRole.canAddUserToGroup}/>
                        <Checkbox label={translate(TranslateIds.canRemoveMemberFromGroup)}
                                  disabled
                                  checked={groupRole.canRemoveMemberFromGroup}/>
                        <Checkbox label={translate(TranslateIds.canChangeOtherMembersRole)}
                                  disabled
                                  checked={groupRole.canChangeOtherMembersRole}/>

                      </Card.Description>

                    </Card.Content>

                    <Card.Content extra>
                      <div className="two-flexed">
                        {
                          (this.props.userSystemRole && this.props.userSystemRole.canChangeRoles) &&
                          <div className="clickable"
                               onClick={() => {
                                 this.props.editGroupRole(groupRole)
                               }}
                          >
                            <Icon name="write"/>
                            <span>
                            {
                              translate(TranslateIds.changeGroupRole)
                            }
                          </span>
                          </div>
                        }
                        {
                          (this.props.userSystemRole && this.props.userSystemRole.canDeleteRoles) &&
                          <div className="clickable"
                               onClick={async () => {

                                 const shouldRemove = await askDialog(translate(TranslateIds.deleteGroupRole_title),
                                   `${translate(TranslateIds.deleteGroupRole_content)} (${groupRole.displayName})`)

                                 if (shouldRemove) {
                                   this.props.deleteGroupRoleAsync(groupRole)
                                 }

                               }}
                          >
                            <Icon name="remove"/>
                            <span>
                            {
                              translate(TranslateIds.deleteGroupRole)
                            }
                          </span>
                          </div>
                        }
                      </div>
                    </Card.Content>
                  </Card>
                )
              })
            }
          </Card.Group>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupRolesCardsView)