/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Card, Icon} from "semantic-ui-react";
import * as constants from '../../../constants'
import {Link} from "react-router-dom";
import {openEditGroupDialog, quitGroup} from "../../../state/actions/groupsSite/groupsSiteActions";
import {copyToClipboard} from "../../../helpers/clipboard";
import {askDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    groups: rootState.groupsState.groups,
    systemRole: rootState.userDataSettingsSate.userData
    && rootState.userDataSettingsSate.userData.systemRole
      ? rootState.userDataSettingsSate.userData.systemRole
      : null,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  openEditGroupDialog,
  quitGroup
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class GroupsCardsView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>

        <Card.Group>
          {
            this.props.groups.map((group, index) => {
              return (
                <Card key={group.id}>
                  <Card.Content>
                    <Card.Header>
                      <span>{group.displayName}</span>
                    </Card.Header>

                    <Card.Meta>
                      <Icon name="clone" style={{visibility: (group.email ? null : 'hidden')}}
                            className="clickable mar-right-half"
                            onClick={() => {
                              copyToClipboard(group.email)
                            }}
                      />
                      <span>{group.email}</span>
                    </Card.Meta>


                    <Card.Description>
                      <Icon name="users" className="mar-right-half"></Icon>
                      <span className="mar-right-half">{translate(TranslateIds.members)}:</span>
                      <span>???backend???</span>
                    </Card.Description>

                  </Card.Content>

                  <Card.Content extra>
                    <div className="two-flexed">

                      <div className="flexed">

                        <div className="mar-right-half clickable">
                          <Link to={constants.getSingleGroupLinkPath(group.id)}>
                            <Icon name="share"/>
                            <span>
                              {
                                translate(TranslateIds.open)
                              }
                            </span>
                          </Link>
                        </div>

                        {
                          (this.props.systemRole && this.props.systemRole.canChangeGroups) &&
                          <div className="clickable"
                               onClick={() => {
                                 this.props.openEditGroupDialog(group)
                               }}>
                            <Icon name="write"/>
                            <span>
                            {
                              translate(TranslateIds.change)
                            }
                          </span>
                          </div>
                        }
                      </div>

                      {
                        (this.props.systemRole && this.props.systemRole.canDeleteGroups) &&
                        <div className="clickable"
                             onClick={async () => {

                               const shouldRemove = await askDialog(translate(TranslateIds.leaveGroup_title),
                                 `${translate(TranslateIds.leaveGroup_content)} (${group.displayName})`)

                               if (shouldRemove) {
                                 this.props.quitGroup(group)
                               }
                             }}
                        >
                          <Icon name="remove"/>
                          <span>
                          {
                            translate(TranslateIds.leaveGroup)
                          }
                        </span>
                        </div>
                      }


                    </div>
                  </Card.Content>
                </Card>
              )
            })
          }
        </Card.Group>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(GroupsCardsView)