/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Table, Icon, Checkbox} from "semantic-ui-react";
import {
  setActivatedUserUiIsSelected,
  setAllActivatedUsersUiIsSelected,
  setIsChangeUserDataDialogDisplayed, setSortByKey
} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteActions";
import {setTargetUser} from "../../../state/actions/manageActivatedUsersSite/dialogs/changeActivatedUserDataDialogActions";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {deleteActivatedUserAsync} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteCrudActions";
import {SortDirection} from "../../../types/sortDirection";
import {SelectableUserFrontendExclusive} from "../../../types/users";
import {isCurrentSortingColumn} from "../../../helpers/tablesHelper";
import ListViewItem from './listViewItem'
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {HelpPopup} from '../../helpers/helpPopup'
import orderBy from 'lodash-es/orderBy'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    activatedUsers: rootState.manageActivatedUsersState.activatedUsers,
    sortByKey: rootState.manageActivatedUsersState.sortByKey,
    sortDirection: rootState.manageActivatedUsersState.sortDirection,
    userLangId: rootState.userDataSettingsSate.userData.settings
      ? rootState.userDataSettingsSate.userData.settings.langId
      : -1
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsChangeUserDataDialogDisplayed,
  setTargetUser,
  setActivatedUserUiIsSelected,
  setAllActivatedUsersUiIsSelected,
  deleteActivatedUserAsync,
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class ActivatedUsersListView extends React.Component<Props, any> {

  render(): JSX.Element {

    const list: ReadonlyArray<SelectableUserFrontendExclusive> =
      this.props.sortDirection === SortDirection.none
        ? this.props.activatedUsers
        : orderBy(this.props.activatedUsers, user => user[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')


    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <Checkbox
                  checked={
                    this.props.activatedUsers.every(p => p.uiIsSelected) && this.props.activatedUsers.length > 0
                  }
                  onChange={(e: any, data: CheckboxOnChangeEventData) => {
                    this.props.setAllActivatedUsersUiIsSelected(data.checked)
                  }}
                />
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableUserFrontendExclusive>(this.props.sortByKey, 'lastName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.lastName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableUserFrontendExclusive>(this.props.sortByKey, 'firstName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('firstName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.firstName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableUserFrontendExclusive>(this.props.sortByKey, 'token',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('token', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.token)
                  }
              </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableUserFrontendExclusive>(this.props.sortByKey, 'email',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('email', this.props.sortDirection + 1)
                  }}
                >
                  {
                    translate(TranslateIds.email)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableUserFrontendExclusive>(this.props.sortByKey, 'systemRole',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('systemRole', this.props.sortDirection + 1)
                  }}
                >
                  {
                    translate(TranslateIds.systemRole)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableUserFrontendExclusive>(this.props.sortByKey, 'createdAt',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('createdAt', this.props.sortDirection + 1)
                  }}
                >
                  {
                    translate(TranslateIds.createdAt)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<SelectableUserFrontendExclusive>(this.props.sortByKey, 'lastLoginAt',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('lastLoginAt', this.props.sortDirection + 1)
                  }}
                >
                  {
                    translate(TranslateIds.lastLoginAt)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <HelpPopup icon="area graph" defaultText={translate(TranslateIds.assessmentStatisticsForUser)}/>
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  translate(TranslateIds.change)
                }
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  translate(TranslateIds.delete_)
                }
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>
          <Table.Body>
            {
              list
                .map((activatedUser, index) => {
                  return (
                    <ListViewItem key={activatedUser.id} activatedUser={activatedUser} />
                  )
                })
            }

          </Table.Body>
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ActivatedUsersListView)