/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {CustomProjectPreviewFullBase} from "../../../types/customProjects";
import {SortDirection} from "../../../types/sortDirection";
import orderBy from 'lodash-es/orderBy'
import {Table, Icon} from "semantic-ui-react";
import {TranslateIds} from "../../../../i18n/ids";
import {translate} from "../../../../i18n/translationHelper";
import {isCurrentSortingColumn} from "../../../helpers/tablesHelper";
import {Link} from "react-router-dom";
import {displayDateAndTime} from "../../../helpers/convertersAndTransformers";
import {
  deleteOwnCustomProjectAsync,
  setSortByKey
} from "../../../state/actions/manageCustomProjectsSite/manageCustomProjectsSiteActions";
import {askDialog} from "../../../helpers/dialogHelper";
import {getCustomProjectToLinkPath} from "../../../constants";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    customProjectPreviews: rootState.manageCustomProjectsSiteState.customProjectPreviews,

    sortDirection: rootState.manageCustomProjectsSiteState.sortDirection,
    sortByKey: rootState.manageCustomProjectsSiteState.sortByKey,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setSortByKey,
  deleteOwnCustomProjectAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<CustomProjectPreviewFullBase> =
      this.props.sortDirection === SortDirection.none
        ? this.props.customProjectPreviews
        : orderBy(this.props.customProjectPreviews, release => release[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')


    return (
      <div>
        <Table selectable sortable padded striped singleLine className="material-table">

          <Table.Header>
            <Table.Row>

              <Table.HeaderCell>
                {
                  translate(TranslateIds.open)
                }
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<CustomProjectPreviewFullBase>(this.props.sortByKey,
                    'displayName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('displayName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.displayName)
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<CustomProjectPreviewFullBase>(this.props.sortByKey,
                    'lastUpdatedAt',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastUpdatedAt', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.lastUpdatedAt)
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell>
                {
                  translate(TranslateIds.delete_)
                }
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((preview, index) => {
                return (
                  <Table.Row key={preview.id}>

                    <Table.Cell>
                      <Link to={getCustomProjectToLinkPath(preview.id)} className="clickable">
                        <Icon name="share"/>
                      </Link>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        {
                          preview.displayName
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(preview.lastUpdatedAt, -1)
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>

                      <div className="clickable"
                           onClick={async () => {

                             const shouldRemove = await askDialog(translate(TranslateIds.deleteCustomProject_title),
                               `${translate(
                                 TranslateIds.deleteCustomProject_content)} (${preview.displayName})`)

                             if (shouldRemove) {
                               this.props.deleteOwnCustomProjectAsync(preview)
                             }

                           }}
                      >
                        <Icon name="remove"></Icon>
                      </div>

                    </Table.Cell>

                  </Table.Row>
                )
              })
            }
          </Table.Body>

        </Table>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listView)