/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Table, Icon, Checkbox, Popup, List} from "semantic-ui-react";
import SelectOptionList, {SelectableOption} from "../../helpers/selectOptionList";
import {
  setAllNewUsersUiIsSelected,
  setNewUserSystemRole,
  setNewUserUiIsChangeSystemRolePopupDisplayed, setNewUserUiIsSelected, setSortByKey
} from "../../../state/actions/manageNewUsersSite/manageNewUsersActions";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {
  activateNewUserAsync,
  deleteNewUserAsync
} from "../../../state/actions/manageNewUsersSite/manageNewUserSiteCrudActions";
import {NewUserFrontendExclusive} from "../../../types/users";
import {isCurrentSortingColumn} from "../../../helpers/tablesHelper";
import {SortDirection} from "../../../types/sortDirection";
import ListViewItem from './listViewItem'
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import orderBy from 'lodash-es/orderBy'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemRoles: rootState.manageNewUsersState.systemRoles,
    newUsers: rootState.manageNewUsersState.newUsers,
    sortByKey: rootState.manageNewUsersState.sortByKey,
    sortDirection: rootState.manageNewUsersState.sortDirection,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setNewUserSystemRole,
  setNewUserUiIsChangeSystemRolePopupDisplayed,
  setNewUserUiIsSelected,
  setAllNewUsersUiIsSelected,
  activateNewUserAsync,
  deleteNewUserAsync,
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class NewUsersListView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<NewUserFrontendExclusive> =
      this.props.sortDirection === SortDirection.none
        ? this.props.newUsers
        : orderBy(this.props.newUsers, user => user[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')

    return (
      <div>
        <Table sortable selectable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <Checkbox
                  checked={
                    this.props.newUsers.every(p => p.uiIsSelected) && this.props.newUsers.length > 0
                  }
                  onChange={(e: any, data: CheckboxOnChangeEventData) => {
                    this.props.setAllNewUsersUiIsSelected(data.checked)
                  }}
                />
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<NewUserFrontendExclusive>(this.props.sortByKey, 'lastName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.lastName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<NewUserFrontendExclusive>(this.props.sortByKey, 'firstName',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('firstName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.firstName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<NewUserFrontendExclusive>(this.props.sortByKey, 'token',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('token', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.token)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<NewUserFrontendExclusive>(this.props.sortByKey, 'email',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('email', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.email)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<NewUserFrontendExclusive>(this.props.sortByKey, 'systemRole',
                    this.props.sortDirection)
                }
              >
                <span>
                  {
                    translate(TranslateIds.systemRole)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<NewUserFrontendExclusive>(this.props.sortByKey, 'createdAt',
                    this.props.sortDirection)
                }
              >
                <span>
                  {
                    translate(TranslateIds.createdAt)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    translate(TranslateIds.activateNewUser)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                  {
                    translate(TranslateIds.delete_)
                  }
                </span>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((newUser, index) => {
                return (
                  <ListViewItem key={newUser.id} newUser={newUser}/>
                )
              })
            }

          </Table.Body>
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(NewUsersListView)