/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Icon, Divider} from "semantic-ui-react";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'
import TableView from '../manageOwnOrGroupExerciseComponents/listView'
import CardsView from '../manageOwnOrGroupExerciseComponents/cardsView'
import {SiteViewState} from "../../../types/siteView";
import Spinner from "../../helpers/Spinner";
import {
  deleteOwnExerciseAsync,
  getOwnExercisesAsync, loadOwnExercisesSite,
  setSortByKey
} from "../../../state/actions/manageOwnExercisesSite/ownExercisesSiteActions";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import TagFilterPanel from '../../tagComponents/tagFilterPanel'
import {sortAndFilterExercisePreviews} from "../../../helpers/tablesHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    tags: rootState.manageTagsState.tags,
    exercisePreviews: rootState.ownExercisesState.ownExercises,
    viewState: rootState.ownExercisesState.viewState,
    isLoading: rootState.ownExercisesState.isLoading
    || rootState.manageTagsState.isLoading,
    sortByKey: rootState.ownExercisesState.sortByKey,
    sortDirection: rootState.ownExercisesState.sortDirection,

    selectedFilterTagsIds: rootState.tagsFilterState.selectedFilterTagsIds,
    selectedNegativeFilterTagsIds: rootState.tagsFilterState.selectedNegativeFilterTagsIds,

    isTagsFilterDisplayed: rootState.ownExercisesState.isTagsFilterDisplayed,

  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  loadOwnExercisesSite,
  setSortByKey,
  deleteOwnExerciseAsync,

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class OwnExercisesSite extends React.Component<Props, any> {

  componentWillMount(): void {
    this.props.loadOwnExercisesSite()
  }

  render(): JSX.Element {

    const previewsList = sortAndFilterExercisePreviews(this.props.exercisePreviews, this.props.sortDirection,
      this.props.sortByKey, this.props.selectedFilterTagsIds, this.props.selectedNegativeFilterTagsIds)

    return (

      <div className="multiple-single-panels">

        {
          this.props.isTagsFilterDisplayed &&
          <TagFilterPanel
            isLoading={this.props.isLoading}
            allTags={this.props.tags}
          />
        }

        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <h1>
                  {
                    translate(TranslateIds.ownExercises)
                  }
                </h1>
              </div>
              <div className="view-options">
                <MaterialSearchInput/>
              </div>
            </div>

            <div className="view-content">
              {
                this.props.viewState === SiteViewState.List &&
                <TableView exercisePreviews={previewsList} comesFromOwnExercisesSite={true}
                           sortByKey={this.props.sortByKey}
                           sortDirection={this.props.sortDirection}
                           setSortByKey={this.props.setSortByKey}
                           onDeleteExercise={(exercisePreview: EditableExercisePreviewFromBackend) => {
                             this.props.deleteOwnExerciseAsync(exercisePreview.id)
                           }}
                           currentUserGroupRole={null}
                />
              }

              {
                this.props.viewState === SiteViewState.Cards &&
                <CardsView exercisePreviews={previewsList} comesFromOwnExercisesSite={true}
                           onDeleteExercise={(exercisePreview: EditableExercisePreviewFromBackend) => {
                             this.props.deleteOwnExerciseAsync(exercisePreview.id)
                           }}
                           currentUserGroupRole={null}
                />
              }
            </div>
          </div>

        </SinglePanelSiteWrapper>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(OwnExercisesSite)