/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Card, Icon, Checkbox, Divider} from "semantic-ui-react";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import {setSelectedExerciseId} from "../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/exerciseReleasesSiteActions";
import {Link} from "react-router-dom";
import * as constants from '../../../constants'
import {getExerciseEditorEditToLink} from "../../../constants";
import {displayDateAndTime} from "../../../helpers/convertersAndTransformers";
import {askDialog, infoDialog} from "../../../helpers/dialogHelper";
import {HelpPopup} from "../../helpers/helpPopup";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";

//const css = require('./styles.styl');

export interface MyProps {
  readonly exercisePreviews: ReadonlyArray<EditableExercisePreviewFromBackend>
  readonly comesFromOwnExercisesSite: boolean

  readonly onDeleteExercise: (exercisePreview: EditableExercisePreviewFromBackend) => void

  /**
   * the group role, null = own exercises --> allow all
   */
  readonly currentUserGroupRole: GroupRoleWithPermissionFullBase | null
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    exercisePreviews: props.exercisePreviews,
    comesFromOwnExercisesSite: props.comesFromOwnExercisesSite,
    onDeleteExercise: props.onDeleteExercise,
    currentUserGroupRole: props.currentUserGroupRole
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setSelectedExerciseId
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class CardsView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <Card.Group>
          {
            this.props.exercisePreviews.map((exercisePreview, index) => {
              return (<Card key={exercisePreview.id}>
                  <Card.Content>

                    <div className="right floated">
                      <Icon name="flag" className="mar-right-half"/>
                      <span>{exercisePreview.solutions}</span>
                    </div>

                    <Card.Header>
                      <span>{exercisePreview.displayName}</span>
                    </Card.Header>

                    <Card.Meta>
                      <span className="mar-right-half">
                        {
                          translate(TranslateIds.lastUpdatedAt)
                        }
                      </span>
                      <span>
                        {
                          displayDateAndTime(exercisePreview.lastUpdatedAt, -1)
                        }
                      </span>
                    </Card.Meta>

                    <Card.Description>

                      <div>
                        <Checkbox checked={exercisePreview.isOnlyVisibleToMe}
                                  label={translate(TranslateIds.isOnlyVisibleToOwner)} disabled/>
                      </div>

                      <Divider/>
                      <div style={{visibility: (exercisePreview.shortDescription ? null : 'hidden')}}>
                        <Icon name="help circle"/>
                        <span>{exercisePreview.shortDescription}</span>
                      </div>


                    </Card.Description>

                  </Card.Content>

                  <Card.Content extra>
                    <div className="two-flexed">
                      <div className="flexed">

                        {
                          (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canChangeExercises) &&
                          <div className="clickable mar-right-half">
                            <Link to={getExerciseEditorEditToLink(exercisePreview.id)}>
                              <Icon name="write"/>
                              <span>
                              {
                                translate(TranslateIds.change)
                              }
                            </span>
                            </Link>
                          </div>
                        }

                        {
                          (this.props.currentUserGroupRole === null
                            || this.props.currentUserGroupRole.canReleaseExercises
                            || this.props.currentUserGroupRole.canAssessExercises) &&
                          <div>
                            <Link to={constants.getExerciseReleaseLinkPath(exercisePreview.id,
                              this.props.comesFromOwnExercisesSite)}>
                              <Icon name="privacy"/>
                              <span>
                              {
                                translate(TranslateIds.releases)
                              }
                            </span>
                            </Link>
                          </div>
                        }


                      </div>


                      {
                        exercisePreview.releases > 0 && (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canDeleteExercises) &&
                        <HelpPopup
                          defaultText={translate(TranslateIds.deleteExerciseOnlyIfNoReleasesLeft)}/>
                      }
                      {
                        exercisePreview.releases === 0 && (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canDeleteExercises) &&
                        <div className="clickable"
                             onClick={async () => {

                               if (exercisePreview.releases > 0) {
                                 //only allowed if it has no exercises
                                 //show never happen because we check above
                                 return
                               }

                               const shouldRemove = await askDialog(translate(TranslateIds.deleteExercise_title),
                                 `${translate(TranslateIds.deleteExercise_content)} (${exercisePreview.displayName})`
                               )

                               if (shouldRemove) {
                                 this.props.onDeleteExercise(exercisePreview)
                               }


                             }}
                        >
                          <Icon name="remove"/>
                        </div>
                      }

                    </div>
                  </Card.Content>
                </Card>
              )
            })
          }
        </Card.Group>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(CardsView)