/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {Table, Icon, Checkbox, Popup} from "semantic-ui-react";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import {Link} from "react-router-dom";
import history from '../../../routerHistory'
import * as constants from '../../../constants'
import {SortDirection} from "../../../types/sortDirection";
import {isCurrentSortingColumn} from "../../../helpers/tablesHelper";
import {
  getCreateCopyFromOldExerciseToLinkPath, getExerciseEditorEditToLink,
  leftIconPopupOffset
} from "../../../constants";
import {displayDateAndTime} from "../../../helpers/convertersAndTransformers";
import {HelpPopup} from "../../helpers/helpPopup";
import {askDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import Tag from '../../tagComponents/tag'


//const css = require('./styles.styl');

export interface MyProps {
  readonly exercisePreviews: ReadonlyArray<EditableExercisePreviewFromBackend>
  readonly comesFromOwnExercisesSite: boolean

  readonly sortByKey: keyof EditableExercisePreviewFromBackend
  readonly sortDirection: SortDirection
  readonly setSortByKey: (sortByKey: keyof EditableExercisePreviewFromBackend, sortDirection: SortDirection) => void

  readonly onDeleteExercise: (exercisePreview: EditableExercisePreviewFromBackend) => void

  /**
   * the group role, null = own exercises --> allow all
   */
  readonly currentUserGroupRole: GroupRoleWithPermissionFullBase | null
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    exercisePreviews: props.exercisePreviews,
    comesFromOwnExercisesSite: props.comesFromOwnExercisesSite,
    sortByKey: props.sortByKey,
    sortDirection: props.sortDirection,
    setSortByKey: props.setSortByKey,
    onDeleteExercise: props.onDeleteExercise,
    currentUserGroupRole: props.currentUserGroupRole
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class TableView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<EditableExercisePreviewFromBackend> = this.props.exercisePreviews

    return (
      <div>

        <Table sortable selectable padded striped singleLine className="material-table">

          <Table.Header>
            <Table.Row>
              <Table.HeaderCell sorted={
                isCurrentSortingColumn<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'displayName',
                  this.props.sortDirection)
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('displayName', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.displayName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumn<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'shortDescription',
                  this.props.sortDirection)
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('shortDescription', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.description)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                {
                  translate(TranslateIds.tags)
                }
              </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumn<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'isOnlyVisibleToMe',
                  this.props.sortDirection)
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('isOnlyVisibleToMe', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.isOnlyVisibleToOwner)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumn<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'lastUpdatedAt',
                  this.props.sortDirection)
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('lastUpdatedAt', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.lastUpdatedAt)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'solutions',
                    this.props.sortDirection)
                }
              >
                <div className="clickable"
                     onClick={() => {
                       this.props.setSortByKey('solutions', this.props.sortDirection + 1)
                     }}
                >
                  <Icon name="flag" className="mar-right-half"/>
                  <span>
                    {
                      translate(TranslateIds.solutions)
                    }
                  </span>
                </div>

              </Table.HeaderCell>
              <Table.HeaderCell sorted={
                isCurrentSortingColumn<EditableExercisePreviewFromBackend>(this.props.sortByKey, 'releases',
                  this.props.sortDirection)
              }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('releases', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.releases)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>Bearbeiten</span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span>
                {
                  translate(TranslateIds.delete_)
                }
              </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span className="mar-right-half">
                  {
                    translate(TranslateIds.copyExercise)
                  }
                </span>
                <HelpPopup defaultText={translate(TranslateIds.copyExercise_helpText)}/>
              </Table.HeaderCell>

            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((exercisePreview, index) => {
                return (
                  <Table.Row key={exercisePreview.id}>

                    <Table.Cell>
                      <span>{exercisePreview.displayName}</span>
                    </Table.Cell>

                    <Table.Cell>
                      {
                        exercisePreview.shortDescription &&
                        <Popup
                          trigger={
                            <div className="hoverable">
                              <Icon name="help circle"/>
                            </div>
                          }
                          content={exercisePreview.shortDescription}
                          wide
                        />
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        (exercisePreview.tags && exercisePreview.tags.length > 0) &&
                        <HelpPopup
                          wide="very"
                          defaultText={
                            <div className="exercise-tag-wrapper">
                              {
                                exercisePreview.tags.map((tag) => {
                                  return (
                                    <Tag key={tag.id} tag={tag}/>
                                  )
                                })
                              }
                            </div>
                          }
                        />
                      }
                    </Table.Cell>

                    <Table.Cell>
                      <Checkbox disabled checked={exercisePreview.isOnlyVisibleToMe}/>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(exercisePreview.lastUpdatedAt, -1)
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>
                      <span>{exercisePreview.solutions}</span>

                    </Table.Cell>

                    <Table.Cell>
                      {
                        (this.props.currentUserGroupRole === null
                          || this.props.currentUserGroupRole.canReleaseExercises
                          || this.props.currentUserGroupRole.canAssessExercises) &&
                        <div>
                          <Link to={constants.getExerciseReleaseLinkPath(exercisePreview.id,
                            this.props.comesFromOwnExercisesSite)}>
                            <Icon name="privacy"/>
                            <span>{exercisePreview.releases}</span>
                          </Link>
                        </div>
                      }

                    </Table.Cell>

                    <Table.Cell>
                      {
                        (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canChangeExercises) &&
                        <div>
                          <Link to={getExerciseEditorEditToLink(exercisePreview.id)}>
                            <Icon name="write"/>
                          </Link>
                        </div>
                      }

                    </Table.Cell>

                    <Table.Cell>
                      {
                        exercisePreview.releases > 0 && (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canDeleteExercises) &&
                        <HelpPopup
                          defaultText={translate(TranslateIds.deleteExerciseOnlyIfNoReleasesLeft)}/>
                      }
                      {
                        exercisePreview.releases === 0 && (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canDeleteExercises) &&
                        <div className="clickable"
                             onClick={async () => {

                               if (exercisePreview.releases > 0) {
                                 //only allowed if it has no exercises
                                 //show never happen because we check above
                                 return
                               }

                               const shouldRemove = await askDialog(translate(TranslateIds.deleteExercise_title),
                                 `${translate(TranslateIds.deleteExercise_content)} (${exercisePreview.displayName})`
                               )

                               if (shouldRemove) {
                                 this.props.onDeleteExercise(exercisePreview)
                               }

                             }}
                        >
                          <Icon name="remove"/>
                        </div>
                      }

                    </Table.Cell>

                    <Table.Cell>
                      {
                        (this.props.currentUserGroupRole === null || this.props.currentUserGroupRole.canCopyExercises) &&
                        <div>
                          <Link to={getCreateCopyFromOldExerciseToLinkPath(exercisePreview.id)}>
                            <Icon name="copy"/>
                          </Link>
                        </div>
                      }
                    </Table.Cell>

                  </Table.Row>
                )
              })
            }
          </Table.Body>
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(TableView)