/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../../../state/reducers/index";
import {Checkbox, Form, Popup, Radio, Icon, List} from "semantic-ui-react";
import MaterialInput from '../../../../../material/materialInput'
import MaterialSelectWrapper from '../../../../../material/materialSelectWrapper'
import {ReleaseStartType} from "../../../../../../types/ReleaseStartType";
import {CheckboxOnChangeEventData} from "../../../../../../types/reactEvents";
import {
  setAutomaticEndAt,
  setAutomaticStartAt, setAvailableWorkingTimeInMinutes,
  setIsReleased, setHasLimitedWorkingTime, setPLangId,
  setReleaseStartType, setNote, setIsVisibleToAllAfterRelease, setShouldAutomaticAssessSubmissions,
  setRunAlsoNormalTests,
  setReleaseDurationType, setHideInOverviews, setShouldClearClipboard, setHideSiteHeaderBar,
} from "../../../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/dialogs/changeExerciseReleaseDialog/setExerciseReleaseActions";
import {ReleaseDurationType} from "../../../../../../types/ReleaseDurationType";
import {ChangeEvent, SyntheticEvent} from "react";
import {selectAllTextEventHandler} from "../../../../../../helpers/commonEventHandlers";
import {parseId} from "../../../../../../helpers/convertersAndTransformers";
import {Moment} from "moment";
import {leftIconPopupOffset} from "../../../../../../constants";
import MaterialDatetimeInput from '../../../../../material/materialDatetimeInput'
import MaterialTextArea from '../../../../../material/materialTextArea'
import {HelpPopup} from "../../../../../helpers/helpPopup";
import {translate} from "../../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../../i18n/ids";
import {
  validationMessageKeys,
  validationRules
} from "../../../../../../state/reducers/ownOrGroupExerciseReducers/exerciseReleasesSite/dialogs/changeExerciseReleaseDialog/setExerciseReleaseReducer";


//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    exerciseRelease: rootState.changeExerciseReleaseDialogState.exerciseRelease,
    initialIsReleased: rootState.changeExerciseReleaseDialogState.initialIsReleased,
    pLangs: rootState.pLangsState.pLangs
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setIsReleased,
  setPLangId,
  setAutomaticStartAt,
  setAutomaticEndAt,
  setHasLimitedWorkingTime,
  setAvailableWorkingTimeInMinutes,
  setNote,
  setIsVisibleToAllAfterRelease,
  setShouldAutomaticAssessSubmissions,
  setRunAlsoNormalTests,
  setReleaseDurationType,
  setReleaseStartType,
  setHideInOverviews,
  setShouldClearClipboard,
  setHideSiteHeaderBar,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class changeReleseView extends React.Component<Props, any> {

  componentWillMount(): void {

    if ((this.props.exerciseRelease.pLangId === null || this.props.exerciseRelease.pLangId <= 0) && this.props.pLangs.length > 0) {
      //select the first plang
      this.props.setPLangId(this.props.pLangs[0].id)
    }
  }

  render(): JSX.Element {

    return (
      <div>

        <div className="ui form divided-checkboxes">

          <Form.Group widths='equal'>
            <Form.Field>
              <label>
                {
                  translate(TranslateIds.startAndEndOfTheRelease)
                }
              </label>

              <div className="flexed">
                <Radio name="startReleaseMode" label={translate(TranslateIds.manual)} className="mar-right"
                       checked={this.props.exerciseRelease.releaseStartType === ReleaseStartType.Manual}
                       value={ReleaseStartType.Manual}
                       onChange={(e, value: CheckboxOnChangeEventData) => {
                         this.props.setReleaseStartType(value.value)
                       }}
                />
                <Radio name="startReleaseMode" label={translate(TranslateIds.automatic)}
                       checked={this.props.exerciseRelease.releaseStartType === ReleaseStartType.Automatic}
                       value={ReleaseStartType.Automatic}
                       onChange={(e, value: CheckboxOnChangeEventData) => {
                         this.props.setReleaseStartType(value.value)
                       }}

                />

              </div>
            </Form.Field>
          </Form.Group>

          {
            //one of the following manual start or automatic start
          }

          {
            this.props.exerciseRelease.releaseStartType === ReleaseStartType.Manual &&
            <div>
              <h4 className="ui dividing header">
                {
                  translate(TranslateIds.manual)
                }
              </h4>

              <Form.Group widths='equal'>
                <Form.Field>

                  {
                    (this.props.exerciseRelease.hasAutomaticAssessmentStarted &&
                      !this.props.exerciseRelease.hasAutomaticAssessmentFinished) &&
                    <div>
                      {
                        translate(TranslateIds.automaticAssessmentIsRunning)
                      }
                    </div>
                  }
                  {
                    (!this.props.exerciseRelease.hasAutomaticAssessmentStarted ||
                      this.props.exerciseRelease.hasAutomaticAssessmentFinished) &&
                    <div>

                      <Checkbox className="mar-right-half"
                                label={translate(TranslateIds.isReleased)}
                                checked={this.props.exerciseRelease.isReleased}
                                onChange={(e, data: CheckboxOnChangeEventData) => {
                                  this.props.setIsReleased(data.checked)
                                }}
                      />
                      <HelpPopup
                        wide
                        defaultText={
                          <div>
                            {
                              translate(TranslateIds.isReleased_helpText)
                            }
                          </div>
                        }
                      />
                    </div>
                  }


                  {
                    //unrelease
                    //or
                    //unrelease (not the first unrelease)
                    (!this.props.exerciseRelease.isReleased && this.props.initialIsReleased === true
                      && this.props.exerciseRelease.releasedAt !== null
                      && !this.props.exerciseRelease.hasAutomaticAssessmentStarted
                      && !this.props.exerciseRelease.hasAutomaticAssessmentFinished
                      && this.props.exerciseRelease.shouldAutomaticAssessSubmissions
                      ||
                      !this.props.exerciseRelease.isReleased
                      && this.props.exerciseRelease.releasedAt !== null
                      && this.props.exerciseRelease.hasAutomaticAssessmentStarted
                      && this.props.exerciseRelease.hasAutomaticAssessmentFinished
                      && this.props.exerciseRelease.shouldAutomaticAssessSubmissions
                    ) &&
                    <div>
                      {
                        translate(TranslateIds.releaseState_automaticTestsWillRun)
                      }
                    </div>
                  }

                  {
                    (!this.props.exerciseRelease.isReleased && this.props.initialIsReleased === true
                      && this.props.exerciseRelease.releasedAt !== null
                      && !this.props.exerciseRelease.hasAutomaticAssessmentStarted
                      && !this.props.exerciseRelease.hasAutomaticAssessmentFinished
                      && !this.props.exerciseRelease.shouldAutomaticAssessSubmissions
                      ||
                      !this.props.exerciseRelease.isReleased && this.props.initialIsReleased === true
                      && this.props.exerciseRelease.releasedAt !== null
                      && this.props.exerciseRelease.hasAutomaticAssessmentStarted
                      && this.props.exerciseRelease.hasAutomaticAssessmentFinished
                      && !this.props.exerciseRelease.shouldAutomaticAssessSubmissions
                    ) &&
                    <div>
                      {
                        translate(TranslateIds.releaseState_automaticTestsWontRun)
                      }
                    </div>
                  }

                </Form.Field>
              </Form.Group>
            </div>
          }
          {
            //not supported (not tested + not used much)
            this.props.exerciseRelease.releaseStartType === ReleaseStartType.Automatic &&
            <div>
              <h4 className="ui dividing header">
                {
                  translate(TranslateIds.automatic)
                }
                <Popup
                  trigger={
                    <Icon name="help circle" className="hoverable mar-left-half"/>
                  }
                  content={
                    <span>
                      {
                        translate(TranslateIds.automaticRelease_helpText)
                      }
                  </span>
                  }
                  wide
                  offset={leftIconPopupOffset}
                />
              </h4>

              <Form.Group widths='equal'>
                <Form.Field>
                  <label>
                    {
                      translate(TranslateIds.automaticStartAt)
                    }
                  </label>
                  <div className="flexed">
                    {
                      /*
                       <MaterialInput icon="calendar" className="mar-right-half"/>
                    <MaterialInput icon="time"/>
                       */
                    }
                    <MaterialDatetimeInput
                      locale="de"
                      value={this.props.exerciseRelease.automaticStartAt}
                      onChange={(e: Moment | string) => {
                        if (typeof e === 'string') {
                          //if the value is invalid...
                          this.props.setAutomaticStartAt(null)

                        } else {
                          this.props.setAutomaticStartAt(e)
                        }
                      }}
                    />

                  </div>
                </Form.Field>

                <Form.Field>
                  <label>
                    {
                      translate(TranslateIds.automaticEndAt)
                    }
                  </label>
                  <div
                    className={this.props.exerciseRelease.releaseDurationType === ReleaseDurationType.Limited ? null : 'div-disabled'}>
                    <div className="flexed">
                      {
                        /*
                        <MaterialInput icon="calendar" className="mar-right-half"/>
                        <MaterialInput icon="time"/>
                        TODO make datetime material like...
                        */
                      }

                      <MaterialDatetimeInput
                        locale="de"
                        value={this.props.exerciseRelease.automaticEndAt}
                        onChange={(e: Moment | string) => {
                          if (typeof e === 'string') {
                            //if the value is invalid...
                            this.props.setAutomaticEndAt(null)

                          } else {
                            this.props.setAutomaticEndAt(e)
                          }
                        }}
                      />

                    </div>
                  </div>
                </Form.Field>
              </Form.Group>

              <Form.Field>
                <label>
                  {
                    translate(TranslateIds.durationOfTheAutomaticRelease)
                  }
                </label>
                <div className="flexed">

                  <Popup
                    trigger={
                      <Radio name="releaseDurationMode" label={translate(TranslateIds.infinitely)} className="mar-right"
                             checked={this.props.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely}
                             value={ReleaseDurationType.Infinitely}
                             onChange={(e, data: CheckboxOnChangeEventData) => {
                               this.props.setReleaseDurationType(data.value)
                             }}
                      />
                    }
                    content={translate(TranslateIds.durationOfTheAutomaticReleaseInfinitely_helpText)}
                    offset={leftIconPopupOffset}
                    wide
                  >
                  </Popup>
                  {
                    //then enable automatic end
                  }
                  <Radio name="releaseDurationMode" label={translate(TranslateIds.limited)}
                         checked={this.props.exerciseRelease.releaseDurationType === ReleaseDurationType.Limited}
                         value={ReleaseDurationType.Limited}
                         onChange={(e, data: CheckboxOnChangeEventData) => {
                           this.props.setReleaseDurationType(data.value)
                         }}
                  />
                </div>
              </Form.Field>
            </div>
          }

          <h4 className="ui dividing header">
            {
              translate(TranslateIds.options)
            }
          </h4>

          <Form.Group widths='equal'>
            <Form.Field>
              <label>
                {
                  translate(TranslateIds.pLang)
                }
              </label>
              <MaterialSelectWrapper
                value={this.props.exerciseRelease.pLangId === null ? 0 : this.props.exerciseRelease.pLangId}
              >
                <select
                  value={this.props.exerciseRelease.pLangId === null ? 0 : this.props.exerciseRelease.pLangId}
                  onChange={(e: SyntheticEvent<HTMLSelectElement>) => {
                    const pLangId = parseId(e.currentTarget.value)
                    this.props.setPLangId(pLangId)

                  }}
                >
                  {
                    //we now longer support creating a release for multiple languages...
                    //(because normally we release it for X langs and not all OR just for one)
                    // because for X langs the backend needs to be massively changed we only support one language per release
                    //<option key={0} value={'0'}>Alle</option>
                  }

                  {
                    this.props.pLangs.map((pLang, index) => {
                      return (
                        <option key={pLang.id} value={pLang.id}>{pLang.displayName}</option>
                      )
                    })
                  }
                </select>
              </MaterialSelectWrapper>
            </Form.Field>

            <Form.Field>
              <div className="form-field-label-with-icon">
                <label>
                  {
                    translate(TranslateIds.availableWorkingTimeInMin)
                  }
                </label>
                <HelpPopup defaultText={translate(TranslateIds.availableWorkingTimeInMin_helpText)}/>
              </div>

              <div className="flexed">
                <Checkbox label={translate(TranslateIds.hasLimitedWorkingTimeCheckbox_text)}
                          className="v-centered mar-right"
                          checked={this.props.exerciseRelease.hasLimitedWorkingTime}
                          onChange={(e, data: CheckboxOnChangeEventData) => {
                            this.props.setHasLimitedWorkingTime(data.checked)
                          }}
                />
                <MaterialInput
                  type="number"
                  value={this.props.exerciseRelease.availableWorkingTimeInMinutes === null ? '' : this.props.exerciseRelease.availableWorkingTimeInMinutes.toString()}

                  validations={validationRules.availableWorkingTimeInMinutes}
                  validationMessageKey={validationMessageKeys.availableWorkingTimeInMinutes}

                  onFocus={selectAllTextEventHandler}
                  onChange={(e) => {
                    const time = parseInt(e.currentTarget.value, 10)

                    if (isNaN(time)) {
                      //TODO some error
                      this.props.setAvailableWorkingTimeInMinutes(null)
                    } else {

                      if (time < 1) {
                        //TODO some error
                      } else {
                        this.props.setAvailableWorkingTimeInMinutes(time)
                      }
                    }
                  }}
                  disabled={!this.props.exerciseRelease.hasLimitedWorkingTime}
                />
              </div>

            </Form.Field>

          </Form.Group>

          <Form.Field>

            <Checkbox className="mar-right-half"
                      label={translate(TranslateIds.releaseIsVisibleForAll)}
                      checked={this.props.exerciseRelease.isVisibleToAllAfterRelease}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setIsVisibleToAllAfterRelease(data.checked)
                      }}
            />
            <HelpPopup
              wide
              defaultText={translate(TranslateIds.releaseIsVisibleForAll_helpText)}/>
          </Form.Field>


          <Form.Group widths='equal'>

            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={translate(TranslateIds.shouldRunAutomaticAssessment)}
                        checked={this.props.exerciseRelease.shouldAutomaticAssessSubmissions}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setShouldAutomaticAssessSubmissions(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={translate(TranslateIds.shouldRunAutomaticAssessment_helpText)}/>
            </Form.Field>
            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={translate(TranslateIds.shouldRunNormalTestsAlso)}
                        checked={this.props.exerciseRelease.runAlsoNormalTests}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setRunAlsoNormalTests(data.checked)
                        }}
              />

              <HelpPopup
                wide='very'
                defaultText={translate(TranslateIds.shouldRunNormalTestsAlso_helpText)}/>
            </Form.Field>
          </Form.Group>


          <Form.Field>
            <div>
              <label style={{
                color: 'rgba(0,0,0, 0.87)',
                fontWeight: 'bold',
                fontSize: '0.92857143em'
              }}
                     className="mar-right-half"
              >
                {
                  translate(TranslateIds.note)
                }
              </label>
            </div>
            <MaterialTextArea
              rows={3}
              value={this.props.exerciseRelease.note}
              onChange={(e) => {
                this.props.setNote(e.currentTarget.value)
              }}
            />
          </Form.Field>

          <h4 className="ui dividing header">
            {
              translate(TranslateIds.examOptions)
            }
          </h4>

          <Form.Group widths='equal'>

            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={translate(TranslateIds.hideReleaseInOverviews)}
                        checked={this.props.exerciseRelease.hideInOverviews}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setHideInOverviews(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={translate(TranslateIds.hideReleaseInOverviews_helpText)}/>
            </Form.Field>

            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={translate(TranslateIds.shouldClearClipboardOnFocus)}
                        checked={this.props.exerciseRelease.shouldClearClipboard}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setShouldClearClipboard(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={translate(TranslateIds.shouldClearClipboardOnFocus_helpText)}/>
            </Form.Field>

          </Form.Group>


          <Form.Group widths='equal'>

            <Form.Field>

              <Checkbox className="mar-right-half"
                        label={translate(TranslateIds.hideSiteHeaderBar)}
                        checked={this.props.exerciseRelease.hideSiteHeaderBar}
                        onChange={(e, data: CheckboxOnChangeEventData) => {
                          this.props.setHideSiteHeaderBar(data.checked)
                        }}
              />
              <HelpPopup
                wide
                defaultText={translate(TranslateIds.hideSiteHeaderBar_helpText)}/>
            </Form.Field>


          </Form.Group>

          <h4 className="ui dividing header">
            {
              translate(TranslateIds.hints)
            }
          </h4>

          <Form.Field>
            <List divided relaxed>
              <List.Item>
                {
                  translate(TranslateIds.doExerciseLinkExamMode_hint)
                }
              </List.Item>
            </List>
          </Form.Field>

        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(changeReleseView)