/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Table, Icon, Popup, Checkbox} from "semantic-ui-react";
import {setZoomedExerciseRelease} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/dialogs/zoomedRelease/zoomedReleaseDialogActions";
import {
  deleteExerciseReleaseAsyncAndReload,
  setIsZoomedExerciseReleaseDisplayed, setSortByKey
} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/exerciseReleasesSiteActions";
import {openEditExerciseRelease} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/dialogs/changeExerciseReleaseDialog/changeExerciseReleaseDialogActions";
import {Link} from "react-router-dom";
import * as constants from '../../../../constants'
import {copyToClipboard} from "../../../../helpers/clipboard";
import {displayDateAndTime} from "../../../../helpers/convertersAndTransformers";
import {SortDirection} from "../../../../types/sortDirection";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {isCurrentSortingColumn} from "../../../../helpers/tablesHelper";
import {HelpPopup} from "../../../helpers/helpPopup";
import {askDialog} from "../../../../helpers/dialogHelper";
import {ReleaseStartType} from "../../../../types/ReleaseStartType";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import orderBy from 'lodash-es/orderBy'
import {leftIconPopupOffset} from "../../../../constants";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    releases: rootState.releaseSiteState.releases,
    comesFromOwnExercisesSite: rootState.releaseSiteState.comesFromOwnExercisesSite,
    selectedExerciseId: rootState.releaseSiteState.selectedExerciseId,

    sortByKey: rootState.releaseSiteState.sortByKey,
    sortDirection: rootState.releaseSiteState.sortDirection,

    groupRole: rootState.groupStorageState.userGroupRole ? rootState.groupStorageState.userGroupRole : null
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setZoomedExerciseRelease,
  setIsZoomedExerciseReleaseDisplayed,
  openEditExerciseRelease,

  deleteExerciseReleaseAsyncAndReload,
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

//TODO enforce group role ... currently we don't know the group role ... maybe ask server exercise.userGroupId --> member --> role
class ListView extends React.Component<Props, any> {
  render(): JSX.Element {

    const list: ReadonlyArray<ExerciseReleaseFromBackend> =
      this.props.sortDirection === SortDirection.none
        ? this.props.releases
        : orderBy(this.props.releases, release => release[this.props.sortByKey],
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')

    return (
      <div>

        <Table selectable sortable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<ExerciseReleaseFromBackend>(this.props.sortByKey, 'isReleased',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('isReleased', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.releaseIsActive)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  translate(TranslateIds.releaseCode)
                }
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span className="mar-right-half">
                  {
                    translate(TranslateIds.startAt)
                  }
                </span>
                <HelpPopup defaultText={translate(TranslateIds.startAt_helpText)}/>
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  translate(TranslateIds.endAt)
                }
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  //this somehow cannot properly sorted if we have no working time (unlimited) this is the default value...
                }
                <span>
                  {
                    translate(TranslateIds.availableWorkingTimeInMin)
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<ExerciseReleaseFromBackend>(this.props.sortByKey,
                    'isVisibleToAllAfterRelease',
                    this.props.sortDirection)
                }
              >
                <div className="inline">
                  <span className="clickable mar-right-half"
                        onClick={() => {
                          this.props.setSortByKey('isVisibleToAllAfterRelease', this.props.sortDirection + 1)
                        }}
                  >
                    {
                      translate(TranslateIds.releaseIsVisibleForAll)
                    }
                  </span>
                  <HelpPopup
                    defaultText={translate(TranslateIds.releaseIsVisibleForAll_helpText)}
                    wide/>
                </div>
              </Table.HeaderCell>

              <Table.HeaderCell>

                <span>
                  {
                    translate(TranslateIds.misc)
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell>
                {
                  translate(TranslateIds.note)
                }
              </Table.HeaderCell>

              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<ExerciseReleaseFromBackend>(this.props.sortByKey,
                    'solutionsCount',
                    this.props.sortDirection)
                }
              >
                <span className="clickable"
                      onClick={() => {
                        this.props.setSortByKey('solutionsCount', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.submissions)
                  }
                </span>
              </Table.HeaderCell>

              <Table.HeaderCell>
                {
                  translate(TranslateIds.change)
                }
              </Table.HeaderCell>

              <Table.HeaderCell>
                {
                  translate(TranslateIds.delete_)
                }
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((exerciseRelease, index) => {

                return (
                  <Table.Row key={exerciseRelease.id}>
                    <Table.Cell>

                      <Checkbox disabled checked={exerciseRelease.isReleased}/>

                    </Table.Cell>
                    <Table.Cell>
                      <div className="flexed">
                        <Icon name="zoom" className="clickable mar-right-half"
                              onClick={() => {
                                this.props.setZoomedExerciseRelease(exerciseRelease)
                                this.props.setIsZoomedExerciseReleaseDisplayed(true)
                              }}
                        />
                        <Icon name="clone" className="clickable mar-right-half"
                              onClick={() => {
                                copyToClipboard(exerciseRelease.generatedCode)
                              }}
                        />
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      {
                        (exerciseRelease.releaseStartType === ReleaseStartType.Manual && exerciseRelease.releasedAt) &&
                        displayDateAndTime(exerciseRelease.releasedAt, -1)
                      }
                      {
                        exerciseRelease.releaseStartType === ReleaseStartType.Automatic
                        && exerciseRelease.automaticStartAt && exerciseRelease.releasedAt === null
                        &&
                        `${displayDateAndTime(exerciseRelease.automaticStartAt, -1)} (${translate(
                          TranslateIds.automatic)})`
                      }

                      {
                        //after release display the actual release time
                      }
                      {
                        exerciseRelease.releaseStartType === ReleaseStartType.Automatic
                        && exerciseRelease.automaticStartAt && exerciseRelease.releasedAt !== null
                        &&
                        `${displayDateAndTime(exerciseRelease.releasedAt, -1)} (${translate(
                          TranslateIds.automatic)})`
                      }
                    </Table.Cell>
                    <Table.Cell>
                      <div>
                        {
                          (exerciseRelease.releaseStartType === ReleaseStartType.Manual) &&
                          translate(TranslateIds.manual)
                        }
                        {
                          (exerciseRelease.releaseStartType === ReleaseStartType.Automatic
                            && exerciseRelease.automaticEndAt) &&
                          `${displayDateAndTime(exerciseRelease.automaticEndAt, -1)} (${translate(
                            TranslateIds.automatic)})`
                        }
                      </div>
                    </Table.Cell>

                    <Table.Cell>
                      <div>
                        {
                          exerciseRelease.hasLimitedWorkingTime
                            ? `${exerciseRelease.availableWorkingTimeInMinutes.toString()} ${translate(
                            TranslateIds.minutes_abbr)}`
                            : <span>&infin;</span>
                        }
                      </div>
                    </Table.Cell>

                    <Table.Cell>

                      <Checkbox disabled checked={exerciseRelease.isVisibleToAllAfterRelease}/>

                    </Table.Cell>

                    <Table.Cell>

                      <div>

                        <Popup
                          trigger={<Icon name='unhide'
                                         className={exerciseRelease.hideInOverviews ? 'hoverable' : 'hoverable-disabled'}/>}
                          content={translate(TranslateIds.hideReleaseInOverviews_helpText)}
                          offset={leftIconPopupOffset}
                          header={translate(TranslateIds.hideReleaseInOverviews)}
                          wide
                        />

                        <Popup
                          trigger={<Icon name='sidebar'
                                         className={exerciseRelease.hideSiteHeaderBar ? 'hoverable' : 'hoverable-disabled'}/>}
                          content={translate(TranslateIds.hideSiteHeaderBar_helpText)}
                          offset={leftIconPopupOffset}
                          header={translate(TranslateIds.hideSiteHeaderBar)}
                          wide
                        />

                        <Popup
                          trigger={<Icon name='clipboard'
                                         className={exerciseRelease.shouldClearClipboard ? 'hoverable' : 'hoverable-disabled'}/>}
                          content={translate(TranslateIds.shouldClearClipboardOnFocus_helpText)}
                          offset={leftIconPopupOffset}
                          header={translate(TranslateIds.shouldClearClipboardOnFocus)}
                          wide
                        />

                      </div>

                      {
                        /*
                         <Checkbox disabled checked={exerciseRelease.hideInOverviews}/>
                      <Checkbox disabled checked={exerciseRelease.shouldClearClipboard}/>
                         */
                      }

                    </Table.Cell>

                    <Table.Cell>
                      <span>{exerciseRelease.note}</span>
                    </Table.Cell>


                    <Table.Cell>
                      <div>
                        <Link to={constants.getSubmissionsSiteToLinkPath(this.props.selectedExerciseId,
                          this.props.comesFromOwnExercisesSite, exerciseRelease.id)}>
                          <Icon name="send"/>
                          <span>{exerciseRelease.solutionsCount}</span>
                        </Link>
                      </div>

                    </Table.Cell>

                    <Table.Cell>
                      {
                        (this.props.groupRole && this.props.groupRole.canReleaseExercises) &&
                        <div className="clickable"
                             onClick={() => {
                               this.props.openEditExerciseRelease(exerciseRelease)
                             }}
                        >
                          <Icon name="write"/>
                        </div>
                      }

                    </Table.Cell>


                    <Table.Cell>
                      {
                        (this.props.groupRole && this.props.groupRole.canReleaseExercises) &&
                        <div className="clickable"
                             onClick={async () => {

                               const shouldRemove = await askDialog(translate(TranslateIds.deleteRelease_title),
                                 `${translate(TranslateIds.deleteRelease_content)} (${exerciseRelease.generatedCode})`
                               )

                               if (shouldRemove) {
                                 this.props.deleteExerciseReleaseAsyncAndReload(exerciseRelease.id,
                                   this.props.selectedExerciseId, true, this.props.comesFromOwnExercisesSite)
                               }

                             }}
                        >
                          <Icon name="remove"/>
                        </div>
                      }

                    </Table.Cell>

                  </Table.Row>
                )
              })
            }

          </Table.Body>

        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ListView)