/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import ListView from './listView'
import MaterialSearchInput from '../../../material/materialSearchInput'
import {RouteComponentProps, withRouter} from "react-router";
import {parseBoolean, parseId} from "../../../../helpers/convertersAndTransformers";
import {
  loadExerciseReleasesSite, resetExerciseReleasesSite, setComesFromOwnExercisesSite,
  setSelectedExerciseId
} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/exerciseReleasesSiteActions";
import SinglePanelSiteWrapper from '../../../singlePanelSiteWrapper'
import Spinner from "../../../helpers/Spinner";
import {Link} from "react-router-dom";
import {Icon} from "semantic-ui-react";
import * as constants from '../../../../constants'
import ZoomedExerciseReleaseDialog from './dialogs/zoomedExerciseReleaseDialog/zoomedExerciseReleaseDialog'
import ChangeExerciseReleaseDialog from './dialogs/changeExerciseReleaseDialog/changeExerciseReleaseDialog'
import {BackArrow} from "../../../helpers/backArrow";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    exercises: rootState.releaseSiteState.exercises,
    selectedExerciseId: rootState.releaseSiteState.selectedExerciseId,
    isLoading: rootState.releaseSiteState.isLoading
    || rootState.pLangsState.isLoading
    || rootState.groupStorageState.isLoading
    ,
    comesFromOwnExercisesSite: rootState.releaseSiteState.comesFromOwnExercisesSite,
    isZoomedExerciseReleaseDisplayed: rootState.releaseSiteState.isZoomedExerciseReleaseDisplayed,
    isChangeDialogDisplayed: rootState.releaseSiteState.isChangeDialogDisplayed,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setSelectedExerciseId,
  loadExerciseReleasesSite,
  setComesFromOwnExercisesSite,
  resetExerciseReleasesSite,
}, dispatch)

interface MatchedProps {
  readonly exerciseId: string
  readonly comesFromOwnExercisesSite: string
}

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//TODO enforce group role ... currently we don't know the group role ... maybe ask server exercise.userGroupId --> member --> role
class ReleasesView extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  componentWillMount(): void {
    const exerciseId = parseId(this.props.match.params.exerciseId)
    const comesFromOwnExercisesSite = parseBoolean(this.props.match.params.comesFromOwnExercisesSite)

    //don't do this in the load action because we need to set som props before (do we really need to??)
    //we don't need to clear the exercises or load them because normally
    //the previous site loaded them...
    this.props.resetExerciseReleasesSite(false)

    this.props.setComesFromOwnExercisesSite(comesFromOwnExercisesSite)

    if (exerciseId !== null && comesFromOwnExercisesSite !== null) {

      this.props.setSelectedExerciseId(exerciseId)

      //exercises.length === 0 this is possible if a user opens this page directly in the browser
      this.props.loadExerciseReleasesSite(
        exerciseId, this.props.exercises === null || this.props.exercises.length === 0, comesFromOwnExercisesSite)

    }
  }

  render(): JSX.Element {
    const selectedExercise = this.props.exercises.find(p => p.id === this.props.selectedExerciseId)
    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        {
          !this.props.isZoomedExerciseReleaseDisplayed &&
          !this.props.isChangeDialogDisplayed &&
          <div className="view-padding">
            <div className="view-head-line">
              <div className="view-caption">
                <div className="mar-right back-arrow">
                  <Link
                    to={this.props.comesFromOwnExercisesSite ? constants.ownExercisesLinkPath : constants.groupExercisesLinkPath}>
                    <BackArrow/>
                  </Link>
                </div>
                <h2 className="clear-margin">
                  {
                    selectedExercise &&
                    `${translate(TranslateIds.releasesFor)} ${selectedExercise.displayName}`
                  }
                </h2>
              </div>
              <div className="view-options">
                <MaterialSearchInput/>
              </div>
            </div>

            <div className="view-content">
              <ListView/>
            </div>
          </div>
        }

        {
          this.props.isZoomedExerciseReleaseDisplayed &&
          <ZoomedExerciseReleaseDialog exercise={selectedExercise}/>
        }

        {
          this.props.isChangeDialogDisplayed &&
          <ChangeExerciseReleaseDialog/>
        }

      </SinglePanelSiteWrapper>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(ReleasesView))