/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Icon, Table, Popup, Checkbox} from "semantic-ui-react";
import {
  getAssessmentStatisticsSiteToLinkPath, getSubmissionDownloadLink,
  getTutorViewForSubmission
} from "../../../../constants";
import {AssessmentFullBase} from "../../../../types/submissions";
import {
  rerunSingleSubmissionAsync,
  setRunningAssessment, setSortByKey
} from "../../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsSiteActions";
import {isCurrentSortingColumn} from "../../../../helpers/tablesHelper";
import {SortDirection} from "../../../../types/sortDirection";
import orderBy from 'lodash-es/orderBy'
import {HelpPopup} from "../../../helpers/helpPopup";
import {Link} from "react-router-dom";
import {isNotEmpty} from "../../../../validation/validations";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {ArrayHelper} from "../../../../helpers/arrayHelper";
import {displayNumber, displayTime} from "../../../../helpers/convertersAndTransformers";
import {HelpPopupHtmlContent} from "../../../helpers/helpPopupHtmlContent";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    releases: rootState.submissionsSiteState.releases,
    selectedReleaseId: rootState.submissionsSiteState.selectedReleaseId,
    submissions: rootState.submissionsSiteState.submissionTuple
      ? rootState.submissionsSiteState.submissionTuple.submissions
      : [],
    pLangs: rootState.pLangsState.pLangs,
    runningAssessment: rootState.submissionsSiteState.runningAssessment,
    sortByKey: rootState.submissionsSiteState.sortByKey,
    sortDirection: rootState.submissionsSiteState.sortDirection,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  rerunSingleSubmissionAsync,
  setRunningAssessment,
  setSortByKey
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ListView extends React.Component<Props, any> {
  render(): JSX.Element {
    const selectedRelease = this.props.releases.find(p => p.id === this.props.selectedReleaseId)

    if (!selectedRelease) {
      //TODO only error if really not found
      //on first render this might not be set because the list was not yet set
      return null
    }

    const canRerunAssessment =
      selectedRelease.hasAutomaticAssessmentStarted
      && selectedRelease.hasAutomaticAssessmentFinished

    const list: ReadonlyArray<AssessmentFullBase> =
      this.props.sortDirection === SortDirection.none
        ? this.props.submissions
        : orderBy(this.props.submissions, submissions => {

          if (submissions[this.props.sortByKey] === null || submissions[this.props.sortByKey] === undefined) {
            //we always want these to be at the bottom of the sorted list...
            return this.props.sortDirection === SortDirection.ascending ? Number.POSITIVE_INFINITY : Number.NEGATIVE_INFINITY
          }

          return submissions[this.props.sortByKey]
        },
        this.props.sortDirection === SortDirection.ascending ? 'asc' : 'desc')

    //calculate passed percentages (normal, submit)

    // noinspection TsLint
    let passedAllNormalTestsPercentage: number = null
    // noinspection TsLint
    let passedAllNormalTestsCount: number = null
    // noinspection TsLint
    let passedAllSubmitTestsPercentage: number = null
    // noinspection TsLint
    let passedAllSubmitTestsCount: number = null

    if (canRerunAssessment && list.length > 0) {
      passedAllNormalTestsCount = ArrayHelper.count(list,
        (el => el.normalTestPoints !== null && el.normalTestPoints === el.maxNormalTestPoints && el.shouldNotCount === false))

      // noinspection TsLint
      passedAllNormalTestsPercentage = passedAllNormalTestsCount * 100 / list.length

      passedAllSubmitTestsCount = ArrayHelper.count(list,
        (el => el.submitTestPoints !== null && el.submitTestPoints === el.maxSubmitTestPoints && el.shouldNotCount === false))

      // noinspection TsLint
      passedAllSubmitTestsPercentage = passedAllSubmitTestsCount * 100 / list.length
    }

    // noinspection TsLint
    let percentageShouldNotCount: number = null

    // noinspection TsLint
    let shouldNotCountCount: number = null
    if (list.length > 0) {
      shouldNotCountCount = ArrayHelper.count(list, el => el.shouldNotCount)
      // noinspection TsLint
      percentageShouldNotCount = shouldNotCountCount * 100 / list.length
    }

    return (
      <div>

        <Table selectable sortable padded striped singleLine className="material-table">
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell>
                <span>
                  {
                    '#'
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AssessmentFullBase>(this.props.sortByKey, 'lastName',
                    this.props.sortDirection)
                }
              >
              <span className="clickable"
                    onClick={() => {
                      this.props.setSortByKey('lastName', this.props.sortDirection + 1)
                    }}
              >
                {
                  translate(TranslateIds.lastName)
                }
              </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AssessmentFullBase>(this.props.sortByKey, 'firstName',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('firstName', this.props.sortDirection + 1)
                  }}
                >
                  {
                    translate(TranslateIds.firstName)
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AssessmentFullBase>(this.props.sortByKey, 'pLangId',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('pLangId', this.props.sortDirection + 1)
                  }}
                >
                  <HelpPopup icon="puzzle" defaultText={translate(TranslateIds.pLang)}/>
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AssessmentFullBase>(this.props.sortByKey, 'shouldNotCount',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('shouldNotCount', this.props.sortDirection + 1)
                  }}
                >
                  <HelpPopup icon="flag outline"
                             defaultText={translate(TranslateIds.shouldNotCount)}/>
                  {
                    shouldNotCountCount !== null &&
                    <Popup
                      trigger={
                        <span>
                          {
                            percentageShouldNotCount !== null &&
                            ` (${displayNumber(percentageShouldNotCount, 2)}%)`
                          }
                        </span>
                      }
                      content={
                        <div>
                          {shouldNotCountCount}/{list.length}
                        </div>
                      }
                    />
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AssessmentFullBase>(this.props.sortByKey, 'participationCount',
                    this.props.sortDirection)
                }
              >
                <span
                  className="clickable"
                  onClick={() => {
                    this.props.setSortByKey('participationCount', this.props.sortDirection + 1)
                  }}
                >
                  <HelpPopup icon="repeat" wide
                             defaultText={translate(TranslateIds.participationCount_helpText)}/>
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AssessmentFullBase>(this.props.sortByKey, 'normalTestPoints',
                    this.props.sortDirection)
                }
              >

                <span
                  className="clickable mar-right-half"
                  onClick={() => {
                    this.props.setSortByKey('normalTestPoints', this.props.sortDirection + 1)
                  }}
                >
                  <HelpPopup wide icon="flag"
                             defaultText={translate(TranslateIds.normalTestsAssessment_helpText)}/>

                  {
                    passedAllNormalTestsCount !== null &&
                    <Popup
                      trigger={
                        <span>
                          {
                            passedAllNormalTestsPercentage !== null &&
                            ` (${displayNumber(passedAllNormalTestsPercentage, 2)}%)`
                          }
                        </span>
                      }
                      content={
                        <div>
                          {passedAllNormalTestsCount}/{list.length}
                        </div>
                      }
                    />
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AssessmentFullBase>(this.props.sortByKey, 'submitTestPoints',
                    this.props.sortDirection)
                }
              >
                <span className="clickable mar-right-half"
                      onClick={() => {
                        this.props.setSortByKey('submitTestPoints', this.props.sortDirection + 1)
                      }}
                >
                  <HelpPopup wide icon="flag checkered"
                             defaultText={translate(TranslateIds.submissionTestsAssessment_helpText)}/>

                  {
                    passedAllSubmitTestsCount !== null &&
                    <Popup
                      trigger={
                        <span>
                          {
                            passedAllSubmitTestsPercentage !== null &&
                            ` (${displayNumber(passedAllSubmitTestsPercentage, 2)}%)`
                          }
                        </span>
                      }
                      content={
                        <div>
                          {passedAllSubmitTestsCount}/{list.length}
                        </div>
                      }
                    />
                  }
                </span>
              </Table.HeaderCell>
              <Table.HeaderCell
                sorted={
                  isCurrentSortingColumn<AssessmentFullBase>(this.props.sortByKey, 'manualPoints',
                    this.props.sortDirection)
                }
              >
                <span className="clickable mar-right-half"
                      onClick={() => {
                        this.props.setSortByKey('manualPoints', this.props.sortDirection + 1)
                      }}
                >
                  {
                    translate(TranslateIds.manual)
                  }
                </span>
                <HelpPopup wide
                           defaultText={translate(TranslateIds.manualPoints_helpText)}/>
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  translate(TranslateIds.noteForOtherTutors)
                }
              </Table.HeaderCell>
              <Table.HeaderCell>
                <span className="mar-right-half">
                  {
                    translate(TranslateIds.open)
                  }
                </span>
                <HelpPopup
                  wide
                  defaultText={translate(TranslateIds.openTutorView_helpText)}/>
              </Table.HeaderCell>
              <Table.HeaderCell>

                {
                  /*
                  <Icon name="sticky note" />
                <HelpPopup defaultText={translate(TranslateIds.feedbackForStudent)}/>
                   */
                }
                <HelpPopup icon="talk" defaultText={translate(TranslateIds.feedbackForStudent)}/>

              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  translate(TranslateIds.download)
                }
              </Table.HeaderCell>
              <Table.HeaderCell>
                <HelpPopup icon="area graph" defaultText={translate(TranslateIds.assessmentStatisticsForUser)}/>
              </Table.HeaderCell>
              <Table.HeaderCell>
                {
                  translate(TranslateIds.rerunSingleAutomaticAssessment)
                }
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {
              list.map((submission: AssessmentFullBase, index) => {
                return (
                  <Table.Row key={`${submission.userId}-${submission.pLangId}`}>
                    <Table.Cell>
                      <div className={
                        this.props.runningAssessment
                        && this.props.runningAssessment.pLangId === submission.pLangId
                        && this.props.runningAssessment.userId === submission.userId
                          ? `table-line-active`
                          : null
                      }>
                        {
                          (index + 1)
                        }
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      <div>
                        {submission.lastName}
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      <span>
                        {submission.firstName}
                      </span>
                    </Table.Cell>
                    <Table.Cell>
                      <span>
                        {
                          this.props.pLangs.find(p => p.id === submission.pLangId) &&
                          this.props.pLangs.find(p => p.id === submission.pLangId).displayName
                        }
                      </span>
                    </Table.Cell>
                    <Table.Cell>
                      <div>
                        <Checkbox checked={submission.shouldNotCount} disabled/>
                      </div>
                    </Table.Cell>
                    <Table.Cell>
                      <span>
                        {
                          submission.participationCount === -1 ? translate(
                            TranslateIds.error) : submission.participationCount
                        }
                      </span>
                    </Table.Cell>
                    <Table.Cell>
                      <span>
                        {
                          submission.normalTestPoints !== null &&
                          `${submission.normalTestPoints} / ${submission.maxNormalTestPoints}`
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>
                      <span>
                        {
                          submission.submitTestPoints !== null &&
                          `${submission.submitTestPoints} / ${submission.maxSubmitTestPoints}`
                        }
                      </span>
                    </Table.Cell>

                    <Table.Cell>
                      {
                        submission.manualPoints !== null &&
                        `${submission.manualPoints}`
                      }
                    </Table.Cell>

                    <Table.Cell>
                      {
                        isNotEmpty(submission.noteForOtherTutors) &&
                        <HelpPopupHtmlContent defaultContent={
                          <pre>
                            {
                              submission.noteForOtherTutors
                            }
                          </pre>
                        }
                                              wide='very'
                        />
                      }
                    </Table.Cell>

                    <Table.Cell>
                      <div>
                        <Link to={getTutorViewForSubmission(this.props.selectedReleaseId, submission.userId,
                          submission.pLangId)}>
                          <Icon name="life ring"/>
                        </Link>
                      </div>
                    </Table.Cell>

                    <Table.Cell>
                      {
                        isNotEmpty(submission.feedbackForStudent) &&
                        <HelpPopupHtmlContent defaultContent={
                          <pre>
                            {
                              submission.feedbackForStudent
                            }
                          </pre>
                        }
                                              style={{margin: '0'}}
                                              wide='very'
                        />
                      }
                    </Table.Cell>

                    <Table.Cell>
                      <div>
                        <a target="_blank"
                           download
                           href={
                             getSubmissionDownloadLink(
                               (selectedRelease ? selectedRelease.generatedCode : null),
                               submission.userId,
                               submission.pLangId
                             )
                           }>
                          <Icon name="download"/>
                        </a>
                      </div>
                    </Table.Cell>

                    <Table.Cell>
                      <div>
                        <Link style={{margin: '0'}} to={getAssessmentStatisticsSiteToLinkPath(submission.userId)}>
                          <Icon name="area graph"/>
                        </Link>
                      </div>
                    </Table.Cell>

                    <Table.Cell>
                      <div className={canRerunAssessment ? 'clickable' : ' div-disabled'}
                           onClick={async () => {

                             this.props.setRunningAssessment(submission)

                             await this.props.rerunSingleSubmissionAsync(
                               selectedRelease ? selectedRelease.generatedCode : null, submission, true)

                             this.props.setRunningAssessment(null)
                           }}


                      >
                        <Icon name="refresh"/>
                      </div>
                    </Table.Cell>

                  </Table.Row>
                )
              })
            }

          </Table.Body>
        </Table>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ListView)