/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";

import ListView from './listView'
import {tryGet} from "../../../../helpers/typeHelpers";
import {Icon} from "semantic-ui-react";
import SinglePanelSiteWrapper, {default as singlePanelSiteWrapper} from '../../../singlePanelSiteWrapper'
import {Link} from "react-router-dom";
import * as constants from '../../../../constants'
import MaterialSearchInput from '../../../material/materialSearchInput'
import {RouteComponentProps, withRouter} from "react-router";
import {parseBoolean, parseId} from "../../../../helpers/convertersAndTransformers";
import {
  setComesFromOwnExercisesSite,
  setSelectedExerciseId
} from "../../../../state/actions/ownOrGroupExerciseActions/releasesSiteActions/exerciseReleasesSiteActions";
import {
  loadSubmissionsSite, resetSubmissionsSite,
  setSelectedReleaseId
} from "../../../../state/actions/ownOrGroupExerciseActions/submissionsSite/submissionsSiteActions";
import Spinner from "../../../helpers/Spinner";
import {BackArrow} from "../../../helpers/backArrow";
import {DoubleBackArrow} from "../../../helpers/doubleBackArrow";
import {getExerciseReleaseLinkPath} from "../../../../constants";
import {ExerciseReleaseFromBackend} from "../../../../types/exerciseRelease";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {HelpPopup} from "../../../helpers/helpPopup";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    comesFromOwnExercisesSite: rootState.submissionsSiteState.comesFromOwnExercisesSite,
    releases: rootState.submissionsSiteState.releases,
    selectedReleaseId: rootState.submissionsSiteState.selectedReleaseId,
    selectedExerciseId: rootState.submissionsSiteState.selectedExerciseId,
    exercises: rootState.submissionsSiteState.exercises,

    isLoading: rootState.submissionsSiteState.isLoading
    || rootState.pLangsState.isLoading,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setComesFromOwnExercisesSite,
  loadSubmissionsSite,
  resetSubmissionsSite,
  setSelectedReleaseId,
  setSelectedExerciseId
}, dispatch)

interface MatchedProps {
  readonly exerciseId: string
  readonly comesFromOwnExercisesSite: string
  readonly releaseId: string
}

const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


//TODO maybe move this to helpers
function getAssessmentState(selectedRelease: ExerciseReleaseFromBackend): string {

  if (selectedRelease.isReleased) return translate(TranslateIds.sillReleased)

  if (!selectedRelease.shouldAutomaticAssessSubmissions &&
    selectedRelease.hasAutomaticAssessmentStarted && selectedRelease.hasAutomaticAssessmentFinished === false)
    return translate(TranslateIds.automaticAssessmentOptionNotSelected)

  if (selectedRelease.shouldAutomaticAssessSubmissions &&
    selectedRelease.hasAutomaticAssessmentStarted === false && selectedRelease.hasAutomaticAssessmentFinished === false)
    return translate(TranslateIds.automaticAssessmentHasNotRanYet)

  if (!selectedRelease.releasedAt)
    return translate(TranslateIds.notReleasedYet)

  if (selectedRelease.hasAutomaticAssessmentStarted && selectedRelease.hasAutomaticAssessmentFinished === false)
    return translate(TranslateIds.automaticAssessmentIsCurrentlyRunning)

  if (selectedRelease.shouldAutomaticAssessSubmissions
    && selectedRelease.hasAutomaticAssessmentStarted && selectedRelease.hasAutomaticAssessmentFinished
    && !selectedRelease.hadAutomaticAssessmentErrors)
    return translate(TranslateIds.automaticAssessmentHasFinished)

  if (!selectedRelease.shouldAutomaticAssessSubmissions
    && selectedRelease.hasAutomaticAssessmentStarted && selectedRelease.hasAutomaticAssessmentFinished)
    return translate(TranslateIds.manualStartedAssessmentHasFinished)

  if (
    selectedRelease.hasAutomaticAssessmentStarted && selectedRelease.hasAutomaticAssessmentFinished
    && selectedRelease.hadAutomaticAssessmentErrors)
    return translate(TranslateIds.automaticAssessmentHasFinishedWithErrors)

  return ''
}

//TODO enforce group role ... currently we don't know the group role ... maybe ask server exercise.userGroupId --> member --> role
class SubmissionsView extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  componentWillMount(): void {
    const exerciseId = parseId(this.props.match.params.exerciseId)
    const comesFromOwnExercisesSite = parseBoolean(this.props.match.params.comesFromOwnExercisesSite)
    const releaseId = parseId(this.props.match.params.releaseId)


    this.props.resetSubmissionsSite(false, false)

    //don't do this in the load action because we need to set som props before (do we really need to??)
    //this.props.resetSubmissionsSite(false, false)

    this.props.setComesFromOwnExercisesSite(comesFromOwnExercisesSite)

    if (exerciseId !== null && releaseId !== null && comesFromOwnExercisesSite !== null) {

      this.props.setSelectedReleaseId(releaseId)
      this.props.setSelectedExerciseId(exerciseId)

      //this is possible if a user opens this page directly in the browser
      //then we need to load some more data e.g. from the releases site...
      //as a side effect (because we use the releases site actions
      //the data from the releases site is also set
      this.props.loadSubmissionsSite(exerciseId, releaseId,
        this.props.exercises === null || this.props.exercises.length === 0,
        comesFromOwnExercisesSite,
        this.props.releases === null || this.props.releases.length === 0,
        true
      )

    }
  }

  render(): JSX.Element {

    const selectedExercise = this.props.exercises.find(p => p.id === this.props.selectedExerciseId)
    const selectedRelease = this.props.releases.find(p => p.id === this.props.selectedReleaseId)

    if (!selectedRelease) {
      //TODO only error if really not found
      //on first render this might not be set because the list was not yet set

      //this can also happen if the user opens this url and releases are not yet loaded...
    }

    return (
      <SinglePanelSiteWrapper>
        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">
          <div className="view-head-line">
            <div className="view-caption">
              <div className="mar-right back-arrow">
                <Link
                  to={this.props.comesFromOwnExercisesSite ? constants.ownExercisesLinkPath : constants.groupExercisesLinkPath}>
                  <DoubleBackArrow/>
                </Link>
              </div>
              <div className="mar-right back-arrow">
                <Link
                  to={getExerciseReleaseLinkPath(this.props.selectedExerciseId, this.props.comesFromOwnExercisesSite)}>
                  <BackArrow/>
                </Link>
              </div>
              <h2 className="clear-margin">
                {
                  translate(TranslateIds.submissionsOf)
                }
                {
                  selectedExercise &&
                  ` ${selectedExercise.displayName}`
                }
                {
                  selectedRelease &&
                  ` (${selectedRelease.generatedCode}) `
                }
              </h2>
            </div>
            <div className="view-options">
              <MaterialSearchInput/>
            </div>
          </div>

          <div className="view-content">

            <div className="flexed pad-left-right">
              <div className="mar-right-half">
                {translate(TranslateIds.state)}:
              </div>
              {
                selectedRelease &&
                <div className={selectedRelease.hadAutomaticAssessmentErrors ? 'error-colored mar-right-half' : 'mar-right-half'}>
                  {
                    getAssessmentState(selectedRelease)
                  }
                </div>
              }
              {
                selectedRelease && selectedRelease.isReleased &&
                <div>
                  <span className="mar-right-half">{translate(TranslateIds.releaseStillActiveButUserNormalTestResultsAreDisplayed)}</span>
                  <HelpPopup defaultText={translate(TranslateIds.releaseStillActiveButUserNormalTestResultsAreDisplayed_helpText)}/>
                </div>
              }
            </div>

            <ListView/>
          </div>
        </div>


      </SinglePanelSiteWrapper>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(SubmissionsView))