/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import {Icon, Form, Dropdown, DropdownItemProps} from "semantic-ui-react";
import MaterialInput from '../../material/materialInput'
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import Spinner from "../../helpers/Spinner";
import {ChangeState} from "../../../state/reducers/settingsSite/settingsSiteReducers";
import MiscSettingsChangeView from './miscSettingsChangeView'
import {setUserSettingsChangeStateAction} from "../../../state/actions/settingsSite/userSettingsSiteActions";
import {resetUserSettings} from "../../../state/actions/settingsSite/userSettingsActions";
import {updateUserSettingsAsync} from "../../../state/actions/settingsSite/userSettingsSiteCrudActions";
import {LanguageFromBackend} from "../../../types/language";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {getEditorInternalThemeName} from "../../../helpers/themes";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.userDataSettingsSate.isLoadingMiscSettings,
    changeState: rootState.userDataSettingsSate.userSettingsChangeState,
    userSettings: rootState.userDataSettingsSate.userData.settings
      ? rootState.userDataSettingsSate.userData.settings
      : null,
    langs: rootState.userDataSettingsSate.langs,
    userSettingsCopy: rootState.userDataSettingsSate.userSettingsCopy
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setUserSettingsChangeStateAction,
  resetUserSettings,
  updateUserSettingsAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


const langs: ReadonlyArray<string> = []

class MiscSettingsView extends React.Component<Props, any> {
  render(): JSX.Element {

    // noinspection TsLint
    let themeName = translate(TranslateIds.noThemeSelected)

    if (this.props.userSettings) {
      // noinspection TsLint
      themeName = getEditorInternalThemeName(this.props.userSettings.theme)
    }


    // noinspection TsLint
    let lang: LanguageFromBackend = null

    if (this.props.userSettings) {
      lang = this.props.langs.find(p => p.id === this.props.userSettings.langId)
    }

    const langName = lang
      ? lang.language
      :
      translate(TranslateIds.automatic)

    return (
      <SinglePanelSiteWrapper>

        {
          this.props.isLoading &&
          <Spinner/>
        }

        <div className="view-padding">

          <div className="view-head-line">

            <div className="view-caption">
              <h2>
                {
                  translate(TranslateIds.misc)
                }
              </h2>
            </div>

            <div className="view-options">
              {
                this.props.changeState === ChangeState.notActive &&
                <div className="flexed view-head-line-centered">
                  <div className="clickable"
                       onClick={() => this.props.setUserSettingsChangeStateAction(ChangeState.active)}
                  >
                    <Icon name="write"/>
                    <span>
                      {
                        translate(TranslateIds.change)
                      }
                    </span>
                  </div>

                  {
                    /*
                  not fully implemented...
                  <SimpleVDivider/>

                  <div className="clickable"
                       onClick={() => {
                         this.props.resetUserSettings()
                       }}
                  >
                    <Icon name="undo"/>
                    <span>Zurücksetzen</span>
                  </div>
                  */
                  }
                </div>
              }

              {
                this.props.changeState === ChangeState.active &&
                <div className="flexed view-head-line-centered">
                  <div className="clickable"
                       onClick={() => {
                         this.props.updateUserSettingsAsync(
                           {
                             id: this.props.userSettingsCopy.id,
                             langId: this.props.userSettingsCopy.langId,
                             theme: this.props.userSettingsCopy.theme
                           }
                         ),
                           this.props.setUserSettingsChangeStateAction(ChangeState.notActive)
                       }}
                  >
                    <Icon name="save"/>
                    <span>
                      {
                        translate(TranslateIds.save)
                      }
                    </span>
                  </div>

                  <SimpleVDivider/>

                  <div className="clickable"
                       onClick={() => {
                         this.props.setUserSettingsChangeStateAction(ChangeState.notActive)
                         this.props.resetUserSettings()
                       }}
                  >
                    <Icon name="remove"/>
                    <span>
                      {
                        translate(TranslateIds.discard)
                      }
                    </span>
                  </div>
                </div>
              }
            </div>

          </div>

          <div className="view-content">
            <div className="ui form">

              {
                this.props.changeState === ChangeState.notActive &&
                <Form.Group widths='equal'>
                  <Form.Field>
                    <label>
                      {
                        translate(TranslateIds.language)
                      }
                    </label>
                    <MaterialInput
                      value={langName}
                      disabled
                    />
                  </Form.Field>

                  <Form.Field>
                    <label>
                      {
                        translate(TranslateIds.design)
                      }
                    </label>
                    <MaterialInput
                      // value={themeName}
                      //TODO remove when we have real theme option
                      value={
                        translate(TranslateIds.defaultLightTheme)
                      }
                      disabled
                    />
                  </Form.Field>

                </Form.Group>
              }

              {
                this.props.changeState === ChangeState.active &&
                <MiscSettingsChangeView/>
              }

            </div>
          </div>

        </div>

      </SinglePanelSiteWrapper>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(MiscSettingsView)