/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {SelectableGroupUserFrontend} from "../../../types/group";
import {
  removeMemberAndRefreshAsync, setAllMemberUiIsSelected, setMemberGroupRoleAndRefreshAsync,
  setMemberUiIsSelected, setUiIsChangeGroupRolePopupDisplayedAction
} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {Card, Icon, Popup, Checkbox} from "semantic-ui-react";
import {copyToClipboard} from "../../../helpers/clipboard";
import {leftIconPopupOffset} from "../../../constants";
import SelectOptionsList, {SelectableOption} from '../../helpers/selectOptionList'
import {askDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly member: SelectableGroupUserFrontend
  readonly currentUserGroupRole: GroupRoleWithPermissionFullBase
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    member: props.member,
    groupRoles: rootState.singleGroupState.groupRoles,
    groupId: rootState.singleGroupState.selectedGroupId,
    currentUserGroupRole: props.currentUserGroupRole
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setMemberGroupRoleAndRefreshAsync,
  setUiIsChangeGroupRolePopupDisplayedAction,
  removeMemberAndRefreshAsync,
  setAllMemberUiIsSelected,
  setMemberUiIsSelected,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class cardViewItem extends React.Component<Props, any> {
  render(): JSX.Element {
    const member = this.props.member

    return (
      <Card key={member.id}>
        <Card.Content>

          <span className="right floated">{member.token}</span>

          <Card.Header>
            <span>{member.firstName + ' ' + member.lastName}</span>
          </Card.Header>

          <Card.Meta>
            <Icon name="clone" style={{visibility: (member.email ? null : 'hidden')}}
                  className="clickable mar-right-half"
                  onClick={() => {
                    copyToClipboard(member.email)
                  }}
            />
            <span>{member.email}</span>
          </Card.Meta>


          <Card.Description>
            <div className="flexed">
              <Icon name="law" className="mar-right-half"></Icon>
              <span className="mar-right">{translate(TranslateIds.groupRole)}:</span>
              {
                member.groupRole && !member.isCurrentUser && this.props.currentUserGroupRole && this.props.currentUserGroupRole.canChangeOtherMembersRole &&
                <Popup
                  trigger={
                    <div className="clickable">
                      <Icon name="caret down" className="mar-right-half"/>
                      <span>{member.groupRole.displayName}</span>
                    </div>
                  }
                  on="click"
                  position="bottom left"
                  offset={leftIconPopupOffset}
                  content={
                    <div>
                      <SelectOptionsList
                        options={this.props.groupRoles}
                        optionClicked={(option: SelectableOption) => {
                          const groupRole = this.props.groupRoles.find(p => p.id === option.id)

                          if (!groupRole) {
                            //TODO some error
                          } else {
                            this.props.setMemberGroupRoleAndRefreshAsync(member.id, this.props.groupId,
                              groupRole.id)
                          }
                        }}
                      />
                    </div>
                  }
                  open={member.uiIsChangeGroupRolePopupDisplayed}
                  onOpen={() => this.props.setUiIsChangeGroupRolePopupDisplayedAction(true, member.id)}
                  onClose={() => this.props.setUiIsChangeGroupRolePopupDisplayedAction(false, member.id)}

                />
              }
              {
                member.groupRole && member.isCurrentUser &&
                <div>
                  <span>{member.groupRole.displayName}</span>
                </div>
              }
              {
                !member.groupRole &&
                <span>error</span>
              }
            </div>


            <div>
              <Checkbox checked={member.isActivated} disabled
                        label={(member.isActivated ? translate(TranslateIds.isActivated) : translate(
                          TranslateIds.isNotActivated))}/>
            </div>

          </Card.Description>

        </Card.Content>

        <Card.Content extra>
          <div className="two-flexed">
            <div className="clickable">
            </div>
            <div className="clickable">
              {
                member.isCurrentUser &&
                <Popup
                  trigger={
                    <div className="hoverable">
                      <Icon name="help circle"/>
                      <span>
                        {
                          translate(TranslateIds.releaseMemberFromGroup)
                        }
                      </span>
                    </div>
                  }
                  content={translate(TranslateIds.thisIsYou)}
                />
              }
              {
                !member.isCurrentUser && this.props.currentUserGroupRole && this.props.currentUserGroupRole.canRemoveMemberFromGroup &&
                <div className="clickable"
                     onClick={async () => {

                       const shouldRemove = await askDialog(translate(TranslateIds.releaseMemberFromGroup_title),
                         `${translate(TranslateIds.releaseMemberFromGroup_content)} (${member.lastName}, ${member.firstName})`)

                       if (shouldRemove) {
                         this.props.removeMemberAndRefreshAsync(member.id, this.props.groupId)
                       }

                     }}
                >
                  <Icon name="remove"/>
                  <span>
                    {
                      translate(TranslateIds.releaseMemberFromGroup)
                    }
                  </span>
                </div>
              }

            </div>

          </div>
        </Card.Content>
      </Card>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(cardViewItem)