/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers/index";
import {SelectableGroupUserFrontend} from "../../../types/group";
import {Table, Icon, Checkbox, Popup} from "semantic-ui-react";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {
  removeMemberAndRefreshAsync,
  setAllMemberUiIsSelected, setMemberGroupRoleAndRefreshAsync,
  setMemberUiIsSelected, setUiIsChangeGroupRolePopupDisplayedAction
} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {copyToClipboard} from "../../../helpers/clipboard";
import {leftIconPopupOffset} from "../../../constants";
import SelectOptionsList, {SelectableOption} from '../../helpers/selectOptionList'
import {askDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly member: SelectableGroupUserFrontend
  readonly currentUserGroupRole: GroupRoleWithPermissionFullBase
}

const mapStateToProps = (rootState: RootState , props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    member: props.member,
    groupRoles: rootState.singleGroupState.groupRoles,
    groupId: rootState.singleGroupState.selectedGroupId,
    currentUserGroupRole: props.currentUserGroupRole
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

  setMemberGroupRoleAndRefreshAsync,
  setUiIsChangeGroupRolePopupDisplayedAction,
  removeMemberAndRefreshAsync,
  setAllMemberUiIsSelected,
  setMemberUiIsSelected,

}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class listViewItem extends React.Component<Props, any> {

  render(): JSX.Element {

    const member = this.props.member

    return (
      <Table.Row>
        <Table.Cell>
          {
            !member.isCurrentUser &&
            <Checkbox checked={member.uiIsSelected}
                      onChange={(e, data: CheckboxOnChangeEventData) => {
                        this.props.setMemberUiIsSelected(member.id, data.checked)
                      }}
            />
          }
        </Table.Cell>

        <Table.Cell>
          <span>{member.lastName}</span>
        </Table.Cell>

        <Table.Cell>
          <span>{member.firstName}</span>
        </Table.Cell>

        <Table.Cell>
          <span>{member.token}</span>
        </Table.Cell>

        <Table.Cell>
          <div>
            {
              //TODO icon or checkbox??
              //checkbox is more precises than different icons??
              //<Icon name={member.isActivated ? 'user' : 'user outline'} />
            }
            {
              <Checkbox checked={member.isActivated} disabled/>
            }
          </div>
        </Table.Cell>

        <Table.Cell>
          <Icon name="clone" className="clickable"
                onClick={() => {
                  copyToClipboard(member.email)
                }}
          />
          <span>{member.email}</span>
        </Table.Cell>

        <Table.Cell>
          {
            member.groupRole && !member.isCurrentUser && this.props.currentUserGroupRole && this.props.currentUserGroupRole.canChangeOtherMembersRole &&
            <Popup
              trigger={
                <div className="clickable">
                  <Icon name="caret down" className="mar-right-half"/>
                  <span>{member.groupRole.displayName}</span>
                </div>
              }
              on="click"
              position="bottom left"
              offset={leftIconPopupOffset}
              content={
                <div>
                  <SelectOptionsList
                    options={this.props.groupRoles}
                    optionClicked={(option: SelectableOption) => {
                      const groupRole = this.props.groupRoles.find(p => p.id === option.id)

                      if (!groupRole) {
                        //TODO some error
                      } else {
                        this.props.setMemberGroupRoleAndRefreshAsync(member.id, this.props.groupId,
                          groupRole.id)
                      }
                    }}
                  />
                </div>
              }
              open={member.uiIsChangeGroupRolePopupDisplayed}
              onOpen={() => this.props.setUiIsChangeGroupRolePopupDisplayedAction(true, member.id)}
              onClose={() => this.props.setUiIsChangeGroupRolePopupDisplayedAction(false, member.id)}

            />
          }
          {
            member.groupRole && member.isCurrentUser &&
            <div>
              <span>{member.groupRole.displayName}</span>
            </div>
          }
          {
            !member.groupRole &&
            <span>error</span>
          }
        </Table.Cell>

        <Table.Cell>
          {
            member.isCurrentUser &&
            <Popup
              trigger={
                <div className="hoverable">
                  <Icon name="help circle"/>
                </div>
              }
              content={translate(TranslateIds.thisIsYou)}
            />
          }
          {
            !member.isCurrentUser && this.props.currentUserGroupRole && this.props.currentUserGroupRole.canRemoveMemberFromGroup &&
            <div className="clickable"
                 onClick={async () => {

                   const shouldRemove = await askDialog(translate(TranslateIds.releaseMemberFromGroup_title),
                     `${translate(TranslateIds.releaseMemberFromGroup_content)} (${member.lastName}, ${member.firstName})`)

                   if (shouldRemove) {
                     this.props.removeMemberAndRefreshAsync(member.id, this.props.groupId)
                   }

                 }}
            >
              <Icon name="remove" className="clickable"/>
            </div>

          }

        </Table.Cell>

      </Table.Row>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(listViewItem)