/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import {Popup, Form, Icon, Divider, Checkbox} from "semantic-ui-react";
import MaterialInput from '../../../material/materialInput'
import {
  setDisplayName,
  setEmail,
  setCanCreateGroups,
  setCanChangeGroups,
  setCanDeleteGroups,
  setCanCreateRoles,
  setCanChangeRoles,
  setCanDeleteRoles,
  setCanManageNewUsers,
  setCanChangeOtherUsersSystemRole,
  setCanDeleteActivatedUsers,
  setCanChangeUserData,
  setCanChangeSystemSettings,
  setCanManageTags, setCanViewDashboard
} from "../../../../state/actions/systemRolesSite/dialog/systemRoleSetActions";
import {
  validationMessageKeys,
  validationRules
} from "../../../../state/reducers/systemRolesSite/dialog/systemRoleReducer";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    systemRole: rootState.systemRolesChangeDialogState.systemRole
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setDisplayName,
  setEmail,
  setCanCreateGroups,
  setCanChangeGroups,
  setCanDeleteGroups,
  setCanCreateRoles,
  setCanChangeRoles,
  setCanDeleteRoles,
  setCanManageNewUsers,
  setCanChangeOtherUsersSystemRole,
  setCanDeleteActivatedUsers,
  setCanChangeUserData,
  setCanChangeSystemSettings,
  setCanManageTags,
  setCanViewDashboard
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class ChangeSystemRoleView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>
        <div className="ui form divided-checkboxes">
          <Form.Group widths='equal'>

            <Form.Field required>
              <label>
                {
                  translate(TranslateIds.displayName)
                }
              </label>
              <MaterialInput value={this.props.systemRole.displayName}
                             onChange={(e) => this.props.setDisplayName(e.currentTarget.value)}
                             validations={validationRules.displayName}
                             validationMessageKey={validationMessageKeys.displayName}
              />
            </Form.Field>
            <Form.Field>
              <label>
                {
                  translate(TranslateIds.email)
                }
              </label>
              <MaterialInput value={this.props.systemRole.email}
                             onChange={(e) => this.props.setEmail(e.currentTarget.value)}
                             validations={validationRules.email}
                             validationMessageKey={validationMessageKeys.email}
              />
            </Form.Field>
          </Form.Group>


          <Form.Group widths='equal'>
            <Form.Field>
              <h4 className="ui dividing header">
                {
                  translate(TranslateIds.groups)
                }
              </h4>

              <Form.Field>
                <Checkbox label={translate(TranslateIds.canCreateGroups)}
                          checked={this.props.systemRole.canCreateGroups}
                          onChange={(e, data) => {
                            this.props.setCanCreateGroups(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canChangeGroups)}
                          checked={this.props.systemRole.canChangeGroups}
                          onChange={(e, data) => {
                            this.props.setCanChangeGroups(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canDeleteGroups)}
                          checked={this.props.systemRole.canDeleteGroups}
                          onChange={(e, data) => {
                            this.props.setCanDeleteGroups(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
            <Form.Field>
              <h4 className="ui dividing header">
                {
                  translate(TranslateIds.systemRoles)
                }
              </h4>

              <Form.Field>
                <Checkbox label={translate(TranslateIds.canCreateRoles)}
                          checked={this.props.systemRole.canCreateRoles}
                          onChange={(e, data) => {
                            this.props.setCanCreateRoles(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canChangeRoles)}
                          checked={this.props.systemRole.canChangeRoles}
                          onChange={(e, data) => {
                            this.props.setCanChangeRoles(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canDeleteRoles)}
                          checked={this.props.systemRole.canDeleteRoles}
                          onChange={(e, data) => {
                            this.props.setCanDeleteRoles(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
          </Form.Group>


          <h4 className="ui dividing header">
            {
              translate(TranslateIds.misc)
            }
          </h4>
          <Form.Group widths='equal'>


            <Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canManageNewUsers)}
                          checked={this.props.systemRole.canManageNewUsers}
                          onChange={(e, data) => {
                            this.props.setCanManageNewUsers(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canChangeOtherUsersSystemRole)}
                          checked={this.props.systemRole.canChangeOtherUsersSystemRole}
                          onChange={(e, data) => {
                            this.props.setCanChangeOtherUsersSystemRole(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canDeleteActivatedUsers)}
                          checked={this.props.systemRole.canDeleteActivatedUsers}
                          onChange={(e, data) => {
                            this.props.setCanDeleteActivatedUsers(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
            <Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canChangeOtherUserData)}
                          checked={this.props.systemRole.canChangeUserData}
                          onChange={(e, data) => {
                            this.props.setCanChangeUserData(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canChangeSystemSettings)}
                          checked={this.props.systemRole.canChangeSystemSettings}
                          onChange={(e, data) => {
                            this.props.setCanChangeSystemSettings(data.checked)
                          }}
                />
              </Form.Field>
              <Form.Field>
                <Checkbox label={translate(TranslateIds.canManageTags)}
                          checked={this.props.systemRole.canManageTags}
                          onChange={(e, data) => {
                            this.props.setCanManageTags(data.checked)
                          }}
                />
              </Form.Field>

              <Form.Field>
                <Checkbox label={translate(TranslateIds.canViewDashboard)}
                          checked={this.props.systemRole.canViewDashboard}
                          onChange={(e, data) => {
                            this.props.setCanViewDashboard(data.checked)
                          }}
                />
              </Form.Field>
            </Form.Field>
          </Form.Group>
        </div>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(ChangeSystemRoleView)