/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import MaterialTextArea from '../../../material/materialTextArea'
import MaterialInput from '../../../material/materialInput'
import {Form} from "semantic-ui-react";
import {
  setFeedbackForStudent,
  setManualAssessmentNote,
  setManualAssessmentPoints
} from "../../../../state/actions/tutorViewSite/sub/setAssessmentActions";
import {ChangeEvent} from "react";
import {parseNotNegativeOrNullNumber} from "../../../../helpers/convertersAndTransformers";
import {changeSingleAssessmentTutorViewAsync} from "../../../../state/actions/tutorViewSite/tutorViewSiteCrudActions";
import {HelpPopup} from "../../../helpers/helpPopup";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {errorDialog} from "../../../../helpers/dialogHelper";
import Logger from "../../../../helpers/logger";
import {SimpleVDivider} from "../../../helpers/simpleVDivider";
import {SimpleDivider} from "../../../helpers/simpleDivider";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    assessment: rootState.tutorViewSiteState.assessment,
  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here
  setManualAssessmentPoints,
  setManualAssessmentNote,
  setFeedbackForStudent,

  changeSingleAssessmentTutorView: changeSingleAssessmentTutorViewAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class manualAssessmentPanelView extends React.Component<Props, any> {


  render(): JSX.Element {

    const submitTestPointsString = this.props.assessment
    && this.props.assessment.submitTestPoints
    && this.props.assessment.maxSubmitTestPoints
      ? `${this.props.assessment.submitTestPoints}/${this.props.assessment.maxSubmitTestPoints}`
      : translate(TranslateIds.notAssessedPoints)

    const normalTestPointsString = this.props.assessment
    && this.props.assessment.normalTestPoints
    && this.props.assessment.maxNormalTestPoints
      ? `${this.props.assessment.normalTestPoints}/${this.props.assessment.maxNormalTestPoints}`
      : translate(TranslateIds.notAssessedPoints)

    return (
      <div className="fh mar-right-half">
        <div className="ui form">

          {
            !this.props.assessment &&
            <div>
              {
                translate(TranslateIds.thereIsNoAutomaticAssessmentYet)
              }
            </div>
          }


          <Form.Group widths='equal'>
            <Form.Field>
              <label>
                {
                  translate(TranslateIds.lastName)
                }
              </label>
              <MaterialInput
                value={this.props.assessment.lastName}
                disabled
              />
            </Form.Field>

            <Form.Field>
              <label>
                {
                  translate(TranslateIds.firstName)
                }
              </label>
              <MaterialInput
                value={this.props.assessment.firstName}
                disabled
              />
            </Form.Field>

          </Form.Group>

          {
            //-- points
          }
          <Form.Group widths='equal'>

            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {
                    translate(TranslateIds.normalTestPoints)
                  }
                </label>
                <HelpPopup
                  wide
                  defaultText={translate(TranslateIds.noNormalTestPoints_helpText)}/>
              </div>
              <MaterialInput
                value={normalTestPointsString}
                disabled
              />
            </Form.Field>

            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {
                    translate(TranslateIds.submitTestPoints)
                  }
                </label>
                <HelpPopup
                  wide
                  defaultText={translate(TranslateIds.noSubmitTestPoints_helpText)}/>
              </div>
              <MaterialInput
                value={submitTestPointsString}
                disabled
              />
            </Form.Field>

            <Form.Field>
              <div className="form-field-label-with-icon">
                <label className="mar-right-half">
                  {
                    translate(TranslateIds.manualPoints)
                  }
                </label>
                <HelpPopup defaultText={translate(TranslateIds.whenAreManualPointsOrFeedbackIsDisplayed_tutor_helpText)}/>
              </div>
              <MaterialInput
                value={this.props.assessment.manualPoints ? this.props.assessment.manualPoints.toString() : null}
                type="number"
                onChange={(e) => {

                  if (e.currentTarget.value === '') {
                    this.props.setManualAssessmentPoints(null)
                    return
                  }

                  const manualPoints = parseNotNegativeOrNullNumber(e.currentTarget.value)

                  if (!manualPoints) {
                    Logger.debug('manualAssessmentPanelView',
                      `could not convert string to int: ${e.currentTarget.value}`)

                    errorDialog(translate(TranslateIds.error),
                      translate(TranslateIds.couldNotConvertStringToInt))

                    return
                  }
                  this.props.setManualAssessmentPoints(manualPoints)
                }}
              />
            </Form.Field>

          </Form.Group>

          <Form.Field>
            <label>
              {
                translate(TranslateIds.noteForOtherTutors)
              }
            </label>

            <MaterialTextArea rows={5}
                              placeholder={`${translate(TranslateIds.noteForOtherTutors)}...`}
                              value={this.props.assessment.noteForOtherTutors ? this.props.assessment.noteForOtherTutors : null}
                              onChange={(event: ChangeEvent<HTMLTextAreaElement>, value: string) => {
                                this.props.setManualAssessmentNote(value)
                              }}
            />
          </Form.Field>

          <SimpleDivider/>

          <Form.Field>
            <div className="form-field-label-with-icon">
            <label className="mar-right-half">
              {
                translate(TranslateIds.feedbackForStudent)
              }
            </label>
              <HelpPopup defaultText={translate(TranslateIds.whenAreManualPointsOrFeedbackIsDisplayed_tutor_helpText)}/>
            </div>

            <MaterialTextArea rows={5}
                              placeholder={`${translate(TranslateIds.feedbackForStudent)}...`}
                              value={this.props.assessment.feedbackForStudent ? this.props.assessment.feedbackForStudent : null}
                              onChange={(event: ChangeEvent<HTMLTextAreaElement>, value: string) => {
                                this.props.setFeedbackForStudent(value)
                              }}
            />
          </Form.Field>


        </div>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(manualAssessmentPanelView)