/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers/index";
import MultiTabCodeEditorWrapper from "../../../codeEditors/multiTabCodeEditorWrapper";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import Logger from "../../../../helpers/logger";
import {
  closeTabChangeTutorEditor,
  openTabChangeTutorEditor,
  setHasUnsavedChangesTutorEditor,
  setIsFileExplorerDisplayedTutorEditor, setIsRenamingFileTutorEditor, setNewFileNameTutorEditor,
  setRenamingFileIdTutorEditor, setRenamingFileNameTutorEditor,
  setSelectedTabIdTutorEditor
} from "../../../../state/actions/tutorViewSite/editorActions/tutorEditorActions";
import {setSolutionFilesTutorEditor} from "../../../../state/actions/tutorViewSite/tutorViewSiteActions";
import {errorDialog} from "../../../../helpers/dialogHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {EditorTab} from "../../../codeEditors/multiTabCodeEditorProps";
import {setForceEditorUpdateSessionDummyTutorEditor} from "../../../../state/actions/editorHelperState/editorHelperStateActions";
import {setSolutionFiles_AndRefreshEditor} from "../../../../helpers/editorHelper";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    solution: rootState.tutorViewSiteState.tutorEditorSolution,

    pLangs: rootState.pLangsState.pLangs,

    selectedTabIdDoEditor: rootState.tutorViewSiteState.tutorEditorState.selectedTabIdDoEditor,
    isFileExplorerDisplayedDoEditor: rootState.tutorViewSiteState.tutorEditorState.isFileExplorerDisplayedDoEditor,
    newFileNameDoEditor: rootState.tutorViewSiteState.tutorEditorState.newFileNameDoEditor,
    renamingFileIdDoEditor: rootState.tutorViewSiteState.tutorEditorState.renamingFileIdDoEditor,
    isRenamingFileDoEditor: rootState.tutorViewSiteState.tutorEditorState.isRenamingFileDoEditor,
    renamingFileNameDoEditor: rootState.tutorViewSiteState.tutorEditorState.renamingFileNameDoEditor,
    hasUnsavedChanges: rootState.tutorViewSiteState.tutorEditorState.hasUnsavedChanges,
    hasFileChangedMap: rootState.tutorViewSiteState.tutorEditorState.hasFileChangedMap,

    forceEditorUpdateSessionDummy: rootState.editorHelperState.forceEditorUpdateSessionDummyTutorEditor,

  }
}

const mapDispatchToProps = (dispatch: Dispatch<any>) => bindActionCreators({
  //imported reducer funcs here

  setSolutionFilesTutorEditor,
  setSelectedTabIdTutorEditor,
  setIsFileExplorerDisplayedTutorEditor,
  setNewFileNameTutorEditor,
  setRenamingFileIdTutorEditor,
  setIsRenamingFileTutorEditor,
  setRenamingFileNameTutorEditor,
  setHasUnsavedChangesTutorEditor,
  openTabChangeTutorEditor,
  closeTabChangeTutorEditor,

  setSolutionFiles_AndRefreshEditor,
  setForceEditorUpdateSessionDummyTutorEditor,
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class editorPanelView extends React.Component<Props, any> {


  render(): JSX.Element {


    const pLang = this.props.pLangs.find(p => p.id === this.props.solution.pLangId)

    if (!pLang) {
      //TODO only error if really not found
      //on first render we have no plang but thats no and error!!

      return null
    }

    const editorTabs = this.props.solution.solutionFiles.map<EditorTab>(p => {
      return {
        id: p.id,
        fileNameWithExtensions: p.fileNameWithExtension,
        content: p.content,
        isMainFile: p.id === this.props.solution.mainFileId,
        isReadonly: !p.isEditableByUser,
        displayIndex: p.displayIndex,
        language: pLang.internalName,
        editSession: p.uiEditSession
      }
    })

    return (
      <div className="fh hidden-overflow-i">
        <MultiTabCodeEditorWrapper
          id="editorPanelView"
          height="100%"
          forceEditorUpdateSessionDummy={this.props.forceEditorUpdateSessionDummy}
          selectedTabId={this.props.selectedTabIdDoEditor}
          onSelectedTabIdChanged={(tabId: number) => {
            this.props.setSelectedTabIdTutorEditor(tabId)
          }}

          isFileExplorerDisplayed={this.props.isFileExplorerDisplayedDoEditor}
          onChangeIsFileExplorerDisplayed={(isDisplayed: boolean) => {
            this.props.setIsFileExplorerDisplayedTutorEditor(isDisplayed)
          }}

          newFileName={this.props.newFileNameDoEditor}
          onNewFileNameChanged={(name: string) => {
            this.props.setNewFileNameTutorEditor(name)
          }}

          renamingFileId={this.props.renamingFileIdDoEditor}
          onRenamingFileIdChanged={(fileId: number) => {
            this.props.setRenamingFileIdTutorEditor(fileId)
          }}

          isRenamingFile={this.props.isRenamingFileDoEditor}
          onIsRenamingFileChanged={(isRenamingFile: boolean) => {
            this.props.setIsRenamingFileTutorEditor(isRenamingFile)
          }}

          renamingFileName={this.props.renamingFileNameDoEditor}
          onRenamingFileNameChanged={(newName: string) => {
            this.props.setRenamingFileNameTutorEditor(newName)
          }}

          hasUnsavedChanges={this.props.hasUnsavedChanges}
          onHasUnsavedChangesChanged={(hasChanged: boolean) => {
            this.props.setHasUnsavedChangesTutorEditor(hasChanged)
          }}

          userCanCreateNewFiles={true}

          language={pLang.internalName}

          onCreateFileClicked={(editorTab: EditorTab) => {

            const newFile: SolutionFileDoExerciseFullBase = {
              id: editorTab.id,
              content: editorTab.content,
              fileNameWithExtension: editorTab.fileNameWithExtensions,
              uiIsDisplayed: editorTab.displayIndex >= 0,
              displayIndex: editorTab.displayIndex,
              isEditableByUser: true,
              templateFileId: null,
              uiEditSession: editorTab.editSession
            }

            // this.props.setSolutionFilesTutorEditor(
            //   [...this.props.solution.solutionFiles, newFile]
            // )

            this.props.setSolutionFiles_AndRefreshEditor(
              [...this.props.solution.solutionFiles, newFile],
              this.props.setSolutionFilesTutorEditor,
              this.props.setForceEditorUpdateSessionDummyTutorEditor
            )

          }}

          onOpenTabClicked={(editorTab: EditorTab) => {
            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (onOpenTabClicked)")

              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
                )

              return
            }

            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: true,
                }
                : p
            })

            this.props.openTabChangeTutorEditor(file, mappedFiles, this.props.selectedTabIdDoEditor)
          }}

          onCloseTabClicked={(editorTab: EditorTab) => {

            const file = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!file) {
              Logger.error("file not found (onCloseTabClicked)")

              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )

              return
            }

            const mappedFiles = this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(p => {
              return p.id === file.id
                ? {
                  ...file,
                  uiIsDisplayed: false,
                }
                : p
            })

            this.props.closeTabChangeTutorEditor(file, mappedFiles,
              this.props.selectedTabIdDoEditor)
          }}

          onFileRename={(editorTab: EditorTab, newName: string) => {

            const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {

              Logger.error("file not found (onFileRename)")

              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )

              return
            }

            this.props.setSolutionFilesTutorEditor(
              this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                  ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: newName,
                    isEditableByUser: file.isEditableByUser,
                    content: file.content,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    uiEditSession: editorTab.editSession
                  }
                  : file
              })
            )

          }}

          onRemoveFile={(editorTab: EditorTab) => {

            this.props.setSolutionFiles_AndRefreshEditor(
              this.props.solution.solutionFiles.filter(p => p.id !== editorTab.id),
              this.props.setSolutionFilesTutorEditor,
              this.props.setForceEditorUpdateSessionDummyTutorEditor
            )

            // this.props.setSolutionFilesTutorEditor(
            //   this.props.solution.solutionFiles.filter(p => p.id !== editorTab.id)
            // )
          }}

          editorTabs={editorTabs}
          onChange={(editorTab: EditorTab, newValue: string) => {

            const currentFile = this.props.solution.solutionFiles.find(p => p.id === editorTab.id)

            if (!currentFile) {

              Logger.error("file not found (onChange)")

              errorDialog(translate(TranslateIds.error),
                translate(TranslateIds.editorFileNotFound)
              )

              return
            }

            this.props.setSolutionFilesTutorEditor(
              this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                return file.id === currentFile.id
                  ? {
                    id: file.id,
                    displayIndex: file.displayIndex,
                    fileNameWithExtension: file.fileNameWithExtension,
                    isEditableByUser: file.isEditableByUser,
                    content: newValue,
                    templateFileId: file.templateFileId,
                    uiIsDisplayed: file.uiIsDisplayed,
                    uiEditSession: editorTab.editSession
                  }
                  : file
              })
            )

          }}

          onSomeEditorSessionChanged={(editorTabs: ReadonlyArray<EditorTab>) => {
            this.props.setSolutionFilesTutorEditor(
              this.props.solution.solutionFiles.map<SolutionFileDoExerciseFullBase>(file => {
                const changedTab = editorTabs.find(p => p.id === file.id)
                return changedTab
                  ? {
                    ...file,
                    uiEditSession: changedTab.editSession
                  }
                  : file
              })
            )
          }}

          hasFileChangedMap={this.props.hasFileChangedMap}
          onUpdateHasFileChangedMap={(changedFile: EditorTab, allTabs: ReadonlyArray<EditorTab>) => {
            //performance is tooooo bad...
            //this.props.setHasFileChangedMapDoEditor(updateHasFileChangedMap(changedFile, allTabs, this.props.hasFileChangedMap))
          }}

        />
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(editorPanelView)