/**
 * Created by janis dähne.
 */
import {ExerciseAssetFullBase} from "../types/exerciseEditor";
import Logger from "./logger";


interface AssetWithUrl {
  readonly url: string,
  readonly asset: ExerciseAssetFullBase
}

interface ExerciseAssetsStorage {
  // noinspection TsLint
  [assetId: string]: AssetWithUrl
}


// noinspection TsLint
let currentAssets: ExerciseAssetsStorage = {}


/**
 * tries to return the asset url from an asset
 * @param {ExerciseAssetFullBase} asset
 * @returns {string}
 */
export function getObjectUrlForAsset(asset: ExerciseAssetFullBase): string | null {

  // noinspection TsLint
  for (const key in currentAssets) {
    const tuple = currentAssets[key]

    if (tuple.asset.id === asset.id) {
      return tuple.url
    }
  }
  return null
}


/**
 * frees all asset resources
 *
 * use this also if the site is left (navigated away)
 *
 * TODO must this does not work... maybe the created dom img tag still hold a referenced??
 * or it's a browser bug? when watching the sources panel in chrome, the (no domain) tree we see the blob but calling
 * revokeObjectURL won't remove it
 */
export function freeAssetUrlsAndObjects(): void {

  // noinspection TsLint
  for (const key in currentAssets) {
    const tuple = currentAssets[key]
    URL.revokeObjectURL(tuple.url)

    Logger.debug("assetHelper", `revoking object url for asset (tuple) with id ${tuple.asset.id}`, tuple)
  }

  currentAssets = {}

}

/**
 * deletes all old object urls and releases resources, then creates new object urls for all given assets
 * @param {ReadonlyArray<ExerciseAssetFullBase>} assets
 */
export function createAndRefreshAssetUrlsAndObjects(assets: ReadonlyArray<ExerciseAssetFullBase>): void {

  //revoke all old urls
  //freeAssetUrlsAndObjects()

  // noinspection TsLint
  for (let i = 0; i < assets.length; i++) {
    const asset = assets[i]

    const bytes = new Uint8Array(asset.content);
    const blob = new Blob([bytes], {type: asset.mimeType})
    const url = URL.createObjectURL(blob)

    const tuple = {
      asset: asset,
      url: url
    }
    currentAssets[asset.id.toString()] = tuple

    Logger.debug("assetHelper", `creating object url for asset (tuple) with id ${tuple.asset.id}`, tuple)

  }
}

export function contentWithAssetsToContentWithUrlsFilter(contentWithAssetUrls: string): string {

  // noinspection TsLint
  let newContent = contentWithAssetUrls

  // noinspection TsLint
  for (const key in currentAssets) {
    const tuple = currentAssets[key]

    newContent = newContent.split(tuple.asset.assetUrl).join(tuple.url);
  }

  return newContent
}

export function getCopyAssetUrlText(asset: ExerciseAssetFullBase): string {
  return `<img alt="img not found" src="${asset.assetUrl}" width="80%" />`
}
