/**
 * Created by janis dähne.
 */
import {ExerciseAssetFullBase, TestAssetBase} from "../types/exerciseEditor";


const fileEncoding = 'utf-8'


export async function readFileAsText(file: File | null,
                                     onProgress?: (e: ProgressEvent) => void
): Promise<string> {

  return new Promise<string>((resolve, reject) => {

    if (!file) return null

    const fileReader = new FileReader()
    fileReader.readAsText(file, fileEncoding)

    if (onProgress)
      fileReader.onprogress = onProgress

    fileReader.onload = e => {
      resolve(fileReader.result as string)
    }

    // fileReader.onerror = ev => {
    //   reject(fileReader.error.message)
    // }

    fileReader.onerror = ev => {
      throw new Error(`could not read file: ${file.name}, error: ${fileReader.error.message}`)
    }
  })
}

/**
 * read a file as an array buffer converts it to a Uint8Array and finally to a normal js number array
 * @param {File} file
 * @param {(e: ProgressEvent) => void} onProgress
 * @returns {Promise<ReadonlyArray<number>>}
 */
export async function readFileAsAsset(file: File | null,
                                     onProgress?: (e: ProgressEvent) => void
): Promise<ReadonlyArray<number>> {

  return new Promise<ReadonlyArray<number>>((resolve, reject) => {

    if (!file) return null

    const fileReader = new FileReader()
    fileReader.readAsArrayBuffer(file)

    if (onProgress)
      fileReader.onprogress = onProgress

    fileReader.onload = e => {

      const result: ArrayBuffer = fileReader.result as ArrayBuffer
      const array = Array.prototype.slice.call(new Uint8Array(result))

      resolve(array)

    }

    // fileReader.onerror = ev => {
    //   reject(fileReader.error.message)
    // }

    fileReader.onerror = ev => {
      throw new Error(`could not read file: ${file.name}, error: ${fileReader.error.message}`)
    }
  })
}