
/**
 * Created by janis dähne.
 */

import moment = require("moment");

// noinspection TsLint
/**
 * starts with -1 when calling getNextId
 * @type {number}
 */
let currentId = -1


function getNextId(): number {

  currentId--
  return currentId
}


export function toJSONAndReplaceIds(obj: any): string {

  currentId = -1
  const value = JSON.stringify(obj, replaceIdsTransformer, "\t")
  return value
}

/**
 * replaces all id attributes with a new negative id
 *
 * this method will not reset the (next) id
 * @param {string} key
 * @param value
 * @returns {any}
 */
 function replaceIdsTransformer(key: string, value: any): any {
  if (key === 'id') {
    return getNextId().toString()
  }

  return value

}


const escapable = /[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g;
const meta = {    // table of character substitutions
  '\b': '\\b',
  '\t': '\\t',
  '\n': '\\n',
  '\f': '\\f',
  '\r': '\\r',
  '"' : '\\"',
  '\\': '\\\\'
};

/*
  Escape JSON
*/
//from https://gist.github.com/matthewmueller/3459332 which is clearly taken from https://github.com/douglascrockford/JSON-js/blob/ad6079cbd8dc362a3cc42e1f97c01aa5ccd48bfe/json2.js#L211
export function escapeJSON(json: string): string {
  escapable.lastIndex = 0;
  return escapable.test(json) ? '"' + json.replace(escapable, function (a) {
    const c = meta[a];
    return (typeof c === 'string') ? c
      : '\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);
  }) + '"' : '"' + json + '"';

};


const reISO = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*))(?:Z|(\+|-)([\d|:]*))?$/
const reMsAjax = /^\/Date\((d|-|.*)\)[\/|\\]$/
const reWithoutMs = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})Z?$/

/**
 * same as dateTimeReviver but converts the time to local time (was gtm +2)
 * @param key
 * @param value
 * @returns {any}
 */
export function dateTimeReviverLocalTime(key: string, value: any): any {

  if (typeof value === 'string') {
    // noinspection TsLint
    let a = reISO.exec(value);
    if (a) {

      //2016-09-10T14:56:01.129Z
      const _date = moment(value, 'YYYY-MM-DD HH:mm:ss')
      if (!_date.isValid()) {
        return null
      }

      const offsetInMinutes = moment().utcOffset()
      _date.add(offsetInMinutes, 'minutes')
      return _date
    }
    a = reMsAjax.exec(value);
    if (a) {
      const b = a[1].split(/[-+,.]/);
      const date = new Date(b[0] ? +b[0] : 0 - +b[1]);

      const _date = moment(date)

      //my modification to catch invalid dates
      if (!_date.isValid()) {
        //invalid
        return null
      }

      const offsetInMinutes = moment().utcOffset()
      _date.add(offsetInMinutes, 'minutes')

      //TODO maybe timezone?
      return _date

    } else {
      //sometimes the backend omits the ms ... so we just get e.g. 2016-08-26T23:22:00 and not 2016-08-26T23:22:00.972589
      a = reWithoutMs.exec(value)
      if (a) {
        const _date = moment(value, 'YYYY-MM-DD HH:mm:ss')
        if (!_date.isValid()) {
          return null
        }

        const offsetInMinutes = moment().utcOffset()
        _date.add(offsetInMinutes, 'minutes')
        return _date

      }
    }
  }
  return value;
}