/**
 * Created by janis dähne.
 */

import {Moment} from "moment";
import moment = require("moment");

export enum SeverityLevel {
  info,
  warning,
  error,
  debug
}

/**
 * a log item from the logger
 */
export interface LogItem {
  readonly id: number
  readonly file?: string
  readonly level: SeverityLevel
  readonly message: string
  readonly params?: ReadonlyArray<any>
  readonly time?: Moment
}


/**
 * a common logger
 */
export default class Logger {

  static nextItemId = 1

  // noinspection TsLint
  static myLog: LogItem[] = []

  // noinspection TsLint
  static log(message?: any, ...optionalParams: any[]): void {
    Logger.logWithLevel(message, SeverityLevel.info, null, optionalParams)
  }

  // noinspection TsLint
  static warn(message?: any, ...optionalParams: any[]): void {
    Logger.logWithLevel(message, SeverityLevel.warning, null, optionalParams)
  }

  // noinspection TsLint
  static error(message?: any, ...optionalParams: any[]): void {
    Logger.logWithLevel(message, SeverityLevel.error, null, optionalParams)
  }

  // noinspection TsLint
  static todo(message?: any, ...optionalParams: any[]): void {
    Logger.logWithLevel(message, SeverityLevel.debug, null, optionalParams)
  }

  // noinspection TsLint
  /**
   * makes a debug output formatted as '[place] message'
   * @param place the location (file)
   * @param message
   * @param optionalParams
   */
  static debug(place: string, message: any, ...optionalParams: any[]): void {
    Logger.logWithLevel(message, SeverityLevel.debug, place, optionalParams)
  }

  // noinspection TsLint
  private static logWithLevel(message: string, level: SeverityLevel, file: string, ...optionalParams: any[]): void {

    if (file) {
      console.log(`[${file}] ${message}`, optionalParams)
    } else {
      console.log(message, optionalParams)
    }

    //we cannot enable this because if we encountered an error during mapStateToProps
    //this gets called with dispatches
    //causing every (mounted) mapStateToProps to run again...

    Logger.myLog.push({
      id: Logger.nextItemId++,
      message,
      level,
      file,
      params: optionalParams,
      time: moment()
    })

    //TODO find workaround
    // store.dispatch(addLogItem({
    //   id: Logger.nextItemId++,
    //   message,
    //   level,
    //   file,
    //   params: optionalParams,
    //   time: moment()
    // }))

  }

}
