/**
 * Created by janis dähne.
 */
import {SortDirection} from "../types/sortDirection";
import {
  EditableExercisePreviewFromBackend, ExercisePreviewFromBackend,
} from "../types/exercisePreview";
import orderBy from 'lodash-es/orderBy'


//TODO why this does not work when we only use T
//and T is of type keyof T??
//don't forget to set sortable on the table!!
export function isCurrentSortingColumn<T>(sortByKeyProp: keyof T, columnName: keyof T, sortDirection: SortDirection): 'ascending' | 'descending' | null {

  if (sortByKeyProp !== columnName) return null

  if (sortDirection === SortDirection.ascending) return 'ascending'

  if (sortDirection === SortDirection.descending) return 'descending'

  return null
}


/**
 * applies sorting t& filtering to the given exercisePreviews
 *
 * CREATES REFERENCE
 * @param {ReadonlyArray<T extends EditableExercisePreviewFromBackend | ExercisePreviewFromBackend>} exercisePreviews
 * @param {SortDirection} sortDirection
 * @param {string} sortByKey
 * @param {ReadonlyArray<number>} selectedFilterTagsIds the tag ids the exercise must match
 * @param selectedNegativeFilterTagsIds the tag ids the exercise must not match
 * @returns {ReadonlyArray<T extends EditableExercisePreviewFromBackend | ExercisePreviewFromBackend>}
 */
export function sortAndFilterExercisePreviews<T extends EditableExercisePreviewFromBackend | ExercisePreviewFromBackend>
(exercisePreviews: ReadonlyArray<T>,
 sortDirection: SortDirection,
 sortByKey: keyof T,
 selectedFilterTagsIds: ReadonlyArray<number>,
 selectedNegativeFilterTagsIds: ReadonlyArray<number>): ReadonlyArray<T> {

  // noinspection TsLint
  let previewsList: ReadonlyArray<T> =
    sortDirection === SortDirection.none
      ? exercisePreviews
      : orderBy<T>(exercisePreviews, (user: T) => user[sortByKey],
      sortDirection === SortDirection.ascending ? 'asc' : 'desc')

  if (selectedFilterTagsIds.length > 0 || selectedNegativeFilterTagsIds.length > 0) {
    //...uh bad performance...
    //in the preview tag list: look if the tag is found in the filtered tags list

    //only 1 filter tag needs to be in the exercise tags (or filtered)
    // previewsList = previewsList.filter(
    //   p => p.tags.some(tag => selectedFilterTagsIds.some(filterTag => tag.id === filterTag)))

    //all filter tag needs to be in the exercise tags (and filtered)
    previewsList = previewsList.filter(
      p => selectedFilterTagsIds.every(filterTagId =>
        p.tags.some(tag => tag.id === filterTagId)
      ))
    //the selectedNegativeFilterTagsIds must not be in the tags from the exercise
    previewsList = previewsList.filter(
      p => selectedNegativeFilterTagsIds.every(filterTagId =>
        p.tags.every(tag => tag.id !== filterTagId)
      ))


  }

  return previewsList
}