Test Syntax
===

# Black Box Test

Ein Black-Box-Test bezieht sich *immer* auf die als Hauptdatei markierten Datei bei der Aufgabenerstellung. Die erste Spalte (erstes Zeichen) im Editor beinhaltet immer ein spezielles Symbol, welches angibt, ob es sich bei der entsprechenden Zeile um
* Konsolenparameter (Dollarsymbol `$`),
* eine Eingabe (Kleiner-als-Zeichen `<`),
* eine Ausgaben (Größer-als-Zeichen `>`) oder
* einen auf eine Ausgabe bezogenen regulären Ausdruck (`r`)
* Rückgabewert des Programms (`e`) **TODO Ist das implementiert/was ist bei Fehlern?**

handelt. 

### Konsolenparameter
Konsolenparameter werden durch das Dollarymbol (`$`) gekennzeichnet und durch Leerzeichen getrennt in eine Zeile geschrieben. Wenn mehrere durch ein Leerzeichen getrennte Zeichenketten als genau ein Konsolenparameter behandelt werden sollen, müssen Sie in doppelte Anführungsstriche geschrieben werden.

*Beispiel:* `$ 21 ist die "halbe Wahrheit"` *würde also die vier Konsolenparameter `21`, `ist`, `die` und `halbe Wahrheit` liefern.*

**Konsolenparameter müssen vor jeglichen Ein- bzw. Ausgabe angegeben werden.**


### Eingabe
Eine Eingabe beginnt immer mit dem Kleiner-als-Zeichen (`<`). Die Eingabe selbst beginnt direkt danach (eventuelle Leerzeichen nach dem `<` Symbol werden also als Eingabe gewertet) und erstreckt sich über die gesamte Zeile. Hier muss auf möglicherweise ungewollte Leerzeichen am Ende der Zeile Acht gegeben werden.

*Beispiel:* `<Hallo Welt` *würde die Zeichenkette `Hallo Welt` als Eingabe zu dem Programm weiterleiten.*

### Ausgabe

Eine Ausgabe beginnt immer mit dem Größer-als-Zeichen (`>`). Die im Test erwartete Ausgabe beginnt direkt nach dem `>` Symbol. Hier muss insbesondere auf Leerzeichen geachtet werden.

*Beispiel:* <code>> 42 </code> *würde bedeuten, dass der Test erwartet, dass das Programm <code> 42 </code>  (und nicht <code>42</code>) ausgibt (also Leerzeichen, Ziffer 4, Ziffer 2, Leerzeichen). Würde bei der tatsächlichen Ausgabe das Leerzeichen am Anfang oder Ende fehlen, würde auch der Test fehlschlagen.*

Ist die spezielle Ausgabe egal, kann mittels `>*` jede Ausgabe akzeptiert werden 
(eine Ausgabe muss jedoch erfolgen, d.h. mindestens der leere String muss ausgegeben werden).

### Ausgabe mit regulären Ausdruck
Soll eine Ausgabe mit einem regulären Ausdruck überprüft werden, so muss die Zeile mit einem `r` beginnen. Direkt danach wird der reguläre Ausdruck hingeschrieben. Der reguläre Ausdruck wird automatisch so angewandt, dass die gesamte Ausgabe einer Zeile dem regulären Ausdruck genügen muss!

*Beispiel:* `r18\.20433[0-9]*` *würde bedeuten, dass alle Ausgaben, die mit `18.20433`beginnen gefolgt von einer beliebigen Folge von Ziffern akzeptiert werden. Ausgaben wie `8.20433` oder `18.20433x` würden somit abgelehnt werden.*



### Rückgabewert des Programms

Die erste Zeile, welche mit **e** beginnt, wird als erwarteten Exit-Code vom zu testenden Programm erwartet (die folgende Zahl nach **e**[Zahl], z.B. e5). Wenn der falsche Exit-Code zurückgegeben wird, gilt der Test als **nicht** bestanden!  
**TODO Ist das implementiert/was ist bei Fehlern?**

## Regex-Test

Die Syntax für Regex-Tests hat eine fest vorgegebene Form:
```
mode:gewählter-Modus
connector:gewählte-Verknüpfung
Datei1:regulärer-Ausdruck
Datei2: regulärer-Ausdruck
...
Datei3:regulärer-Ausdruck
```
Jeder Zeile enthält ein Paar von Werten durch `:` getrennt.
Leerzeilen werden nicht beachtet und alle Leerzeichen am Zeilenanfang, Zeilenende und nach bzw. vor dem `:` werden ignoriert.

Bei dem Modus wird zwischen *whitelist* und *blacklist* unterschieden (Default: *blacklist*, falls nicht angegeben). Der connector kann `&&` oder `||` sein und ist nur wirksam, wenn mehrere Dateien angegeben werden. Anschließend kann für eine oder mehrere Dateien ein regulärer Ausdruck vorgegeben werden (Default: `&&`, falls nicht angegeben). Die Dateien müssen sich dabei unterscheiden, die regulären Ausdrücke aber nicht.

Für `regulärer-Ausdruck` muss ein entsprechend regulärer Ausdruck eingesetzt werden. Hier ist zu beachten, dass der reguläre Ausdruck immer mit `/` beginnt und endet.

Hinter dem abschließenden `/` eines Regex können die Flags `i` und `m` verwendet werden. Z.B. `/abc/m`, `/abc/i`, `/abc/im`, `/abc/mi`

Eine Datei genügt dem regulären Ausdruck, wenn sie eine entsprechende Zeichenkette aufweist. Mit Metazeichen wie `^` oder `$`kann angegeben werden, dass die Zeichenkette am Anfang oder Ende der Datei steht.

Außerdem darf der `regulärer-Ausdruck` für eine Datei nicht die Zeichen `&&` (logishc und) und/oder `||` (logisch oder) enhthalten,
da diese zum Verbinden mehrerer `regulärer-Ausdrücke` reserviert sind.
`\&\&` bzw. `\|\|` sind erlaubt.

Beispiel: `/42[0-9]*/ || /40[0-9]*/` oder 
`/if \(x \&\& y\)/ && /if \(y \&\& x\)/`

Es kann als Dateinamen `*` angegeben werden (nur 1 Mal erlaubt im Test). Dieser Ausdruck wird dann auf alle Dateien angewendet und mittels des `connector`s zu den einzelnen Ergebnissen der Ausdrucke für die jeweilige Datei zusammengeführt.


### Beispiel für whitelist
Wenn als Modus *whitelist* gewählt wurde, müssen bei `&&` als connector alle angegebenen oder bei `||` als connector mindestens einer der regulären Ausdrücke vorkommen.
```
mode:whitelist
connector:&&
Datei1.java:/42.*(Antwort|Wahrheit)+/
Datei2.java:/4\.2.*nicht\s*42/
```
Für *Datei1.java* würde verlangt werden, dass `42` und das Wort `Antwort` oder `Wahrheit` mindestens einmal vorkommt. Dazwischen können beliebige Zeichen stehen.  Bei der *Datei2.java* muss `4.2`, gefolgt von beliebig vielen Zeichen, dem Wort `nicht`, beliebig vielen Leerraumzeichen und `42` als Zeichenkette vorkommen. Nur wenn (wegen dem `&&` als connector) beide reguläre Ausdrücke erfüllt sind, ist der Test erfolgreich.

### Beispiel für blacklist
Wenn als Modus *blacklist* gewählt wurde, darf bei `&&` als connector keiner der angegebenen regulären Ausdrücke zutreffen. Bei `||` als connector muss mindestens einer der regulären Ausdrücke nicht zutreffen.
```
mode:blacklist
connector:&&
Datei1.java:/$[aA]nfang/
Datei2.java:/^[eE]nde/
```

Diese Test ist nur erfolgreich, wenn bei *Datei1.java* am Ende weder `Anfang` noch `anfang` steht und bei *Datei2.java* am Anfang weder `Ende` noch `ende` steht.


Um den regulären Ausdruck zu testen, empfiehlt sich [https://regex101.com](https://regex101.com) oder auch [https://www.regextester.com](https://www.regextester.com), da man hier genauere Informationen darüber bekommt, warum ein reguläre Ausdruck nicht funktioniert. Z.B. funktioniert 
`/(1\.0|1\.34(\s|.)*1\.34)/` nicht, falls der Quelltext mehr als 20 Zeichen lang ist (und der Ausdruck gefunden werden kann). Unter [https://regex101.com](https://regex101.com) wird angezeigt, dass es sich hierbei um einen *schlechten* Ausdruck handelt, da viel Rekursion nötig ist (die benötigten Schritte werden ebenfalls angezeigt).


## Syntaxtest

Syntaxtests benötigen keine Eingabe.