/**
 * Created by janis dähne.
 */
import {
  ResetAction, SET_isEnterReleaseCodeDialogDisplayedAction, SET_isTagsFilterDisplayedAction, SET_openExercisesAction,
  SET_sortByKeyAction,
  SET_viewStateAction
} from "../../../reducers/allAccessibleExercises/openExercisesSite/openExercisesSiteReducer";
import {delay} from "../../../../../debug";
import {
  getOpenExercisePreviews
} from "../../../../communicationLayer/exercisesPreviewLayer";
import {SiteViewState} from "../../../../types/siteView";
import {GET_openExercisePreviewsAction} from "../../../reducers/allAccessibleExercises/openExercisesSite/getAllOpenExercisesReducer";
import {ActionType} from "../../../reducers/allAccessibleExercises/openExercisesSite/openExercisesSiteActionTypes";
import {SortDirection} from "../../../../types/sortDirection";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {ExercisePreviewsFrontendExclusive} from "../../../../components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {AwaitActions} from "../../types";
import {loadManageTagsSite, resetManageTagsSite} from "../../manageTagsSite/manageTagsActions";
import {resetTagsFilter} from "../../tagsFilterActions";


export function setOpenExercises(openExercises: ReadonlyArray<ExercisePreviewsFrontendExclusive>): SET_openExercisesAction {
  return {
    type: ActionType.SET_openExercises,
    openExercises
  }
}

export function loadOpenExercisePreviewsAsync(): AwaitActions {
  return async(dispatch )=> {

    dispatch(resetOpenExercisesSite())

    dispatch(resetTagsFilter())
    await dispatch(loadManageTagsSite())

    await dispatch(getOpenExercisePreviewsAsync())
  }
}

export function getOpenExercisePreviewsAsync(): GET_openExercisePreviewsAction {
  return {
    type: ActionType.GET_openExercisePreviews,
    payload: delay(getOpenExercisePreviews())
  }
}

export function setOpenExercisesSiteViewState(viewState: SiteViewState): SET_viewStateAction {
  return {
    type: ActionType.SET_viewState,
    viewState
  }
}

export function setIsEnterReleaseCodeDialogDisplayed(isDisplayed: boolean): SET_isEnterReleaseCodeDialogDisplayedAction {
  return {
    type: ActionType.SET_isEnterReleaseCodeDialogDisplayed,
    isDisplayed
  }
}

export function setSortByKey(sortByKey: keyof ExercisePreviewFromBackend, sortDirection: SortDirection): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setIsTagsFilterDisplayed(isTagsFilterDisplayed: boolean): SET_isTagsFilterDisplayedAction {
  return {
    type: ActionType.SET_isTagsFilterDisplayed,
    isTagsFilterDisplayed,
  }
}

export function resetOpenExercisesSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}