/**
 * Created by janis dähne.
 */
import {changeTabVisibility, HasFileChangedMap} from "../../../../helpers/editorHelper";
import {
  SET_hasFileChangedMapAction,
  SET_hasUnsavedChangesAction,
  SET_isFileExplorerDisplayedCustomProjectSolutionEditorAction,
  SET_isRenamingFileCustomProjectSolutionEditorAction,
  SET_newFileNameCustomProjectSolutionEditorAction,
  SET_renamingFileIdCustomProjectSolutionEditorAction,
  SET_renamingFileNameCustomProjectSolutionEditorAction,
  SET_selectedTabIdCustomProjectSolutionEditorAction
} from "../../../reducers/editCustomProjectSite/editorReducers/customProjectSolutionEditorReducer";
import {ActionType} from "../../../reducers/editCustomProjectSite/editCustomProjectSiteActionTypes";
import {CustomProjectSolutionFileFullBase} from "../../../../types/customProjects";
import {MultiActions} from "../../types";
import {setSolutionFilesCustomProject} from "../editCustomProjectActions";


export function setHasUnsavedChanges(hasUnsavedChanges: boolean): SET_hasUnsavedChangesAction {
  return {
    type: ActionType.SET_hasUnsavedChanges,
    hasUnsavedChanges
  }
}

/**
 * sets the file changed map to determine what files changed
 *
 *
 * pass {} to indicate that no file has changed
 * @param {HasFileChangedMap} hasFileChangedMap
 * @returns {SET_hasFileChangedMapAction}
 */
export function setHasFileChangedMapCustomProjectSolutionEditor(hasFileChangedMap: HasFileChangedMap): SET_hasFileChangedMapAction {
  return {
    type: ActionType.SET_hasFileChangedMap,
    hasFileChangedMap
  }
}


/**
 * changes the visibility of a tab
 * @param {SolutionFileDoExerciseFullBase} file the file (won't be used to change something)
 * @param {ReadonlyArray<SolutionFileDoExerciseFullBase>} templateFiles
 * @param {number} selectedFileId needed because the user can close another file than the existing one
 * @returns {MultiActions}
 */
export function openTabChangeCustomProjectSolutionEditor(file: CustomProjectSolutionFileFullBase,
                                                         templateFiles: ReadonlyArray<CustomProjectSolutionFileFullBase>,
                                                         selectedFileId: number): MultiActions {
  return changeTabVisibility(file, true, templateFiles, selectedFileId, setSolutionFilesCustomProject,
    setSelectedTabIdCustomProjectSolutionEditor)
}

export function closeTabChangeCustomProjectSolutionEditor(file: CustomProjectSolutionFileFullBase,
                                                          templateFiles: ReadonlyArray<CustomProjectSolutionFileFullBase>, //SolutionFileDoExerciseFullBase
                                                          selectedFileId: number): MultiActions {
  return changeTabVisibility(file, false, templateFiles, selectedFileId, setSolutionFilesCustomProject,
    setSelectedTabIdCustomProjectSolutionEditor)
}


export function setSelectedTabIdCustomProjectSolutionEditor(selectedTabIdCustomProjectSolutionEditor: number): SET_selectedTabIdCustomProjectSolutionEditorAction {
  return {
    type: ActionType.SET_selectedTabIdCustomProjectSolutionEditor,
    selectedTabIdCustomProjectSolutionEditor
  }
}

export function setIsFileExplorerDisplayedCustomProjectSolutionEditor(isFileExplorerDisplayedCustomProjectSolutionEditor: boolean): SET_isFileExplorerDisplayedCustomProjectSolutionEditorAction {
  return {
    type: ActionType.SET_isFileExplorerDisplayedCustomProjectSolutionEditor,
    isFileExplorerDisplayedCustomProjectSolutionEditor
  }
}

export function setNewFileNameCustomProjectSolutionEditor(newFileNameCustomProjectSolutionEditor: string): SET_newFileNameCustomProjectSolutionEditorAction {
  return {
    type: ActionType.SET_newFileNameCustomProjectSolutionEditor,
    newFileNameCustomProjectSolutionEditor
  }
}

export function setRenamingFileIdCustomProjectSolutionEditor(renamingFileIdCustomProjectSolutionEditor: number): SET_renamingFileIdCustomProjectSolutionEditorAction {
  return {
    type: ActionType.SET_renamingFileIdCustomProjectSolutionEditor,
    renamingFileIdCustomProjectSolutionEditor
  }
}

export function setIsRenamingFileCustomProjectSolutionEditor(isRenamingFileCustomProjectSolutionEditor: boolean): SET_isRenamingFileCustomProjectSolutionEditorAction {
  return {
    type: ActionType.SET_isRenamingFileCustomProjectSolutionEditor,
    isRenamingFileCustomProjectSolutionEditor
  }
}

export function setRenamingFileNameCustomProjectSolutionEditor(renamingFileNameCustomProjectSolutionEditor: string): SET_renamingFileNameCustomProjectSolutionEditorAction {
  return {
    type: ActionType.SET_renamingFileNameCustomProjectSolutionEditor,
    renamingFileNameCustomProjectSolutionEditor
  }
}