/**
 * Created by janis dähne.
 */
import {GET_editorExerciseAction} from "../../reducers/exerciseEditorSite/getEditorExerciseReducer";
import {ActionType} from "../../reducers/exerciseEditorSite/editorExerciseSiteActionTypes";
import {delay} from "../../../../debug";
import {
  createEditorExercise, getEditorExercise,
  updateEditorExercise
} from "../../../communicationLayer/exerciseEditorLayer";
import {CREATE_editorExerciseAction} from "../../reducers/exerciseEditorSite/createEditorExerciseReducer";
import {EditorExerciseExerciseForBackend} from "../../../types/exerciseEditor";
import {UPDATE_editorExerciseAction} from "../../reducers/exerciseEditorSite/updateEditorExerciseReducer";
import {AwaitActions, AwaitActionsWithResult} from "../types";
import {createAndRefreshAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import Logger from "../../../helpers/logger";


export function getEditorExerciseAsync(exerciseId: number, loadTestAssetContents: boolean): AwaitActions {
  return async(dispatch, getState) => {

    try {
      await dispatch(_getEditorExerciseAsync(exerciseId, loadTestAssetContents))
    } catch(err) {
      return
    }

    //create asset urls
    createAndRefreshAssetUrlsAndObjects(getState().exerciseEditorState.editorExercise.exerciseDescription.assets)
  }
}


function _getEditorExerciseAsync(exerciseId: number, loadTestAssetContents: boolean): GET_editorExerciseAction {
  return {
    type: ActionType.GET_editorExercise,
    payload: delay(getEditorExercise(exerciseId, loadTestAssetContents))
  }
}

export function createEditorExerciseAsync(exercise: EditorExerciseExerciseForBackend): AwaitActionsWithResult<boolean> {
  return async(dispatch, getState) => {

    try {
      await dispatch(_createEditorExerciseAsync(exercise))
    } catch(err) {
      Logger.error(`could not create new exercise, err: ${err.message}`)
      return false
    }

    return true
  }
}

export function _createEditorExerciseAsync(exercise: EditorExerciseExerciseForBackend): CREATE_editorExerciseAction {
  return {
    type: ActionType.CREATE_editorExercise,
    payload: delay(createEditorExercise(exercise))
  }
}

export function updateEditorExerciseAsync(exercise: EditorExerciseExerciseForBackend): AwaitActionsWithResult<boolean> {
  return async(dispatch, getState) => {

    try {
      await dispatch(_updateEditorExerciseAsync(exercise))
    } catch(err) {
      Logger.error(`could not update exercise, err: ${err.message}`)
      return false
    }

    return true
  }
}

export function _updateEditorExerciseAsync(exercise: EditorExerciseExerciseForBackend): UPDATE_editorExerciseAction {
  return {
    type: ActionType.UPDATE_editorExercise,
    payload: delay(updateEditorExercise(exercise))
  }
}
