/**
 * Created by janis dähne.
 */
import {ActionType} from "../../reducers/exerciseEditorSite/editorExerciseSiteActionTypes";
import {
  ResetAction, SET_botLeftPanelViewSizeAction, SET_botRightPanelViewSizeAction, SET_bottomLeftSelectedTabIndexAction,
  SET_bottomRightSelectedTabIndexAction, SET_editorExerciseAction,
  SET_hDividerPosOffsetAction,
  SET_isChangeCodeTemplateDialogDisplayedAction, SET_isChangeEditorSettingsDialogDisplayedAction,
  SET_isChangeSelectedTagsDialogDisplayedAction,
  SET_isChangeTestDialogDisplayedAction, SET_topLeftPanelViewSizeAction, SET_topLeftSelectedTabIndexAction,
  SET_topRightPanelViewSizeAction,
  SET_topRightSelectedTabIndexAction,
  SET_vDividerPosOffsetAction
} from "../../reducers/exerciseEditorSite/exerciseEditorSiteReducer";
import {AwaitActions, MultiActions} from "../types";
import {getTestTypesAsync} from "../testTypesActions";
import {getEditorExerciseAsync} from "./exerciseEditorCrudActions";
import {getTagsAsync} from "../manageTagsSite/manageTagsSiteCrudActions";
import {getGroupsAsync} from "../groupsSite/groupsCrudActions";
import {getPLangsAsync} from "../pLangsActions";
import {EditorExerciseExerciseForBackend, ExerciseAssetFullBase} from "../../../types/exerciseEditor";
import {createAndRefreshAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {getNextFrontendId} from "../../../helpers/frontendIdHelper";
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";


/**
 *
 * @param {number} exerciseId the exercise id or null (if we want to create a new exercise)
 * @param {boolean} resetIds true: resets the ids of the exercise (useful when we want to copy an exercise), because backend treats all ids <= 0 as new entities
 * false: not
 * @param loadTestAssetContents
 * @returns {AwaitActions}
 */
export function loadExerciseEditor(exerciseId: number | null, resetIds: boolean, loadTestAssetContents: boolean): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetExerciseEditor())

    await dispatch(getTestTypesAsync())
    await dispatch(getTagsAsync())
    await dispatch(getGroupsAsync())
    await dispatch(getPLangsAsync())

    if (exerciseId !== null) {
      await dispatch(getEditorExerciseAsync(exerciseId, loadTestAssetContents))

      if (resetIds) {

        //just setting the id to -x of the exercise is enough all child entities will be created because of the backend...
        //if fact we have 2 separate methods for this and the frontend calls save/create based on the exercise id > 0 / <= 0
        const copy: EditorExerciseExerciseForBackend = {
          ...getState().exerciseEditorState.editorExercise,
          id: getNextFrontendId(),
          exerciseProperties: {
            ...getState().exerciseEditorState.editorExercise.exerciseProperties,
            displayName: `${getState().exerciseEditorState.editorExercise.exerciseProperties.displayName} - ${translate(TranslateIds.copy)}`
          }
        }

        dispatch(setEditorExercise(copy))
      }

    }
  }
}


/**
 * this is e.g. used when importing an exercise from ja json file
 * @param {EditorExerciseExerciseForBackend} editorExercise
 * @returns {SET_editorExerciseAction}
 */
function _setEditorExercise(editorExercise: EditorExerciseExerciseForBackend): SET_editorExerciseAction {
  return {
    type: ActionType.SET_editorExercise,
    editorExercise
  }
}

//--- panel indices

export function setTopRightSelectedTabIndex(index: number): SET_topRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_topRightSelectedTabIndex,
    topRightSelectedTabIndex: index
  }
}

export function setTopLeftSelectedTabIndex(index: number): SET_topLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_topLeftSelectedTabIndex,
    topLeftSelectedTabIndex: index
  }
}

export function setBottomRightSelectedTabIndex(index: number): SET_bottomRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomRightSelectedTabIndex,
    bottomRightSelectedTabIndex: index
  }
}

export function setBottomLeftSelectedTabIndex(index: number): SET_bottomLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomLeftSelectedTabIndex,
    bottomLeftSelectedTabIndex: index
  }
}

//--- dialog actions

export function setIsChangeTestDialogDisplayed(isDisplayed: boolean): SET_isChangeTestDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeTestDialogDisplayed,
    isDisplayed
  }
}

export function setIsChangeSelectedTagsDialogDisplayed(isDisplayed: boolean): SET_isChangeSelectedTagsDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeSelectedTagsDialogDisplayed,
    isDisplayed
  }
}


export function setIsChangeCodeTemplateDialogDisplayed(isDisplayed: boolean): SET_isChangeCodeTemplateDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeCodeTemplateDialogDisplayed,
    isDisplayed
  }
}

export function setIsChangeExerciseEditorSettingsDialogDisplayed(isDisplayed: boolean): SET_isChangeEditorSettingsDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeEditorSettingsDialogDisplayed,
    isDisplayed
  }
}

//--- panels grippers
export function setExerciseEditorHDividerPosOffset(hDividerPosOffset: number): SET_hDividerPosOffsetAction {
  return {
    type: ActionType.SET_hDividerPosOffset,
    hDividerPosOffset
  }
}

export function setExerciseEditorVDividerPosOffset(vDividerPosOffset: number): SET_vDividerPosOffsetAction {
  return {
    type: ActionType.SET_vDividerPosOffset,
    vDividerPosOffset
  }
}


//--- panel view sizes

export function setTopRightPanelViewSize(topRightPanelViewSize: PanelViewSize): SET_topRightPanelViewSizeAction {
  return {
    type: ActionType.SET_topRightPanelViewSize,
    topRightPanelViewSize
  }
}

export function setTopLeftPanelViewSize(topLeftPanelViewSize: PanelViewSize): SET_topLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_topLeftPanelViewSize,
    topLeftPanelViewSize
  }
}

export function setBotRightPanelViewSize(botRightPanelViewSize: PanelViewSize): SET_botRightPanelViewSizeAction {
  return {
    type: ActionType.SET_botRightPanelViewSize,
    botRightPanelViewSize
  }
}

export function setBotLeftPanelViewSize(botLeftPanelViewSize: PanelViewSize): SET_botLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_botLeftPanelViewSize,
    botLeftPanelViewSize
  }
}


export function setEditorExercise(editorExercise: EditorExerciseExerciseForBackend): MultiActions {
  return dispatch => {

    dispatch(_setEditorExercise(editorExercise))

    //then create asset urls
    createAndRefreshAssetUrlsAndObjects(editorExercise.exerciseDescription.assets)

  }
}



export function resetExerciseEditor(): ResetAction {
  return {
    type: ActionType.RESET,
  }
}