/**
 * Created by janis dähne.
 */

import {UserGroupFullBase} from "../../../types/group";
import {SiteViewState} from "../../../types/siteView";
import {
  ResetAction,
  SET_viewStateAction,
  SET_isChangeDialogDisplayedAction, SET_groupsAction, SET_sortByKeyAction
} from "../../reducers/groupsSite/groupsSiteReducer";
import {ActionType} from '../../reducers/groupsSite/groupsSiteActionTypes'
import {ThunkAction} from "redux-thunk";
import {resetChangeGroupDialog, setIsCreating} from "./dialog/changeGroupDialogActions";
import {setTargetGroup} from "./dialog/groupSetActions";
import {getGroupsAsync, quitGroupAsync} from "./groupsCrudActions";
import {AwaitActions} from "../types";
import {SortDirection} from "../../../types/sortDirection";


export function setGroups(groups: ReadonlyArray<UserGroupFullBase>): SET_groupsAction {
  return {
    type: ActionType.SET_groups,
    groups
  }
}

export function setSortByKey(sortByKey: keyof UserGroupFullBase, sortDirection: SortDirection): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}


export function openCreateGroupDialog(): ThunkAction<void, void, void> {
  return dispatch => {

    dispatch(resetChangeGroupDialog())
    dispatch(setIsCreating(true))
    dispatch(setIsChangeDialogDisplayed(true))
  }
}

export function openEditGroupDialog(group: UserGroupFullBase): ThunkAction<void, void, void> {
  return dispatch => {

    dispatch(resetChangeGroupDialog())
    dispatch(setIsCreating(false))
    dispatch(setTargetGroup(group))
    dispatch(setIsChangeDialogDisplayed(true))

  }
}

export function cancelEditGroup(): ThunkAction<void, void, void> {
  return dispatch => {

    dispatch(setIsChangeDialogDisplayed(false))
    dispatch(resetChangeGroupDialog())

  }
}

export function quitGroup(group: UserGroupFullBase): AwaitActions {
  return async (dispatch) => {

    await dispatch(quitGroupAsync(group))
    dispatch(getGroupsAsync())

  }
}

export function setIsChangeDialogDisplayed(isDisplayed: boolean): SET_isChangeDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeDialogDisplayed,
    isDisplayed
  }
}

export function setViewState(viewState: SiteViewState): SET_viewStateAction {
  return {
    type: ActionType.SET_viewState,
    viewState
  }
}


export function loadGroupsSite(): AwaitActions {
  return async(dispatch) => {

    dispatch(resetGroupsSite())
    await dispatch(getGroupsAsync())
  }
}

export function resetGroupsSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}