/**
 * Created by janis dähne.
 */
import {ActionType} from "../../reducers/manageCustomProjectsSite/manageCustomProjectsSiteActionTypes";
import {delay} from "../../../../debug";
import {
  deleteOwnCustomProject,
  getOwnCustomProjectOverviews
} from "../../../communicationLayer/customProjectsOverviewLayer";
import {AwaitActions, MultiActions} from "../types";
import {
  ResetAction,
  SET_sortByKeyAction
} from "../../reducers/manageCustomProjectsSite/manageCustomProjectsSiteReducer";
import {getSystemSettingsAsync} from "../systemSettingsActions";
import {SortDirection} from "../../../types/sortDirection";
import {CustomProjectPreviewFullBase} from "../../../types/customProjects";
import {DELETE_customProjectAction} from "../../reducers/manageCustomProjectsSite/deleteCustomProjectReducer";
import {errorDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";
import Logger from "../../../helpers/logger";
import {GET_customProjectOverviewsAction} from "../../reducers/manageCustomProjectsSite/getCustomProjectOverviewsReducer";


export function setSortByKey(sortByKey: keyof CustomProjectPreviewFullBase, sortDirection: SortDirection): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function getOwnCustomProjectsAsync(): GET_customProjectOverviewsAction {
  return {
    type: ActionType.GET_customProjectOverviews,
    payload: delay(getOwnCustomProjectOverviews())
  }
}

export function deleteOwnCustomProjectAsync(customProjectPreview: CustomProjectPreviewFullBase): AwaitActions {
  return async (dispatch, getState) => {

    try {
      await dispatch(_deleteOwnCustomProjectAsync(customProjectPreview))
    } catch (err) {
        errorDialog(translate(TranslateIds.serverError), translate(TranslateIds.couldNotDeleteCustomProject))
      Logger.error(`could not delete custom project with id: ${customProjectPreview.id}, err: ${err.message}`)
    }

    //refresh always
    dispatch(loadManageCustomProjectSite())

  }
}

export function _deleteOwnCustomProjectAsync(customProjectPreview: CustomProjectPreviewFullBase): DELETE_customProjectAction {
  return {
    type: ActionType.DELETE_customProject,
    payload: delay(deleteOwnCustomProject(customProjectPreview))
  }
}



export function loadManageCustomProjectSite(): AwaitActions {
  return async (dispatch) => {

    dispatch(resetManageCustomProjectsSite())

    await dispatch(getSystemSettingsAsync())

    await dispatch(getOwnCustomProjectsAsync())

  }
}

export function resetManageCustomProjectsSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}