/**
 * Created by janis dähne.
 */
import {
  ResetAction,
  SET_groupExercisesAction, SET_isTagsFilterDisplayedAction, SET_sortByKeyAction, SET_viewStateAction
} from "../../reducers/manageGroupExercisesSite/groupExercisesSiteReducer";
import {EditableExercisePreviewFromBackend, GroupPreviewExerciseTupleFromBackend} from "../../../types/exercisePreview";
import {SiteViewState} from "../../../types/siteView";
import {deleteExercise, getAccessibleExercisePreviewsByGroup} from "../../../communicationLayer/exercisesPreviewLayer";
import {delay} from "../../../../debug";
import {SortDirection} from "../../../types/sortDirection";
import {ActionType} from "../../reducers/manageGroupExercisesSite/groupExercisesSiteActionTypes";
import {GET_groupExercisesAction} from "../../reducers/manageGroupExercisesSite/getGroupExercisesReducer";
import {AwaitActions} from "../types";
import {DELETE_groupExerciseAction} from "../../reducers/manageGroupExercisesSite/deleteGroupExerciseReducer";
import {loadManageTagsSite} from "../manageTagsSite/manageTagsActions";
import {resetTagsFilter} from "../tagsFilterActions";


export function setGroupExercises(groupExercises: ReadonlyArray<GroupPreviewExerciseTupleFromBackend>): SET_groupExercisesAction {
  return {
    type: ActionType.SET_groupExercises,
    exercises: groupExercises
  }
}

export function getGroupExercisesAsync(): GET_groupExercisesAction {
  return {
    type: ActionType.GET_groupExercises,
    payload: delay(getAccessibleExercisePreviewsByGroup())
  }
}


export function deleteGroupExercisesAsync(exerciseId: number): AwaitActions {
  return async (dispatch, getState) => {

    await dispatch(_deleteGroupExercisesAsync(exerciseId))

    dispatch(loadManageGroupExercisesSite())

  }
}

export function _deleteGroupExercisesAsync(exerciseId: number): DELETE_groupExerciseAction {
  return {
    type: ActionType.DELETE_groupExercise,
    payload: delay(deleteExercise(exerciseId))
  }
}


export function setViewState(viewState: SiteViewState): SET_viewStateAction {
  return {
    type: ActionType.SET_viewState,
    viewState
  }
}

export function setSortByKey(sortByKey: keyof EditableExercisePreviewFromBackend, sortDirection: SortDirection): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setIsTagsFilterDisplayed(isTagsFilterDisplayed: boolean): SET_isTagsFilterDisplayedAction {
  return {
    type: ActionType.SET_isTagsFilterDisplayed,
    isTagsFilterDisplayed,
  }
}

export function loadManageGroupExercisesSite(): AwaitActions {
  return async (dispatch) => {

    dispatch(resetManageGroupExercisesSite())

    dispatch(resetTagsFilter())
    await dispatch(loadManageTagsSite())

    await dispatch(getGroupExercisesAsync())
  }
}

export function resetManageGroupExercisesSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}