/**
 * Created by janis dähne.
 */
import {GET_newUsersAction} from "../../reducers/manageNewUsersSite/getNewUsersReducers";
import {ActionType} from "../../reducers/manageNewUsersSite/manageNewUsersSiteActionTypes";
import {delay} from "../../../../debug";
import {activateUser, deleteNotActive, getNotActivatedUsers} from "../../../communicationLayer/usersLayer";
import {UserPreviewFromBackend, UserSystemRoleTupleForBackend} from "../../../types/users";
import {AwaitActions} from "../types";
import {DELETE_newUserAction} from "../../reducers/manageNewUsersSite/deleteNewUserReducer";
import {ACTIVATE_newUserAction} from "../../reducers/manageNewUsersSite/activateNewUserReducer";
import {GET_systemRolesAction} from "../../reducers/manageNewUsersSite/getSystemRolesReducer";
import {getSystemRoles} from "../../../communicationLayer/systemRolesLayer";

export function deleteMultipleNewUserAsync(newUsers: ReadonlyArray<UserPreviewFromBackend>): AwaitActions {
  return async (dispatch) => {

    for(const tuple of newUsers) {
      await dispatch(_deleteNewUserAsync(tuple, false))
    }

    dispatch(getNewUsersAsync())
  }
}

export function deleteNewUserAsync(newUser: UserPreviewFromBackend): AwaitActions {
  return async (dispatch) => {
    await dispatch(_deleteNewUserAsync(newUser, false))
    dispatch(getNewUsersAsync())
  }
}

function _deleteNewUserAsync(newUser: UserPreviewFromBackend, shouldHideLoading: boolean): DELETE_newUserAction {
  return {
    type: ActionType.DELETE_newUser,
    payload: delay(deleteNotActive(newUser)),
    meta: {
      shouldHideLoading
    }
  }
}

export function activateMultipleNewUserAsync(newUserWithSystemRole: ReadonlyArray<UserSystemRoleTupleForBackend>): AwaitActions {
  return async (dispatch) => {

    for(const tuple of newUserWithSystemRole) {
      await dispatch(_activateNewUserAsync(tuple, false))
    }

    dispatch(getNewUsersAsync())
  }
}

export function activateNewUserAsync(newUserWithSystemRole: UserSystemRoleTupleForBackend): AwaitActions {
  return async (dispatch) => {
    await dispatch(_activateNewUserAsync(newUserWithSystemRole, false))
    dispatch(getNewUsersAsync())
  }
}

function _activateNewUserAsync(newUserWithSystemRole: UserSystemRoleTupleForBackend, shouldHideLoading: boolean): ACTIVATE_newUserAction {
  return {
    type: ActionType.ACTIVATE_newUser,
    payload: delay(activateUser(newUserWithSystemRole)),
    meta: {
      shouldHideLoading
    }
  }
}

/**
 * gets the new users and hides the loader
 * @returns {GET_newUsersAction}
 */
export function getNewUsersAsync(): GET_newUsersAction {
  return {
    type: ActionType.GET_newUsers,
    payload: delay(getNotActivatedUsers())
  }
}


export function getSystemRolesAsync(shouldHideLoading: boolean): GET_systemRolesAction {
  return {
    type: ActionType.GET_systemRoles,
    payload: delay(getSystemRoles()),
    meta: {
      shouldHideLoading: shouldHideLoading
    }
  }
}



