/**
 * Created by janis dähne.
 */
import {
  ResetAction, SET_isTagsFilterDisplayedAction, SET_sortByKeyAction, SET_viewStateAction
} from "../../reducers/manageOwnExercisesSite/ownExercisesSiteReducer";
import {SiteViewState} from "../../../types/siteView";
import {deleteExercise, getOwnExercisePreviews} from "../../../communicationLayer/exercisesPreviewLayer";
import {delay} from "../../../../debug";
import {GET_ownExercisesAction} from "../../reducers/manageOwnExercisesSite/getOwnExercisesReducer";
import {ActionType} from "../../reducers/manageOwnExercisesSite/ownExercisesSiteActionTypes";
import {EditableExercisePreviewFromBackend} from "../../../types/exercisePreview";
import {SortDirection} from "../../../types/sortDirection";
import {AwaitActions} from "../types";
import {DELETE_ownExerciseAction} from "../../reducers/manageOwnExercisesSite/deleteOwnExerciseReducer";
import {loadManageTagsSite} from "../manageTagsSite/manageTagsActions";
import {resetTagsFilter} from "../tagsFilterActions";


export function getOwnExercisesAsync(): GET_ownExercisesAction {
  return {
    type: ActionType.GET_ownExercises,
    payload: delay(getOwnExercisePreviews())
  }
}

export function deleteOwnExerciseAsync(exerciseId:number): AwaitActions {
  return async (dispatch, getState) => {

    await dispatch(_deleteOwnExerciseAsync(exerciseId))

    dispatch(loadOwnExercisesSite())
  }
}


export function _deleteOwnExerciseAsync(exerciseId:number): DELETE_ownExerciseAction {
  return {
    type: ActionType.DELETE_ownExercise,
    payload: delay(deleteExercise(exerciseId))
  }
}



//getOwnExercisePreviews
export function setViewState(viewState: SiteViewState): SET_viewStateAction {
  return {
    type: ActionType.SET_viewState,
    viewState
  }
}

export function setSortByKey(sortByKey: keyof EditableExercisePreviewFromBackend, sortDirection: SortDirection): SET_sortByKeyAction {
  return {
    type: ActionType.SET_sortByKey,
    sortByKey,
    sortDirection
  }
}

export function setIsTagsFilterDisplayed(isTagsFilterDisplayed: boolean): SET_isTagsFilterDisplayedAction {
  return {
    type: ActionType.SET_isTagsFilterDisplayed,
    isTagsFilterDisplayed,
  }
}

export function loadOwnExercisesSite(): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetOwnExercisesSite())

    dispatch(resetTagsFilter())

    await dispatch(loadManageTagsSite())

    await dispatch(getOwnExercisesAsync())
  }
}

export function resetOwnExercisesSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}