/**
 * Created by janis dähne.
 */
import {changeTabVisibility, HasFileChangedMap} from "../../../../helpers/editorHelper";
import {MultiActions} from "../../types";
import {
  SET_tutorEditor_hasFileChangedMapAction,
  SET_tutorEditor_hasUnsavedChangesAction, SET_tutorEditor_isFileExplorerDisplayedDoEditorAction,
  SET_tutorEditor_isRenamingFileDoEditorAction,
  SET_tutorEditor_newFileNameDoEditorAction, SET_tutorEditor_renamingFileIdDoEditorAction,
  SET_tutorEditor_renamingFileNameDoEditorAction,
  SET_tutorEditor_selectedTabIdDoEditorAction
} from "../../../reducers/tutorViewSite/editorReducer/tutorEditorReducer";
import {SolutionFileDoExerciseFullBase} from "../../../../types/exerciseSolution";
import {ActionType} from "../../../reducers/tutorViewSite/tutorViewSiteActionTypes";
import {setSolutionFilesTutorEditor} from "../tutorViewSiteActions";




/**
 * the file
 * @param {SolutionFileDoExerciseFullBase} file the file (won't be used to change something)
 * @param {ReadonlyArray<SolutionFileDoExerciseFullBase>} templateFiles
 * @param {number} selectedFileId
 * @returns {MultiActions}
 */
export function openTabChangeTutorEditor(file: SolutionFileDoExerciseFullBase,
                                         templateFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>,
                                         selectedFileId: number): MultiActions {
  return changeTabVisibility(file, true, templateFiles, selectedFileId, setSolutionFilesTutorEditor,
    setSelectedTabIdTutorEditor)
}

export function closeTabChangeTutorEditor(file: SolutionFileDoExerciseFullBase,
                                          templateFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>,
                                          selectedFileId: number): MultiActions {
  return changeTabVisibility(file, false, templateFiles, selectedFileId, setSolutionFilesTutorEditor,
    setSelectedTabIdTutorEditor)
}

export function setSelectedTabIdTutorEditor(selectedTabIdDoEditor: number): SET_tutorEditor_selectedTabIdDoEditorAction {
  return {
    type: ActionType.SET_tutorEditor_selectedTabIdDoEditor,
    selectedTabIdDoEditor
  }
}

export function setIsFileExplorerDisplayedTutorEditor(isFileExplorerDisplayedDoEditor: boolean): SET_tutorEditor_isFileExplorerDisplayedDoEditorAction {
  return {
    type: ActionType.SET_tutorEditor_isFileExplorerDisplayedDoEditor,
    isFileExplorerDisplayedDoEditor
  }
}

export function setNewFileNameTutorEditor(newFileNameDoEditor: string): SET_tutorEditor_newFileNameDoEditorAction {
  return {
    type: ActionType.SET_tutorEditor_newFileNameDoEditor,
    newFileNameDoEditor
  }
}

export function setRenamingFileIdTutorEditor(renamingFileIdDoEditor: number): SET_tutorEditor_renamingFileIdDoEditorAction {
  return {
    type: ActionType.SET_tutorEditor_renamingFileIdDoEditor,
    renamingFileIdDoEditor
  }
}

export function setIsRenamingFileTutorEditor(isRenamingFileDoEditor: boolean): SET_tutorEditor_isRenamingFileDoEditorAction {
  return {
    type: ActionType.SET_tutorEditor_isRenamingFileDoEditor,
    isRenamingFileDoEditor
  }
}

export function setRenamingFileNameTutorEditor(renamingFileNameDoEditor: string): SET_tutorEditor_renamingFileNameDoEditorAction {
  return {
    type: ActionType.SET_tutorEditor_renamingFileNameDoEditor,
    renamingFileNameDoEditor
  }
}

export function setHasUnsavedChangesTutorEditor(hasUnsavedChanges: boolean): SET_tutorEditor_hasUnsavedChangesAction {
  return {
    type: ActionType.SET_tutorEditor_hasUnsavedChanges,
    hasUnsavedChanges
  }
}

/**
 * sets the file changed map to determine what files changed
 *
 *
 * pass {} to indicate that no file has changed
 * @param {HasFileChangedMap} hasFileChangedMap
 * @returns {SET_hasFileChangedMapAction}
 */
export function setHasFileChangedMapTutorEditor(hasFileChangedMap: HasFileChangedMap): SET_tutorEditor_hasFileChangedMapAction {
  return {
    type: ActionType.SET_tutorEditor_hasFileChangedMap,
    hasFileChangedMap
  }
}