/**
 * Created by janis dähne.
 */
import {ActionType} from "../../reducers/tutorViewSite/tutorViewSiteActionTypes";
import {
  SET_botLeftPanelViewSizeAction,
  SET_botRightPanelViewSizeAction,
  SET_topLeftPanelViewSizeAction,
  SET_topRightSelectedTabIndexAction,
  SET_topLeftSelectedTabIndexAction,
  SET_bottomRightSelectedTabIndexAction,
  SET_bottomLeftSelectedTabIndexAction,
  SET_hDividerPosOffsetAction,
  SET_vDividerPosOffsetAction,
  SET_topRightPanelViewSizeAction, ResetAction, SET_dummyAction, SET_selectedConsoleViewTestAction,
  SET_isViewTestDialogDisplayedAction, SET_isCodeEditorSettingsDialogDisplayedAction, SET_normalTestsForTutorAction,
  SET_submitTestsForTutorAction, SET_useDiffIfOutputMismatchedAction, SET_useReplaceWhiteSpacesWithBulletsAction
} from "../../reducers/tutorViewSite/tutorViewSiteReducer";
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {AwaitActions, MultiActions} from "../types";
import {
  getAllUserTestResultsAsTutorAsnc, getExerciseTutorViewAsync, getSingleAssessmentTutorViewAsync,
  getUserSolutionAsTutorViewAsync,
  getUserSubmissionTestResultsAsTutorAsync
} from "./tutorViewSiteCrudActions";
import {createAndRefreshAssetUrlsAndObjects} from "../../../helpers/assetHelper";
import {getSystemSettingsAsync} from "../systemSettingsActions";
import {getTestTypesAsync} from "../testTypesActions";
import {getPLangsAsync} from "../pLangsActions";
import {AssessedSubmitTestFromBackend, AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {SolutionFileDoExerciseFullBase, SolutionVersionDoExerciseFullBase} from "../../../types/exerciseSolution";
import {SET_userSolution_solutionFilesAction} from "../../reducers/tutorViewSite/subSets/setUserSolutionReducer";
import {setSelectedTabIdUserSolutionEditorTutorView} from "./editorActions/userSolutionEditorActions";
import {
  SET_tutorEditorSolution_mainFileIdAction,
  SET_tutorEditorSolution_solutionFilesAction
} from "../../reducers/tutorViewSite/subSets/setTutorEditorSolutionReducer";
import {
  setHasFileChangedMapTutorEditor, setHasUnsavedChangesTutorEditor,
  setSelectedTabIdTutorEditor
} from "./editorActions/tutorEditorActions";
import Logger from "../../../helpers/logger";
import {setSubmitTestsForTutor} from "./sub/submitTestActions";
import {mergeNormalTestResults, mergeSubmitTestResults} from "../../../helpers/convertersAndTransformers";
import {setNormalTestsForTutor} from "./sub/normalTestActions";
import {MultiTabEditorState} from "../../../components/codeEditors/multiTabCodeEditorProps";
import {setForceEditorUpdateSessionDummyTutorEditor} from "../editorHelperState/editorHelperStateActions";
import {refreshEditorDelay} from "../../../helpers/editorHelper";


export function setSelectedConsoleViewTest(selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive
  | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null): SET_selectedConsoleViewTestAction {
  return {
    type: ActionType.SET_selectedConsoleViewTest,
    selectedConsoleViewTest
  }
}

//used to open/close the user solution editor tabs
export function setSolutionFilesUserSolutionTutorView(solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>): SET_userSolution_solutionFilesAction {
  return {
    type: ActionType.SET_userSolution_solutionFiles,
    solutionFiles
  }
}

export function setSolutionFilesTutorEditor(solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>): SET_tutorEditorSolution_solutionFilesAction {
  return {
    type: ActionType.SET_tutorEditorSolution_solutionFiles,
    solutionFiles
  }
}

//used for resetting the tutor editor
export function setMainFileIdTutorEditor(mainFileId: number): SET_tutorEditorSolution_mainFileIdAction {
  return {
    type: ActionType.SET_tutorEditorSolution_mainFileId,
    mainFileId
  }
}

export function resetTutorEditorToUserSolution(userSolution: SolutionVersionDoExerciseFullBase): AwaitActions {
  return async (dispatch) => {


    dispatch(setForceEditorUpdateSessionDummyTutorEditor(MultiTabEditorState.beforeChangingFiles))

    dispatch(setSolutionFilesTutorEditor([...userSolution.solutionFiles]))
    dispatch(setMainFileIdTutorEditor(userSolution.mainFileId))


    dispatch(setForceEditorUpdateSessionDummyTutorEditor(MultiTabEditorState.afterChangingFiles))
    dispatch(setForceEditorUpdateSessionDummyTutorEditor(MultiTabEditorState.normal))

    //also reset the editor changes state...
    dispatch(setHasUnsavedChangesTutorEditor(false))
    dispatch(setHasFileChangedMapTutorEditor({}))



    //clear all new results
    dispatch(resetTutorNormalTestResults())
    dispatch(resetTutorSubmitTestResults())

  }
}

export function resetTutorNormalTestResults(): MultiActions {
  return (dispatch, getState) => {
      dispatch(setNormalTestsForTutor(mergeNormalTestResults(getState().tutorViewSiteState.normalTestsForTutor, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
    }
}
export function resetTutorSubmitTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setSubmitTestsForTutor(mergeSubmitTestResults(getState().tutorViewSiteState.submitTestsForTutor, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}


//-- console view settings

export function setUseDiffIfOutputMismatched(useDiffIfOutputMismatched: boolean): SET_useDiffIfOutputMismatchedAction {
  return {
    type: ActionType.SET_useDiffIfOutputMismatched,
    useDiffIfOutputMismatched
  }
}
export function setUseReplaceWhiteSpacesWithBullets(useReplaceWhiteSpacesWithBullets: boolean): SET_useReplaceWhiteSpacesWithBulletsAction {
  return {
    type: ActionType.SET_useReplaceWhiteSpacesWithBullets,
    useReplaceWhiteSpacesWithBullets
  }
}


//--- dialogs

export function setTutorViewIsCodeEditorSettingsDialogDisplayed(isDisplayed: boolean): SET_isCodeEditorSettingsDialogDisplayedAction {
  return {
    type: ActionType.SET_isCodeEditorSettingsDialogDisplayed,
    isDisplayed
  }
}

export function setIsViewTestDialogDisplayed(isViewTestDialogDisplayed: boolean): SET_isViewTestDialogDisplayedAction {
  return {
    type: ActionType.SET_isViewTestDialogDisplayed,
    isViewTestDialogDisplayed
  }
}

//--- grid system, tab indices
export function setTopRightSelectedTabIndex(topRightSelectedTabIndex: number): SET_topRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_topRightSelectedTabIndex,
    topRightSelectedTabIndex
  }
}

export function setTopLeftSelectedTabIndex(topLeftSelectedTabIndex: number): SET_topLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_topLeftSelectedTabIndex,
    topLeftSelectedTabIndex
  }
}

export function setBottomRightSelectedTabIndex(bottomRightSelectedTabIndex: number): SET_bottomRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomRightSelectedTabIndex,
    bottomRightSelectedTabIndex
  }
}

export function setBottomLeftSelectedTabIndex(bottomLeftSelectedTabIndex: number): SET_bottomLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomLeftSelectedTabIndex,
    bottomLeftSelectedTabIndex
  }
}

//--- gripper
export function setDoExerciseHDividerPosOffset(hDividerPosOffset: number): SET_hDividerPosOffsetAction {
  return {
    type: ActionType.SET_hDividerPosOffset,
    hDividerPosOffset
  }
}

export function setDoExerciseVDividerPosOffset(vDividerPosOffset: number): SET_vDividerPosOffsetAction {
  return {
    type: ActionType.SET_vDividerPosOffset,
    vDividerPosOffset
  }
}

//--- panel view sizes

export function setTopRightPanelViewSize(topRightPanelViewSize: PanelViewSize): SET_topRightPanelViewSizeAction {
  return {
    type: ActionType.SET_topRightPanelViewSize,
    topRightPanelViewSize
  }
}

export function setTopLeftPanelViewSize(topLeftPanelViewSize: PanelViewSize): SET_topLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_topLeftPanelViewSize,
    topLeftPanelViewSize
  }
}

export function setBotRightPanelViewSize(botRightPanelViewSize: PanelViewSize): SET_botRightPanelViewSizeAction {
  return {
    type: ActionType.SET_botRightPanelViewSize,
    botRightPanelViewSize
  }
}

export function setBotLeftPanelViewSize(botLeftPanelViewSize: PanelViewSize): SET_botLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_botLeftPanelViewSize,
    botLeftPanelViewSize
  }
}




export function loadTutorViewSite(releaseId: number, userId: number, pLangId: number): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetTutorViewSite())

    //load exercise
    //with tests, custom tests
    await dispatch(getExerciseTutorViewAsync(releaseId, userId, pLangId))

    //load test types, system settings ... same as do exercise site loading
    await dispatch(getSystemSettingsAsync())

    await dispatch(getTestTypesAsync())

    await dispatch(getPLangsAsync())

    //get solution
    //this sets the user solution
    //and also the tutor editor solution (which is a copy of the user solution initially)
    await dispatch(getUserSolutionAsTutorViewAsync(releaseId, userId, pLangId))

    const userSolutionId = getState().tutorViewSiteState.userSolution.id

    //custom, test results
    await dispatch(getAllUserTestResultsAsTutorAsnc(releaseId, userId, pLangId, userSolutionId))

    //submit test results only if they can be present
    if (!getState().tutorViewSiteState.exercise.exerciseRelease.isReleased
      && getState().tutorViewSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished) {
      try {
        //this might fail if the assessment has not started yet...
        await dispatch(getUserSubmissionTestResultsAsTutorAsync(releaseId, userId, pLangId))
      } catch(err) {
        //could be ok
        Logger.warn("could not get submission test result as tutor, error (automatic assessment might not be finished yet): ", err)
      }
    }

    //try to select the first displayed tab (if any)
    if (getState().tutorViewSiteState.userSolution.solutionFiles.length > 0
      && getState().tutorViewSiteState.userSolution.solutionFiles.filter(p => p.uiIsDisplayed).length > 0) {
      dispatch(setSelectedTabIdUserSolutionEditorTutorView(
        getState().tutorViewSiteState.userSolution.solutionFiles.filter(p => p.uiIsDisplayed)[0].id))
    }

    //try to select the first displayed tab (if any) also for the tutor editor
    if (getState().tutorViewSiteState.tutorEditorSolution.solutionFiles.length > 0
      && getState().tutorViewSiteState.tutorEditorSolution.solutionFiles.filter(p => p.uiIsDisplayed).length > 0) {
      dispatch(setSelectedTabIdTutorEditor(
        getState().tutorViewSiteState.tutorEditorSolution.solutionFiles.filter(p => p.uiIsDisplayed)[0].id))
    }

    //create the assets
    //this is not working because we already set the exercise description and queried the wrong url...
    createAndRefreshAssetUrlsAndObjects(getState().tutorViewSiteState.exercise.exerciseDescription.assets)

    //then load the assessment...
    await dispatch(getSingleAssessmentTutorViewAsync(releaseId, userId, pLangId))

    //somehow force the preview panel to re render
    dispatch(setTutorViewDummy(false))

  }
}

export function setTutorViewDummy(dummy: boolean): SET_dummyAction {
  return {
    type: ActionType.SET_dummy,
    dummy
  }
}

export function resetTutorViewSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}