/**
 * Created by janis dähne.
 */
import {GET_exerciseAction} from "../../reducers/tutorViewSite/getExerciseReducer";
import {ActionType} from "../../reducers/tutorViewSite/tutorViewSiteActionTypes";
import {delay} from "../../../../debug";
import {
  getAllTestResultsAsTutor, getExerciseAsTutor, getSingleAssessment, getSubmissionTestResultsAsTutor,
  getUserSolutionAsTutor, updateSingleAssessment,
} from "../../../communicationLayer/tutorViewAssessmentLayer";
import {GET_allUserTestResultsAction} from "../../reducers/tutorViewSite/crud/getAllUserTestResultsReducer";
import {GET_user_solutionVersionAction} from "../../reducers/tutorViewSite/crud/getUserSolutionReducer";
import {GET_userSubmitTestResultsAction} from "../../reducers/tutorViewSite/crud/getUserSubmitTestResults";
import {RUN_normalTestAsTutorAction} from "../../reducers/tutorViewSite/crud/runNormalTestAsTutorReducer";
import {RunAfterTestCommandForBackend} from "../../../types/testsAndTestProtocol";
import {runTestAsTutor} from "../../../communicationLayer/testingLayer";
import {RUN_submitTestAsTutorAction} from "../../reducers/tutorViewSite/crud/runSubmitTestAsTutorReducer";
import {AwaitActions, AwaitActionsWithResult} from "../types";
import {setSelectedConsoleViewTest} from "./tutorViewSiteActions";
import {setNormalTestsForTutorAreRunningAndResetResult} from "./sub/normalTestActions";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {setSubmitTestsForTutorAreRunningAndResetResult} from "./sub/submitTestActions";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {GET_singleAssessmentAction} from "../../reducers/tutorViewSite/crud/getSingleAssessmentReducer";
import {AssessmentFullBase} from "../../../types/submissions";
import {UPDATE_singleAssessmentAction} from "../../reducers/tutorViewSite/crud/updateSingleAssessmentReducer";
import {notifySuccess} from "../../../helpers/notificationHelper";
import {warningDialog} from "../../../helpers/dialogHelper";
import {translate} from "../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../i18n/ids";


export function getExerciseTutorViewAsync(releaseId: number, userId: number, pLangId: number): GET_exerciseAction {
  return {
    type: ActionType.GET_exercise,
    payload: delay(getExerciseAsTutor(releaseId, userId, pLangId))
  }
}

export function getAllUserTestResultsAsTutorAsnc(releaseId: number, userId: number, pLangId: number, solutionVersionId: number): GET_allUserTestResultsAction {
  return {
    type: ActionType.GET_allUserTestResults,
    payload: delay(getAllTestResultsAsTutor(releaseId, userId, pLangId, solutionVersionId))
  }
}

export function getUserSolutionAsTutorViewAsync(releaseId: number, userId: number, pLangId: number): GET_user_solutionVersionAction {
  return {
    type: ActionType.GET_user_solutionVersion,
    payload: delay(getUserSolutionAsTutor(releaseId, userId, pLangId))
  }
}

export function getUserSubmissionTestResultsAsTutorAsync(releaseId: number, userId: number, pLangId: number): GET_userSubmitTestResultsAction {
  return {
    type: ActionType.GET_userSubmitTestResults,
    payload: delay(getSubmissionTestResultsAsTutor(releaseId, userId, pLangId))
  }
}

export function getSingleAssessmentTutorViewAsync(releaseId: number, userId: number, pLangId: number): GET_singleAssessmentAction {
  return {
    type: ActionType.GET_singleAssessment,
    payload: delay(getSingleAssessment(releaseId, userId, pLangId))
  }
}

export function changeSingleAssessmentTutorViewAsync(assessment: AssessmentFullBase): AwaitActions {
    return async(dispatch, getState) => {

      await dispatch(_changeSingleAssessmentTutorViewAsync(assessment))

      //TODO only if no error

      notifySuccess(translate(TranslateIds.manualAssessmentSaved))
    }
}

export function _changeSingleAssessmentTutorViewAsync(assessment: AssessmentFullBase): UPDATE_singleAssessmentAction {
  return {
    type: ActionType.UPDATE_singleAssessment,
    payload: delay(updateSingleAssessment(assessment))
  }
}


export function runNormalTestAsTutor(runTestAsTutorCommand: RunAfterTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {
    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runTestAsTutorCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().tutorViewSiteState.normalTestsForTutor
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(
      setNormalTestsForTutorAreRunningAndResetResult(runningTests, getState().tutorViewSiteState.normalTestsForTutor))

    //run the test
    await dispatch(_runNormalTestAsTutor(runTestAsTutorCommand))


    const ranTests = runTestAsTutorCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        getState().tutorViewSiteState.normalTestsForTutor
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {

      warningDialog(translate(TranslateIds.warning),
        translate(TranslateIds.probablyNoTestRun)
        )
      return
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return !!firstFailedTest
  }
}

export function _runNormalTestAsTutor(runTestAsTutorCommand: RunAfterTestCommandForBackend): RUN_normalTestAsTutorAction {
  return {
    type: ActionType.RUN_normalTestAsTutor,
    payload: delay(runTestAsTutor(runTestAsTutorCommand))
  }
}

export function runSubmitTestAsTutor(runTestAsTutorCommand: RunAfterTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {

    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runTestAsTutorCommand.testIds
      .map<AssessedSubmitTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().tutorViewSiteState.submitTestsForTutor
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(
      setSubmitTestsForTutorAreRunningAndResetResult(runningTests, getState().tutorViewSiteState.submitTestsForTutor))

    //run the test
    await dispatch(_runSubmitTestAsTutor(runTestAsTutorCommand))

    const ranTests = runTestAsTutorCommand.testIds
      .map<AssessedSubmitTestFrontendExclusive>(testId =>
        getState().tutorViewSiteState.submitTestsForTutor
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {

      warningDialog(translate(TranslateIds.warning),
        translate(TranslateIds.probablyNoTestRun)
      )

      return
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return !!firstFailedTest
  }
}

export function _runSubmitTestAsTutor(runTestAsTutorCommand: RunAfterTestCommandForBackend): RUN_submitTestAsTutorAction {
  return {
    type: ActionType.RUN_submitTestAsTutor,
    payload: delay(runTestAsTutor(runTestAsTutorCommand))
  }
}