/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {SiteViewState} from "../../../../types/siteView";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {ActionBase, ActionType} from "./closedExercisesSiteActionTypes";
import {
  AllActions as GetClosedExercisesActions,
  reducer as getClosedExercisesReducer
} from './getClosedExercisesReducer'
import {SortDirection} from "../../../../types/sortDirection";
import {FrontendSettingsManager} from "../../../../helpers/frontendSettingsManager";


export type State = {
  readonly closedExercisePreviews: ReadonlyArray<ExercisePreviewFromBackend>
  readonly closedExercisePreviewsViaCode: ReadonlyArray<ExercisePreviewFromBackend>
  readonly closedExercisePreviewsViaVisibility: ReadonlyArray<ExercisePreviewFromBackend>
  readonly viewState: SiteViewState
  readonly isLoading: boolean

  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: SortDirection

  readonly isTagsFilterDisplayed: boolean
}

export const initial: State = {
  closedExercisePreviews: [],
  closedExercisePreviewsViaCode: [],
  closedExercisePreviewsViaVisibility: [],
  viewState: SiteViewState.List,
  isLoading: false,

  sortByKey: 'displayName',
  sortDirection: SortDirection.ascending,

  isTagsFilterDisplayed: false,
}


export interface SET_closedExercisesAction extends ActionBase {
  readonly type: ActionType.SET_closedExercises
  readonly closedExercises: ReadonlyArray<ExercisePreviewFromBackend>
}

export interface SET_viewStateAction extends ActionBase {
  readonly type: ActionType.SET_viewState
  readonly viewState: SiteViewState
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: SortDirection
}

export interface SET_isTagsFilterDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isTagsFilterDisplayed
  readonly isTagsFilterDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_viewStateAction
  | SET_sortByKeyAction
  | SET_closedExercisesAction
  | SET_isTagsFilterDisplayedAction

  | GetClosedExercisesActions

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_closedExercisePreviews_PENDING:
    case ActionType.GET_closedExercisePreviews_FULFILLED:
    case ActionType.GET_closedExercisePreviews_REJECTED:
      return getClosedExercisesReducer(state, action)

    case ActionType.SET_viewState:

      FrontendSettingsManager.setAllClosedExercisesSiteViewState(action.viewState)

      return {
        ...state,
        viewState: action.viewState
      }

    case ActionType.SET_isTagsFilterDisplayed:

      FrontendSettingsManager.setAllClosedExercisesSiteTagsFilterIsDisplayed(action.isTagsFilterDisplayed)

      return {
        ...state,
        isTagsFilterDisplayed: action.isTagsFilterDisplayed
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.SET_closedExercises:
      return {
        ...state,
        closedExercisePreviews: action.closedExercises,
        closedExercisePreviewsViaCode: action.closedExercises.filter(p => p.isVisibleToAllAfterRelease === false),
        closedExercisePreviewsViaVisibility: action.closedExercises.filter(p => p.isVisibleToAllAfterRelease === true)
      }

    case ActionType.RESET:
      return {
        ...initial,
        viewState: FrontendSettingsManager.getFrontendSettings().allClosedExercisesSiteViewState,
        isTagsFilterDisplayed: FrontendSettingsManager.getFrontendSettings().allClosedExercisesSiteTagsFilterIsDisplayed
      }

    default:
      notExhaustive(action)
      return state
  }
}

