/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionBase, ActionType} from "./closedExercisesSiteActionTypes";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {State, initial} from "./closedExercisesSiteReducer";
import Logger from "../../../../helpers/logger";


export interface GET_closedExercisePreviewsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<ExercisePreviewFromBackend>, GET_closedExercisePreviews_PENDINGAction, GET_closedExercisePreviews_FULFILLEDAction, GET_closedExercisePreviews_REJECTEDAction> {
  readonly type: ActionType.GET_closedExercisePreviews
  readonly payload: Promise<ReadonlyArray<ExercisePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisePreviews_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_closedExercisePreviews_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisePreviews_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<ExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_closedExercisePreviews_FULFILLED
  readonly payload: ReadonlyArray<ExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_closedExercisePreviews_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<ExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_closedExercisePreviews_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_closedExercisePreviews_PENDINGAction
  | GET_closedExercisePreviews_FULFILLEDAction
  | GET_closedExercisePreviews_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_closedExercisePreviews_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_closedExercisePreviews_FULFILLED:
      return {
        ...state,
        isLoading: false,
        closedExercisePreviews: action.payload,
        closedExercisePreviewsViaCode: action.payload.filter(p => p.isVisibleToAllAfterRelease === false),
        closedExercisePreviewsViaVisibility: action.payload.filter(p => p.isVisibleToAllAfterRelease === true)
      }
    case ActionType.GET_closedExercisePreviews_REJECTED:
      return {
        ...state,
        isLoading: false,
        closedExercisePreviews: [],
        closedExercisePreviewsViaCode: [],
        closedExercisePreviewsViaVisibility: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

