/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {
  getValidationCollection, getValidationMessagesCollection,
  isNotEmptyOrSpaces
} from "../../../../../validation/validations";
import {isValidDialog} from "../../../../../validation/dialogValidations";
import {ExercisePreviewFromBackend} from "../../../../../types/exercisePreview";
import {AllActions as AccessExerciseReleaseActions, reducer as accessExercisWithCode} from './accessExercisWithCode'
import {TranslateIds} from "../../../../../../i18n/ids";
import {translate} from "../../../../../../i18n/translationHelper";

export type PreState = {
  readonly releaseCode: string
}

export type State = PreState & {

  readonly accessedExerciseRelease: ExercisePreviewFromBackend | null

  readonly isValid: boolean
  readonly isBusy: boolean
}
export const validationRules = getValidationCollection<PreState>({
  releaseCode: [isNotEmptyOrSpaces],
})

export const validationMessageKeys = getValidationMessagesCollection<PreState>({
  releaseCode: translate(TranslateIds.fieldShouldNotBeEmptyOrSpaces),
})


const preInitial: PreState = {
  releaseCode: ''

}

export const initial: State = {
  ...preInitial,
  isBusy: false,
  accessedExerciseRelease: null,
  isValid: isValidDialog(preInitial, validationRules)
}


export enum ActionType {
  SET_releaseCode = 'enterReleseCodeDialogReducer_SET_releaseCode',

  ACCESS_exerciseWithCode = 'enterReleseCodeDialogReducer_ACCESS_exerciseWithCode',
  ACCESS_exerciseWithCode_PENDING = 'enterReleseCodeDialogReducer_ACCESS_exerciseWithCode_PENDING',
  ACCESS_exerciseWithCode_FULFILLED = 'enterReleseCodeDialogReducer_ACCESS_exerciseWithCode_FULFILLED',
  ACCESS_exerciseWithCode_REJECTED = 'enterReleseCodeDialogReducer_ACCESS_exerciseWithCode_REJECTED',

  RESET = 'enterReleseCodeDialogReducer_RESET',
}

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface SET_releaseCodeAction extends ActionBase {
  readonly type: ActionType.SET_releaseCode
  readonly releaseCode: string
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_releaseCodeAction

  | AccessExerciseReleaseActions


export function reducer(state: State = initial, action: AllActions): State {

  // noinspection TsLint
  let temp: State = null

  switch (action.type) {


    case ActionType.ACCESS_exerciseWithCode_PENDING:
    case ActionType.ACCESS_exerciseWithCode_FULFILLED:
    case ActionType.ACCESS_exerciseWithCode_REJECTED:
      return accessExercisWithCode(state, action)

    case ActionType.SET_releaseCode:
      temp = {
        ...state,
        releaseCode: action.releaseCode
      }
      return {
        ...temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

