/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionBase, ActionType} from "./openExercisesSiteActionTypes";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {initial, State} from "./openExercisesSiteReducer";
import Logger from "../../../../helpers/logger";
import {convertOpenExercisePreviewToFrontendOpenExercisePreview} from "../../../../helpers/convertersAndTransformers";


export interface GET_openExercisePreviewsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<ExercisePreviewFromBackend>, GET_openExercisePreviews_PENDINGAction, GET_openExercisePreviews_FULFILLEDAction, GET_openExercisePreviews_REJECTEDAction> {
  readonly type: ActionType.GET_openExercisePreviews
  readonly payload: Promise<ReadonlyArray<ExercisePreviewFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisePreviews_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_openExercisePreviews_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisePreviews_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<ExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_openExercisePreviews_FULFILLED
  readonly payload: ReadonlyArray<ExercisePreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_openExercisePreviews_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<ExercisePreviewFromBackend>> {
  readonly type: ActionType.GET_openExercisePreviews_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_openExercisePreviews_PENDINGAction
  | GET_openExercisePreviews_FULFILLEDAction
  | GET_openExercisePreviews_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_openExercisePreviews_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_openExercisePreviews_FULFILLED:

      const convertedPreviews = convertOpenExercisePreviewToFrontendOpenExercisePreview(action.payload)

      return {
        ...state,
        isLoading: false,
        openExercisePreviews: convertedPreviews,
        openExercisePreviewsViaCode: convertedPreviews.filter(p => p.isVisibleToAllAfterRelease === false),
        openExercisePreviewsViaVisibility: convertedPreviews.filter(p => p.isVisibleToAllAfterRelease === true)
      }
    case ActionType.GET_openExercisePreviews_REJECTED:
      return {
        ...state,
        isLoading: false,
        openExercisePreviews: [],
        openExercisePreviewsViaCode: [],
        openExercisePreviewsViaVisibility: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

