/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {SiteViewState} from "../../../../types/siteView";
import {ExercisePreviewFromBackend} from "../../../../types/exercisePreview";
import {ActionBase, ActionType} from "./openExercisesSiteActionTypes";
import {
  AllActions as GetAllOpenExercisesAction,
  reducer as getAllOpenExercisesReducer
} from './getAllOpenExercisesReducer'
import {SortDirection} from "../../../../types/sortDirection";
import {ExercisePreviewsFrontendExclusive} from "../../../../components/sites/allAccessibleExercisesSite/openExercises/allOpenExercisesSite";
import {FrontendSettingsManager} from "../../../../helpers/frontendSettingsManager";


export type State = {
  readonly openExercisePreviews: ReadonlyArray<ExercisePreviewsFrontendExclusive>
  //todo make own reducer and use todo new api
  //trade memory for performance (else we would calculate the array new with filter after every rerender)
  readonly openExercisePreviewsViaCode: ReadonlyArray<ExercisePreviewsFrontendExclusive>
  //todo make own reducer and use todo new api
  //trade memory for performance (else we would calculate the array new with filter after every rerender)
  readonly openExercisePreviewsViaVisibility: ReadonlyArray<ExercisePreviewsFrontendExclusive>

  readonly viewState: SiteViewState
  readonly isLoading: boolean
  readonly isEnterReleaseCodeDialogDisplayed: boolean

  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: SortDirection

  readonly isTagsFilterDisplayed: boolean
}

export const initial: State = {
  openExercisePreviews: [],
  openExercisePreviewsViaCode: [],
  openExercisePreviewsViaVisibility: [],
  viewState: SiteViewState.List,
  isLoading: false,
  isEnterReleaseCodeDialogDisplayed: false,

  sortByKey: 'displayName',
  sortDirection: SortDirection.ascending,

  isTagsFilterDisplayed: false,
}


//used to update the left time
export interface SET_openExercisesAction extends ActionBase {
  readonly type: ActionType.SET_openExercises
  readonly openExercises: ReadonlyArray<ExercisePreviewsFrontendExclusive>
}

export interface SET_viewStateAction extends ActionBase {
  readonly type: ActionType.SET_viewState
  readonly viewState: SiteViewState
}

export interface SET_isEnterReleaseCodeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isEnterReleaseCodeDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof ExercisePreviewFromBackend
  readonly sortDirection: SortDirection
}

export interface SET_isTagsFilterDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isTagsFilterDisplayed
  readonly isTagsFilterDisplayed: boolean
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_viewStateAction
  | SET_sortByKeyAction
  | SET_isEnterReleaseCodeDialogDisplayedAction
  | SET_openExercisesAction
  | SET_isTagsFilterDisplayedAction

  | GetAllOpenExercisesAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_openExercisePreviews_PENDING:
    case ActionType.GET_openExercisePreviews_FULFILLED:
    case ActionType.GET_openExercisePreviews_REJECTED:
      return getAllOpenExercisesReducer(state, action)

    case ActionType.SET_viewState:

      FrontendSettingsManager.setAllOpenExercisesSiteViewState(action.viewState)

      return {
        ...state,
        viewState: action.viewState
      }


    case ActionType.SET_isTagsFilterDisplayed:

      FrontendSettingsManager.setAllOpenExercisesSiteTagsFilterIsDisplayed(action.isTagsFilterDisplayed)

      return {
        ...state,
        isTagsFilterDisplayed: action.isTagsFilterDisplayed
      }

    case ActionType.SET_isEnterReleaseCodeDialogDisplayed:
      return {
        ...state,
        isEnterReleaseCodeDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }
    case ActionType.SET_openExercises:
      return {
        ...state,
        openExercisePreviews: action.openExercises,
        openExercisePreviewsViaCode: action.openExercises.filter(p => p.isVisibleToAllAfterRelease === false),
        openExercisePreviewsViaVisibility: action.openExercises.filter(p => p.isVisibleToAllAfterRelease === true)
      }


    case ActionType.RESET:
      return {
        ...initial,
        viewState: FrontendSettingsManager.getFrontendSettings().allOpenExercisesSiteViewState,
        isTagsFilterDisplayed: FrontendSettingsManager.getFrontendSettings().allOpenExercisesSiteTagsFilterIsDisplayed
      }

    default:
      notExhaustive(action)
      return state
  }
}

