/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./assessmentStatisticsSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import {AssessmentStatisticsFullBase, SingleAssessmentStatistic} from "../../../types/assessmentStatistics";
import {SortDirection} from "../../../types/sortDirection";
import {
  AllActions as GetAssessmentStatisticsActions,
  reducer as getAssessmentStatistics
} from './getAssessmentStatistics'

export type State = {
  readonly assessmentStatistics: AssessmentStatisticsFullBase | null

  readonly isLoading: boolean

  readonly sortByKey: keyof SingleAssessmentStatistic
  readonly sortDirection: SortDirection


  readonly displaySubmitTestLine: boolean
  readonly displayNormalTestLine: boolean

}


export const initial: State = {
  assessmentStatistics: null,

  sortByKey: 'lastUpdatedAt',
  sortDirection: SortDirection.ascending,

  isLoading: false,

  displaySubmitTestLine: true,
  displayNormalTestLine: true

}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof SingleAssessmentStatistic
  readonly sortDirection: SortDirection
}

export interface SET_displaySubmitTestLineAction extends ActionBase {
  readonly type: ActionType.SET_displaySubmitTestLine
  readonly displaySubmitTestLine: boolean
}

export interface SET_displayNormalTestLineAction extends ActionBase {
  readonly type: ActionType.SET_displayNormalTestLine
  readonly displayNormalTestLine: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_sortByKeyAction
  | SET_displaySubmitTestLineAction
  | SET_displayNormalTestLineAction

  | GetAssessmentStatisticsActions


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {


    case ActionType.GET_assessmentStatistics_PENDING:
    case ActionType.GET_assessmentStatistics_FULFILLED:
    case ActionType.GET_assessmentStatistics_REJECTED:
      return getAssessmentStatistics(state, action)

    case ActionType.SET_displaySubmitTestLine:
      return {
        ...state,
        displaySubmitTestLine: action.displaySubmitTestLine
      }

    case ActionType.SET_displayNormalTestLine:
      return {
        ...state,
        displayNormalTestLine: action.displayNormalTestLine
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.RESET:
      return initial

    default:
      //enable if we have > 2 actions
      notExhaustive(action)
      return state
  }
}

