/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "../dashboardSiteActionTypes";
import {HardwareUsageStatistic} from "../../../../types/dashboard";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {initial, State} from "../dashboardSiteReducer";
import {notExhaustive} from "../../_notExhausiveHelper";
import moment = require("moment");
import {dashboardRefreshIntervalInS} from "../../../../components/sites/dashboardSite/dashboardSite";


export interface GET_hardwareUsageAction
  extends ActionBase, ReduxPromiseAction<HardwareUsageStatistic, GET_hardwareUsage_PENDINGAction, GET_hardwareUsage_FULFILLEDAction, GET_hardwareUsage_REJECTEDAction, boolean> {
  readonly type: ActionType.GET_hardwareUsage
  readonly payload: Promise<HardwareUsageStatistic>
}

//automatically dispatched by redux promise middleware
export interface GET_hardwareUsage_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_hardwareUsage_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_hardwareUsage_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<HardwareUsageStatistic, boolean> {
  readonly type: ActionType.GET_hardwareUsage_FULFILLED
  readonly payload: HardwareUsageStatistic
}

//automatically dispatched by redux promise middleware
export interface GET_hardwareUsage_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, boolean> {
  readonly type: ActionType.GET_hardwareUsage_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_hardwareUsage_PENDINGAction
  | GET_hardwareUsage_FULFILLEDAction
  | GET_hardwareUsage_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_hardwareUsage_PENDING:
      return {
        ...state,
        //isLoading: true //handled manually
      }
    case ActionType.GET_hardwareUsage_FULFILLED:
      return {
        ...state,
        //isLoading: false,  //handled manually
        hardwareUsageStatistic: addAndClampArray(state.hardwareUsageStatistic, {
          cpuUsageInPercentage: action.payload.cpuUsageInPercentage === -1 ? Number.NaN : action.payload.cpuUsageInPercentage,
          freeRamInKb: action.payload.freeRamInKb === -1 ? Number.NaN : action.payload.freeRamInKb,
          totalRamInKb: action.payload.totalRamInKb === -1 ? Number.NaN : action.payload.totalRamInKb,
          time: moment().format('HH:mm:ss')
        }, maxItems)
      }
    case ActionType.GET_hardwareUsage_REJECTED:
      return {
        ...state,
        //isLoading: false,  //handled manually
        hardwareUsageStatistic: addAndClampArray(state.hardwareUsageStatistic, {
          cpuUsageInPercentage: Number.NaN,
          freeRamInKb: Number.NaN,
          totalRamInKb: Number.NaN,
          time: moment().format('HH:mm:ss')
        }, maxItems)
      }

    default:
      notExhaustive(action)
      return state
  }
}

// noinspection TsLint
const maxItems = 60 / dashboardRefreshIntervalInS //over 1 min

function addAndClampArray<T>(array: ReadonlyArray<T>, itemToAdd: T, maxItems: number): ReadonlyArray<T> {


  if (array.length < maxItems) {
    return [...array, itemToAdd]
  }

  //remove the oldest
  const [first, ...rest] = array

  return [...rest, itemToAdd]


}

