/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {SolutionFileDoExerciseFullBase, SolutionVersionTupleFromBackend} from "../../../../types/exerciseSolution";
import {initial, State} from "../doExerciseSiteReducer";
import Logger from "../../../../helpers/logger";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {getNewSelectedTabId, updateFilesReference} from "../../../../helpers/editorHelper";
import {FrontendSettingsManager} from "../../../../helpers/frontendSettingsManager";


export interface UPDATE_doExerciseSolutionAction
  extends ActionBase, ReduxPromiseAction<SolutionVersionTupleFromBackend, UPDATE_doExerciseSolution_PENDINGAction, UPDATE_doExerciseSolution_FULFILLEDAction, UPDATE_doExerciseSolution_REJECTEDAction> {
  readonly type: ActionType.UPDATE_doExerciseSolution
  readonly payload: Promise<SolutionVersionTupleFromBackend>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExerciseSolution_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_doExerciseSolution_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExerciseSolution_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<SolutionVersionTupleFromBackend> {
  readonly type: ActionType.UPDATE_doExerciseSolution_FULFILLED
  readonly payload: SolutionVersionTupleFromBackend
}

//automatically dispatched by redux promise middleware
export interface UPDATE_doExerciseSolution_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.UPDATE_doExerciseSolution_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_doExerciseSolution_PENDINGAction
  | UPDATE_doExerciseSolution_FULFILLEDAction
  | UPDATE_doExerciseSolution_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_doExerciseSolution_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.UPDATE_doExerciseSolution_FULFILLED:

      FrontendSettingsManager.setDoEditorSolution({...action.payload.solutionVersion})

      return {
        ...state,
        isLoading: false,
        allSolutionVersionPreviews: action.payload.allSolutionVersionPreviews,

        //this needs to be BEFORE setting solution somehow...
        doEditorState: {
          ...state.doEditorState,
          selectedTabIdDoEditor: getNewSelectedTabId(state.solution.solutionFiles,
            action.payload.solutionVersion.solutionFiles, state.doEditorState.selectedTabIdDoEditor),
        },
        solution: {
          ...state.solution,
          id: action.payload.solutionVersion.id,
          createdAt: action.payload.solutionVersion.createdAt,
          pLangId: action.payload.solutionVersion.pLangId,
          mainFileId: action.payload.solutionVersion.mainFileId,
          //the display order is preserved because we only take the new ids with updateFilesReference
          solutionFiles: updateFilesReference(state.solution.solutionFiles,
            action.payload.solutionVersion.solutionFiles, action.payload.fileIdAssociationMap)
        },


        //solution: action.payload.solutionVersion //do NOT take the new version else the editor has no undo action??
      }
    case ActionType.UPDATE_doExerciseSolution_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

