/**
 * Created by janis dähne.
 */

import {CustomTestForBackend} from "../../../../../types/customTests";
import {getNextFrontendId} from "../../../../../helpers/frontendIdHelper";
import {ActionBase, ActionType} from "./changeCustomTestDialogActionTypes";
import {TestAssetForBackend} from "../../../../../types/exerciseEditor";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {
  getValidationCollection, getValidationMessagesCollection,
  isNotEmptyOrSpaces
} from "../../../../../validation/validations";
import {translate} from "../../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../../i18n/ids";

export type State = CustomTestForBackend


export const initial: () => State = () => {
  return {
    id: getNextFrontendId(),
    testTypeId: -1,
    displayName: '',
    content: '',
    files: [],
    displayIndex: -1
  }
}

export const validationRules = getValidationCollection<State>({
  id: [],
  testTypeId: [],
  displayName: [isNotEmptyOrSpaces],
  content: [],
  files: [],
  displayIndex: []
})

export const validationMessageKeys = getValidationMessagesCollection<State>({
  id: '',
  testTypeId: '',
  displayName: translate(TranslateIds.fieldShouldNotBeEmptyOrSpaces),
  content: '',
  files: '',
  displayIndex:'',
})

export interface SET_idAction extends ActionBase {
  readonly type: ActionType.SET_id
  readonly id: number
}

export interface SET_testTypeIdAction extends ActionBase {
  readonly type: ActionType.SET_testTypeId
  readonly testTypeId: number
}

export interface SET_displayNameAction extends ActionBase {
  readonly type: ActionType.SET_displayName
  readonly displayName: string
}

export interface SET_contentAction extends ActionBase {
  readonly type: ActionType.SET_content
  readonly content: string
}

export interface SET_filesAction extends ActionBase {
  readonly type: ActionType.SET_files
  readonly files: ReadonlyArray<TestAssetForBackend>
}

export interface SET_displayIndexAction extends ActionBase {
  readonly type: ActionType.SET_displayIndex
  readonly displayIndex: number
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  SET_idAction
  | SET_testTypeIdAction
  | SET_displayNameAction
  | SET_contentAction
  | SET_filesAction
  | SET_displayIndexAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_id:
      return {
        ...state,
        id: action.id
      }

    case ActionType.SET_testTypeId:
      return {
        ...state,
        testTypeId: action.testTypeId
      }
    case ActionType.SET_displayName:
      return {
        ...state,
        displayName: action.displayName
      }

    case ActionType.SET_content:
      return {
        ...state,
        content: action.content
      }
    case ActionType.SET_files:
      return {
        ...state,
        files: action.files
      }

    case ActionType.SET_displayIndex:
      return {
        ...state,
        displayIndex: action.displayIndex
      }


    default:
      notExhaustive(action)
      return state
  }
}

