/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "../doExerciseSiteActionTypes";
import {notExhaustive} from "../../_notExhausiveHelper";
import {HasFileChangedMap} from "../../../../helpers/editorHelper";
import {MultiTabEditorState} from "../../../../components/codeEditors/multiTabCodeEditorProps";

export type DoEditorState = {

  readonly selectedTabIdDoEditor: number
  readonly isFileExplorerDisplayedDoEditor: boolean
  readonly newFileNameDoEditor: string
  readonly renamingFileIdDoEditor: number
  readonly isRenamingFileDoEditor: boolean
  readonly renamingFileNameDoEditor: string
  readonly hasUnsavedChanges: boolean

  /**
   * we don't use this because performance is too bad ... the whole editor rerenders every time?
   */
  readonly hasFileChangedMap: HasFileChangedMap
}

export const initial: DoEditorState = {

  selectedTabIdDoEditor: -1,
  isFileExplorerDisplayedDoEditor: false,
  newFileNameDoEditor: '',
  renamingFileIdDoEditor: NaN, //use nan because a new file coud have id -1
  isRenamingFileDoEditor: false,
  renamingFileNameDoEditor: '',

  hasUnsavedChanges: false,
  hasFileChangedMap: {}
}


//--- do editor

export interface SET_hasFileChangedMapAction extends ActionBase {
  readonly type: ActionType.SET_hasFileChangedMap
  readonly hasFileChangedMap: HasFileChangedMap
}

export interface SET_hasUnsavedChangesAction extends ActionBase {
  readonly type: ActionType.SET_hasUnsavedChanges
  readonly hasUnsavedChanges: boolean
}

export interface SET_selectedTabIdDoEditorAction extends ActionBase {
  readonly type: ActionType.SET_selectedTabIdDoEditor
  readonly selectedTabIdDoEditor: number
}

export interface SET_isFileExplorerDisplayedDoEditorAction extends ActionBase {
  readonly type: ActionType.SET_isFileExplorerDisplayedDoEditor
  readonly isFileExplorerDisplayedDoEditor: boolean
}

export interface SET_newFileNameDoEditorAction extends ActionBase {
  readonly type: ActionType.SET_newFileNameDoEditor
  readonly newFileNameDoEditor: string
}

export interface SET_renamingFileIdDoEditorAction extends ActionBase {
  readonly type: ActionType.SET_renamingFileIdDoEditor
  readonly renamingFileIdDoEditor: number
}

export interface SET_isRenamingFileDoEditorAction extends ActionBase {
  readonly type: ActionType.SET_isRenamingFileDoEditor
  readonly isRenamingFileDoEditor: boolean
}

export interface SET_renamingFileNameDoEditorAction extends ActionBase {
  readonly type: ActionType.SET_renamingFileNameDoEditor
  readonly renamingFileNameDoEditor: string
}


export type AllActions =
  SET_selectedTabIdDoEditorAction
  | SET_isFileExplorerDisplayedDoEditorAction
  | SET_newFileNameDoEditorAction
  | SET_renamingFileIdDoEditorAction
  | SET_isRenamingFileDoEditorAction
  | SET_renamingFileNameDoEditorAction
  | SET_hasUnsavedChangesAction
  | SET_hasFileChangedMapAction


export function reducer(state: DoEditorState = initial, action: AllActions): DoEditorState {

  switch (action.type) {

    case ActionType.SET_selectedTabIdDoEditor:
      return {
        ...state,
        selectedTabIdDoEditor: action.selectedTabIdDoEditor
      }
    case ActionType.SET_isFileExplorerDisplayedDoEditor:
      return {
        ...state,
        isFileExplorerDisplayedDoEditor: action.isFileExplorerDisplayedDoEditor
      }
    case ActionType.SET_newFileNameDoEditor:
      return {
        ...state,
        newFileNameDoEditor: action.newFileNameDoEditor
      }
    case ActionType.SET_renamingFileIdDoEditor:
      return {
        ...state,
        renamingFileIdDoEditor: action.renamingFileIdDoEditor
      }
    case ActionType.SET_isRenamingFileDoEditor:
      return {
        ...state,
        isRenamingFileDoEditor: action.isRenamingFileDoEditor
      }
    case ActionType.SET_renamingFileNameDoEditor:
      return {
        ...state,
        renamingFileNameDoEditor: action.renamingFileNameDoEditor
      }
    case ActionType.SET_hasUnsavedChanges:
      return {
        ...state,
        hasUnsavedChanges: action.hasUnsavedChanges
      }
    case ActionType.SET_hasFileChangedMap:
      return {
        ...state,
        hasFileChangedMap: action.hasFileChangedMap
      }


    default:
      notExhaustive(action)
      return state
  }
}

