/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "../editCustomProjectSiteActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {State, initial} from "../editCustomProjectSiteReducer";
import {notExhaustive} from "../../_notExhausiveHelper";

export interface CREATE_customProjectAction
  extends ActionBase, ReduxPromiseAction<number, CREATE_customProject_PENDINGAction, CREATE_customProject_FULFILLEDAction, CREATE_customProject_REJECTEDAction> {
  readonly type: ActionType.CREATE_customProject
  readonly payload: Promise<number>
}

//automatically dispatched by redux promise middleware
export interface CREATE_customProject_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.CREATE_customProject_PENDING
}

//automatically dispatched by redux promise middleware
export interface CREATE_customProject_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<number> {
  readonly type: ActionType.CREATE_customProject_FULFILLED
  readonly payload: number
}

//automatically dispatched by redux promise middleware
export interface CREATE_customProject_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.CREATE_customProject_REJECTED
  readonly payload: any
}


export type AllActions =
  CREATE_customProject_PENDINGAction
  | CREATE_customProject_FULFILLEDAction
  | CREATE_customProject_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.CREATE_customProject_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.CREATE_customProject_FULFILLED:
      return {
        ...state,
        isLoading: false,
        customProject: {
          ...state.customProject,
          id: action.payload
        },
        savedNewCustomProject: true
      }
    case ActionType.CREATE_customProject_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

