/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../editCustomProjectSiteActionTypes";
import {State, initial} from "../editCustomProjectSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {StoredTestResultFromBackend} from "../../../../types/testResults";
import {mergeTestResultsIntoCustomProject} from "../../../../helpers/convertersAndTransformers";

export interface GET_customProjectTestResultsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<StoredTestResultFromBackend>, GET_customProjectTestResults_PENDINGAction, GET_customProjectTestResults_FULFILLEDAction, GET_customProjectTestResults_REJECTEDAction> {
  readonly type: ActionType.GET_customProjectTestResults
  readonly payload: Promise<ReadonlyArray<StoredTestResultFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectTestResults_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_customProjectTestResults_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectTestResults_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<StoredTestResultFromBackend>> {
  readonly type: ActionType.GET_customProjectTestResults_FULFILLED
  readonly payload: ReadonlyArray<StoredTestResultFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_customProjectTestResults_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_customProjectTestResults_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_customProjectTestResults_PENDINGAction
  | GET_customProjectTestResults_FULFILLEDAction
  | GET_customProjectTestResults_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_customProjectTestResults_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_customProjectTestResults_FULFILLED:
      return {
        ...state,
        isLoading: false,
        customProject: mergeTestResultsIntoCustomProject(state.customProject, action.payload)
      }
    case ActionType.GET_customProjectTestResults_REJECTED:
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

