/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../editCustomProjectSiteActionTypes";
import {State, initial} from "../editCustomProjectSiteReducer";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {TestAnswerFromBackend} from "../../../../types/testsAndTestProtocol";
import {notExhaustive} from "../../_notExhausiveHelper";
import {
  mergeCustomTestResultsCustomProject
} from "../../../../helpers/convertersAndTransformers";


export interface RUN_customProjectTestAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestAnswerFromBackend>, RUN_customProjectTest_PENDINGAction, RUN_customProjectTest_FULFILLEDAction, RUN_customProjectTest_REJECTEDAction> {
  readonly type: ActionType.RUN_customProjectTest
  readonly payload: Promise<ReadonlyArray<TestAnswerFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface RUN_customProjectTest_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_customProjectTest_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_customProjectTest_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestAnswerFromBackend>> {
  readonly type: ActionType.RUN_customProjectTest_FULFILLED
  readonly payload: ReadonlyArray<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_customProjectTest_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_customProjectTest_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_customProjectTest_PENDINGAction
  | RUN_customProjectTest_FULFILLEDAction
  | RUN_customProjectTest_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_customProjectTest_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.RUN_customProjectTest_FULFILLED:
      return {
        ...state,
        isLoading: false,
        customProject: {
          ...state.customProject,
          tests: mergeCustomTestResultsCustomProject(state.customProject.tests, action.payload)
        }
      }
    case ActionType.RUN_customProjectTest_REJECTED:
      return {
        ...state,
        isLoading: false,
        customProject: {
          ...state.customProject,
          tests: mergeCustomTestResultsCustomProject(state.customProject.tests, [])
        }
      }

    default:
      notExhaustive(action)
      return state
  }
}

