/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "./changeCustomProjectTestDialogActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {State, initial} from "./changeCustomProjectTestDialogReducer";
import {notExhaustive} from "../../../_notExhausiveHelper";
import {convertCustomProjectTestFromBackendToRunnableFrontendCustomProjectTest} from "../../../../../helpers/convertersAndTransformers";
import {CustomProjectTestFullBase} from "../../../../../types/customProjects";


export interface CREATE_customProjectTestAction
  extends ActionBase, ReduxPromiseAction<CustomProjectTestFullBase, CREATE_customProjectTest_PENDINGAction, CREATE_customProjectTest_FULFILLEDAction, CREATE_customProjectTest_REJECTEDAction> {
  readonly type: ActionType.CREATE_customProjectTest
  readonly payload: Promise<CustomProjectTestFullBase>
}

//automatically dispatched by redux promise middleware
export interface CREATE_customProjectTest_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.CREATE_customProjectTest_PENDING
}

//automatically dispatched by redux promise middleware
export interface CREATE_customProjectTest_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<CustomProjectTestFullBase> {
  readonly type: ActionType.CREATE_customProjectTest_FULFILLED
  readonly payload: CustomProjectTestFullBase
}

//automatically dispatched by redux promise middleware
export interface CREATE_customProjectTest_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.CREATE_customProjectTest_REJECTED
  readonly payload: any
}


export type AllActions =
  CREATE_customProjectTest_PENDINGAction
  | CREATE_customProjectTest_FULFILLEDAction
  | CREATE_customProjectTest_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.CREATE_customProjectTest_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.CREATE_customProjectTest_FULFILLED:
      return {
        ...state,
        isBusy: false,
        test: convertCustomProjectTestFromBackendToRunnableFrontendCustomProjectTest(action.payload)
      }
    case ActionType.CREATE_customProjectTest_REJECTED:
      return {
        ...state,
        isBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

