/**
 * Created by janis dähne.
 */

import {Action} from "redux";
import {MultiTabEditorState} from "../../../components/codeEditors/multiTabCodeEditorProps";
import {notExhaustive} from "../_notExhausiveHelper";


/*
 * this is used so that the editor's componentWillUpdate method will be called with the right ForceEditorUpdateSessionDummy values
 * so that we get always a valid editor state (with undo, selection...)
 *
 * see MultiTabEditorState
 */

export type State = {

  readonly forceEditorUpdateSessionDummyDoEditor: MultiTabEditorState
  readonly forceEditorUpdateSessionDummyAfterEditor: MultiTabEditorState
  readonly forceEditorUpdateSessionDummyTutorEditor: MultiTabEditorState
  readonly forceEditorUpdateSessionDummyCreateExerciseTemplateEditor: MultiTabEditorState
  readonly forceEditorUpdateSessionDummyCustomProjectSolutionEditor: MultiTabEditorState

}

export const initial: State = {
  forceEditorUpdateSessionDummyDoEditor: MultiTabEditorState.normal,
  forceEditorUpdateSessionDummyAfterEditor: MultiTabEditorState.normal,
  forceEditorUpdateSessionDummyTutorEditor: MultiTabEditorState.normal,
  forceEditorUpdateSessionDummyCreateExerciseTemplateEditor: MultiTabEditorState.normal,
  forceEditorUpdateSessionDummyCustomProjectSolutionEditor: MultiTabEditorState.normal,
}

export interface ActionBase extends Action {
  readonly type: ActionType
}


export enum ActionType {
  SET_forceEditorUpdateSessionDummyDoEditor = 'editorHelperStateReducer_SET_forceEditorUpdateSessionDummyDoEditorAction',
  SET_forceEditorUpdateSessionDummyAfterEditor = 'editorHelperStateReducer_SET_forceEditorUpdateSessionDummyAfterEditorAction',
  SET_forceEditorUpdateSessionDummyTutorEditor = 'editorHelperStateReducer_SET_forceEditorUpdateSessionDummyTutorEditorAction',
  SET_forceEditorUpdateSessionDummyCreateExerciseTemplateEditor = 'editorHelperStateReducer_SET_forceEditorUpdateSessionDummyCreateExerciseTemplateEditorAction',
  SET_forceEditorUpdateSessionDummyCustomProjectSolutionEditor = 'editorHelperStateReducer_SET_forceEditorUpdateSessionDummyCustomProjectSolutionEditor',


  RESET = 'editorHelperStateReducer_RESET',
}


export interface SET_forceEditorUpdateSessionDummyDoEditorAction extends ActionBase {
  readonly type: ActionType.SET_forceEditorUpdateSessionDummyDoEditor
  readonly editorState: MultiTabEditorState
}

export interface SET_forceEditorUpdateSessionDummyAfterEditorAction extends ActionBase {
  readonly type: ActionType.SET_forceEditorUpdateSessionDummyAfterEditor
  readonly editorState: MultiTabEditorState
}

export interface SET_forceEditorUpdateSessionDummyTutorEditorAction extends ActionBase {
  readonly type: ActionType.SET_forceEditorUpdateSessionDummyTutorEditor
  readonly editorState: MultiTabEditorState
}

export interface SET_forceEditorUpdateSessionDummyCreateExerciseTemplateEditorAction extends ActionBase {
  readonly type: ActionType.SET_forceEditorUpdateSessionDummyCreateExerciseTemplateEditor
  readonly editorState: MultiTabEditorState
}

export interface SET_forceEditorUpdateSessionDummyCustomProjectSolutionEditorAction extends ActionBase {
  readonly type: ActionType.SET_forceEditorUpdateSessionDummyCustomProjectSolutionEditor
  readonly editorState: MultiTabEditorState
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_forceEditorUpdateSessionDummyDoEditorAction
  | SET_forceEditorUpdateSessionDummyAfterEditorAction
  | SET_forceEditorUpdateSessionDummyTutorEditorAction
  | SET_forceEditorUpdateSessionDummyCreateExerciseTemplateEditorAction
  | SET_forceEditorUpdateSessionDummyCustomProjectSolutionEditorAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_forceEditorUpdateSessionDummyDoEditor:
      return {
        ...state,
        forceEditorUpdateSessionDummyDoEditor: action.editorState
      }

    case ActionType.SET_forceEditorUpdateSessionDummyAfterEditor:
      return {
        ...state,
        forceEditorUpdateSessionDummyAfterEditor: action.editorState
      }

    case ActionType.SET_forceEditorUpdateSessionDummyTutorEditor:
      return {
        ...state,
        forceEditorUpdateSessionDummyTutorEditor: action.editorState
      }

    case ActionType.SET_forceEditorUpdateSessionDummyCreateExerciseTemplateEditor:
      return {
        ...state,
        forceEditorUpdateSessionDummyCreateExerciseTemplateEditor: action.editorState
      }

    case ActionType.SET_forceEditorUpdateSessionDummyCustomProjectSolutionEditor:
      return {
        ...state,
        forceEditorUpdateSessionDummyCustomProjectSolutionEditor: action.editorState
      }


    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

