/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./editorExerciseSiteActionTypes";
import {EditorExerciseExerciseForBackend} from "../../../types/exerciseEditor";
import {
  AllActions as GetEditorExerciseActionTypes,
  reducer as getEditorExerciseReducer
} from './getEditorExerciseReducer'

import {
  AllActions as CreateEditorExerciseActions,
  reducer as createEditorExerciseReducer
} from './createEditorExerciseReducer'
import {
  AllActions as UpdateEditorExerciseActions,
  reducer as updateEditorExerciseReducer
} from './updateEditorExerciseReducer'

import {
  AllActions as SetEditorExercisePropertiesActions,
  initial as initialExerciseProperties,
  reducer as setEditorExercisePropertiesReducer
} from './subSets/setEditorExercisePropertiesReducer'
import {
  AllActions as SetEditorExerciseDescriptionActions,
  initial as initialExerciseDescription,
  reducer as setEditorExerciseDescriptionReducer
} from './subSets/setEditorExerciseDescriptionReducer'

import {
  AllActions as SetEditorExerciseMetaDataActions,
  initial as initialExerciseMetaData,
  reducer as setEditorExerciseMetaDataReducer
} from './subSets/setEditorExerciseMetaDataReducer'
import {
  AllActions as SetEditorExerciseDefaultTestSettingsActions,
  initial as initialDefaultTestSettings,
  reducer as setEditorExerciseDefaultTestSettingsReducer
} from './subSets/setEditorExerciseDefaultTestSettingsReducer'

import {
  AllActions as SetEditorExerciseCodeTemplatesActions,
  initial as initialCodeTemplates,
  reducer as setEditorExerciseCodeTemplatesReducer
} from './subSets/setEditorExerciseCodeTemplatesReducer'
import {
  AllActions as SetEditorExerciseTestsActions,
  initial as initialTests,
  reducer as setEditorExerciseTestsReducer
} from './subSets/setEditorExerciseTestsReducer'
import {getNextFrontendId} from "../../../helpers/frontendIdHelper";


//TODO move assets to own reducer because they are big and we don't want to copy them every time

export type State = EditorExerciseExerciseForBackend


//function to evaluate getNextFrontendId every time
export const initial: () => State = () => {
  return {
    id: getNextFrontendId(),
    note: '',
    exerciseProperties: initialExerciseProperties,
    exerciseDescription: initialExerciseDescription(),
    metaData: initialExerciseMetaData(),
    defaultTestSettings: initialDefaultTestSettings(),
    codeTemplates: initialCodeTemplates,
    creatorId: -1,
    tests: initialTests
  }
}


export interface SET_editorExerciseIdAction extends ActionBase {
  readonly type: ActionType.SET_editorExerciseId
  readonly id: number
}

export interface SET_noteAction extends ActionBase {
  readonly type: ActionType.SET_note
  readonly note: string
}

export interface SET_editorExerciseCreatorIdAction extends ActionBase {
  readonly type: ActionType.SET_editorExerciseCreatorId
  readonly creatorId: number
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_noteAction
  | SET_editorExerciseCreatorIdAction
  | SET_editorExerciseIdAction

  | SetEditorExercisePropertiesActions
  | SetEditorExerciseDescriptionActions
  | SetEditorExerciseMetaDataActions
  | SetEditorExerciseDefaultTestSettingsActions
  | SetEditorExerciseCodeTemplatesActions
  | SetEditorExerciseTestsActions


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {


    case ActionType.SET_editorExerciseId:
      return {
        ...state,
        id: action.id
      }

    case ActionType.SET_note:
      return {
        ...state,
        note: action.note
      }

    case ActionType.SET_editorExerciseCreatorId:
      return {
        ...state,
        creatorId: action.creatorId
      }

    case ActionType.SET_tests:
      return {
        ...state,
        tests: setEditorExerciseTestsReducer(state.tests, action)
      }

    case ActionType.SET_codeTemplates:
      return {
        ...state,
        codeTemplates: setEditorExerciseCodeTemplatesReducer(state.codeTemplates, action)
      }

    case ActionType.SET_editorExerciseDefaultTestSettingsId:
    case ActionType.SET_timeoutInMs:
    case ActionType.SET_memoryLimitInKb:
    case ActionType.SET_maxDiskSpaceInKb:
      return {
        ...state,
        defaultTestSettings: setEditorExerciseDefaultTestSettingsReducer(state.defaultTestSettings, action),
      }

    case ActionType.SET_editorExerciseMetaDataId:
    case ActionType.SET_tagIds:
      return {
        ...state,
        metaData: setEditorExerciseMetaDataReducer(state.metaData, action)
      }

    case ActionType.SET_editorExerciseDescriptionId:
    case ActionType.SET_editorExerciseContent:
    case ActionType.SET_assets:
      return {
        ...state,
        exerciseDescription: setEditorExerciseDescriptionReducer(state.exerciseDescription, action)
      }

    case ActionType.SET_displayName:
    case ActionType.SET_shortDescription:
    case ActionType.SET_isOnlyVisibleToMe:
    case ActionType.SET_canUserCreateCustomTests:
    case ActionType.SET_canUserCreateFiles:
    case ActionType.SET_showSubmitResults:
    case ActionType.SET_userGroupId:
      return {
        ...state,
        exerciseProperties: setEditorExercisePropertiesReducer(state.exerciseProperties, action)
      }

    case ActionType.RESET:
      return initial()

    default:
      notExhaustive(action)
      return state
  }
}

