/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./editorExerciseSiteActionTypes";
import {initial, State} from "./exerciseEditorSiteReducer";
import Logger from "../../../helpers/logger";
import {EditorExerciseFromBackend} from "../../../types/exerciseEditor";
import {convertEditorExerciseFromBackendToFrontend} from "../../../helpers/convertersAndTransformers";


export interface GET_editorExerciseAction
  extends ActionBase, ReduxPromiseAction<EditorExerciseFromBackend, GET_editorExercise_PENDINGAction, GET_editorExercise_FULFILLEDAction, GET_editorExercise_REJECTEDAction> {
  readonly type: ActionType.GET_editorExercise
  readonly payload: Promise<EditorExerciseFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_editorExercise_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_editorExercise_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_editorExercise_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<EditorExerciseFromBackend> {
  readonly type: ActionType.GET_editorExercise_FULFILLED
  readonly payload: EditorExerciseFromBackend
}

//automatically dispatched by redux promise middleware
export interface GET_editorExercise_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_editorExercise_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_editorExercise_PENDINGAction
  | GET_editorExercise_FULFILLEDAction
  | GET_editorExercise_REJECTEDAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_editorExercise_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_editorExercise_FULFILLED:
      return {
        ...state,
        isLoading: false,
        editorExercise: convertEditorExerciseFromBackendToFrontend(action.payload)
      }
    case ActionType.GET_editorExercise_REJECTED:
      return {
        ...state,
        isLoading: false,
        editorExercise: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

