/**
 * Created by janis dähne.
 */


import {GroupRoleWithPermissionFullBase} from "../../../../types/groupRoles";
import {
  getValidationCollection, getValidationMessagesCollection,
  isNotEmptyOrSpaces
} from "../../../../validation/validations";
import {ActionType} from "./groupRolesDialogActionTypes";
import {Action} from "redux";
import {notExhaustive} from "../../_notExhausiveHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

export type State = GroupRoleWithPermissionFullBase


export const initial: State = {
  id: -1,
  displayName: '',

  canCreateExercises: false,
  canChangeExercises: false,
  canDeleteExercises: false,

  canReleaseExercises: false,
  canCopyExercises: false,
  canAssessExercises: false,

  canSeeExercisesFromOthersInGroup: false,
  canAddUserToGroup: false,
  canRemoveMemberFromGroup: false,
  canChangeOtherMembersRole: false,
  canSeeOtherUsers: false
}


export const validationRules = getValidationCollection<State>({
  id: [],
  displayName: [isNotEmptyOrSpaces],

  canCreateExercises: [],
  canChangeExercises: [],
  canDeleteExercises: [],

  canReleaseExercises: [],
  canCopyExercises: [],
  canAssessExercises: [],

  canSeeExercisesFromOthersInGroup: [],

  canAddUserToGroup: [],
  canRemoveMemberFromGroup: [],
  canChangeOtherMembersRole: [],

  canSeeOtherUsers: [],
})

export const validationMessageKeys = getValidationMessagesCollection<State>({
  id: '',
  displayName: translate(TranslateIds.fieldShouldNotBeEmptyOrSpaces),
  canCreateExercises: '',
  canChangeExercises: '',
  canDeleteExercises: '',

  canReleaseExercises: '',
  canCopyExercises: '',
  canAssessExercises: '',

  canSeeExercisesFromOthersInGroup: '',

  canAddUserToGroup: '',
  canRemoveMemberFromGroup: '',
  canChangeOtherMembersRole: '',

  canSeeOtherUsers: '',

})


export interface ActionBase extends Action {
  readonly type: ActionType
}


export interface SET_idAction extends ActionBase {
  readonly type: ActionType.SET_id
  readonly id: number
}

export interface SET_displayNameAction extends ActionBase {
  readonly type: ActionType.SET_displayName
  readonly displayName: string
}

export interface SET_canCreateExercisesAction extends ActionBase {
  readonly type: ActionType.SET_canCreateExercises
  readonly can: boolean
}

export interface SET_canChangeExercisesAction extends ActionBase {
  readonly type: ActionType.SET_canChangeExercises
  readonly can: boolean
}

export interface SET_canDeleteExercisesAction extends ActionBase {
  readonly type: ActionType.SET_canDeleteExercises
  readonly can: boolean
}

export interface SET_canReleaseExercisesAction extends ActionBase {
  readonly type: ActionType.SET_canReleaseExercises
  readonly can: boolean
}

export interface SET_canCopyExercisesAction extends ActionBase {
  readonly type: ActionType.SET_canCopyExercises
  readonly can: boolean
}

export interface SET_canAssessExercisesAction extends ActionBase {
  readonly type: ActionType.SET_canAssessExercises
  readonly can: boolean
}

export interface SET_canSeeExercisesFromOthersInGroupAction extends ActionBase {
  readonly type: ActionType.SET_canSeeExercisesFromOthersInGroup
  readonly can: boolean
}

export interface SET_canAddUserToGroupAction extends ActionBase {
  readonly type: ActionType.SET_canAddUserToGroup
  readonly can: boolean
}

export interface SET_canRemoveMemberFromGroupAction extends ActionBase {
  readonly type: ActionType.SET_canRemoveMemberFromGroup
  readonly can: boolean
}

export interface SET_canChangeOtherMembersRoleAction extends ActionBase {
  readonly type: ActionType.SET_canChangeOtherMembersRole
  readonly can: boolean
}

export interface SET_canSeeOtherUsersAction extends ActionBase {
  readonly type: ActionType.SET_canSeeOtherUsers
  readonly can: boolean
}

export type AllActions =
  SET_idAction
  | SET_displayNameAction
  | SET_canCreateExercisesAction
  | SET_canChangeExercisesAction
  | SET_canDeleteExercisesAction
  | SET_canReleaseExercisesAction
  | SET_canCopyExercisesAction
  | SET_canAssessExercisesAction
  | SET_canSeeExercisesFromOthersInGroupAction
  | SET_canAddUserToGroupAction
  | SET_canRemoveMemberFromGroupAction
  | SET_canChangeOtherMembersRoleAction
  | SET_canSeeOtherUsersAction


export function reducer(state: State = initial, action: AllActions): State {


  switch (action.type) {
    case ActionType.SET_id:

      return {
        ...state,
        id: action.id
      }
    case ActionType.SET_displayName:

      return {
        ...state,
        displayName: action.displayName
      }

    case ActionType.SET_canCreateExercises:

      return {
        ...state,
        canCreateExercises: action.can
      }

    case ActionType.SET_canChangeExercises:
      return {
        ...state,
        canChangeExercises: action.can
      }

    case ActionType.SET_canDeleteExercises:
      return {
        ...state,
        canDeleteExercises: action.can
      }

    case ActionType.SET_canReleaseExercises:
      return {
        ...state,
        canReleaseExercises: action.can
      }

    case ActionType.SET_canCopyExercises:
      return {
        ...state,
        canCopyExercises: action.can
      }

    case ActionType.SET_canAssessExercises:
      return {
        ...state,
        canAssessExercises: action.can
      }

    case ActionType.SET_canSeeExercisesFromOthersInGroup:
      return {
        ...state,
        canSeeExercisesFromOthersInGroup: action.can
      }

    case ActionType.SET_canAddUserToGroup:

      return {
        ...state,
        canAddUserToGroup: action.can
      }

    case ActionType.SET_canRemoveMemberFromGroup:
      return {
        ...state,
        canRemoveMemberFromGroup: action.can
      }

    case ActionType.SET_canChangeOtherMembersRole:
      return {
        ...state,
        canChangeOtherMembersRole: action.can
      }

    case ActionType.SET_canSeeOtherUsers:

      return {
        ...state,
        canSeeOtherUsers: action.can
      }

    default:
      notExhaustive(action)
      return state
  }
}
