/**
 * Created by janis dähne.
 */
import {State, initial} from "./groupStorageReducer";
import {ActionBase, ActionType} from "./groupStorageActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import {notExhaustive} from "../_notExhausiveHelper";


export interface GET_userGroupRoleAction
  extends ActionBase, ReduxPromiseAction<GroupRoleWithPermissionFullBase | null, GET_userGroupRole_PENDINGAction, GET_userGroupRole_FULFILLEDAction, GET_userGroupRole_REJECTEDAction> {
  readonly type: ActionType.GET_userGroupRole
  readonly payload: Promise<GroupRoleWithPermissionFullBase | null>
}

//automatically dispatched by redux promise middleware
export interface GET_userGroupRole_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_userGroupRole_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_userGroupRole_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<GroupRoleWithPermissionFullBase | null> {
  readonly type: ActionType.GET_userGroupRole_FULFILLED
  readonly payload: GroupRoleWithPermissionFullBase | null
}

//automatically dispatched by redux promise middleware
export interface GET_userGroupRole_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_userGroupRole_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_userGroupRole_PENDINGAction
  | GET_userGroupRole_FULFILLEDAction
  | GET_userGroupRole_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_userGroupRole_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_userGroupRole_FULFILLED:
      return {
        ...state,
        isLoading: false,
        userGroupRole: action.payload
      }
    case ActionType.GET_userGroupRole_REJECTED:

      return {
        ...state,
        isLoading: false,
        userGroupRole: null
      }

    default:
      notExhaustive(action)
      return state
  }
}

