/**
 * Created by janis dähne.
 */
import {Action} from "redux";

import {UserGroupFullBase} from "../../../../types/group";
import {isValidDialog} from "../../../../validation/dialogValidations";
import {
  AllActions as SetActions,
  reducer as groupReducer,
  initial as preInitial,
  validationRules,
  validationMessageKeys
} from './groupReducer'
import {ActionType, ActionBase} from './actionTypes'
import {notExhaustive} from "../../_notExhausiveHelper";

import {AllActions as CreateActions, reducer as createReducer} from './createGroupReducer'
import {AllActions as UpdateActions, reducer as updateReducer} from './updateGroupReducer'
import {AllActions as DeleteActions, reducer as deleteReducer} from './deleteGroupReducer'

export type State = {
  readonly isValid: boolean
  /**
   * true: show loader,
   * false: not
   */
  readonly isBusy: boolean

  /**
   * true: create entity
   * false: change entity,
   */
  readonly isCreating: boolean

  readonly group: UserGroupFullBase

}

export const initial: State = {
  group: preInitial,
  isValid: isValidDialog(preInitial, validationRules),
  isBusy: false,
  isCreating: false
}


export interface SET_isCreatingAction extends ActionBase {
  readonly type: ActionType.SET_isCreating
  readonly isCreating: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isCreatingAction
  | SetActions
  | CreateActions
  | UpdateActions
  | DeleteActions


export function reducer(state: State = initial, action: AllActions): State {


  switch (action.type) {

    case ActionType.SET_isCreating:
      return {
        ...state,
        isCreating: action.isCreating
      }

    case ActionType.SET_id:
    case ActionType.SET_email:
    case ActionType.SET_displayName:

      const temp = groupReducer(state.group, action)

      return {
        ...state,
        group: temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.CREATE_group_PENDING:
    case ActionType.CREATE_group_FULFILLED:
    case ActionType.CREATE_group_REJECTED:
      return createReducer(state, action)


    case ActionType.UPDATE_group_PENDING:
    case ActionType.UPDATE_group_FULFILLED:
    case ActionType.UPDATE_group_REJECTED:
      return updateReducer(state, action)

    case ActionType.DELETE_group_PENDING:
    case ActionType.DELETE_group_FULFILLED:
    case ActionType.DELETE_group_REJECTED:
      return deleteReducer(state, action)

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

