/**
 * Created by janis dähne.
 */
import {Action} from "redux";

import {UserGroupFullBase} from "../../../../types/group";
import {
  getValidationCollection, getValidationMessagesCollection, isEmailOrEmpty,
  isNotEmptyOrSpaces
} from "../../../../validation/validations";
import {ActionBase, ActionType} from './actionTypes'
import {notExhaustive} from "../../_notExhausiveHelper";
import {translate} from "../../../../../i18n/translationHelper";
import {TranslateIds} from "../../../../../i18n/ids";

export type State = UserGroupFullBase


export const initial: State = {
  id: -1,
  email: '',
  displayName: '',
  groupRole: null
}


export const validationRules = getValidationCollection<UserGroupFullBase>({
  id: [],
  displayName: [isNotEmptyOrSpaces],
  email: [isEmailOrEmpty],
  groupRole: [],
})

export const validationMessageKeys = getValidationMessagesCollection<UserGroupFullBase>({
  id: '',
  displayName: translate(TranslateIds.fieldShouldNotBeEmptyOrSpaces),
  email: translate(TranslateIds.invalidEmail),

  groupRole: '',
})


export interface SET_idAction extends ActionBase {
  readonly type: ActionType.SET_id
  readonly id: number
}

export interface SET_emailAction extends ActionBase {
  readonly type: ActionType.SET_email
  readonly email: string
}

export interface SET_displayNameAction extends ActionBase {
  readonly type: ActionType.SET_displayName
  readonly displayName: string
}


export type AllActions =
  SET_idAction
  | SET_emailAction
  | SET_displayNameAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_id:
      return {
        ...state,
        id: action.id
      }
    case ActionType.SET_email:
      return {
        ...state,
        email: action.email
      }
    case ActionType.SET_displayName:
      return {
        ...state,
        displayName: action.displayName
      }


    default:
      notExhaustive(action)
      return state
  }
}

