/**
 * Created by janis dähne.
 */
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./groupsSiteActionTypes";
import {UserGroupFullBase} from "../../../types/group";
import Logger from "../../../helpers/logger";
import {notExhaustive} from "../_notExhausiveHelper";
import {State, initial} from "./groupsSiteReducer";


export interface GET_groupsAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<UserGroupFullBase>, GET_groups_PENDINGAction, GET_groups_FULFILLEDAction, GET_groups_REJECTEDAction> {
  readonly type: ActionType.GET_groups
  readonly payload: Promise<ReadonlyArray<UserGroupFullBase>>
}

//automatically dispatched by redux promise middleware
export interface GET_groups_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_groups_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_groups_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<UserGroupFullBase>> {
  readonly type: ActionType.GET_groups_FULFILLED
  readonly payload: ReadonlyArray<UserGroupFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_groups_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<UserGroupFullBase>> {
  readonly type: ActionType.GET_groups_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_groups_PENDINGAction
  | GET_groups_FULFILLEDAction
  | GET_groups_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_groups_PENDING:
      return {
        ...state,
        isLoading: true
      }

    case ActionType.GET_groups_FULFILLED:
      return {
        ...state,
        groups: action.payload,
        isLoading: false
      }

    case ActionType.GET_groups_REJECTED:
      return {
        ...state,
        groups: [],
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

