/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {UserGroupFullBase} from "../../../types/group";
import {notExhaustive} from "../_notExhausiveHelper";
import {SiteViewState} from "../../../types/siteView";
import {ActionBase, ActionType} from "./groupsSiteActionTypes";
import {AllActions as GetActions, reducer as getReducer} from './getGroupsReducer'
import {AllActions as QuitActions, reducer as quitReducer} from './quitGroupReducer'
import {SortDirection} from "../../../types/sortDirection";
import {AllActions as DeleteGroupActions, reducer as deleteGroupReducer} from './deleteGroupReducer'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";

export type State = {
  readonly groups: ReadonlyArray<UserGroupFullBase>
  readonly isChangeDialogDisplayed: boolean
  readonly viewState: SiteViewState
  readonly isLoading: boolean

  readonly sortByKey: keyof UserGroupFullBase
  readonly sortDirection: SortDirection
}


export const initial: State = {
  groups: [],
  viewState: SiteViewState.List,
  isLoading: false,
  isChangeDialogDisplayed: false,

  sortByKey: 'displayName',
  sortDirection: SortDirection.ascending
}


export interface SET_groupsAction extends ActionBase {
  readonly type: ActionType.SET_groups
  readonly groups: ReadonlyArray<UserGroupFullBase>
}

export interface SET_isChangeDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_viewStateAction extends ActionBase {
  readonly type: ActionType.SET_viewState
  readonly viewState: SiteViewState
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof UserGroupFullBase
  readonly sortDirection: SortDirection
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isChangeDialogDisplayedAction
  | SET_groupsAction
  | SET_viewStateAction
  | SET_sortByKeyAction
  | GetActions
  | QuitActions
  | DeleteGroupActions



export function reducer(state: State = initial, action: AllActions): State {
  switch (action.type) {

    case ActionType.SET_groups:
      return {
        ...state,
        groups: action.groups
      }

    case ActionType.SET_isChangeDialogDisplayed:
      return {
        ...state,
        isChangeDialogDisplayed: action.isDisplayed
      }

    case ActionType.DELETE_userGroup_PENDING:
    case ActionType.DELETE_userGroup_FULFILLED:
    case ActionType.DELETE_userGroup_REJECTED:
      return deleteGroupReducer(state, action)

    case ActionType.GET_groups_PENDING:
    case ActionType.GET_groups_FULFILLED:
    case ActionType.GET_groups_REJECTED:
      return getReducer(state, action)

    case ActionType.QUIT_group_PENDING:
    case ActionType.QUIT_group_FULFILLED:
    case ActionType.QUIT_group_REJECTED:
      return quitReducer(state, action)

    case ActionType.SET_viewState:

      FrontendSettingsManager.setGroupsSiteViewState(action.viewState)

      return {
        ...state,
        viewState: action.viewState
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.RESET:
      return {
        ...initial,
        viewState: FrontendSettingsManager.getFrontendSettings().groupsSiteViewState
      }

    default:
      notExhaustive(action)
      return state
  }
}

