/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./manageActivatedUsersSiteActionTypes";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {State, initial} from './manageActivatedUsersSiteReducer'
import Logger from "../../../helpers/logger";



export interface DELETE_activatedUserAction
  extends ActionBase, ReduxPromiseAction<void, DELETE_activatedUser_PENDINGAction, DELETE_activatedUser_FULFILLEDAction, DELETE_activatedUser_REJECTEDAction> {
  readonly type: ActionType.DELETE_activatedUser
  readonly payload: Promise<void>
}

//automatically dispatched by redux promise middleware
export interface DELETE_activatedUser_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.DELETE_activatedUser_PENDING
}

//automatically dispatched by redux promise middleware
export interface DELETE_activatedUser_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<void> {
  readonly type: ActionType.DELETE_activatedUser_FULFILLED
  readonly payload: void
}

//automatically dispatched by redux promise middleware
export interface DELETE_activatedUser_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.DELETE_activatedUser_REJECTED
  readonly payload: any
}

export type AllActions =
  DELETE_activatedUser_PENDINGAction
  | DELETE_activatedUser_FULFILLEDAction
  | DELETE_activatedUser_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_activatedUser_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.DELETE_activatedUser_FULFILLED:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.DELETE_activatedUser_REJECTED:
      Logger.error('rejected delete data, prev state', state)
      return {
        ...state,
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

