/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {isValidDialog} from "../../../../validation/dialogValidations";
import {ActionBase, ActionType} from "./changeActivatedUserDataDialogActionTypes";
import {
  State as PreState,
  initial as preInitial,
  validationRules,
  AllActions as SetActions,
  reducer as setReducer
} from './otherUserDataReducer'
import {reducer as updateReducer, AllActions as UpdateActions} from './updateActivatedUserDataReducer'
import {
  reducer as updateSystemRoleReducer,
  AllActions as UpdateSystemRoleActions
} from './updateActivatedUserSystemRoleReducer'
import {reducer as resetPasswordReducer, AllActions as ResetPasswordActions} from './resetActivatedUserPasswordReducer'


export type State = {

  readonly userData: PreState

  readonly isValid: boolean
  readonly isBusy: boolean
}

export const initial: State = {
  userData: preInitial,
  isBusy: false,
  isValid: isValidDialog(preInitial, validationRules)
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SetActions
  | UpdateActions
  | UpdateSystemRoleActions
  | ResetPasswordActions

export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_activatedUserData_PENDING:
    case ActionType.UPDATE_activatedUserData_FULFILLED:
    case ActionType.UPDATE_activatedUserData_REJECTED:
      return updateReducer(state, action)


    case ActionType.UPDATE_activatedUserSystemRoleData_PENDING:
    case ActionType.UPDATE_activatedUserSystemRoleData_FULFILLED:
    case ActionType.UPDATE_activatedUserSystemRoleData_REJECTED:
      return updateSystemRoleReducer(state, action)

    case ActionType.RESET_activateUserPassword_PENDING:
    case ActionType.RESET_activateUserPassword_FULFILLED:
    case ActionType.RESET_activateUserPassword_REJECTED:
      return resetPasswordReducer(state, action)

    case ActionType.SET_targetUserId:
    case ActionType.SET_email:
    case ActionType.SET_lastName:
    case ActionType.SET_firstName:
    case ActionType.SET_password:
    case ActionType.SET_systemRole:
      const temp = setReducer(state.userData, action)
      return {
        ...state,
        userData: temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

