/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionBase, ActionType} from "./changeActivatedUserDataDialogActionTypes";
import {State, initial} from "./changeActivatedUserDataDialogReducer";
import Logger from "../../../../helpers/logger";


export interface RESET_activateUserPasswordAction
  extends ActionBase, ReduxPromiseAction<string, RESET_activateUserPassword_PENDINGAction, RESET_activateUserPassword_FULFILLEDAction, RESET_activateUserPassword_REJECTEDAction> {
  readonly type: ActionType.RESET_activateUserPassword
  readonly payload: Promise<string>
}

//automatically dispatched by redux promise middleware
export interface RESET_activateUserPassword_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RESET_activateUserPassword_PENDING
}

//automatically dispatched by redux promise middleware
export interface RESET_activateUserPassword_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<string> {
  readonly type: ActionType.RESET_activateUserPassword_FULFILLED
  readonly payload: string
}

//automatically dispatched by redux promise middleware
export interface RESET_activateUserPassword_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RESET_activateUserPassword_REJECTED
  readonly payload: any
}


export type AllActions =
  RESET_activateUserPassword_PENDINGAction
  | RESET_activateUserPassword_FULFILLEDAction
  | RESET_activateUserPassword_REJECTEDAction


//password reset comes from server and should be always valid --> no isValid update
export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RESET_activateUserPassword_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.RESET_activateUserPassword_FULFILLED:
      return {
        ...state,
        isBusy: false,
        userData: {
          ...state.userData,
          password: action.payload
        }
      }
    case ActionType.RESET_activateUserPassword_REJECTED:
      return {
        ...state,
        isBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

