/**
 * Created by janis dähne.
 */

import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./manageActivatedUsersSiteActionTypes";
import {State, initial} from "./manageActivatedUsersSiteReducer";
import Logger from "../../../helpers/logger";
import {notExhaustive} from "../_notExhausiveHelper";
import {UserPreviewFromBackend} from "../../../types/users";
import {toSelectableUsersWithSystemRoles} from "../../../communicationLayer/usersLayer";
import {SystemRoleFullBase} from "../../../types/userData";


export type Meta = {
  readonly systemRoles: ReadonlyArray<SystemRoleFullBase>
  readonly currentUserId: number
}

export interface GET_activatedUsersAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<UserPreviewFromBackend>, GET_activatedUsers_PENDINGAction, GET_activatedUsers_FULFILLEDAction, GET_activatedUsers_REJECTEDAction, Meta> {
  readonly type: ActionType.GET_activatedUsers
  readonly payload: Promise<ReadonlyArray<UserPreviewFromBackend>>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface GET_activatedUsers_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_activatedUsers_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_activatedUsers_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<UserPreviewFromBackend>, Meta> {
  readonly type: ActionType.GET_activatedUsers_FULFILLED
  readonly payload: ReadonlyArray<UserPreviewFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_activatedUsers_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<ReadonlyArray<UserPreviewFromBackend>, Meta> {
  readonly type: ActionType.GET_activatedUsers_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_activatedUsers_PENDINGAction
  | GET_activatedUsers_FULFILLEDAction
  | GET_activatedUsers_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_activatedUsers_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_activatedUsers_FULFILLED:
      return {
        ...state,
        activatedUsers: toSelectableUsersWithSystemRoles(action.payload, action.meta.systemRoles,
          action.meta.currentUserId),
        isLoading: false
      }
    case ActionType.GET_activatedUsers_REJECTED:
      return {
        ...state,
        activatedUsers: [],
        isLoading: false
      }

    default:
      notExhaustive(action)
      return state
  }
}

