/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./manageCustomProjectsSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import {CustomProjectPreviewFullBase} from "../../../types/customProjects";

import {
  AllActions as GetCustomProjectOverviewActions,
  reducer as getCustomProjectOverviews
} from './getCustomProjectOverviewsReducer'
import {SortDirection} from "../../../types/sortDirection";

import {
  AllActions as DeleteCustomProjectActions,
  reducer as deleteCustomProjectReducer
} from './deleteCustomProjectReducer'

export type State = {

  readonly isLoading: boolean
  readonly customProjectPreviews: ReadonlyArray<CustomProjectPreviewFullBase>

  readonly sortByKey: keyof CustomProjectPreviewFullBase
  readonly sortDirection: SortDirection
}

export const initial: State = {
  isLoading: false,
  customProjectPreviews: [],
  sortByKey: 'displayName',
  sortDirection: SortDirection.ascending
}


export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof CustomProjectPreviewFullBase
  readonly sortDirection: SortDirection
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_sortByKeyAction

  | GetCustomProjectOverviewActions
  | DeleteCustomProjectActions


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.DELETE_customProject_PENDING:
    case ActionType.DELETE_customProject_FULFILLED:
    case ActionType.DELETE_customProject_REJECTED:
      return deleteCustomProjectReducer(state, action)

    case ActionType.GET_customProjectOverviews_PENDING:
    case ActionType.GET_customProjectOverviews_FULFILLED:
    case ActionType.GET_customProjectOverviews_REJECTED:
      return getCustomProjectOverviews(state, action)

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.RESET:
      return initial

    default:
      //notExhaustive(action)
      return state
  }
}

