/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./manageNewUsersSiteActionTypes";
import {State, initial} from "./manageNewUsersSiteReducer";
import {notExhaustive} from "../_notExhausiveHelper";
import {SystemRoleFullBase} from "../../../types/userData";
import Logger from "../../../helpers/logger";


export type Meta = {
  readonly shouldHideLoading: boolean
}

export interface GET_systemRolesAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<SystemRoleFullBase>, GET_systemRoles_PENDINGAction, GET_systemRoles_FULFILLEDAction, GET_systemRoles_REJECTEDAction, Meta> {
  readonly type: ActionType.GET_systemRoles
  readonly payload: Promise<ReadonlyArray<SystemRoleFullBase>>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface GET_systemRoles_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_systemRoles_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_systemRoles_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<SystemRoleFullBase>, Meta> {
  readonly type: ActionType.GET_systemRoles_FULFILLED
  readonly payload: ReadonlyArray<SystemRoleFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_systemRoles_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.GET_systemRoles_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_systemRoles_PENDINGAction
  | GET_systemRoles_FULFILLEDAction
  | GET_systemRoles_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_systemRoles_PENDING:
      return {
        ...state,
        isLoading: true
      }
    case ActionType.GET_systemRoles_FULFILLED:
      return {
        ...state,
        systemRoles: action.payload,
        isLoading: action.meta.shouldHideLoading ? false : state.isLoading
      }
    case ActionType.GET_systemRoles_REJECTED:
      return {
        ...state,
        systemRoles: [],
        isLoading: action.meta.shouldHideLoading ? false : state.isLoading
      }

    default:
      notExhaustive(action)
      return state
  }
}

