/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../_notExhausiveHelper";
import {TagFullBase} from "../../../../types/tag";
import {isValidDialog} from "../../../../validation/dialogValidations";
import {ActionType, ActionBase} from "./changeTagDialogActionTypes";
import {AllActions as TagActions, reducer as tagReducer, validationRules, initial as preInitial} from './tagReducer'
import {AllActions as CreateActions, reducer as createReducer} from './createTagReducer'
import {AllActions as UpdateActions, reducer as updateReducer} from './updateTagReducer'

export type State = {
  readonly tag: TagFullBase
  readonly isValid: boolean
  readonly isBusy: boolean

  readonly isColorPickerOpened: boolean
  readonly isBackgroundColorPickerOpened: boolean

  /**
   * true: create entity
   * false: change entity,
   */
  readonly isCreating: boolean
}


export const initial: State = {
  tag: preInitial,
  isBusy: false,
  isCreating: false,
  isColorPickerOpened: false,
  isBackgroundColorPickerOpened: false,
  isValid: isValidDialog(preInitial, validationRules)
}

export interface SET_isCreatingAction extends ActionBase {
  readonly type: ActionType.SET_isCreating
  readonly isCreating: boolean
}

export interface SET_isColorPickerOpenedAction extends ActionBase {
  readonly type: ActionType.SET_isColorPickerOpened
  readonly isOpened: boolean
}

export interface SET_isBackgroundColorPickerOpenedAction extends ActionBase {
  readonly type: ActionType.SET_isBackgroundColorPickerOpened
  readonly isOpened: boolean
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_isCreatingAction
  | SET_isColorPickerOpenedAction
  | SET_isBackgroundColorPickerOpenedAction
  | TagActions
  | CreateActions
  | UpdateActions


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {
    case ActionType.SET_id:
    case ActionType.SET_displayName:
    case ActionType.SET_description:
    case ActionType.SET_htmlColor:
    case ActionType.SET_htmlBackgroundColor:
    case ActionType.SET_htmlIcon:
      const temp = tagReducer(state.tag, action)
      return {
        ...state,
        tag: temp,
        isValid: isValidDialog(temp, validationRules)
      }

    case ActionType.CREATE_tag_PENDING:
    case ActionType.CREATE_tag_FULFILLED:
    case ActionType.CREATE_tag_REJECTED:
      return createReducer(state, action)


    case ActionType.UPDATE_tag_PENDING:
    case ActionType.UPDATE_tag_FULFILLED:
    case ActionType.UPDATE_tag_REJECTED:
      return updateReducer(state, action)

    case ActionType.SET_isCreating:
      return {
        ...state,
        isCreating: action.isCreating
      }

    case ActionType.SET_isColorPickerOpened:
      return {
        ...state,
        isColorPickerOpened: action.isOpened
      }

    case ActionType.SET_isBackgroundColorPickerOpened:
      return {
        ...state,
        isBackgroundColorPickerOpened: action.isOpened
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

