/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "./changeTagDialogActionTypes";
import {TagFullBase} from "../../../../types/tag";
import {
  getValidationCollection, getValidationMessagesCollection,
  isNotEmptyOrSpaces
} from "../../../../validation/validations";
import {notExhaustive} from "../../_notExhausiveHelper";
import {TranslateIds} from "../../../../../i18n/ids";
import {translate} from "../../../../../i18n/translationHelper";

export type State = TagFullBase

export const validationRules = getValidationCollection<State>({
  id: [],
  displayName: [isNotEmptyOrSpaces],
  description: [],
  htmlColor: [],
  htmlBackgroundColor: [],
  htmlIcon: [],
})

export const validationMessageKeys = getValidationMessagesCollection<State>({
  id: '',
  displayName: translate(TranslateIds.fieldShouldNotBeEmptyOrSpaces),
  description: '',
  htmlColor: '',
  htmlBackgroundColor: '',
  htmlIcon: '',
})


export const initial: State = {
  id: -1,
  displayName: '',
  description: '',
  htmlColor: '',
  htmlBackgroundColor: '',
  htmlIcon: '',
}

export interface SET_idAction extends ActionBase {
  readonly type: ActionType.SET_id
  readonly id: number
}

export interface SET_displayNameAction extends ActionBase {
  readonly type: ActionType.SET_displayName
  readonly displayName: string
}

export interface SET_descriptionAction extends ActionBase {
  readonly type: ActionType.SET_description
  readonly description: string
}

export interface SET_htmlColorAction extends ActionBase {
  readonly type: ActionType.SET_htmlColor
  readonly htmlColor: string
}

export interface SET_htmlBackgroundColorAction extends ActionBase {
  readonly type: ActionType.SET_htmlBackgroundColor
  readonly htmlBackgroundColor: string
}

export interface SET_htmlIconAction extends ActionBase {
  readonly type: ActionType.SET_htmlIcon
  readonly htmlIcon: string
}

export type AllActions =
  SET_idAction
  | SET_displayNameAction
  | SET_descriptionAction
  | SET_htmlColorAction
  | SET_htmlBackgroundColorAction
  | SET_htmlIconAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {
    case ActionType.SET_id:
      return {
        ...state,
        id: action.id
      }
    case ActionType.SET_displayName:
      return {
        ...state,
        displayName: action.displayName
      }
    case ActionType.SET_description:
      return {
        ...state,
        description: action.description
      }
    case ActionType.SET_htmlColor:
      return {
        ...state,
        htmlColor: action.htmlColor
      }
    case ActionType.SET_htmlBackgroundColor:
      return {
        ...state,
        htmlBackgroundColor: action.htmlBackgroundColor
      }
    case ActionType.SET_htmlIcon:
      return {
        ...state,
        htmlIcon: action.htmlIcon
      }

    default:
      notExhaustive(action)
      return state
  }
}