/**
 * Created by janis dähne.
 */

import {Action} from "redux";
import {notExhaustive} from "../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {ActionType, ActionBase} from './changeTagDialogActionTypes'
import {TagFullBase} from "../../../../types/tag";
import {initial, State} from "./changeTagDialogReducer";

export interface ActionBase extends Action {
  readonly type: ActionType
}

export interface UPDATE_tagAction
  extends ActionBase, ReduxPromiseAction<TagFullBase, UPDATE_tag_PENDINGAction, UPDATE_tag_FULFILLEDAction, UPDATE_tag_REJECTEDAction> {
  readonly type: ActionType.UPDATE_tag
  readonly payload: Promise<TagFullBase>
}

//automatically dispatched by redux promise middleware
export interface UPDATE_tag_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.UPDATE_tag_PENDING
}

//automatically dispatched by redux promise middleware
export interface UPDATE_tag_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<TagFullBase> {
  readonly type: ActionType.UPDATE_tag_FULFILLED
  readonly payload: TagFullBase
}

//automatically dispatched by redux promise middleware
export interface UPDATE_tag_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<TagFullBase> {
  readonly type: ActionType.UPDATE_tag_REJECTED
  readonly payload: any
}


export type AllActions =
  UPDATE_tag_PENDINGAction
  | UPDATE_tag_FULFILLEDAction
  | UPDATE_tag_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.UPDATE_tag_PENDING:
      return {
        ...state,
        isBusy: true
      }
    case ActionType.UPDATE_tag_FULFILLED:
      return {
        ...state,
        isBusy: false
      }
    case ActionType.UPDATE_tag_REJECTED:
      return {
        ...state,
        isBusy: false
      }

    default:
      notExhaustive(action)
      return state
  }
}


