/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionType, ActionBase} from "./settingsSiteActionTypes";
import {initial, State} from "./settingsSiteReducers";
import Logger from "../../../helpers/logger";
import {LanguageFromBackend} from "../../../types/language";


export interface GET_languagesAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<LanguageFromBackend>, GET_languages_PENDINGAction, GET_languages_FULFILLEDAction, GET_languages_REJECTEDAction> {
  readonly type: ActionType.GET_languages
  readonly payload: Promise<ReadonlyArray<LanguageFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface GET_languages_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_languages_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_languages_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<LanguageFromBackend>> {
  readonly type: ActionType.GET_languages_FULFILLED
  readonly payload: ReadonlyArray<LanguageFromBackend>
}

//automatically dispatched by redux promise middleware
export interface GET_languages_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_languages_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_languages_PENDINGAction
  | GET_languages_FULFILLEDAction
  | GET_languages_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_languages_PENDING:
      return {
        ...state,
        isLoadingMiscSettings: true,
        isLoadingUserData: true,
      }
    case ActionType.GET_languages_FULFILLED:
      return {
        ...state,
        isLoadingMiscSettings: true,
        isLoadingUserData: true,
        langs: action.payload
      }
    case ActionType.GET_languages_REJECTED:
      return {
        ...state,
        isLoadingMiscSettings: true,
        isLoadingUserData: true,
        langs: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

