/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../index";
import {ActionBase, ActionType} from "./settingsSiteActionTypes";
import {initial, State, userDataInitial} from "./settingsSiteReducers";
import {UserDataFullBase} from "../../../types/userData";
import Logger from "../../../helpers/logger";
import {initial as userBaseDataInitial} from './userDataCopyReducer'
import {defaultUserSettings as userSettingsInitial} from './userSettingsCopyReducer'

export interface GET_userDataAction
  extends ActionBase, ReduxPromiseAction<UserDataFullBase, GET_userData_PENDINGAction, GET_userData_FULFILLEDAction, GET_userData_REJECTEDAction> {
  readonly type: ActionType.GET_userData
  readonly payload: Promise<UserDataFullBase>
}

//automatically dispatched by redux promise middleware
export interface GET_userData_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_userData_PENDING
}

//automatically dispatched by redux promise middleware
export interface GET_userData_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<UserDataFullBase> {
  readonly type: ActionType.GET_userData_FULFILLED
  readonly payload: UserDataFullBase
}

//automatically dispatched by redux promise middleware
export interface GET_userData_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.GET_userData_REJECTED
  readonly payload: any
}


export type AllActions =
  GET_userData_PENDINGAction
  | GET_userData_FULFILLEDAction
  | GET_userData_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_userData_PENDING:
      return {
        ...state,
        isLoadingMiscSettings: true,
        isLoadingUserData: true,
      }
    case ActionType.GET_userData_FULFILLED:
      return {
        ...state,
        isLoadingMiscSettings: false,
        isLoadingUserData: false,
        userData: action.payload,
        userDataCopy: {
          email: action.payload.email,
          oldPassword: '',
          newPassword: '',
          newPasswordCheck: '',
          isValid: false
        },
        userSettingsCopy: action.payload.settings
          ? {
            ...action.payload.settings
          }
          : userSettingsInitial
      }

    case ActionType.GET_userData_REJECTED:
      return {
        ...state,
        isLoadingMiscSettings: false,
        isLoadingUserData: false,
        userData: userDataInitial,
        userDataCopy: userBaseDataInitial,
        userSettingsCopy: userSettingsInitial
      }

    default:
      notExhaustive(action)
      return state
  }
}

