/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../../_notExhausiveHelper";
import {ActionBase, ActionType} from "./addsUserDialogActionTypes";
import {AddableUserFrontendExclusive} from "../../../../../types/users";
import {AllActions as GetUsersActions, reducer as getUsersReducer} from './getUsersReducer'
import {AllActions as AddMembersActions, reducer as addMembersReducer} from './addMembersReducer'
import {SelectableGroupUserFrontend} from "../../../../../types/group";
import {SortDirection} from "../../../../../types/sortDirection";


export type State = {
  /**
   * all possible users that could be added to a group
   * the system roles are not set!
   */
  readonly users: ReadonlyArray<AddableUserFrontendExclusive>


  /**
   * true: if at least 1 user is selected,
   * false: else
   */
  readonly isValid: boolean

  readonly isBusy: boolean

  readonly sortByKey: keyof AddableUserFrontendExclusive
  readonly sortDirection: SortDirection
}

export const initial: State = {
  users: [],
  isValid: false,
  isBusy: false,
  sortByKey: 'lastName',
  sortDirection: SortDirection.ascending,
}

export interface SET_user_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_user_uiIsSelected
  readonly isSelected: boolean
  readonly user: AddableUserFrontendExclusive
}

export interface SET_all_users_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_all_users_uiIsSelected
  readonly isSelected: boolean
}

export interface INVERT_allSelectionsAction extends ActionBase {
  readonly type: ActionType.INVERT_allSelections
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof AddableUserFrontendExclusive
  readonly sortDirection: SortDirection
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_user_uiIsSelectedAction
  | SET_all_users_uiIsSelectedAction
  | GetUsersActions
  | INVERT_allSelectionsAction
  | AddMembersActions
  | SET_sortByKeyAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_users_PENDING:
    case ActionType.GET_users_FULFILLED:
    case ActionType.GET_users_REJECTED:
      return getUsersReducer(state, action)

    case ActionType.ADD_members_PENDING:
    case ActionType.ADD_members_FULFILLED:
    case ActionType.ADD_members_REJECTED:
      return addMembersReducer(state, action)

    case ActionType.SET_user_uiIsSelected:
      return {
        ...state,
        users: state.users.map(user => {
          return user.id === action.user.id ?
            {
              ...user,
              uiIsSelected: action.isSelected
            }
            : user
        })
      }

    case ActionType.SET_all_users_uiIsSelected:
      return {
        ...state,
        users: state.users.map(user => {
          return {
            ...user,
            uiIsSelected: action.isSelected
          }
        })
      }

    case ActionType.INVERT_allSelections:
      return {
        ...state,
        users: state.users.map(user => {
          return {
            ...user,
            uiIsSelected: !user.uiIsSelected
          }
        })
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.RESET:
      return initial

    default:
      notExhaustive(action)
      return state
  }
}

