/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../../../_notExhausiveHelper";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../../index";
import {ActionBase, ActionType} from "./addsUserDialogActionTypes";
import {initial, State} from "./addUsersDialogReducer";
import Logger from "../../../../../helpers/logger";
import {SelectableUserFrontendExclusive, UserPreviewFromBackend} from "../../../../../types/users";
import {toAddableUser} from "../../../../../communicationLayer/groupsLayer";
import {SelectableGroupUserFrontend} from "../../../../../types/group";


export type Meta = {
  readonly currentUserId: number
  readonly members: ReadonlyArray<SelectableGroupUserFrontend>
}

export interface GET_usersAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<UserPreviewFromBackend>, GET_users_PENDINGAction, GET_users_FULFILLEDAction, GET_users_REJECTEDAction, Meta> {
  readonly type: ActionType.GET_users
  readonly payload: Promise<ReadonlyArray<UserPreviewFromBackend>>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface GET_users_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.GET_users_PENDING
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface GET_users_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<UserPreviewFromBackend>, Meta> {
  readonly type: ActionType.GET_users_FULFILLED
  readonly payload: ReadonlyArray<UserPreviewFromBackend>
  readonly meta: Meta
}

//automatically dispatched by redux promise middleware
export interface GET_users_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void, Meta> {
  readonly type: ActionType.GET_users_REJECTED
  readonly payload: any
  readonly meta: Meta
}


export type AllActions =
  GET_users_PENDINGAction
  | GET_users_FULFILLEDAction
  | GET_users_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.GET_users_PENDING:
      return {
        ...state,
        isBusy: true,
      }
    case ActionType.GET_users_FULFILLED:
      return {
        ...state,
        isBusy: false,
        users: toAddableUser( //only add user if they are not already in this group (a member)
          action.payload
            .filter(p =>
              action.meta.members.some(member => member.id === p.id) === false
            ),
          action.meta.currentUserId)
      }
    case ActionType.GET_users_REJECTED:
      return {
        ...state,
        isBusy: false,
        users: []
      }

    default:
      notExhaustive(action)
      return state
  }
}

