/**
 * Created by janis dähne.
 */
import {SelectableGroupUserFrontend, UserGroupFullBase} from "../../../types/group";
import {notExhaustive} from "../_notExhausiveHelper";
import {SiteViewState} from "../../../types/siteView";
import {ActionBase, ActionType} from "./singleGroupSiteActionTypes";
import {GroupRoleWithPermissionFullBase} from "../../../types/groupRoles";
import {AllActions as GetActions, reducer as getReducer} from './getSingleGroupReducer'
import {AllActions as GetGroupRolesActions, reducer as getGroupRolesReducer} from './getGroupRolesReducer'
import {AllActions as GetGroupsActions, reducer as getGroupsReducer} from './getGroupsReducer'
import {
  AllActions as SetMemberGroupRoleActions,
  reducer as setMemberGroupRoleReducer
} from './setMemberGroupRoleReducer'
import {AllActions as RemoveMemberActions, reducer as removeMemberReducer} from './removeMemberReducer'
import {ActionType as GroupsSiteActionTypes} from '../groupsSite/groupsSiteActionTypes'
import {ActionType as GroupRolesSiteActionTypes} from '../groupRolesSite/groupRolesSiteActionTypes'
import {SortDirection} from "../../../types/sortDirection";
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";


//TODO make only 1 request for members + roles + group data
export type State = {
  readonly selectedGroupId: number
  readonly members: ReadonlyArray<SelectableGroupUserFrontend>
  readonly groupRoles: ReadonlyArray<GroupRoleWithPermissionFullBase>
  readonly groups: ReadonlyArray<UserGroupFullBase>
  readonly viewState: SiteViewState
  readonly isLoading: boolean
  readonly isAddUsersDialogDisplayed: boolean

  readonly sortByKey: keyof SelectableGroupUserFrontend
  readonly sortDirection: SortDirection
}


export const initial: State = {
  members: [],
  groupRoles: [],
  groups: [],
  selectedGroupId: -1,
  viewState: SiteViewState.List,
  isLoading: false,
  isAddUsersDialogDisplayed: false,

  sortByKey: 'lastName',
  sortDirection: SortDirection.ascending
}


export interface SET_selectedGroupIdAction extends ActionBase {
  readonly type: ActionType.SET_selectedGroupId
  readonly groupId: number
}

export interface SET_isAddUsersDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isAddUsersDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_member_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_member_uiIsSelected
  readonly memberId: number
  readonly isSelected: boolean
}

export interface SET_all_member_uiIsSelectedAction extends ActionBase {
  readonly type: ActionType.SET_all_member_uiIsSelected
  readonly isSelected: boolean
}


export interface SET_viewStateAction extends ActionBase {
  readonly type: ActionType.SET_viewState
  readonly viewState: SiteViewState
}

export interface SET_user_uiIsChangeGroupRolePopupDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_user_uiIsChangeGroupRolePopupDisplayed
  readonly isDisplayed: boolean
  readonly userId: number
}

export interface SET_sortByKeyAction extends ActionBase {
  readonly type: ActionType.SET_sortByKey
  readonly sortByKey: keyof SelectableGroupUserFrontend
  readonly sortDirection: SortDirection
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
  readonly resetGroups: boolean
}

export type AllActions =
  ResetAction
  | SET_selectedGroupIdAction
  | GetActions
  | SET_viewStateAction
  | GetGroupRolesActions
  | GetGroupsActions

  | SET_isAddUsersDialogDisplayedAction
  | SetMemberGroupRoleActions
  | SET_user_uiIsChangeGroupRolePopupDisplayedAction
  | RemoveMemberActions
  | SET_member_uiIsSelectedAction
  | SET_all_member_uiIsSelectedAction
  | SET_sortByKeyAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.REMOVE_member_PENDING:
    case ActionType.REMOVE_member_FULFILLED:
    case ActionType.REMOVE_member_REJECTED:
      return removeMemberReducer(state, action)

    case GroupsSiteActionTypes.GET_groups_PENDING:
    case GroupsSiteActionTypes.GET_groups_FULFILLED:
    case GroupsSiteActionTypes.GET_groups_REJECTED:
      return getGroupsReducer(state, action)

    case GroupRolesSiteActionTypes.GET_groupRoles_PENDING:
    case GroupRolesSiteActionTypes.GET_groupRoles_FULFILLED:
    case GroupRolesSiteActionTypes.GET_groupRoles_REJECTED:
      return getGroupRolesReducer(state, action)

    case ActionType.GET_members_PENDING:
    case ActionType.GET_members_FULFILLED:
    case ActionType.GET_members_REJECTED:
      return getReducer(state, action)

    case ActionType.SET_memberGroupRole_PENDING:
    case ActionType.SET_memberGroupRole_FULFILLED:
    case ActionType.SET_memberGroupRole_REJECTED:
      return setMemberGroupRoleReducer(state, action)


    case ActionType.SET_member_uiIsSelected:
      return {
        ...state,
        members: state.members.map(member => {
          return member.id === action.memberId ?
            {
              ...member,
              uiIsSelected: action.isSelected
            }
            : member
        })
      }

    case ActionType.SET_all_member_uiIsSelected:
      return {
        ...state,
        members: state.members.map(member => {
          return {
            ...member,
            uiIsSelected: member.isCurrentUser ? false : action.isSelected
          }
        })
      }

    case ActionType.SET_user_uiIsChangeGroupRolePopupDisplayed:
      return {
        ...state,
        members: state.members.map(member => {
          return member.id === action.userId ?
            {
              ...member,
              uiIsChangeGroupRolePopupDisplayed: action.isDisplayed
            }
            : member
        })
      }

    case ActionType.SET_isAddUsersDialogDisplayed:
      return {
        ...state,
        isAddUsersDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_selectedGroupId:
      return {
        ...state,
        selectedGroupId: action.groupId
      }

    case ActionType.SET_viewState:

      FrontendSettingsManager.setSingleGroupSiteViewState(action.viewState)

      return {
        ...state,
        viewState: action.viewState
      }

    case ActionType.SET_sortByKey:
      return {
        ...state,
        sortByKey: action.sortByKey,
        sortDirection: state.sortByKey !== action.sortByKey ? SortDirection.ascending : action.sortDirection % 2
      }

    case ActionType.RESET:

      console.log("reset")
      console.log(FrontendSettingsManager.getFrontendSettings().singleGroupSiteViewState)
      console.log(FrontendSettingsManager.getFrontendSettings().singleGroupSiteViewState)

      return {
        ...initial,
        groups: action.resetGroups ? [] : state.groups,
        viewState: FrontendSettingsManager.getFrontendSettings().singleGroupSiteViewState
      }

    default:
      notExhaustive(action)
      return state
  }
}

