/**
 * Created by janis dähne.
 */
import {ActionBase, ActionType} from "../tutorViewSiteActionTypes";
import {initial, State} from "../tutorViewSiteReducer";
import Logger from "../../../../helpers/logger";
import {ReduxPromiseAction, ReduxPromiseFulfilledAction, ReduxPromiseRejectedAction} from "../../index";
import {notExhaustive} from "../../_notExhausiveHelper";
import {TestAnswerFromBackend} from "../../../../types/testsAndTestProtocol";
import {mergeSubmitTestResults} from "../../../../helpers/convertersAndTransformers";


export interface RUN_submitTestAsTutorAction
  extends ActionBase, ReduxPromiseAction<ReadonlyArray<TestAnswerFromBackend>, RUN_submitTestAsTutor_PENDINGAction, RUN_submitTestAsTutor_FULFILLEDAction, RUN_submitTestAsTutor_REJECTEDAction> {
  readonly type: ActionType.RUN_submitTestAsTutor
  readonly payload: Promise<ReadonlyArray<TestAnswerFromBackend>>
}

//automatically dispatched by redux promise middleware
export interface RUN_submitTestAsTutor_PENDINGAction
  extends ActionBase {
  readonly type: ActionType.RUN_submitTestAsTutor_PENDING
}

//automatically dispatched by redux promise middleware
export interface RUN_submitTestAsTutor_FULFILLEDAction
  extends ActionBase, ReduxPromiseFulfilledAction<ReadonlyArray<TestAnswerFromBackend>> {
  readonly type: ActionType.RUN_submitTestAsTutor_FULFILLED
  readonly payload: ReadonlyArray<TestAnswerFromBackend>
}

//automatically dispatched by redux promise middleware
export interface RUN_submitTestAsTutor_REJECTEDAction
  extends ActionBase, ReduxPromiseRejectedAction<void> {
  readonly type: ActionType.RUN_submitTestAsTutor_REJECTED
  readonly payload: any
}


export type AllActions =
  RUN_submitTestAsTutor_PENDINGAction
  | RUN_submitTestAsTutor_FULFILLEDAction
  | RUN_submitTestAsTutor_REJECTEDAction


export function reducer(state: State = initial, action: AllActions): State {

  switch (action.type) {

    case ActionType.RUN_submitTestAsTutor_PENDING:
      return {
        ...state,
        isLoading: false
      }
    case ActionType.RUN_submitTestAsTutor_FULFILLED:
      return {
        ...state,
        isLoading: false,
        submitTestsForTutor: mergeSubmitTestResults(state.submitTestsForTutor, action.payload)
      }
    case ActionType.RUN_submitTestAsTutor_REJECTED:
      return {
        ...state,
        isLoading: false,
        submitTestsForTutor: mergeSubmitTestResults(state.submitTestsForTutor, [])
      }

    default:
      notExhaustive(action)
      return state
  }
}

