/**
 * Created by janis dähne.
 */

import {ActionBase, ActionType} from "./tutorViewSiteActionTypes";
import {notExhaustive} from "../_notExhausiveHelper";
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {DoExerciseFrontendExclusive} from "../../../types/doExercise";
import {initial as initalDoExercise} from '../doExerciseSite/doExerciseReducer'
import {AllActions as GetExerciseActions, reducer as getExerciseReducer} from './getExerciseReducer'
import {TestAnswerFromBackend} from "../../../types/testsAndTestProtocol";
import {AssessedSubmitTestFromBackend, AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {SolutionVersionDoExerciseFullBase, SolutionVersionPreview} from "../../../types/exerciseSolution";
import {AllActions as GetUserSolutionActions, reducer as getUserSolutionReducer} from './crud/getUserSolutionReducer'
import {
  initial as initialSolution,
} from '../doExerciseSite/subSets/setDoExerciseSolutionReducer'
import {DoEditorState} from "../doExerciseSite/editorReducers/doEditorReducer";
import {
  AllActions as AllUserSolutionEditorActions,
  reducer as userSolutionEditorReducer,
  initial as initialUserSolutionEditorState
} from './editorReducer/userSolutionTutorViewEditorReducer'

import {
  AllActions as TutorEditorActions,
  reducer as tutorEditorReducer,
  initial as initialTutorEditorState
} from './editorReducer/tutorEditorReducer'

import {
  AllActions as GetAllUserTestResultActions,
  reducer as getAllUserTestResults
} from './crud/getAllUserTestResultsReducer'
import {
  AllActions as GetSubmitTestResultActions,
  reducer as getUserSubmitTestResults
} from './crud/getUserSubmitTestResults'

import {AllActions as SetUserSolutionActions, reducer as setUserSolutionReducer} from './subSets/setUserSolutionReducer'

import {
  AllActions as SetTutorEditorSolutionActions,
  reducer as setTutorEditorSolutionReducer,
  initial as initialTutorEditorSolution
} from './subSets/setTutorEditorSolutionReducer'

import {
  AllActions as RunNormalTestActions,
  reducer as runNormalTestAsTutorReducer
} from './crud/runNormalTestAsTutorReducer'
import {
  AllActions as RunSubmitTestActions,
  reducer as runSubmitTestAsTutorReducer
} from './crud/runSubmitTestAsTutorReducer'
import {AssessmentFullBase} from "../../../types/submissions";
import {
  AllActions as SetAssessmentActions,
  reducer as setAssessmentReducer,
  initial as initialAssessment
} from './subSets/setAssessmentReducer'
import {
  AllActions as GetSingleAssessmentActions,
  reducer as getSingleAssessmentReducer
} from './crud/getSingleAssessmentReducer'

import {
  AllActions as UpdateSingleAssessmentActions,
  reducer as updateSingleAssessmentReducer
} from './crud/updateSingleAssessmentReducer'
import {FrontendSettingsManager} from "../../../helpers/frontendSettingsManager";

export type State = {

  /**
   * null on error
   * set via CRUD
   * managed via setAssessmentReducer
   */
  readonly assessment: AssessmentFullBase | null

  /**
   * set via getExerciseReducer
   * managed via getExerciseReducer
   */
  readonly exercise: DoExerciseFrontendExclusive


  /**
   * normal tests for the tutor to run (results won't be saved to backend)
   */
  readonly normalTestsForTutor: ReadonlyArray<RunnableExerciseTestFrontendExclusive>
  /**
   * submit tests for the tutor to run (results won't be saved to backend)
   */
  readonly submitTestsForTutor: ReadonlyArray<AssessedSubmitTestFrontendExclusive>


  /**
   * the submit test results, only available if the release has ended
   *
   * set via CRUD
   */
  readonly submitTestResults: ReadonlyArray<AssessedSubmitTestFrontendExclusive>

  /**
   * the user solution
   * set via CRUD
   */
  readonly userSolution: SolutionVersionDoExerciseFullBase
  readonly allUserSolutionVersionPreviews: ReadonlyArray<SolutionVersionPreview>

  /**
   * the tutor solution to exeriment with the users solution
   */
  readonly tutorEditorSolution: SolutionVersionDoExerciseFullBase

  /**
   * normally the test results are attached to the tests... but for this command we have no test...
   * we only check if the CURRENT (not saved in db) state of the current file compiles
   *
   * this is set via async crud action callback
   */
  readonly lastCompileSingleFileResult: TestAnswerFromBackend | null

  /**
   * the currently selected test to display in the console view
   *
   * no id because it could be RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive
   */
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null


  /**
   * the editor state for the user solution ... should be read only
   */
  readonly userSolutionEditorState: DoEditorState

  /**
   * editor to test / experiment with the users solution
   */
  readonly tutorEditorState: DoEditorState

  readonly isLoading: boolean

  /**
   * for the console view
   */
  readonly useDiffIfOutputMismatched: boolean
  /**
   * for the console view
   */
  readonly useReplaceWhiteSpacesWithBullets: boolean

  readonly isViewTestDialogDisplayed: boolean
  readonly isCodeEditorSettingsDialogDisplayed: boolean

  readonly topRightSelectedTabIndex: number
  readonly topLeftSelectedTabIndex: number
  readonly bottomRightSelectedTabIndex: number
  readonly bottomLeftSelectedTabIndex: number

  readonly topRightPanelViewSize: PanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize

  readonly hDividerPosOffset: number
  readonly vDividerPosOffset: number



  /**
   * toggle this to force a re render of hte preview panel (when assets are finished creating)
   */
  readonly dummy: boolean
}

export const initial: State = {

  assessment: initialAssessment,
  exercise: initalDoExercise,

  normalTestsForTutor: [],
  submitTestsForTutor: [],


  submitTestResults: [],

  allUserSolutionVersionPreviews: [],
  userSolution: initialSolution,

  tutorEditorSolution: initialTutorEditorSolution,

  lastCompileSingleFileResult: null,
  selectedConsoleViewTest: null,

  userSolutionEditorState: initialUserSolutionEditorState,
  tutorEditorState: initialTutorEditorState,

  isLoading: false,
  useDiffIfOutputMismatched: false,
  useReplaceWhiteSpacesWithBullets: true,

  isViewTestDialogDisplayed: false,
  isCodeEditorSettingsDialogDisplayed: false,

  topRightSelectedTabIndex: 1,
  topLeftSelectedTabIndex: 0,
  bottomRightSelectedTabIndex: 0,
  bottomLeftSelectedTabIndex: 0,

  topRightPanelViewSize: PanelViewSize.normal,
  topLeftPanelViewSize: PanelViewSize.normal,
  botRightPanelViewSize: PanelViewSize.normal,
  botLeftPanelViewSize: PanelViewSize.normal,

  hDividerPosOffset: 0,
  vDividerPosOffset: 150,

  dummy: false
}


export interface SET_normalTestsForTutorAction extends ActionBase {
  readonly type: ActionType.SET_normalTestsForTutor
  readonly testsForTutor: ReadonlyArray<RunnableExerciseTestFrontendExclusive>
}

export interface SET_submitTestsForTutorAction extends ActionBase {
  readonly type: ActionType.SET_submitTestsForTutor
  readonly submitTestsForTutor: ReadonlyArray<AssessedSubmitTestFrontendExclusive>
}

export interface SET_selectedConsoleViewTestAction extends ActionBase {
  readonly type: ActionType.SET_selectedConsoleViewTest
  readonly selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | null
}

export interface SET_dummyAction extends ActionBase {
  readonly type: ActionType.SET_dummy
  readonly dummy: boolean
}

//--- console view options

export interface SET_useDiffIfOutputMismatchedAction extends ActionBase {
  readonly type: ActionType.SET_useDiffIfOutputMismatched
  readonly useDiffIfOutputMismatched: boolean
}

export interface SET_useReplaceWhiteSpacesWithBulletsAction extends ActionBase {
  readonly type: ActionType.SET_useReplaceWhiteSpacesWithBullets
  readonly useReplaceWhiteSpacesWithBullets: boolean
}

//--- dialogs

export interface SET_isCodeEditorSettingsDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isCodeEditorSettingsDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_isViewTestDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isViewTestDialogDisplayed
  readonly isViewTestDialogDisplayed: boolean
}

// --- panel view sizes

export interface SET_topRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topRightPanelViewSize
  readonly topRightPanelViewSize: PanelViewSize
}

export interface SET_topLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topLeftPanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
}

export interface SET_botRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botRightPanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
}

export interface SET_botLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botLeftPanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize
}

//--- panel selected indices

export interface SET_topRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topRightSelectedTabIndex
  readonly topRightSelectedTabIndex: number
}

export interface SET_topLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topLeftSelectedTabIndex
  readonly topLeftSelectedTabIndex: number
}

export interface SET_bottomRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomRightSelectedTabIndex
  readonly bottomRightSelectedTabIndex: number
}

export interface SET_bottomLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomLeftSelectedTabIndex
  readonly bottomLeftSelectedTabIndex: number
}

//--- panel grip

export interface SET_hDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_hDividerPosOffset
  readonly hDividerPosOffset: number
}

export interface SET_vDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_vDividerPosOffset
  readonly vDividerPosOffset: number
}


export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction

  | SET_topRightSelectedTabIndexAction
  | SET_topLeftSelectedTabIndexAction
  | SET_bottomRightSelectedTabIndexAction
  | SET_bottomLeftSelectedTabIndexAction

  | SET_topRightPanelViewSizeAction
  | SET_topLeftPanelViewSizeAction
  | SET_botRightPanelViewSizeAction
  | SET_botLeftPanelViewSizeAction

  | SET_hDividerPosOffsetAction
  | SET_vDividerPosOffsetAction
  | SET_dummyAction

  | SetUserSolutionActions

  | SET_normalTestsForTutorAction
  | SET_submitTestsForTutorAction

  | TutorEditorActions
  | SetTutorEditorSolutionActions
  | SetAssessmentActions

  | RunNormalTestActions
  | RunSubmitTestActions

  | SET_selectedConsoleViewTestAction
  | SET_isViewTestDialogDisplayedAction
  | SET_isCodeEditorSettingsDialogDisplayedAction
  | SET_useDiffIfOutputMismatchedAction
  | SET_useReplaceWhiteSpacesWithBulletsAction

  | AllUserSolutionEditorActions

  | GetExerciseActions
  | GetUserSolutionActions
  | GetAllUserTestResultActions
  | GetSubmitTestResultActions
  | GetSingleAssessmentActions

  | UpdateSingleAssessmentActions


export function reducer(state: State = initial, action: AllActions): State {


  switch (action.type) {

    case ActionType.UPDATE_singleAssessment_PENDING:
    case ActionType.UPDATE_singleAssessment_FULFILLED:
    case ActionType.UPDATE_singleAssessment_REJECTED:
      return updateSingleAssessmentReducer(state, action)

    case ActionType.SET_feedbackForStudent:
    case ActionType.SET_manualAssessmentPoints:
    case ActionType.SET_manualAssessmentNote:
      return {
        ...state,
        assessment: setAssessmentReducer(state.assessment, action)
      }

    case ActionType.GET_singleAssessment_PENDING:
    case ActionType.GET_singleAssessment_FULFILLED:
    case ActionType.GET_singleAssessment_REJECTED:
      return getSingleAssessmentReducer(state, action)

    case ActionType.RUN_normalTestAsTutor_PENDING:
    case ActionType.RUN_normalTestAsTutor_FULFILLED:
    case ActionType.RUN_normalTestAsTutor_REJECTED:
      return runNormalTestAsTutorReducer(state, action)

    case ActionType.RUN_submitTestAsTutor_PENDING:
    case ActionType.RUN_submitTestAsTutor_FULFILLED:
    case ActionType.RUN_submitTestAsTutor_REJECTED:
      return runSubmitTestAsTutorReducer(state, action)

    case ActionType.SET_tutorEditor_hasFileChangedMap:
    case ActionType.SET_tutorEditor_hasUnsavedChanges:
    case ActionType.SET_tutorEditor_selectedTabIdDoEditor:
    case ActionType.SET_tutorEditor_isFileExplorerDisplayedDoEditor:
    case ActionType.SET_tutorEditor_newFileNameDoEditor:
    case ActionType.SET_tutorEditor_renamingFileIdDoEditor:
    case ActionType.SET_tutorEditor_isRenamingFileDoEditor:
    case ActionType.SET_tutorEditor_renamingFileNameDoEditor:
      return {
        ...state,
        tutorEditorState: tutorEditorReducer(state.tutorEditorState, action)
      }

    case ActionType.GET_userSubmitTestResults_PENDING:
    case ActionType.GET_userSubmitTestResults_FULFILLED:
    case ActionType.GET_userSubmitTestResults_REJECTED:
      return getUserSubmitTestResults(state, action)

    case ActionType.GET_allUserTestResults_PENDING:
    case ActionType.GET_allUserTestResults_FULFILLED:
    case ActionType.GET_allUserTestResults_REJECTED:
      return getAllUserTestResults(state, action)

    case ActionType.GET_user_solutionVersion_PENDING:
    case ActionType.GET_user_solutionVersion_FULFILLED:
    case ActionType.GET_user_solutionVersion_REJECTED:
      return getUserSolutionReducer(state, action)

    case ActionType.GET_exercise_PENDING:
    case ActionType.GET_exercise_FULFILLED:
    case ActionType.GET_exercise_REJECTED:
      return getExerciseReducer(state, action)


    case ActionType.SET_normalTestsForTutor:
      return {
        ...state,
        normalTestsForTutor: action.testsForTutor
      }

    case ActionType.SET_submitTestsForTutor:
      return {
        ...state,
        submitTestsForTutor: action.submitTestsForTutor
      }

    case ActionType.SET_tutorEditorSolution_solutionFiles:
    case ActionType.SET_tutorEditorSolution_mainFileId:
      return {
        ...state,
        tutorEditorSolution: setTutorEditorSolutionReducer(state.tutorEditorSolution, action)
      }

    case ActionType.SET_userSolution_solutionFiles:
    case ActionType.SET_userSolution_mainFileId:
      return {
        ...state,
        userSolution: setUserSolutionReducer(state.userSolution, action)
      }

    case ActionType.SET_userSolutionEditor_selectedTabIdDoEditor:
    case ActionType.SET_userSolutionEditor_isFileExplorerDisplayedDoEditor:
      return {
        ...state,
        userSolutionEditorState: userSolutionEditorReducer(state.userSolutionEditorState, action)
      }

    case ActionType.SET_isCodeEditorSettingsDialogDisplayed:
      return {
        ...state,
        isCodeEditorSettingsDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_isViewTestDialogDisplayed:
      return {
        ...state,
        isViewTestDialogDisplayed: action.isViewTestDialogDisplayed
      }

    case ActionType.SET_selectedConsoleViewTest:
      return {
        ...state,
        selectedConsoleViewTest: action.selectedConsoleViewTest
      }

    case ActionType.SET_dummy:
      return {
        ...state,
        dummy: action.dummy
      }
    case ActionType.SET_useDiffIfOutputMismatched:

      FrontendSettingsManager.setUseDiffIfOutputMismatched(action.useDiffIfOutputMismatched)

      return {
        ...state,
        useDiffIfOutputMismatched: action.useDiffIfOutputMismatched
      }

    case ActionType.SET_useReplaceWhiteSpacesWithBullets:

      FrontendSettingsManager.setUseReplaceWhiteSpacesWithBullets(action.useReplaceWhiteSpacesWithBullets)

      return {
        ...state,
        useReplaceWhiteSpacesWithBullets: action.useReplaceWhiteSpacesWithBullets
      }


    case ActionType.SET_topRightSelectedTabIndex:
      return {
        ...state,
        topRightSelectedTabIndex: action.topRightSelectedTabIndex
      }

    case ActionType.SET_topLeftSelectedTabIndex:
      return {
        ...state,
        topLeftSelectedTabIndex: action.topLeftSelectedTabIndex
      }

    case ActionType.SET_bottomRightSelectedTabIndex:
      return {
        ...state,
        bottomRightSelectedTabIndex: action.bottomRightSelectedTabIndex
      }

    case ActionType.SET_bottomLeftSelectedTabIndex:
      return {
        ...state,
        bottomLeftSelectedTabIndex: action.bottomLeftSelectedTabIndex
      }

    case ActionType.SET_hDividerPosOffset:
      return {
        ...state,
        hDividerPosOffset: action.hDividerPosOffset
      }
    case ActionType.SET_vDividerPosOffset:
      return {
        ...state,
        vDividerPosOffset: action.vDividerPosOffset
      }

    case ActionType.SET_topRightPanelViewSize:
      return {
        ...state,
        topRightPanelViewSize: action.topRightPanelViewSize
      }

    case ActionType.SET_topLeftPanelViewSize:
      return {
        ...state,
        topLeftPanelViewSize: action.topLeftPanelViewSize
      }

    case ActionType.SET_botRightPanelViewSize:
      return {
        ...state,
        botRightPanelViewSize: action.botRightPanelViewSize
      }

    case ActionType.SET_botLeftPanelViewSize:
      return {
        ...state,
        botLeftPanelViewSize: action.botLeftPanelViewSize
      }

    case ActionType.RESET:
      return {
        ...initial,
        useDiffIfOutputMismatched: FrontendSettingsManager.getFrontendSettings().useDiffIfOutputMismatched,
        useReplaceWhiteSpacesWithBullets: FrontendSettingsManager.getFrontendSettings().useReplaceWhiteSpacesWithBullets,
      }

    default:
      notExhaustive(action)
      return state
  }
}

