/**
 * Created by janis dähne.
 */


import {ExerciseCodeTemplateFileFullBase} from "./exerciseEditor";
import {Moment} from "moment";
import {AceEditSession} from "../components/codeEditors/multiTabCodeEditorProps";


/**
 *  a preview of a solution version (includes just some basic data so that the user knows about the version (just the id is not so informative))
 */
export interface SolutionVersionPreview {
  /**
   * the id of the solution version
   */
  readonly id: number
  /**
   * a note about the version
   */
  readonly note: string
  /**
   * when the version was created
   */
  readonly createdAt: Moment
}

/**
 * a solution tuple for the backend (includes one solution version and a list of all known solutions versions from the user)
 */
export interface SolutionVersionTupleFromBackend {
  /**
   * the solution version for the frontend
   */
  readonly solutionVersion: SolutionVersionDoExerciseFullBase
  /**
   * all other solution versions that the user has (the solution version from the <see cref="SolutionVersion"/> in this calls is included!)
   */
  readonly allSolutionVersionPreviews: ReadonlyArray<SolutionVersionPreview>

  /**
   * if we change/create a new file in the frontend then we send back the saved version (backend version) to the frontend
   * because the frontend can replace the file (else the undo history would be lost/corrupted) it needs to set the
   * file content directly. for this we need to know which old/frontend file had which content
   */
  readonly fileIdAssociationMap: ReadonlyArray<FileIdAssociation>
}

export interface FileIdAssociation {
  readonly frontendFileId: number
  readonly backendFileId: number
}

/**
 * a solution version backend sends Dates as strings to the frontend so we need a pre type to convert his to a Date
 * see InternalSolutionVersionFromBackend
 */
export interface SolutionVersionDoExerciseFullBase {
  readonly id: number
  /**
   * the main file id (where the entry point is declared, e.g. public static void main...)
   */
  readonly mainFileId: number
  /**
   * the plang the solution version is for
   */
  readonly pLangId: number

  /**
   * a note about the version
   */
  readonly note: string

  /**
   * the datetime when the version was ?
   */
  readonly createdAt: Moment
  /**
   * the files for this solution version
   */
  readonly solutionFiles: ReadonlyArray<SolutionFileDoExerciseFullBase>

  /**
   * this is taken from the users solution
   * true: the student wants that his/her solution is not counted (for whatever), false: not
   * can only be set once by the student
   * NOT SET through this prop
   */
  readonly shouldNotCount: boolean

}

/**
 * a solution file (part of the solution verion)
 */
export interface SolutionFileDoExerciseFullBase extends ExerciseCodeTemplateFileFullBase {

  /**
   * ui property, true: is displayed as tab, false: not (only displayed in project explorer)
   */
  readonly uiIsDisplayed: boolean
  /**
   * null or the template file id from which the file was created
   */
  readonly templateFileId: number | null


}